# Data Logger

## REQ-2 Data Logging

### REQ-2.1
Support MODBUS equipment-tag data as the logging data source.

### REQ-2.2.1
Each profile should be able to configure its:
   - Tag List
   - Maximum Storage Size
   - Upload URL
   - Upload Schedule
   - File Format

### REQ-2.2.2
Besides **MODBUS equipment-tag data**, log file should contain a **date/time**
field filled with tag-reading-time(not storing-time).

### REQ-2.2.3
Logging system should start logging once a profile is configured.

### REQ-2.2.4
**Maximum Storage Size** for a profile shall have a minimum as **10MB**.

### REQ-2.2.5
Whenever the storage size exceeds limit, the oldest log-file would be deleted.(FIFO)

### REQ-2.2.6
Both **IP** and **hostname** should be acceptable for upload URL.

### REQ-2.2.7
Minimum interval for upload schedule is **1 hour** (by the hour) and **offset**
with **minutes** should be provided.

### REQ-2.2.8
Support **XML**, **CSV** and **JSON** as log file format.

### REQ-2.2.9
Log files are compressed in *.gz format.
Log file compression should be able to be enabled or disabled.

### REQ-2.2.10
Provide an **Upload Now** method to upload data log on-demand.

### REQ-2.3
Log files are segmented at 0 minute every hour or by a system-decided size limit.

### REQ-2.4
Filename will be formatted automatically by date/time with the format:
**yyyyMMddhhmm.log**, with date/time at the time of log file creation.

### REQ-2.5
Upload retry is set to **3** times, if all fail, system will wait for
**next schedule**.

### REQ-2.6
Log files shall be received by a standard **HTTP** server with **gzip**
decompression capability.

### REQ-2.7
Support **Log on Change**.

Enable/disable **Log on Change** by Tag.
Users can either enable or disable "Log on Change" of a Tag in a Log Profile
without effecting the configuration of the same Tag in other Log Profiles.

After a Tag is read, only Tags with their value changed are written to log
files.
For example,

All disabled:

```
at,di0@E1210,di1@E1210,di2@E1210
"2015-10-19T03:08:02.004Z","1","1","1"
"2015-10-19T03:08:03.004Z","0","0","1"
"2015-10-19T03:08:04.004Z","0","0","0"
"2015-10-19T03:08:05.004Z","0","0","0"
```

di0 and di1 enabled, di2 disabled:

```
at,di0@E1210,di1@E1210,di2@E1210
"2015-10-19T03:08:02.004Z","1","1","1"
"2015-10-19T03:08:03.004Z","0","0","1"
"2015-10-19T03:08:04.004Z",,,"0"
"2015-10-19T03:08:05.004Z",,,"0"
```

## Components
- mxlogarchiver
- mxlogconverter
- mxloguploader

## Test Cases

### Environment
- UC-8132-LX deployed with v1.5 firmware

### Case 1 - mxlogarchiver

#### Purpose
Check whether `mxlogarchiver` can run properly with:
- **5 profiles**
- Each profile logs **32 tags**.
- Each profile having different size of storage.
- Log-on-change behaves correctly.

#### Steps
1. Transfer `mxlogarchiver`, `setup-ctest-01-mxlogarchiver.py`, `publish-tag` and `mxlogconverter` to UC-8132.

2. Setup component test environment.
    ```
    $ setup-ctest-01-mxlogarchiver.py
    ```

    - Files to place:
        ```
        # /etc/mxlogarchiver/conf.d/profile-0.conf
        archive = {
            directory = "/data/var/mxlogarchiver/logs/profile-0"
            directory-size-limit-byte = 10485760 # 10MB
            threshold-byte-per-file = 5242880
            threshold-sec = 3600
        }

        equipment-list = (
            {
                equipment = "equipment-0"
                tag-list = (
                    {
                        tag = "tag-0"
                    },
                    {
                        tag = "tag-1"
                    },
                    ...
                    {
                        tag = "tag-19"
                    }
                )
            },
            {
                equipment = "equipment-1"
                tag-list = (
                    {
                        tag = "tag-0"
                    },
                    {
                        tag = "tag-1"
                    },
                    ...
                    {
                        tag = "tag-11"
                    }
                )
            }
        )
        ```
        -----------------------------------------------------------------------
        ```
        # /etc/mxlogarchiver/conf.d/profile-1.conf
        archive = {
            directory = "/data/var/mxlogarchiver/logs/profile-1"
            directory-size-limit-byte = 10485760 # 10MB
            threshold-byte-per-file = 5242880
            threshold-sec = 3600
        }

        equipment-list = (
            {
                equipment = "equipment-1"
                tag-list = (
                    {
                        tag = "tag-0"
                    },
                    {
                        tag = "tag-1"
                    },
                    ...
                    {
                        tag = "tag-19"
                    }
                )
            },
            {
                equipment = "equipment-2"
                tag-list = (
                    {
                        tag = "tag-0"
                    },
                    {
                        tag = "tag-1"
                    },
                    ...
                    {
                        tag = "tag-11"
                    }
                )
            }
        )
        ```
        ------------------------------------------------------------------------
        ```
        # /etc/mxlogarchiver/conf.d/profile-2.conf
        archive = {
            directory = "/data/var/mxlogarchiver/logs/profile-2"
            directory-size-limit-byte = 10485760 # 10MB
            threshold-byte-per-file = 5242880
            threshold-sec = 3600
        }

        equipment-list = (
            {
                equipment = "equipment-2"
                tag-list = (
                    {
                        tag = "tag-0"
                    },
                    {
                        tag = "tag-1"
                    },
                    ...
                    {
                        tag = "tag-19"
                    }
                )
            },
            {
                equipment = "equipment-3"
                tag-list = (
                    {
                        tag = "tag-0"
                    },
                    {
                        tag = "tag-1"
                    },
                    ...
                    {
                        tag = "tag-11"
                    }
                )
            }
        )
        ```
        ------------------------------------------------------------------------
        ```
        # /etc/mxlogarchiver/conf.d/hourly.conf
        archive = {
            directory = "/data/var/mxlogarchiver/logs/hourly"
            directory-size-limit-byte = 10485760 # 10MB
            threshold-byte-per-file = 5242880
            threshold-sec = 3600
        }

        equipment-list = (
            {
                equipment = "equipment-0"
                tag-list = (
                    {
                        tag = "tag-0"
                    }
                )
            }
        )
        ```
        ------------------------------------------------------------------------
        ```
        # /etc/mxlogarchiver/conf.d/log-on-change.conf
        archive = {
            directory = "/data/var/mxlogarchiver/logs/log-on-change"
            directory-size-limit-byte = 10485760 # 10MB
            threshold-byte-per-file = 5242880
            threshold-sec = 3600
        }

        equipment-list = (
            {
                equipment = "equipment-3"
                tag-list = (
                    {
                        tag = "tag-0"
                        log-on-change = true
                    },
                    {
                        tag = "tag-1"
                        log-on-change = true
                    },
                    ...
                    {
                        tag = "tag-19"
                        log-on-change = true
                    }
                )
            },
            {
                equipment = "equipment-4"
                tag-list = (
                    {
                        tag = "tag-0"
                        log-on-change = true
                    },
                    {
                        tag = "tag-1"
                        log-on-change = true
                    },
                    ...
                    {
                        tag = "tag-11"
                        log-on-change = true
                    }
                )
            }
        )
        ```

3. Start mxlogarchiver and leave it there.
    ```
    $ cd / && mxlogarchiver -f
    ```

4. Start `publish-tag` from YOUR_HOST_MACHINE and leave it there.
    ```
    $ publish-tag --equ-count 5 --tag-count 20 --period-ms 1000 --mqtt-broker-host IP_OF_UC_8132_LX
    ```

    - Publish topics like: `/equs/equipment-0/tags/tag-0`

#### Expect
1. The size of each log directory should not exceed usage limit and examing
    every time a log file is removed, it was the oldest one.
    ```
    $ du -h /data/var/mxlogarchiver/logs
    ```

2. Log files should be organized hourly.
    ```
    $ ls /data/var/mxlogarchiver/logs/hourly
    ```

3. Only one record for each tag is logged by `log-on-change` profile.
    ```
    $ cat data/var/mxlogarchiver/logs/log-on-change/<LOG_FILE> | mxlogconverter --dump
    ```

#### Traceability
- REQ-2.1
- REQ-2.2.1
- REQ-2.2.3
- REQ-2.2.5
- REQ-2.2.7
- REQ-2.3
- REQ-2.7

### Case 2 - mxlogconverter

#### Purpose
Check whether `mxlogconverter` can convert internal log file into:
    - CSV
    - XML

#### Steps
(none)

#### Expect
1. Run `mxlogconverter` and check its output. Note that `csvlook` is installed through `pip install csvkit`.
    ```
    $ gen_internal_log | gzip | mxlogconverter --format csv | csvlook
    $ echo $?
    ```

    ```
    $ gen_internal_log | gzip | mxlogconverter --format xml > out.xml
    $ xmllint --noout out.xml
    $ echo $?
    ```

    ```
    $ gen_internal_log | gzip | mxlogconverter --format json | python -m simplejson.tool
    ```

#### Traceability
- REQ-2.2.2
- REQ-2.2.8

### Case 3 - mxloguploader

#### Purpose
`mxloguploader` can send log files to different remote server types:
- HTTP or HTTPS
- With or without Basic Authentication
- Gzipped or not

Upload Test functionality.

#### Setup
1. Setup test environment by:
    ```
    $ setup-ctest-03-mxloguploader.py setup YOUR_HOST_MACHINE_IP
    $ gen_internal_log | gzip > /data/var/logs/20150706-145131.log.gz
    ```

2. Generate self-signed certificate on YOUR_HOST_MACHINE.

    ```
    # on YOUR_HOST_MACHINE
    $ cd /path/to/mxcloud-etest/server
    $ openssl genrsa -out rootCA.key 2048
    $ openssl req -x509 -new -nodes -key rootCA.key -days 1 -out mxcloud-etest.crt
    ```

    Fill-in `YOUR_HOST_MACHINE_IP.xip.io` when prompt for *Common Name*.

3. Upload `mxcloud-etest.crt` to UC-8100 and install it.

    - On UC-8100:
        ```
        $ cp /path/to/mxcloud-etest.crt /usr/share/ca-certificates
        # add a line to /etc/ca-certificates.conf
        $ echo mxcloud-etest.crt >> /etc/ca-certificates.conf
        $ update-ca-certificates --fresh
        ```

3. Start HTTP test server on Host.
    ```
    $ python -m server.http
    ```
    ```
    $ python -m server.http_auth
    ```
    ```
    $ python -m server.https
    ```
    ```
    $ python -m server.https_nocert
    ```

#### Expect
1. Run `mxloguploader` with different profile, check whether all files are
    uploaded to Host properly.

    ```
    $ mxloguploader --upload --profile http
    $ mxloguploader --upload --profile http-auth
    $ mxloguploader --upload --profile https
    $ mxloguploader --upload --profile https-nocert
    $ mxloguploader --upload --profile nogzip
    ```

2. Run `mxloguploader` to check its retry behavior.
    `mxloguploader` should retry 3 times and idle about 10 seconds between retries.

    ```
    $ mxloguploader --upload --profile retry
    ```

3. Run `mxloguploader` to check upload-test functionality. Check whether a test
    file with proper content is uploaded.

    ```
    $ mxloguploader --upload-test --file /etc/mxloguploader/conf.d/http.conf
    ```

#### Cleanup
    ```
    $ mxloguploader --clear --profile http
    $ mxloguploader --clear --profile http-auth
    $ mxloguploader --clear --profile https
    $ mxloguploader --clear --profile https-nocert
    $ mxloguploader --clear --profile nogzip
    $ mxloguploader --clear --profile retry

    $ setup-ctest-03-mxloguploader.py teardown
    $ rm /usr/share/ca-certificates/mxcloud-etest.crt
    # remove mxcloud-etest.crt from /etc/ca-certificates.conf
    $ vim /etc/ca-certificates.conf
    $ update-ca-certificates --fresh
    ```

#### Traceability
- REQ-2.2.4
- REQ-2.2.6
- REQ-2.2.9
- REQ-2.2.10
- REQ-2.4
- REQ-2.5
- REQ-2.6
