# Integration Test

## Case 1

### Purpose
User can add 5 upload profiles, each collecting 32 tags, to UC-8132-LX-CG.

User can use `Upload Now` to upload logs on-demand.

### Environment
- UC-8132-LX-CG
- Host running `test-slave`.(fixme)
- Host running `server.http`, `server.https_nocert`, `server.http_auth`.

### Step
1. Navigate to DASHBOARD.LOG_PROFILE_MANAGEMENT.
2. Add test Modbus device:
    1. Add `Test Modbus Equipment` according to [Test Modbus Equipment Template](#test-modbus-equipment-template).
    2. Add Modbus/TCP device
        1. EQUIPMENT\_NAME: `test`
        2. MODEL:`Test Modbus Equipment`
        3. UNIT\_ID: `255`
3. Add Profile according to [Profile Configuration](#profile-configuration)

### Expect
1. Wait for an hour, and logs should be uploaded to Host correctly.
2. Verify filename(`yyyymmdd-HHMM`) and extension(`.csv`, `.xml.gz`...).
3. Click `Upload Now` on one of the profiles. Latest logs should be uploaded to Host.

### Data

#### Test Modbus Equipment Template
| #   | TAG\_NAME | FUNCTION   | ADDRESS | QUANTITY   | TYPE | INV\_VALUE | BYTE\_ORDER | SCALING |
|:---:| ---------:| ---------- | -------:| ----------:| ----:| ----------:|:-----------:| ------- |
| 1   | v1        | Read Coils | 0x0000  | (disabled) | bool | disable    | disable     | disable |
| 2   | v2        | Read Coils | 0x0001  | (disabled) | bool | disable    | disable     | disable |
| 3   | v3        | Read Coils | 0x0002  | (disabled) | bool | disable    | disable     | disable |
| 4   | v4        | Read Coils | 0x0003  | (disabled) | bool | disable    | disable     | disable |
| 5   | v5        | Read Coils | 0x0004  | (disabled) | bool | disable    | disable     | disable |
| 6   | v6        | Read Coils | 0x0005  | (disabled) | bool | disable    | disable     | disable |
| 7   | v7        | Read Coils | 0x0006  | (disabled) | bool | disable    | disable     | disable |
| 8   | v8        | Read Coils | 0x0007  | (disabled) | bool | disable    | disable     | disable |
| 9   | v9        | Read Coils | 0x0008  | (disabled) | bool | disable    | disable     | disable |
| 10  | v10       | Read Coils | 0x0009  | (disabled) | bool | disable    | disable     | disable |
| 11  | v11       | Read Coils | 0x000A  | (disabled) | bool | disable    | disable     | disable |
| 12  | v12       | Read Coils | 0x000B  | (disabled) | bool | disable    | disable     | disable |
| 13  | v13       | Read Coils | 0x000C  | (disabled) | bool | disable    | disable     | disable |
| 14  | v14       | Read Coils | 0x000D  | (disabled) | bool | disable    | disable     | disable |
| 15  | v15       | Read Coils | 0x000E  | (disabled) | bool | disable    | disable     | disable |
| 16  | v16       | Read Coils | 0x000F  | (disabled) | bool | disable    | disable     | disable |
| 17  | v17       | Read Coils | 0x0010  | (disabled) | bool | disable    | disable     | disable |
| 18  | v18       | Read Coils | 0x0011  | (disabled) | bool | disable    | disable     | disable |
| 19  | v19       | Read Coils | 0x0012  | (disabled) | bool | disable    | disable     | disable |
| 20  | v20       | Read Coils | 0x0013  | (disabled) | bool | disable    | disable     | disable |
| 21  | v21       | Read Coils | 0x0014  | (disabled) | bool | disable    | disable     | disable |
| 22  | v22       | Read Coils | 0x0015  | (disabled) | bool | disable    | disable     | disable |
| 23  | v23       | Read Coils | 0x0016  | (disabled) | bool | disable    | disable     | disable |
| 24  | v24       | Read Coils | 0x0017  | (disabled) | bool | disable    | disable     | disable |
| 25  | v25       | Read Coils | 0x0018  | (disabled) | bool | disable    | disable     | disable |
| 26  | v26       | Read Coils | 0x0019  | (disabled) | bool | disable    | disable     | disable |
| 27  | v27       | Read Coils | 0x001A  | (disabled) | bool | disable    | disable     | disable |
| 28  | v28       | Read Coils | 0x001B  | (disabled) | bool | disable    | disable     | disable |
| 29  | v29       | Read Coils | 0x001C  | (disabled) | bool | disable    | disable     | disable |
| 30  | v30       | Read Coils | 0x001D  | (disabled) | bool | disable    | disable     | disable |
| 31  | v31       | Read Coils | 0x001E  | (disabled) | bool | disable    | disable     | disable |
| 32  | v32       | Read Coils | 0x001F  | (disabled) | bool | disable    | disable     | disable |

#### Profile Configuration
There are 5 Profiles that should be configured.
```
Profile Name:
    http
Target URL:
    http://HOST:10080/upload/http
Enable HTTP Authentication:
    uncheck
No Check Certificate:
    uncheck
Storage Size:
    10 MB
Schedule:
    Every Hour at 5 past the hour
File Format:
    csv
Enable Compression:
    check
Equipment:
    test
        v1~v32
```
```
Profile Name:
    http-auth
Target URL:
    http://HOST:20080/upload/http-auth
Enable HTTP Authentication:
    check
Username:
    admin
Password:
    admin
No Check Certificate:
    uncheck
Storage Size:
    10 MB
Schedule:
    Every Hour at 5 past the hour
File Format:
    xml
Enable Compression:
    check
Equipment:
    test
        v1~v32 with log-on-change checked
```
```
Profile Name:
    https-nocert
Target URL:
    https://HOST.xip.io:20443/upload/https-nocert
Enable HTTP Authentication:
    uncheck
No Check Certificate:
    check
Storage Size:
    10485760
Schedule:
    Every Hour at 5 past the hour
File Format:
    csv
Enable Compression:
    uncheck
Equipment:
    test
        v1~v32
```
```
Profile Name:
    http-json
Target URL:
    http://HOST:10080/upload/http-json
Enable HTTP Authentication:
    uncheck
No Check Certificate:
    uncheck
Storage Size:
    10485760
Schedule:
    Every Hour at 5 past the hour
File Format:
    json
Enable Compression:
    uncheck
Equipment:
    test
        v1~v32 with log-on-change checked
```
```
Profile Name:
    http-xml-gz
Target URL:
    http://HOST:10080/upload/http-xml-gz
Enable HTTP Authentication:
    uncheck
No Check Certificate:
    uncheck
Storage Size:
    10485760
Schedule:
    Every Hour at 5 past the hour
File Format:
    xml
Enable Compression:
    check
Equipment:
    test
        v1~v32
```

## Case 2

### Purpose
User can use upload-test to test upload connectivity.

### Environment
- UC-8132-LX-CG
- Host running `server.http`, `server.https_nocert`, `server.http_auth`.

### Step
1. Navigate to DASHBOARD.LOG_PROFILE_MANAGEMENT.
2. Click `ADD`.
3. For each **Profile Configuration** listed in **Case 1**, fill it in and click `Connection Test`.
4. All connection tests shall pass.
