# Modbus Framework


## REQ-1 Configurable MODBUS Framework

### REQ-1.1
Configurable MODBUS TCP/RTU master.

### REQ-1.2
Minimum scan interval is 1 sec.

### REQ-1.3
Provide point-slope and point-to-point scaling formula.

### REQ-1.4
Accept pre-configured MODBUS equipment template.

### REQ-1.5
Built-in MOXA ioLogik E12XX/R12xx MODBUS remote I/O.

### REQ-1.6
Test equipment template via actual TCP/RTU equipment while editing it.

### REQ-1.7
Test actual equipment while editing it.

### REQ-1.8
Provide **last update time** and **status** for each equipment.


## Components
- mxmodbusmaster
- mxmodbusgateway
- mxmodbusclient

## Test Cases

### 1. mxmodbusmaster

#### Purpose
- Test **5** Modbus slave device.
- Scan interval is **1 second**.
- Test **20** tags for each equipment.

#### Steps
1. Setup test environment.
```
./mxmodbusmaster-test-setup.sh
```
2. **Check** all tags and values by eyes.
```
./mxmodbusmaster-test.sh
```
3. Cleanup test environment.
```
./mxmodbusmaster-test-teardown.sh
```

#### Expect
1. Service mxmodbusmaster SHOULD run in the background.
2. Subscriber SHOULD receive tag data continuously in protocol-buffer format with the correct interval.
    - Topic: `/equs/EQUIPMENT/tags/TAG`
        - EQUIPMENT: eq1, eq2, eq3, eq4, eq5.
        - TAG: rc, rc2, rdi, rdi2, rig, int16, uint32, int32, uint64, int64, float32, float64, sting, DCBA, rhr, border, scaling, scaling2.
    - Message:
        - Value: 0, 1, 1, 0, 65535, -32768, 4294967295, -1, 18446744073709551615, -1, 3.14(approx.), 3.14(approx), "ModbusEmu", 305419896, 0, 305419896, 4.554, -1.0
    - Time Interval: the `at` time between 2 same equipment, need around **1 second**.

#### Traceability
1. REQ-1.1
2. REQ-1.2
3. REQ-1.3


### 2. mxmodbusgateway

#### Purpose
- Test Modbus RTU device.

#### Steps
1. Prepare a Tester(another UC-8100).
    1. Connect UART Port 1 to Tester's Port 1 by RS-232 cable.
    2. Connect UART Port 2 to Tester's Port 2 by RS-232 cable.
2. Setup test environment in Tester.
```
./mxmodbusgateway-tester-setup.sh
```
3. Setup test environment.
```
./mxmodbusgateway-test-setup.sh
```
4. Start tester program in Tester.
```
./mxmodbusgateway-tester.sh
```
5. Start mxmodbusgateway test script and **CHECK** the result.
```
./mxmodbusgateway-test.sh
```
6. Cleanup tester program in Tester.
```
./mxmodbusgateway-tester-teardown.sh
```
7. Cleanup mxmodbusgateway testcase.
```
./mxmodbusgateway-test-teardown.sh
```

#### Expect
1. Service mxmodbusgateway SHOULD run in the background.
2. At stdout SHOULD see the testing result of 2 serial port:
    - Read Coils ... OK
    - Read Discrete Inputs ... OK
    - Read Input Registers ... OK
    - Read Holding Registers ... OK

#### Traceability
1. REQ-1.1(1)


### 3. mxmodbusclient

#### Purpose
- Validate Equipment Template.
- Test Equipment with specific template.

#### Steps
1. Run test script.
    - start `modbusemulator`
    - `mxmodbusclient --dry --equipment-template test-template.met`
    - `mxmodbusclient --equipment-template test-template.met --host 127.0.0.1 --port 55688 --device-id 0`
    - stop `modbusemulator`

#### Expect
1. At stdout SHOULD see the testing result:
    - Validate Equipment Template OK.
    - Validate Equipment Reading OK.

#### Traceability
1. Partial REQ-1.6
2. Partial REQ-1.7
