# Ethernet

## Case 1. Update LAN1 with static IP when cellular is down

### Pre-Conditions
1. Make sure or set LAN1 as [static IP](#static-ip-for-eth0-before-test)

### Steps
1. Navigate to `Settings`.`Ethernet`.`eth0` and `Edit` with
   [static for eth0](#static-ip-for-eth0)
2. Click `Save`

### Expect
1. _IP_ and _netmask_ changed to `192.168.31.39` & `255.255.255.0`
   ```
   $ ip a show dev eth0
   ```
2. Default gateway changed to `192.168.3.254`
   ```
   $ ip route
   ```
3. DNS file should be:
   ```
   # /etc/resolv.conf
   nameserver 8.8.8.8
   ```

### Post-Conditions
- Set LAN1 back (as [before test](#static-ip-for-eth0-before-test))


## Case 2. Update LAN1 with static IP when cellular is up

### Pre-Conditions
1. Make sure or set LAN1 as [static IP](#static-ip-for-eth0-before-test)

### Steps
1. Navigate to `Settings`.`Ethernet`.`eth0` and `Edit` with
   [static for eth0](#static-ip-for-eth0)
2. Click `Save`

### Expect
1. _IP_ and _netmask_ changed to `192.168.31.39` & `255.255.255.0`
   ```
   $ ip a show dev eth0
   ```
2. Default gateway cannot be changed (should be cellular one)
   ```
   $ ip route
   ```
3. DNS file cannot be changed (should be cellular one)

### Post-Conditions
- Set LAN1 back (as [before test](#static-ip-for-eth0-before-test))


## Case 3. Update LAN1 with DHCP when cellular is down

### Pre-Conditions
1. Make sure or set LAN1 as [static IP](#static-ip-for-eth0-before-test)
2. _eth0_ connects with a DHCP server
   ([config](#dhcp-server-configuration-for-test))

### Steps
1. Navigate to `Settings`.`Ethernet`.`eth0` and `Edit` with `DHCP`
2. Click `Save`

### Expect
1. _IP_ and _netmask_ changed to `192.168.5.130` & `255.255.0.0`
   ```
   $ ip a show dev eth0
   ```
2. Default gateway changed to `192.168.1.1`
   ```
   $ ip route
   ```
3. DNS file should be:
   ```
   # /etc/resolv.conf
   nameserver 168.95.1.1
   ```

### Post-Conditions
- Set LAN1 back (as [before test](#static-ip-for-eth0-before-test))


## Case 4. Update LAN2 with DHCP

### Pre-Conditions
1. Make sure or set LAN2 as [static IP](#static-ip-for-eth1-before-test)
2. _eth1_ connects with a DHCP server
   ([config](#dhcp-server-configuration-for-test))

### Steps
1. Navigate to `Settings`.`Ethernet`.`eth1` and `Edit` with `DHCP`
2. Click `Save`
3. Login via `192.168.5.130`

### Expect
1. _admin_ will be logged out from web
2. _IP_ and _netmask_ changed to `192.168.5.130` & `255.255.0.0`
   ```
   $ ip a show dev eth1
   ```
3. Default gateway cannot be changed (should be cellular one)
   ```
   $ ip route
   ```
4. DNS file cannot be changed (should be cellular one)

### Post-Conditions
- Set LAN2 back (as [before test](#static-ip-for-eth1-before-test))


## Test Data
#### Static IP for eth0 Before Test
- _IP Address Mode_: `Static IP`
- _IP_:  `192.168.31.38`
- _Netmask_: `255.255.255.0`
- _Gateway_: `192.168.31.254`
- _DNS 1_: `192.168.50.36`
- _DNS 2_: `192.168.50.33`

#### Static IP for eth1 Before Test
- _IP Address Mode_: `Static IP`
- _IP_:  `192.168.4.127`
- _Netmask_: `255.255.255.0`

#### Static IP for eth0
- _IP Address Mode_: `Static IP`
- _IP_:  `192.168.3.38`
- _Netmask_: `255.255.255.0`
- _Gateway_: `192.168.3.254`
- _DNS 1_: `8.8.8.8`
- _DNS 2_:

#### DHCP Server Configuration for Test
- _subnet_: `192.168.0.0`
- _netmask_: `255.255.0.0`
- _DNS_: `168.95.1.1`
- _address range_: `192.168.5.130` ~ `192.168.5.200`
- _address range_: `192.168.6.130` ~ `192.168.6.200`
- _routers_: `192.168.1.1`
