import unittest
import logging
import sh
import requests

from pyswagger import SwaggerApp
from pyswagger.contrib.client.requests import Client
from pyswagger.utils import jp_compose
from pyswagger.primitives import SwaggerPrimitive
from pyswagger.primitives._int import validate_int, create_int
from pyswagger.primitives._float import validate_float, create_float

factory = SwaggerPrimitive()
factory.register("integer", "", create_int, validate_int)
factory.register("integer", None, create_int, validate_int)
factory.register("number", "", create_float, validate_float)
factory.register("number", None, create_float, validate_float)


class TestSerial(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        app = SwaggerApp.load("./api.json", prim=factory)
        app.prepare()
        self.client = Client(send_opt={"verify": False})
        self.SERIALS = app.resolve(
            jp_compose("/system/serial", base="#/paths"))
        self.SERIAL = app.resolve(
            jp_compose("/system/serial/{id}", base="#/paths"))

    @classmethod
    def tearDownClass(self):
        pass

    def test_get_serial(self):
        """Get current SERIALS"""
        serials = self.client.request(self.SERIALS.get()).data
        for serial in serials:
            self.assertIn("id", serial)
            self.assertIn("dev", serial)
            self.assertIn("devDisplayName", serial)
            self.assertIn("mode", serial)
            self.assertIn(
                serial["mode"],
                str(sh.setinterface(serial["dev"])).lower()
            )

    def test_put_serial(self):
        """Set id 0 to RS422/RS485-4W"""
        serial = self.client.request(
            self.SERIAL.put(
                id=1,
                body={"mode": "rs422/rs485-4w"}
            )
        ).data

        self.assertIn(
            "rs422/rs485-4w",
            str(sh.setinterface(serial["dev"])).lower()
        )

if __name__ == "__main__":
    FORMAT = '%(asctime)s - %(levelname)s - %(lineno)s - %(message)s'
    logging.basicConfig(level=30, format=FORMAT)
    logger = logging.getLogger('TestSerial')
    unittest.main()
