import unittest
import logging
import sh
import requests

from pyswagger import SwaggerApp
from pyswagger.contrib.client.requests import Client
from pyswagger.utils import jp_compose
from pyswagger.primitives import SwaggerPrimitive
from pyswagger.primitives._int import validate_int, create_int
from pyswagger.primitives._float import validate_float, create_float

factory = SwaggerPrimitive()
factory.register("integer", "", create_int, validate_int)
factory.register("integer", None, create_int, validate_int)
factory.register("number", "", create_float, validate_float)
factory.register("number", None, create_float, validate_float)


class TestSerial(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        app = SwaggerApp.load("./api.json", prim=factory)
        app.prepare()
        self.client = Client(send_opt={"verify": False})
        self.SERVICES = app.resolve(
            jp_compose("/system/services", base="#/paths"))
        self.SERVICE = app.resolve(
            jp_compose("/system/services/{name}", base="#/paths"))

    @classmethod
    def tearDownClass(self):
        pass

    def test_get_services(self):
        """Get current SERIALS"""
        services = self.client.request(self.SERVICES.get()).data
        for service in services:
            if "rsyslog" in service["name"]:
                if service["enable"]:
                    self.assertIn("rsyslog", sh.ps("ax"))
                else:
                    self.assertNotIn("rsyslog", sh.ps("ax"))
            if "ssh" in service["name"]:
                if service["enable"]:
                    self.assertIn("ssh", sh.ps("ax"))
                else:
                    self.assertNotIn("ssh", sh.ps("ax"))

    def test_put_service(self):
        """Set rsyslog enable"""
        self.client.request(
            self.SERVICE.put(
                name="rsyslog",
                body={"enable": True}
            )
        ).data

        self.assertIn("rsyslog", sh.ps("ax"))

if __name__ == "__main__":
    FORMAT = '%(asctime)s - %(levelname)s - %(lineno)s - %(message)s'
    logging.basicConfig(level=30, format=FORMAT)
    logger = logging.getLogger('TestServices')
    unittest.main()
