import unittest
import logging
import sh
import requests

from pyswagger import SwaggerApp
from pyswagger.contrib.client.requests import Client
from pyswagger.utils import jp_compose
from pyswagger.primitives import SwaggerPrimitive
from pyswagger.primitives._int import validate_int, create_int
from pyswagger.primitives._float import validate_float, create_float

factory = SwaggerPrimitive()
factory.register("integer", "", create_int, validate_int)
factory.register("integer", None, create_int, validate_int)
factory.register("number", "", create_float, validate_float)
factory.register("number", None, create_float, validate_float)


class TestStatus(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        app = SwaggerApp.load("./api.json", prim=factory)
        app.prepare()
        self.client = Client(send_opt={"verify": False})
        self.STATUS = app.resolve(jp_compose("/system/status", base="#/paths"))
        self.SYSLOG = app.resolve(jp_compose("/system/syslog", base="#/paths"))

    @classmethod
    def tearDownClass(self):
        pass

    def test_get_status(self):
        """Get current STATUS"""
        self.client.request(self.STATUS.get()).data

    def test_put_status_1(self):
        """Change hostname"""
        sh.hostname("BEROFE-MODIFY-HOSTNAME")
        self.client.request(self.STATUS.put(body={
            "hostname": "TEST-HOST-NAME"
        }))
        hostname = sh.hostname()
        self.assertIn("TEST-HOST-NAME", hostname)

    def test_download_syslog(self):
        """Download syslog"""
        resp = self.client.request(self.SYSLOG.post(body={
            "url": "https://localhost/api/v1/files/upload"
        }))

        print resp
        r = requests.get(
            url="https://localhost{}".format(resp["url"]),
            stream=True,
            verify=False
        )
        chunk_size = 1024
        if r.status_code != requests.codes.ok:
            raise RuntimeError("Can't download file.")

        with open("/run/shm/test.tar.gz", "wb") as fd:
            for chunk in r.iter_content(chunk_size):
                fd.write(chunk)

        output = sh.tar("-tvf", "/run/shm/test.tar.gz")
        self.assertIn("syslog", output)
        self.assertIn("syslog.user.info", output)

if __name__ == "__main__":
    FORMAT = '%(asctime)s - %(levelname)s - %(lineno)s - %(message)s'
    logging.basicConfig(level=30, format=FORMAT)
    logger = logging.getLogger('SanjiKeeper')
    unittest.main()
