var Promise = require('bluebird');
var path = require('path');
var fs = require('fs');
var glob = require('glob');
var YAML = require('js-yaml');
var resolve = require('json-refs').resolveRefs;
var express = require('express');
var debug = require('debug')('openapi');
var middleware = require('swagger-express-middleware');
var _ = require('lodash');
var app = express();

var bundlePaths = glob.sync('./MXcloud-baseline/ThingsPro/*/bundle.json');
var bundles = {};
var doc = {
  "swagger": "2.0",
  "info": {
    "version": "0.9.0",
    "title": "ThingsPro API Reference"
  },
  "basePath": "/api/v1",
  "schemes": [
    "http",
    "https"
  ],
  "produces": [
    "application/json"
  ],
  "tags": [],
  "paths": {},
  "definitions": {}
};

var defaultOptions = {
  host: '0.0.0.0',
  port: process.env.PORT || 8000
};

var aggregate = function (data) {
  if (_.isArray(data)) {
    var obj = {}
    data.forEach(function (v) {
      _.merge(obj, v);
    });

    data = obj;
  }

  return data;
}


Promise.map(bundlePaths, function (bundlePath) {
  var bundle = require(bundlePath);
  var schemaPath = path.dirname(bundlePath) + '/schema/index.yaml';
  bundles[bundle.name] = bundle;

  try {
    var stat = fs.statSync(schemaPath);
  } catch (e) {
    return Promise.resolve();
  }

  if (!stat.isFile()) return Promise.resolve();

  bundle.__hasSchema = true;
  bundle.__schemaPath = schemaPath;
  var root = YAML.load(fs.readFileSync(schemaPath).toString());
  var options = {
    relativeBase: path.dirname(schemaPath),
    filter: function (s) {
      if (s.uriDetails.reference === 'same-document' &&
        s.uri.indexOf('externalDocs') !== -1) {
        return true;
      }

      if (s.type === 'relative') return true;

      return false;
    },
    loaderOptions: {
      processContent: function (content, cb) {
        return cb(null, YAML.safeLoad(content.text));
      }
    }
  };

  doc.tags.push({
    name: bundle.name,
    description: bundle.description
  });

  return resolve(root, options)
    .then(function (results) {
      var pathObjects = aggregate(results.resolved.paths);
      for (var key in pathObjects) {
        var pathObject = pathObjects[key];
        for (var method in pathObject) {
          pathObject[method].tags = [bundle.name];
        }
      }

      _.merge(doc.paths, pathObjects);
      _.merge(doc.definitions, aggregate(results.resolved.definitions));
      _.merge(doc.externalDocs, aggregate(results.resolved.externalDocs));
      return Promise.resolve();
    });
})
.then(function () {
  return new Promise(function(resolve, reject) {
    try {
      fs.mkdirSync('./build');
    } catch (e) {
      console.log('It may Already exist.');
    }

    fs.writeFile(
      './build/openapi.json',
      JSON.stringify(doc, null, 2),
      function (err) {
        if (err) return reject(err);
        return resolve();
      }
    );
  });
})
.then(function () {
  if (process.env.BUILD_ONLY === 'yes') {
    return Promise.resolve();
  }

  var options = _.defaults(options, defaultOptions);

  app.get('/', function (req, res, next) {
    if (!req.query.url) return res.redirect('/?url=openapi.json');
    next();
  });

  app.use(express.static(__dirname + '/swagger-ui/dist'));

  app.get('/favicon.ico', function(req, res, next) {
    res.status(404);
  });

  var oapiDoc = JSON.parse(JSON.stringify(doc));
  middleware(oapiDoc, app, function(err, middleware) {
    app.use(
      middleware.metadata(),
      middleware.CORS(),
      middleware.parseRequest(),
      middleware.validateRequest(),
      middleware.mock()
    );

    app.get('/openapi.json', function(req, res, next) {
      res.json(doc);
    });

    app.listen(options.port, function () {
      console.log('Visit http://%s:%d', options.host, options.port);
    });
  });
});
