# Overview

Every ThingsPro Gateway, like UC-8112-LX-CG, comes with *Data Acquisition Framework* installed.
As denoted by its name,
*Data Acquisition Framework* is set of services that helps you develop *Data Acquisition* applications.

Conceptually, *Data Acquisition Framework* consists of following services:

**Broker Service**, dispatches imported Tags to individual sub-services.

**Storage Service**, stores Tags of interest into *Log Files*.

**Cache Service**, caches all Tags imported for further manipulation.

**Upload Service**, upload *Log Files* to remote servers.

![System Overview][tutorial-overview]

Note: About **Modbus Management Framework** please refer to **Software Manual**.

When a Tag is imported to **Broker Service**, from *Modbus Management Framework* or through `libmxidaf`,
it is dispatched to **Storage Service** and **Cache Service**.
If the Tag were configured to be logged, **Storage Service** saves it into *Log Files*.
**Cache Service** would save it directly into internal cache.
Note that the number of Tags saved in **Cache Service** is limited, old Tags are deleted when size exceeded.

In this tutorial, we will introduce the usage of `libmxidaf`. `libmxidaf` is an API for you to make your applications acquire data, log and tags value from other devices or sensors. Through `libmxidaf`, you can:

- Import Tags into Data Acquisition Framework.
- Query previously imported Tags from Data Acquisition Framework.

This document describes the usage of `libmxidaf`.

Note that all code snippets are simplified for clarity,
see [Reference](reference.md) for complete examples.

## libmxidaf

`libmxidaf` provides interfaces in C and Python.

As in the figure below,
components in `libmxidaf` are modeled in *Classes*.

`Service` class serves as the entry class of `libmxidaf`.
Through `Service`, you can

- create `Cache` to retrieve history Tags, or
- create `TagImporter` to import Tags.

`Cache`, `Equipment` and `Stream` are proxies to **Cache** service.

`Tag` models the basic structure of a reading.
It consists of three factors:

- Value of Reading(`value`)
- Time of Reading(`at`), in UTC
- Unit of Reading(`unit`)

![Library Hierarchy][library-hierarchy]


[tutorial-overview]: images/tutorial/overview.png "Overview"
[library-hierarchy]: images/tutorial/hierarchy.png "Hierarchy"
