# Using Python API

Python libraries are pre-installed.
Just import the library directly in your Python source file.

```py
from libmxidaf_py import Service
```

## Library Initialization

`libmxidaf` should be initialized before use.
To initialize the library, simply get the instance of `Service`.

To get `Service` instance in Python:

```py
service = Service.instance()
```

## Import Tags

Tags are imported by `TagImporter`.

```py
tag_importer = Service.instance().get_tag_importer()

at = Time.now()

tag_importer.import_tags(
    "electrical-probe",
    "voltage",
    [Tag(
        Value(5.010),
        at,
        "v")]
)

tag_importer.import_tags(
    "electrical-probe",
    "current",
    [Tag(
        Value(0.344),
        at,
        "a")]
)
```

Note: If the `equipment name` or `tag name` is named start with `-`, the whole tag will be ignored.

### Sample

Sample application can be found at

  `/usr/share/mxdaf/libmxidaf_py/sample/import.py`

This sample imports a `voltage` Tag and a `current` Tag under `electrical-probe` Equipment.

To run sample:

```
$ python import.py
```

You can use `mxtaghistory` to see the result:

```
$ mxtaghistory
"equipment","tag","at","value","unit"
"electrical-probe","current","2016-03-16T16:14:19.206Z","0.344","a"
"electrical-probe","voltage","2016-03-16T16:14:19.206Z","5.01","v"
```

## Query History Tags

To query history Tags, first get a `Cache` object.
Use `Cache` object to get the `Equipment` object of interest.
Use the `Equipment` object to get the `Stream` object you want.
After you obtain the `Stream` object, you can query the `Tags` as needed.

```py
cache = Service.instance().get_cache()

tag_list = cache.get_equipment(
    "electrical-probe").get_stream(
    "voltage").get_last_tag_list(
    3)

# ...do something with tag_list...
```

Note: Please DO NOT query history tags when there is no tag imported.

### Sample

Sample application can be found at

  `/usr/share/mxdaf/libmxidaf_py/sample/query.py`

This sample queries `voltage` Tag of `electrical-probe` Equipment and dumps Tag list to stdout.

To run sample:

```
$ python query.py
queried 2 tags
2016-03-16T16:14:51.994Z:5.0099999999999998:v
2016-03-16T16:14:19.206Z:5.0099999999999998:v
```

## Arithmetic of Tag value

With the ability to *Query History Tags* and *Import Tags*,
you can now inject your data processing logic into *Data Acquisition Framework*.

For example, we want to produce a new value by multiplying two input values:

```py
tag_importer.import(
    "electrical-probe",
    "power",
    voltage_stream.get_last_tag_list(1)[0].as_float() *
        current_stream.get_last_tag_list(1)[0].as_float())
```

### Sample

Sample application can be found at

  `/usr/share/mxdaf/libmxidaf_py/sample/calculate.py`

This sample imports a `power` Tag and its value, which is the latest value of `voltage` multiply the latest value of `current`.

To run sample:

```
$ python calculate.py
```

After running the sample, you can see that `power` Tag is imported:

```
$ mxtaghistory --equipment electrical-probe --tag power
"equipment","tag","at","value","unit"
"electrical-probe","power","2016-03-16T17:03:31.508Z","1.72344","w"
```

## Error Handling

`Error` is thrown whenever a library call fails, for example:

```py
from libmxidaf_py import Error

try:
    tag_list = stream.get_last_tag_list(10);

except Error as err:
    print err
```
