## Integration Test Spec

### Case 1

#### Purpose
User can add Equipment Templates with tags having different configurations.
User can add up to 20 tags to an Equipment Template.

#### Environment
- UC-8132-LX-CG
- Host computer running `test-slave`.

#### Step
1. Navigate to DASHBOARD.EQUIPMENT\_TEMPLATE\_LIST.
2. Add Equipment Template:
    1. TEMPLATE\_NAME: `My_Test_Equipment`
    2. Tag list: (see [Tags to Add](#tags-to-add))
3. Observe that an Equipment Template with name `My_Test_Device` is created
    correctly.
4. Navigate to DASHBOARD.MODBUS\_MANAGEMENT.
5. Add Modbus/TCP equipment.
    1. INTERFACE\_NAME: `test`
    2. HOST: (IP of test-slave)
    3. PORT: (PORT of test-slave)
    4. RESPONSE\_TIMEOUT: `1`
    5. INTERVAL\_PERIOD: `1`
    6. EQUIPMENT\_NAME: `test-equipment`
    7. MODEL: `My_Test_Equipment`
    8. UNIT\_ID: `255`
6. Run `subscribe_tag` to check whether value of each tag received from
    MQTT matches [Expected Tag Value](#expected-tag-value).

#### Data
##### Tags to Add

| #   | TAG\_NAME | FUNCTION             | ADDRESS | QUANTITY   | TYPE    | INV\_VALUE | BYTE\_ORDER | SCALING |
|:---:| ---------:| -------------------- | -------:| ----------:| -------:| ----------:|:-----------:| ------- |
|1    | rc        | Read Coils           | 0x0000  | (disabled) | bool    | disable    | disable     | disable |
|2    | nv        | Read Coils           | 0x0001  | (disabled) | bool    | [0x0000]   | disable     | disable |
|3    | rc2       | Read Coils           | 0x0002  | (disabled) | bool    | disable    | disable     | disable |
|4    | rdi       | Read Discrete Inputs | 0x0000  | (disabled) | bool    | disable    | disable     | disable |
|5    | rdi2      | Read Discrete Inputs | 0x0001  | (disabled) | bool    | disable    | disable     | disable |
|6    | rig       | Read Input Reg.      | 0x0000  | 1          | uint16  | disable    | disable     | disable |
|7    | int16     | Read Input Reg.      | 0x0001  | 1          | int16   | disable    | disable     | disable |
|8    | uint32    | Read Input Reg.      | 0x0002  | 2          | uint32  | disable    | disable     | disable |
|9    | int32     | Read Input Reg.      | 0x0004  | 2          | int32   | disable    | disable     | disable |
|10   | uint64    | Read Input Reg.      | 0x0006  | 4          | uint64  | disable    | disable     | disable |
|11   | int64     | Read Input Reg.      | 0x000A  | 4          | int64   | disable    | disable     | disable |
|12   | float32   | Read Input Reg.      | 0x000E  | 2          | float32 | disable    | disable     | disable |
|13   | float64   | Read Input Reg.      | 0x0010  | 4          | float64 | disable    | disable     | disable |
|14   | string    | Read Input Reg.      | 0x0014  | 5          | string  | disable    | disable     | disable |
|15   | DCBA      | Read Input Reg.      | 0x0019  | 2          | uint32  | disable    | DCBA        | disable |
|16   | rhr       | Read Holding Reg.    | 0x0000  | 1          | uint16  | disable    | disable     | disable |
|17   | border    | Read Holding Reg.    | 0x0001  | 2          | uint32  | disable    | CDAB        | disable |
|18   | scaling   | Read Holding Reg.    | 0x0003  | 1          | uint16  | disable    | disable     | slope-intercept;slope=3.14;offset=1.414 |
|19   | scaling2  | Read Holding Reg.    | 0x0004  | 1          | uint16  | disable    | disable     | point-slope;src-min=-10;src-max=10;target-min=-100;target-max=100 |
|20   | nv2       | Read Holding Reg.    | 0x0005  | 2          | uint32  | [0x0123, 0x4567] | disable | disable |

##### Expected Tag Value
| #   | TAG     | VALUE                |
|:---:| -------:| --------------------:|
| 1   | rc      | 0                    |
| 2   | nv      | (should not present) |
| 3   | rc2     | 1                    |
| 4   | rdi     | 1                    |
| 5   | rdi2    | 0                    |
| 6   | rig     | 65535                |
| 7   | int16   | -32768               |
| 8   | uint32  | 4294967295           |
| 9   | int32   | -1                   |
| 10  | uint64  | 18446744073709551615 |
| 11  | int64   | -1                   |
| 12  | float32 | 3.14(approx.)        |
| 13  | float64 | -3.14(approx.)       |
| 14  | string  | ModbusEmu            |
| 15  | DCBA    | 305419896            |
| 16  | rhr     | 0                    |
| 17  | border  | 305419896            |
| 18  | scaling | 4.554(approx.)       |
| 19  | scaling | -1.0(approx.)        |
| 20  | nv2     | (should not present) |

### Case 2

#### Purpose
User can add up to 5 Modbus Devices.

#### Environment
- UC-8132-LX-CG
- ioLogik
    - 2 E12xx
    - 2 R12xx connected to P1
    - 1 R12xx connected to P2

#### Step
1. Navigate to DASHBOARD.MODBUS\_MANAGEMENT.
2. Add Modbus/TCP devices according to E12xx configuration.
3. Add Modbus/RTU devices according to R12xx configuration.
4. Connect to UC-8132-LX-CG MQTT Broker to check whether all tags are reported.

### Case 3

#### Purpose
User can delete Equipment Templates.

#### Environment
- UC-8132-LX-CG

#### Step
1. Add a new template.
    1. Template name: `My_Template`
2. Click checkbox in front of `My_Template`.
3. Click **Delete** button.

#### Expect
1. `My_Template` be deleted from DASHBOARD.EQUIPMENT\_TEMPLATE\_LIST.

### Case 4

#### Purpose
User can copy Equipment Templates.

#### Environment
- UC-8132-LX-CG

#### Step
1. Add a new template.
    1. Template name: `My_Template`.
2. Click **Copy** button of `My_Template`.

#### Expect
1. A new copy of `My_Template` be created.

### Case 5

#### Purpose
User can't delete Equipment Templates if TCP interfaces depend on it.

#### Environment
- UC-8132-LX-CG

#### Step
1. Add a new template: `My_Template`.
2. Add a new TCP interface with equipment name `test-eq` and equipment template `My_Template`.
3. Delete `My_Template` from equipment template list.

#### Expect
1. Show error message and `My_Template` should not be deleted.

### Case 6

#### Purpose
User can't delete Equipment if Log Profile depend on it.

#### Environment
- UC-8132-LX-CG

#### Step
1. Add a new template: `My_Template`.
2. Add a new TCP interface with equipment name `test-eq` and equipment template `My_Template`.
3. Add a new Log Profile with selected tags from `test-eq`
4. Delete `My_Template` from equipment template list.

#### Expect
1. Show error message and `My_Template` should not be deleted.

### Case 7

#### Purpose
User can't modify Equipment Templates' Tags if Log Profile depend on it.

#### Environment
- UC-8132-LX-CG

#### Step
1. Add a new template: `My_Template` with a tag `tag-1`.
2. Add a new TCP interface with name `test-eq` and equipment template `My_Template`.
3. Add a new Log Profile with tag `tag-1` selected from `test-eq`
4. Modify tag `tag-1` in `My_Template`.

#### Expect
1. Show error message and `My_Template`'s Tags should not be modified.

### Case 8

#### Purpose
Log Profile function should be disabled, if SD card isn't inserted. (UC-8112-LX-CG only)

#### Environment
- UC-8112-LX-CG

#### Step
1. Boot with SD card (filesystem: ext4).
2. Add a Log Profile
3. Power-off
4. Boot without SD card

#### Expect
1. mxlogarchiver should not be running.
2. `/media/sd-mmcblk1p1` shoud not exist.
3. Web should not be able to create/update Log Profile entry.

### Case 9

#### Purpose
User can use test button to test Equipment Template and Equipment.

#### Environment
- UC-8132-LX-CG
- Host computer running `modbusemulator` for Modbus TCP.
- Modbus RTU tester running `modbusemulator`.

#### Step
1. Create Equipment Template `My_Test_Device` as [Case 1](#case-1).
2. Add Modbus TCP equipment, same configuration as [Case 1](#case-1).
3. Click `Test` button in TCP window.
4. Add Modbus RTU equipment of Port1 and Port2.
  1. Set port configuration.
  2. Use `My_Test_Device` template.
5. Clickt `Test` button in RTU window.

#### Expect
1. In TCP window, pop-up a window show the return result, need to
    match with [Expected Tag Value](#expected-tag-value).
2. In RTU window, pop-up a window show the return result, need to
    match with [Expected Tag Value](#expected-tag-value).

### Case 10

#### Purpose
User can get each Equipment's `update time` and `status`.

#### Environment
- UC-8132-LX-CG
- Host computer running `modbusemulator` for Modbus TCP.
- Modbus RTU tester running `modbusemulator`.

#### Step
1. Create Equipment Template `My_Test_Device` as [Case 1](#case-1).
2. Add Modbus TCP equipment, same configuration as [Case 1](#case-1).
3. Click `Refresh Button` in `Modbus Management` window.
4. Click the `view` link of `Connected Equipment` to show the
    `update time` and `status`.
5. Add Modbus RTU equipment of Port1 and Port2.
  1. Set port configuration.
  2. Use `My_Test_Device` template.
6. Click `Refresh Button` in `Modbus Management` window.
7. Click the `view` link of `Connected Equipment` to show the
    `update time` and `status`.

#### Expect
1. For each Equipment, a pop-up window to show `Device Name`,
    `AVG. Success Rate`, `Elapse  Request Count`, `Success Count`,
    and `Update Time`.
