## In/Out Interfaces

### Man Page
```
NAME
    mxmodbusio

SYNOPSIS
    mxmodbusio -r REFERENCE -q QUANTITY [other options]
    mxmodbusio --read=REFERENCE --quantity=QUANTITY [other options]

    mxmodbusio -w REFERENCE -d DATA [other options]
    mxmodbusio --write=REFERENCE --data=DATA [other options]

    mxmodbusio [other options]

DESCRIPTION
    If both -r and -w are not given, mxmodbusio enters command mode and reads
    stdin for commands.

    -r REFERENCE
    --read=REFERENCE
        Do read operation.

        Example:
        $ mxmodbusio -r 0x00333 -q 5 -h 192.168.31.123 -u 0
        0:1:1:0:0

    -w REFERENCE
    --write=REFERENCE
        Do write operation.

        Example:
        $ mxmodbusio -w 0x40FFF -d 03F1:0031:5D67 -h 192.168.31.123 -u 0

    -d DATA
    --data=DATA
        Example, two words:
            0000:55Aa

        Example, four bits:
            0:1:1:0

        Default 0000.

    -q QUANTITY
    --quantity=QUANTITY
        Default 1.

    -u UNIT
    --unit=UNIT
        Default 1.

    -h HOST
    --host=HOST
        Default 127.0.0.1.

    -p PORT
    --port=PORT
        Default 502.

    -t TIMEOUT_MS
    --timeout-ms=TIMEOUT_MS
        Given MODBUS request timeout.

        Default 3000 (3 seconds).

    -s
    --single
        When given with -w option, mxmodbusio uses
        write-single-coil/write-single-register instead of
        write-multiple-coils/write-multiple-registers for write operation.

        Example:
        $ mxmodbusio -w 0x40FFF -d 03f1 -h 192.168.31.123 -u 0 -s

REFERENCE
    Coils
        0x00000~0x0FFFF
    Discrete Inputs
        0x10000~0x1FFFF
    Input Registers
        0x30000~0x3FFFF
    Holding Registers
        0x40000~0x4FFFF

COMMAND
    If both -r and -w are not given, mxmodbusio enters command mode and reads
    stdin for commands.
    -s,--single option is ignored in command mode.

    Read
    Command
        read <reference> <quantity>
    Response
        <data>

    Write
    Command
        write <reference> <data>
    Response
        (empty line)

    Single Write
    Command
        swrite <reference> <sdata>

        <sdata> should contain only one word(or bit, depends on <reference>).
    Response
        (empty line)

    Quit
    Command
        quit
    Response
        (empty line)

    Error Response
    Response
        ERR:<MODBUS exception code>

        <MODBUS exception code> -1 for unknow protocol error.
            01
                ILLEGAL FUNCTION
            02
                ILLEGAL DATA ADDRESS
            03
                ILLEGAL DATA VALUE
            04
                SERVER DEVICE FAILURE
            05
                ACKNOWLEDGE
            06
                SERVER DEVICE BUSY
            08
                MEMORY PARITY ERROR
            0A
                GATEWAY PATH UNAVAILABLE
            0B
                GATEWAY TARGET DEVICE FAILED TO RESPOND

    Example:
        $ mxmodbusio -h 192.168.3.127
        read 0x00303 4
        1:1:0:0
        write 0x00303 1:0:0:0

        read 0x00303 4
        1:0:0:0
        swrite 0x00303 1

        read 0x1FFFF 1
        ERR:2

        quit
        $

EXIT STATUS
    0
        Success
    1
        Unknown error
    2
        Illegal argument
    3
        IO error
    9..19
        MODBUS error(MODBUS exception code +8.)
```
