#ifndef _MXHTAG_DATABASE_HPP_
#define _MXHTAG_DATABASE_HPP_

#include <vector>

#include <boost/filesystem.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxtag/Tag.hpp>

#include <mxhtag/error.hpp>

namespace mxhtag
{

class Database
    : private boost::noncopyable
{
public:
    static void
        create_tables(
            const boost::filesystem::path &); // throws mxhtag::Error

public:
    explicit Database(
        const boost::filesystem::path &,
        int retry_timeout_ms = 10 * 1000,
        int retry_wait_ms = 10);      // throws mxhtag::OpenError, mxhtag::VersionError, mxhtag::Error
    ~Database();

    void
        insert_reading(
            const mxtag::Tag &);                // throws mxhtag::Error
    void
        insert_reading_list(
            const std::vector<mxtag::Tag> &);   // throws mxhtag::Error
    void
        delete_oldest_readings(
            float rate);                        // throws mxhtag::Error

public:
    // query functions
    const std::vector<std::string>
        get_equipment_name_list() const;                 // throws mxhtag::Error

    bool
        is_equipment_exist(
            const std::string &equipment) const;    // throws mxhtag::Error

    const std::vector<std::string>
        get_tag_name_list_by_equipment(
            const std::string &equipment) const;    // throws mxhtag::Error

    bool
        is_equipment_tag_exist(
            const std::string &equipment,
            const std::string &tag) const;          // throws mxhtag::Error

    const std::vector<mxtag::Tag>
        get_tag_list_by_equipment_tag(
            const std::string &equipment,
            const std::string &tag) const;          // throws mxhtag::Error

    const std::vector<mxtag::Tag>
        get_last_tag_list_by_equipment_tag(
            const std::string &equipment,
            const std::string &tag,
            std::size_t count) const;               // throws mxhtag::Error

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
