#ifndef _MXSQLITE3_STATEMENT_HPP_
#define _MXSQLITE3_STATEMENT_HPP_

#include <vector>

#include <boost/cstdint.hpp>
#include <boost/noncopyable.hpp>
#include <boost/optional.hpp>

#include <mxsqlite3/error.hpp>
#include <mxsqlite3/Value.hpp>

namespace mxsqlite3
{

class Statement // interface
    : private boost::noncopyable
{
protected:
    Statement();

public:
    virtual ~Statement();

    void
        reset();    // throws mxsqlite3::Error

    void
        bind(std::size_t index);    // bind null
                                    // throws mxsqlite3::Error
    void
        bind(
            std::size_t index,
            int64_t);               // throws mxsqlite3::Error
    void
        bind(
            std::size_t index,
            double);                // throws mxsqlite3::Error
    void
        bind(
            std::size_t index,
            const std::string &);   // throws mxsqlite3::Error

    boost::optional<std::vector<mxsqlite3::Value> >
        execute();    // throws mxsqlite3::Error

private:
    virtual void
        do_reset() = 0;

    virtual void
        do_bind(std::size_t index) = 0;
    virtual void
        do_bind(
            std::size_t index,
            int64_t) = 0;
    virtual void
        do_bind(
            std::size_t index,
            double) = 0;
    virtual void
        do_bind(
            std::size_t index,
            const std::string &) = 0;

    virtual boost::optional<std::vector<mxsqlite3::Value> >
        do_execute() = 0;
};

}

#endif
