#include <mxsqlite3/Sqlite3.hpp>

#include <sqlite3.h>

#include <mxsqlite3/ConnectionImpl.hpp>

using boost::filesystem::path;
using boost::shared_ptr;

using std::string;

namespace mxsqlite3
{

const Sqlite3 &
    Sqlite3::instance()
{
    static Sqlite3 inst;

    return inst;
}

struct Sqlite3::Impl
{
};

Sqlite3::Sqlite3()
    : pimpl(new Impl())
{
}

Sqlite3::~Sqlite3()
{
}

shared_ptr<Connection>
    Sqlite3::connect(
        const path &dbpath,
        int retry_timeout_ms,
        int retry_wait_ms,
        bool create_if_not_exist) const
{
    BOOST_VERIFY(retry_timeout_ms >= 0);
    BOOST_VERIFY(retry_wait_ms >= 0);

    sqlite3 *handle = 0;
    int flags = SQLITE_OPEN_READWRITE;
    if (create_if_not_exist)
        flags |= SQLITE_OPEN_CREATE;

    const int rtn = sqlite3_open_v2(
        dbpath.string().c_str(),
        &handle,
        flags,
        0);

    if (rtn != SQLITE_OK)
    {
        int errcode = -1;
        string errmsg = "unknown error";
        if (handle)
        {
            errcode = sqlite3_errcode(handle);
            errmsg = sqlite3_errmsg(handle);
        }

        if (handle &&
            SQLITE_OK != sqlite3_close(handle))
        {
            // do nothing
        }

        BOOST_THROW_EXCEPTION(
            Error()
                << Sqlite3ErrCodeInfo(errcode)
                << ErrMsgInfo(errmsg)
        );
    }

    return shared_ptr<Connection>(new ConnectionImpl(
        handle,
        retry_timeout_ms,
        retry_wait_ms));
}

}
