#include <mxsqlite3/Value.hpp>

#include <boost/make_shared.hpp>

namespace mxsqlite3
{

using boost::make_shared;
using boost::shared_ptr;

using std::string;

struct Value::Impl
{
    Impl()
    {
    }

    shared_ptr<int64_t> iinst;
    shared_ptr<double>  dinst;
    shared_ptr<string>  sinst;
};

Value::Value()
    : pimpl(new Impl())
{
}

Value::Value(int64_t v)
    : pimpl(new Impl())
{
    pimpl->iinst = make_shared<int64_t>(v);
}

Value::Value(double v)
    : pimpl(new Impl())
{
    pimpl->dinst = make_shared<double>(v);
}

Value::Value(const string &v)
    : pimpl(new Impl())
{
    pimpl->sinst = make_shared<string>(v);
}

bool
    Value::is_null() const
{
    return
        !pimpl->iinst &&
        !pimpl->dinst &&
        !pimpl->sinst;
}

bool
    Value::is_int() const
{
    return pimpl->iinst;
}

bool
    Value::is_double() const
{
    return pimpl->dinst;
}

bool
    Value::is_string() const
{
    return pimpl->sinst;
}

int64_t
    Value::as_int() const
{
    return *pimpl->iinst;
}

double
    Value::as_double() const
{
    return *pimpl->dinst;
}

const string &
    Value::as_string() const
{
    return *pimpl->sinst;
}

bool
    operator ==(
        const Value &lhs,
        const Value &rhs)
{
    if (&lhs == &rhs)
        return true;

    if (lhs.is_null() && rhs.is_null())
        return true;

    if (lhs.is_int() && rhs.is_int() &&
        lhs.as_int() == rhs.as_int())
        return true;

    if (lhs.is_double() && rhs.is_double() &&
        lhs.as_double() == rhs.as_double())
        return true;

    if (lhs.is_string() && rhs.is_string() &&
        lhs.as_string() == rhs.as_string())
        return true;

    return false;
}

}
