#include <iostream>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#include <mxhtag/Database.hpp>

using boost::lexical_cast;
using boost::posix_time::ptime;
using boost::posix_time::microsec_clock;
using boost::posix_time::seconds;

using mxhtag::Database;
using mxtag::Tag;
using mxtag::Value;

using std::cerr;
using std::endl;
using std::string;
using std::vector;

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 5)
    {
        cerr << "usage: <equ-count> <tag-count-per-equ> <reading-per-tag> <dbpath>" << endl;
        return 1;
    }

    int i = 1;
    const int equ_count = lexical_cast<int>(argv[i++]);
    const int tag_count_per_equ = lexical_cast<int>(argv[i++]);
    const int reading_per_tag = lexical_cast<int>(argv[i++]);
    const string dbpath = argv[i++];

    vector<ptime> at_list;
    {
        ptime at = microsec_clock::universal_time();
        for(int ax = 0; ax < reading_per_tag; ++ax)
        {
            at_list.push_back(at);
            at += seconds(1);
        }
    }

    try
    {
        Database::create_tables(dbpath);

        Database db(dbpath);

        for(int equ = 0; equ < equ_count; ++equ)
        {
            for(int tag = 0; tag < tag_count_per_equ; ++tag)
            {
                string equ_name = "equipment-" + lexical_cast<string>(equ);
                equ_name.resize(64, 'f');

                string tag_name = "tag-" + lexical_cast<string>(tag);
                tag_name.resize(64, 'f');

                BOOST_FOREACH(const ptime &at, at_list)
                {
                    const Tag t(
                        equ_name,
                        tag_name,
                        at,
                        Value(3.14159265),
                        string(8, 'u'));

                    db.insert_reading(t);
                }
            }
        }
    }
    catch(boost::exception &e)
    {
        cerr << diagnostic_information(e) << endl;
        return 1;
    }

    return 0;
}
