#include <boost/test/unit_test.hpp>

#include <mxsqlite3/Connection.hpp>

#include <mxsqlite3/Sqlite3.hpp>

namespace mxsqlite3
{

using boost::shared_ptr;

using std::string;

namespace
{

struct TestFixture
{
    TestFixture()
        : path("/dev/shm/test.sqlite3")
    {
        if (0 != unlink(path.c_str()))
        {
            // do nothing
        }

        connection = Sqlite3::instance().connect(
            path,
            10 * 1000,
            10,
            true);
    }

    ~TestFixture()
    {
        if (0 != unlink(path.c_str()))
        {
            // do nothing
        }
    }

    const string            path;
    shared_ptr<Connection>  connection;
};

}

BOOST_FIXTURE_TEST_CASE(test_Connection_create_statement_ShouldPass, TestFixture)
{
    // arrange
    // act
    const shared_ptr<Statement> result = connection->create_statement(
        "CREATE TABLE hello ("
        " world INTEGER NOT NULL"
        " );"
    );

    // assert
    BOOST_CHECK(result != NULL);
}

BOOST_FIXTURE_TEST_CASE(test_Connection_operator_put_ShouldPass, TestFixture)
{
    // arrange
    // act
    *connection
        <<
            "CREATE TABLE hello("
            " world INTEGER NOT NULL"
            " );"
        <<
            "CREATE TABLE world("
            " hello FLOAT NOT NULL"
            " );";
}

}
