#include <iostream>

#include <boost/lexical_cast.hpp>

#include <mxhtag/Database.hpp>

using boost::lexical_cast;
using boost::posix_time::microsec_clock;
using boost::posix_time::ptime;
using boost::posix_time::seconds;

using mxhtag::Database;
using mxhtag::DiskFullError;

using mxtag::Tag;

using std::cerr;
using std::endl;
using std::string;
using std::vector;

class Testbed
{
public:
    Testbed(
        const string &dbpath)
        : db_(dbpath)
    {
    }

    void
        insert_readings(const vector<Tag> &reading_list)
    {
        try
        {
            db_.insert_reading_list(reading_list);
        }
        catch(DiskFullError &e)
        {
            db_.delete_oldest_readings(0.1);

            this->insert_readings(reading_list);
        }
    }

private:
    Database    db_;
};

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 5)
    {
        cerr << "usage: <dbpath> <equ-count> <tags-per-equ> <readings-per-tag>" << endl;
        return 1;
    }

    int index = 1;
    const string dbpath = argv[index++];
    const unsigned int equ_count = lexical_cast<unsigned int>(argv[index++]);
    const unsigned int tags_per_equ = lexical_cast<unsigned int>(argv[index++]);
    const unsigned int readings_per_tag = lexical_cast<unsigned int>(argv[index++]);

    const ptime now = microsec_clock::universal_time();

    Database::create_tables(dbpath);
    Testbed tb(dbpath);
    for(unsigned int reading = 0; reading < readings_per_tag; ++reading)
    {
        const ptime reading_time = now + seconds(reading);

        vector<Tag> reading_list;
        for(unsigned int equ = 0; equ < equ_count; ++equ)
        {
            for(unsigned int tag = 0; tag < tags_per_equ; ++tag)
            {
                reading_list.push_back(Tag(
                    "equipment" + lexical_cast<string>(equ),
                    "tag" + lexical_cast<string>(tag),
                    reading_time,
                    mxtag::Value("test-value"),
                    "test-unit"));
            }
        }

        tb.insert_readings(reading_list);
    }

    return 0;
}
