/**
 * @file
 * @author tc.huang@moxa.com
 * @brief Cache object.
 *
 * Provide mxidaf_Cache class and its member functions.
 */

#ifndef _MXIDAF_CACHE_H_
#define _MXIDAF_CACHE_H_

#include <stdint.h>

#include <mxidaf/Equipment.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @defgroup mxidaf-cache Cache object
 *
 * @ingroup mxidaf
 */

/**
 * @brief Cache instance placeholder.
 *
 * @ingroup mxidaf-cache
 */
typedef struct mxidaf_Cache_ mxidaf_Cache;

/**
 * @brief Delete Cache instance.
 *
 * @ingroup mxidaf-cache
 */
void
    mxidaf_Cache_delete(
        mxidaf_Cache *self      /**< [in] Cache instance to be deleted */
    );

/**
 * @brief Get Equipment list.
 *
 * @ingroup mxidaf-cache
 *
 * @return
 *      Number of Equipment instances returned.\n
 *      Return -1 on error.\n
 *      If len == 0, this function returns the value of len required to obtain
 *      complete Equipment list.
 *
 * @see
 *      mxidaf_Equipment_delete()
 */
int
    mxidaf_Cache_get_equipment_list(
        const mxidaf_Cache *self,               /**< [in] Cache instance */
        mxidaf_Equipment *equipment_list[],     /**< [out] Buffer to be filled */
        int len                                 /**< [in] Length of equipment_list */
    );

/**
 * @brief Get Equipment instance.
 *
 * @ingroup mxidaf-cache
 *
 * @return
 *      Returns Equipment instance.\n
 *      Returns NULL on error.
 *
 * @see
 *      mxidaf_Equipment_delete()
 */
mxidaf_Equipment *
    mxidaf_Cache_get_equipment(
        const mxidaf_Cache *self,   /**< [in] Cache instance */
        const char *equ_name        /**< [in] Name of the Equipment */
    );

#ifdef __cplusplus
}
#endif

#endif
