/**
 * @file
 * @author tc.huang@moxa.com
 * @brief Equipment object.
 */

#ifndef _MXIDAF_EQUIPMENT_H_
#define _MXIDAF_EQUIPMENT_H_

#include <mxidaf/Stream.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @defgroup mxidaf-equipment Equipment object
 *
 * @ingroup mxidaf
 */

/**
 * @brief Equipment instance placeholder.
 *
 * @ingroup mxidaf-equipment
 */
typedef struct mxidaf_Equipment_ mxidaf_Equipment;

/**
 * @brief Delete Equipment instance.
 *
 * @ingroup mxidaf-equipment
 */
void
    mxidaf_Equipment_delete(
        mxidaf_Equipment *self  /**< [in] Equipment instance to be deleted */
    );

/**
 * @brief Get equipment name associated with this object.
 *
 * @ingroup mxidaf-equipment
 *
 * @return
 *      NULL terminated string.\n
 *      Pointer remains valid during the lifecycle of this object.
 */
const char *
    mxidaf_Equipment_get_name(
        const mxidaf_Equipment *self    /**< [in] Equipment instance */
    );

/**
 * @brief Get Stream list.
 *
 * @ingroup mxidaf-equipment
 *
 * @return
 *      Number of entries filled in stream_list.\n
 *      Return -1 on error.\n
 *      If len == 0, this function returns the value of len required to obtain
 *      complete Stream list.
 *
 * @see
 *      mxidaf_Stream_delete()
 */
int
    mxidaf_Equipment_get_stream_list(
        const mxidaf_Equipment *self,   /**< [in] Equipment instance */
        mxidaf_Stream *stream_list[],   /**< [out] Buffer to be filled */
        int len                         /**< [in] Length of stream_list */
    );

/**
 * @brief Get Stream instance associated with a tag.
 *
 * @ingroup mxidaf-equipment
 *
 * @return
 *      Stream instance.\n
 *      Returns NULL on error.
 *
 * @see
 *      mxidaf_Stream_delete()
 */
mxidaf_Stream *
    mxidaf_Equipment_get_stream(
        const mxidaf_Equipment *self,   /**< [in] Equipment instance */
        const char *tag_name            /**< [in] Tag name of the Stream */
    );

#ifdef __cplusplus
}
#endif

#endif
