/**
 * @file
 * @author tc.huang@moxa.com
 * @brief Stream object.
 */

#ifndef _MXIDAF_STREAM_H_
#define _MXIDAF_STREAM_H_

#include <mxidaf/Tag.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @defgroup mxidaf-stream Stream object
 *
 * @ingroup mxidaf
 */

/**
 * @brief Stream instance placeholder.
 *
 * @ingroup mxidaf-stream
 */
typedef struct mxidaf_Stream_ mxidaf_Stream;

/**
 * @brief Delete Stream instance.
 *
 * @ingroup mxidaf-stream
 */
void
    mxidaf_Stream_delete(
        mxidaf_Stream *self     /**< [in] Stream instance to be deleted */
    );

/**
 * @brief Get equipment name associated with this object.
 *
 * @ingroup mxidaf-stream
 *
 * @return
 *      NULL terminated string.
 *      Pointer remains valid during the lifecycle of this object.
 */
const char *
    mxidaf_Stream_equipment_name(
        const mxidaf_Stream *self   /**< [in] Stream instance */
    );

/**
 * @brief Get tag name associated with this object.
 *
 * @ingroup mxidaf-stream
 *
 * @return
 *      NULL terminated string.
 *      Pointer remains valid during the lifecycle of this object.
 */
const char *
    mxidaf_Stream_tag_name(
        const mxidaf_Stream *self  /**< [in] Stream instance */
    );

/**
 * @brief Get last few Tags.
 *
 * @ingroup mxidaf-stream
 *
 * @return
 *      Number of Tags filled into tag_list.\n
 *      Return -1 on error.\n
 *      If len were given 0, this function returns the value of len required to obtain
 *      complete Tag list.
 *
 * @see
 *      mxidaf_Tag_delete()
 */
int
    mxidaf_Stream_get_last_tag_list(
        const mxidaf_Stream *self, /**< [in] Stream instance */
        mxidaf_Tag *tag_list[],    /**< [out] Buffer to be filled */
        int len                    /**< [in] Length of tag_list */
    );

#ifdef __cplusplus
}
#endif

#endif
