/**
 * @file
 * @author tc.huang@moxa.com
 * @brief String object
 */

#ifndef _MXIDAF_STRING_H_
#define _MXIDAF_STRING_H_

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @defgroup mxidaf-string String utility
 *
 * @ingroup mxidaf
 */

/**
 * @brief String object.
 *
 * @ingroup mxidaf-string
 */
typedef struct mxidaf_String_ mxidaf_String;

/**
 * @brief Create a String instance.
 *
 * @ingroup mxidaf-string
 *
 * @return
 *      Return String instance.\n
 *      Return NULL on failure.
 */
mxidaf_String *
    mxidaf_String_new(
        const char *s       /**< [in] Initial value\n
                                    Should point to a NULL terminated string.\n
                                    Can be NULL. */
    );

/**
 * @brief Delete a String instance.
 *
 * @ingroup mxidaf-string
 *
 */
void
    mxidaf_String_delete(
        mxidaf_String *self     /**< [in] String instance to be deleted */
    );

/**
 * @brief Get actual string.
 *
 * @ingroup mxidaf-string
 *
 * @return
 *      Pointer to a C-style string.\n
 *
 *      Returned pointer is maintained by String object.
 *      Manual free() is not permitted.
 */
const char *
    mxidaf_String_str(
        const mxidaf_String *self   /**< [in] String instance */
    );

/**
 * @brief Make a copy of String object.
 *
 * @ingroup mxidaf-string
 *
 * @return
 *      String instance.\n
 *      Return NULL on failure.
 */
mxidaf_String *
    mxidaf_String_clone(
        const mxidaf_String *from  /**< [in] String instance */
    );

#ifdef __cplusplus
}
#endif

#endif
