/**
 * @file
 * @author tc.huang@moxa.com
 * @brief Tag object
 *
 */

#ifndef _MXIDAF_TAG_H_
#define _MXIDAF_TAG_H_

#include <mxidaf/String.h>
#include <mxidaf/Time.h>
#include <mxidaf/Value.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @defgroup mxidaf-tag Tag object
 *
 * @ingroup mxidaf
 */

/**
 * @brief Tag object
 *
 * @ingroup mxidaf-tag
 */
typedef struct mxidaf_Tag_ mxidaf_Tag;

/**
 * @brief Create a new Tag instance.
 *
 * @ingroup mxidaf-tag
 *
 * @return
 *      Tag instance.\n
 *      Return NULL on failure.
 */
mxidaf_Tag *
    mxidaf_Tag_new(
        const mxidaf_Value *value,   /**< [in] Tag value */
        const mxidaf_Time *at,       /**< [in] Timestamp */
        const char *unit             /**< [in] Unit of value */
    );

/**
 * @brief Create a new Tag instance using system time.
 *
 * @ingroup mxidaf-tag
 *
 * @return
 *      A new Tag object.\n
 *      Return NULL on failure.
 */
mxidaf_Tag *
    mxidaf_Tag_new_system_time(
        const mxidaf_Value *value,   /**< [in] Tag value */
        const char *unit             /**< [in] Unit of value */
    );

/**
 * @brief Delete a Tag instance.
 *
 * @ingroup mxidaf-tag
 */
void
    mxidaf_Tag_delete(
        mxidaf_Tag *self    /**< Tag instance to be deleted */
    );

/**
 * @brief Get contained values.
 *
 * @ingroup mxidaf-tag
 *
 * @return
 *      Return 0 on success.\n
 *      Return -1 on failure.
 *
 * @see
 *      mxidaf_String_delete()
 *      mxidaf_Time_delete()
 *      mxidaf_Value_delete()
 */
int
    mxidaf_Tag_get(
        const mxidaf_Tag *self,     /**< [in] Tag instance */
        mxidaf_Value **value,       /**< [out] Buffer to fill Tag value */
        mxidaf_Time **at,           /**< [out] Buffer to fill timestamp */
        mxidaf_String **unit        /**< [out] Buffer to fill unit */
    );

/**
 * @brief Create a copy of existing Tag object.
 *
 * @ingroup mxidaf-tag
 *
 * @return
 *      A new Tag object.\n
 *      Return NULL on failure.
 */
mxidaf_Tag *
    mxidaf_Tag_clone(
        const mxidaf_Tag *from      /**< [in] Tag instance to be cloned */
    );

#ifdef __cplusplus
}
#endif

#endif
