/**
 * @file
 * @author tc.huang@moxa.com
 * @brief TagImporter object
 *
 */

#ifndef _MXIDAF_TAG_IMPORTER_H_
#define _MXIDAF_TAG_IMPORTER_H_

#include <mxidaf/Tag.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @defgroup mxidaf-tag-importer TagImporter object
 *
 * @ingroup mxidaf
 */

/**
 * @brief TagImporter object.
 *
 * @ingroup mxidaf-tag-importer
 */
typedef struct mxidaf_TagImporter_ mxidaf_TagImporter;

/**
 * @brief Delete TagImporter instance.
 *
 * @ingroup mxidaf-tag-importer
 */
void
    mxidaf_TagImporter_delete(
        mxidaf_TagImporter *self    /**< [in] Instance to be deleted */
    );

/**
 * @brief Import Tag value.
 *
 * This function blocks until
 *   - All Tags are imported.
 *   - An error occurred.
 *
 * @ingroup mxidaf-tag-importer
 *
 * @return
 *      Returns 0 on success.\n
 *      \n
 *      Returns -1 on failure.
 */
int
    mxidaf_TagImporter_import(
        mxidaf_TagImporter *self,       /**< [in] TagImporter instance */
        const char *equipment_name,     /**< [in] Equipment name */
        const char *tag_name,           /**< [in] Tag name */
        mxidaf_Tag * const tag_list[],  /**< [in] Tag values to be imported */
        int len                         /**< [in] Length of tag_list */
    );

#ifdef __cplusplus
}
#endif

#endif
