#include <assert.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>

#include <mxidaf/Service.h>
#include <mxidaf/Value.h>

#include "error.h"

double
    last_value_of_stream(
        mxidaf_Stream *stream);

int
    main(
        int argc,
        const char *argv[])
{
    mxidaf_Service *service = mxidaf_Service_instance();

    mxidaf_Cache *cache = mxidaf_Service_get_cache(
        service);
    assert(cache);

    mxidaf_Equipment *equipment = mxidaf_Cache_get_equipment(
        cache,
        "electrical-probe");
    assert(equipment);

    mxidaf_Stream *voltage_stream = mxidaf_Equipment_get_stream(
        equipment,
        "voltage");
    assert(voltage_stream);

    mxidaf_Stream *current_stream = mxidaf_Equipment_get_stream(
        equipment,
        "current");
    assert(current_stream);

    mxidaf_Value *watt = mxidaf_Value_new_float(
        last_value_of_stream(voltage_stream) *
        last_value_of_stream(current_stream));
    assert(watt);

    mxidaf_Tag *tag = mxidaf_Tag_new_system_time(
        watt,
        "w");
    assert(tag);

    mxidaf_TagImporter *tag_importer = mxidaf_Service_get_tag_importer(service);
    if (!tag_importer)
        handle_error(service);

    const int rc = mxidaf_TagImporter_import(
        tag_importer,
        "electrical-probe",
        "power",
        &tag,
        1);
    if (0 != rc)
        handle_error(service);

    mxidaf_TagImporter_delete(tag_importer);
    mxidaf_Tag_delete(tag);
    mxidaf_Value_delete(watt);
    mxidaf_Stream_delete(current_stream);
    mxidaf_Stream_delete(voltage_stream);
    mxidaf_Equipment_delete(equipment);
    mxidaf_Cache_delete(cache);

    return 0;
}

double
    last_value_of_stream(
        mxidaf_Stream *stream)
{
    mxidaf_Tag *tag;

    const int len = mxidaf_Stream_get_last_tag_list(
        stream,
        &tag,
        1);

    assert(1 == len);

    mxidaf_Value *value;

    mxidaf_Tag_get(
        tag,
        &value,
        NULL,
        NULL);

    mxidaf_Tag_delete(tag);

    assert(mxidaf_Value_is_float(value));

    const double last = mxidaf_Value_as_float(value);
    mxidaf_Value_delete(value);

    return last;
}
