#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <mxidaf/Service.h>

#include "error.h"

int
    main(
        int argc,
        const char *argv[])
{
    mxidaf_Service *service = mxidaf_Service_instance();

    mxidaf_Time *at = mxidaf_Time_new_system_time();
    assert(at);

    mxidaf_Value *volt = mxidaf_Value_new_float(5.010);
    assert(volt);

    mxidaf_Value *curr = mxidaf_Value_new_float(0.344);
    assert(curr);

    mxidaf_Tag *volt_tag = mxidaf_Tag_new(
        volt,
        at,
        "v");
    assert(volt_tag);

    mxidaf_Tag *curr_tag = mxidaf_Tag_new(
        curr,
        at,
        "a");
    assert(curr_tag);

    /*  value and at are copied by mxidaf_Tag_new(),
        so that these instances are not needed anymore. */
    mxidaf_Value_delete(volt);
    mxidaf_Value_delete(curr);
    mxidaf_Time_delete(at);

    mxidaf_TagImporter *tag_importer = mxidaf_Service_get_tag_importer(
        service);
    assert(tag_importer);

    int rc = mxidaf_TagImporter_import(
        tag_importer,
        "electrical-probe",     /* equipment name */
        "voltage",              /* tag name */
        &volt_tag,
        1);
    if (0 != rc)
        handle_error(service);

    rc = mxidaf_TagImporter_import(
        tag_importer,
        "electrical-probe",
        "current",
        &curr_tag,
        1);
    if (0 != rc)
        handle_error(service);

    mxidaf_TagImporter_delete(tag_importer);

    mxidaf_Tag_delete(volt_tag);
    mxidaf_Tag_delete(curr_tag);

    return 0;
}
