#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>

#include <mxidaf/Service.h>
#include <mxidaf/Value.h>

#include "error.h"

int
    main(
        int argc,
        const char *argv[])
{
    mxidaf_Service *service = mxidaf_Service_instance();

    mxidaf_Cache *cache = mxidaf_Service_get_cache(
        service);
    if (!cache)
    {
        handle_error(service);
    }

    mxidaf_Equipment *equipment = mxidaf_Cache_get_equipment(
        cache,
        "electrical-probe");
    if (!equipment)
    {
        handle_error(service);
    }

    mxidaf_Stream *stream = mxidaf_Equipment_get_stream(
        equipment,
        "voltage");
    if (!stream)
    {
        handle_error(service);
    }

    mxidaf_Tag *tag_list[3] = {0};
    const int len = mxidaf_Stream_get_last_tag_list(
        stream,
        tag_list,
        3);

    printf("queried %d tags\n", len);
    int ax;
    for(ax = 0; ax < len; ++ax)
    {
        mxidaf_Value *value = NULL;
        mxidaf_Time *at = NULL;
        mxidaf_String *unit = NULL;

        if (0 != mxidaf_Tag_get(
                tag_list[ax],
                &value,
                &at,
                &unit))
        {
            handle_error(service);
        }

        mxidaf_Time_Value tv;
        mxidaf_Time_get(
            at,
            &tv);

        printf("%02d-%02d-%02dT%02d:%02d:%02d.%03dZ",
            tv.year, tv.month, tv.day, tv.hour, tv.minute, tv.second, tv.millisec);

        if (mxidaf_Value_is_int(value))
            printf(":%" PRId64, mxidaf_Value_as_int(value));
        else if (mxidaf_Value_is_uint(value))
            printf(":%" PRIu64, mxidaf_Value_as_uint(value));
        else if (mxidaf_Value_is_float(value))
            printf(":%f", mxidaf_Value_as_float(value));
        else
            printf(":%s", mxidaf_Value_as_string(value));

        printf(":%s\n", mxidaf_String_str(unit));

        mxidaf_Value_delete(value);
        mxidaf_Time_delete(at);
        mxidaf_String_delete(unit);

        mxidaf_Tag_delete(tag_list[ax]);
    }

    mxidaf_Stream_delete(stream);
    mxidaf_Equipment_delete(equipment);
    mxidaf_Cache_delete(cache);

    return 0;
}
