#include <mxidaf/Cache.h>

#include <cassert>

#include <mxidafpp/Cache.hpp>

#include <mxidaf/impl/Service.h>

using mxidafpp::Equipment;
using mxidafpp::Cache;

using std::min;
using std::vector;

void
    mxidaf_Cache_delete(
        mxidaf_Cache *self)
{
    if (NULL == self)
        return;

    Cache *cache = (Cache *)self;

    delete cache;
}

int
    mxidaf_Cache_get_equipment_list(
        const mxidaf_Cache *self,
        mxidaf_Equipment *equipment_list[],
        int len)
{
    assert(self);

    const Cache *cache = (const Cache *)self;

    vector<Equipment> equipment_list_;

    try
    {
        equipment_list_ = cache->get_equipment_list();
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());
        return -1;
    }

    if (len == 0)
        return equipment_list_.size();

    assert(equipment_list && len > 0);

    len = min<int>(len, equipment_list_.size());
    for(int ax = 0; ax < len; ++ax)
    {
        equipment_list[ax] = (mxidaf_Equipment *)(new Equipment(equipment_list_[ax]));
    }

    return len;
}

mxidaf_Equipment *
    mxidaf_Cache_get_equipment(
        const mxidaf_Cache *self,
        const char *equ_name)
{
    assert(self && equ_name);

    const Cache *cache = (const Cache *)self;

    try
    {
        Equipment *equipment = new Equipment(cache->get_equipment(equ_name));

        return (mxidaf_Equipment *)equipment;
    }
    catch(mxidafpp::NotFoundError &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_NOT_FOUND,
            diagnostic_information(e).c_str());

        return NULL;
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());

        return NULL;
    }
}
