#include <mxidaf/Equipment.h>

#include <mxidafpp/Equipment.hpp>

#include <mxidaf/impl/Service.h>

using mxidafpp::Equipment;
using mxidafpp::Stream;

using std::min;
using std::vector;

void
    mxidaf_Equipment_delete(
        mxidaf_Equipment *self)
{
    if (NULL == self)
        return;

    Equipment *inst = (Equipment *)self;

    delete inst;
}

const char *
    mxidaf_Equipment_get_name(
        const mxidaf_Equipment *self)
{
    assert(self);

    const Equipment *inst = (const Equipment *)self;

    return inst->name().c_str();
}

int
    mxidaf_Equipment_get_stream_list(
        const mxidaf_Equipment *self,
        mxidaf_Stream *stream_list[],
        int len)
{
    assert(self);

    const Equipment *inst = (const Equipment *)self;

    vector<Stream> stream_list_;

    try
    {
        stream_list_ = inst->get_stream_list();
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());

        return -1;
    }

    if (len == 0)
        return stream_list_.size();

    assert(stream_list != NULL && len > 0);

    len = min<int>(len, stream_list_.size());

    for(int ax = 0; ax < len; ++ax)
    {
        stream_list[ax] = (mxidaf_Stream *)(new Stream(stream_list_[ax]));
    }

    return len;
}

mxidaf_Stream *
    mxidaf_Equipment_get_stream(
        const mxidaf_Equipment *self,
        const char *tag_name)
{
    assert(self && tag_name);

    const Equipment *equipment = (const Equipment *)self;

    try
    {
        Stream *stream = new Stream(equipment->get_stream(tag_name));

        return (mxidaf_Stream *)stream;
    }
    catch(mxidafpp::NotFoundError &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_NOT_FOUND,
            diagnostic_information(e).c_str());

        return NULL;
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());

        return NULL;
    }
}
