#include <mxidaf/Service.h>

#include <mxidaf/impl/Service.h>
#include <mxidaf/impl/ServiceContext.hpp>

using mxidaf::impl::ServiceContext;

using mxidafpp::Cache;
using mxidafpp::TagImporter;

using std::string;

namespace
{

ServiceContext      context;

}

mxidaf_Service *
    mxidaf_Service_instance()
{
    return (mxidaf_Service *)&context;
}

mxidaf_Cache *
    mxidaf_Service_get_cache(
        const mxidaf_Service *self)
{
    assert(self);

    if (self != (mxidaf_Service *)&context)
    {
        // TODO
        return NULL;
    }

    try
    {
        Cache *pcache = new Cache(context.service().get_cache());

        return (mxidaf_Cache *)pcache;
    }
    catch(mxidafpp::NotFoundError &e)
    {
        mxidaf_Service_set_last_error(
            const_cast<mxidaf_Service *>(self),
            MXIDAF_ERROR_NOT_FOUND,
            diagnostic_information(e).c_str());
        return NULL;
    }
    catch(mxidafpp::VersionError &e)
    {
        mxidaf_Service_set_last_error(
            const_cast<mxidaf_Service *>(self),
            MXIDAF_ERROR_VERSION,
            diagnostic_information(e).c_str());
        return NULL;
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            const_cast<mxidaf_Service *>(self),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());
        return NULL;
    }
}

mxidaf_Cache *
    mxidaf_Service_get_cache_2(
        const mxidaf_Service *self,
        const char *dbpath)
{
    assert(self);

    if (self != (mxidaf_Service *)&context)
    {
        // TODO
        return NULL;
    }

    try
    {
        Cache *pcache = new Cache(context.service().get_cache(dbpath));

        return (mxidaf_Cache *)pcache;
    }
    catch(mxidafpp::NotFoundError &e)
    {
        mxidaf_Service_set_last_error(
            const_cast<mxidaf_Service *>(self),
            MXIDAF_ERROR_NOT_FOUND,
            diagnostic_information(e).c_str());
        return NULL;
    }
    catch(mxidafpp::VersionError &e)
    {
        mxidaf_Service_set_last_error(
            const_cast<mxidaf_Service *>(self),
            MXIDAF_ERROR_VERSION,
            diagnostic_information(e).c_str());
        return NULL;
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            const_cast<mxidaf_Service *>(self),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());
        return NULL;
    }
}

mxidaf_TagImporter *
    mxidaf_Service_get_tag_importer(
        const mxidaf_Service *self)
{
    if (self != (const mxidaf_Service *)&context)
    {
        // TODO
        return NULL;
    }

    TagImporter *ti = new TagImporter(context.service().get_tag_importer());

    return (mxidaf_TagImporter *)ti;
}

mxidaf_TagImporter *
    mxidaf_Service_get_tag_importer_2(
        const mxidaf_Service *self,
        const char *host,
        int port)
{
    if (self != (const mxidaf_Service *)&context)
    {
        // TODO
        return NULL;
    }

    TagImporter *ti = new TagImporter(context.service().get_tag_importer(
        host,
        port));

    return (mxidaf_TagImporter *)ti;
}

mxidaf_Error
    mxidaf_Service_get_last_error(
        const mxidaf_Service *self)
{
    assert(self);

    if (self != (mxidaf_Service *)&context)
    {
        // TODO
        return MXIDAF_ERROR_ILLEGAL_SERVICE;
    }

    return context.error();
}

const char *
    mxidaf_Service_get_last_error_msg(
        const mxidaf_Service *self)
{
    assert(self);

    if (self != (mxidaf_Service *)&context)
    {
        // TODO
        return NULL;
    }

    return context.error_msg().c_str();
}

void
    mxidaf_Service_set_last_error(
        mxidaf_Service *self,
        mxidaf_Error err,
        const char *errmsg)
{
    assert(self == (mxidaf_Service *)&context);

    return context.set_error(err, errmsg);
}
