#include <mxidaf/Stream.h>

#include <boost/foreach.hpp>
#include <boost/scope_exit.hpp>

#include <mxidaf/impl/Tag.hpp>

#include <mxidafpp/Stream.hpp>

#include <mxidaf/impl/Service.h>

using mxidafpp::Stream;

using mxidaf::impl::mxidaf_from_mxidafpp;

using mxidafpp::Tag;
using std::min;
using std::vector;

void
    mxidaf_Stream_delete(
        mxidaf_Stream *self)
{
    if (NULL == self)
        return;

    Stream *stream = (Stream *)self;

    delete stream;
}

const char *
    mxidaf_Stream_equipment_name(
        const mxidaf_Stream *self)
{
    assert(self);

    const Stream *stream = (const Stream *)self;

    return stream->equipment_name().c_str();
}

const char *
    mxidaf_Stream_tag_name(
        const mxidaf_Stream *self)
{
    assert(self);

    const Stream *stream = (const Stream *)self;

    return stream->tag_name().c_str();
}

int
    mxidaf_Stream_get_last_tag_list(
        const mxidaf_Stream *self,
        mxidaf_Tag *tag_list[],
        int len)
{
    assert(self && len >= 0);

    const Stream *stream = (const Stream *)self;

    vector<Tag> tag_list_;
    try
    {
        tag_list_ = stream->get_last_tag_list(len);
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());

        return -1;
    }

    if (len == 0)
        return tag_list_.size();

    assert(tag_list && len > 0);

    len = min<int>(len, tag_list_.size());

    for(int ax = 0; ax < len; ++ax)
    {
        tag_list[ax] = mxidaf_from_mxidafpp(
            tag_list_[ax]);
    }

    return len;
}
