#include <mxidaf/Tag.h>

#include <cassert>

#include <boost/date_time/posix_time/posix_time.hpp>

#include <mxidafpp/Tag.hpp>
#include <mxidafpp/Value.hpp>

using mxidafpp::Tag;
using mxidafpp::Value;
using mxidafpp::Time;

using std::string;

extern "C" mxidaf_Tag *
    mxidaf_Tag_new(
        const mxidaf_Value *value,
        const mxidaf_Time *at,
        const char *unit)
{
    assert(value && at);

    const mxidafpp::Value *value_inst = (const mxidafpp::Value *)value;
    const mxidafpp::Time *at_inst = (const mxidafpp::Time *)at;

    try
    {
        mxidafpp::Tag *inst = new mxidafpp::Tag(
            *value_inst,
            *at_inst,
            unit);

        return (mxidaf_Tag *)inst;
    }
    catch(...)
    {
        // TODO
        return NULL;
    }
}

extern "C" mxidaf_Tag *
    mxidaf_Tag_new_system_time(
        const mxidaf_Value *value,
        const char *unit)
{
    assert(value);

    const mxidafpp::Value *value_inst = (const mxidafpp::Value *)value;

    try
    {
        mxidafpp::Tag *inst = new mxidafpp::Tag(
            *value_inst,
            mxidafpp::Time::now(),
            unit);

        return (mxidaf_Tag *)inst;
    }
    catch(...)
    {
        // TODO
        return NULL;
    }
}

extern "C" void
    mxidaf_Tag_delete(
        mxidaf_Tag *self)
{
    if (NULL == self)
        return;

    mxidafpp::Tag *inst = (mxidafpp::Tag *)self;

    delete inst;
}

extern "C" int
    mxidaf_Tag_get(
        const mxidaf_Tag *self,
        mxidaf_Value **value,
        mxidaf_Time **at,
        mxidaf_String **unit)
{
    assert(self);

    const Tag *inst = (const Tag *)self;

    try
    {
        if (value)
        {
            Value *value_inst = new Value(
                inst->value());

            *value = (mxidaf_Value *)value_inst;
        }

        if (at)
        {
            Time *at_inst = new Time(
                inst->at());

            *at = (mxidaf_Time *)at_inst;
        }

        if (unit)
        {
            string *unit_inst = new string(
                inst->unit());

            *unit = (mxidaf_String *)unit_inst;
        }

        return 0;
    }
    catch(...)
    {
        // TODO
        return -1;
    }
}

extern "C" mxidaf_Tag *
    mxidaf_Tag_clone(
        const mxidaf_Tag *from)
{
    assert(from);

    const Tag *from_inst = (const Tag *)from;

    try
    {
        Tag *to_inst = new Tag(*from_inst);

        return (mxidaf_Tag *)to_inst;
    }
    catch(...)
    {
        // TODO
        return NULL;
    }
}
