#include <mxidaf/TagImporter.h>

#include <mxidafpp/TagImporter.hpp>

#include <mxidaf/impl/Service.h>
#include <mxidaf/impl/Tag.hpp>

using mxidaf::impl::mxidafpp_from_mxidaf;

using mxidafpp::Tag;
using mxidafpp::TagImporter;

using std::vector;

extern "C" void
    mxidaf_TagImporter_delete(
        mxidaf_TagImporter *self)
{
    if (self == NULL)
        return;

    TagImporter *inst = (TagImporter *)self;

    delete inst;
}

extern "C" int
    mxidaf_TagImporter_import(
        mxidaf_TagImporter *self,
        const char *equipment_name,
        const char *tag_name,
        mxidaf_Tag *const tag_list[],
        int len)
{
    BOOST_VERIFY(self);

    TagImporter *inst = (TagImporter *)self;

    vector<Tag> tag_list_;
    for (int ax = 0; ax < len; ++ax)
    {
        tag_list_.push_back(mxidafpp_from_mxidaf(tag_list[ax]));
    }

    try
    {
        inst->import(
            equipment_name,
            tag_name,
            tag_list_);

        return 0;
    }
    catch(mxidafpp::ArgumentError &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_ILLEGAL_ARGUMENT,
            diagnostic_information(e).c_str());

        return -1;
    }
    catch(mxidafpp::Error &e)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            diagnostic_information(e).c_str());

        return -1;
    }
    catch(...)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            "unknown error");

        return -1;
    }
}
