#include <mxidaf/Time.h>

#include <mxidafpp/Time.hpp>

#include <mxidaf/impl/Service.h>

using mxidafpp::Time;

extern "C" mxidaf_Time *
    mxidaf_Time_new(
        const mxidaf_Time_Value *tv)
{
    try
    {
        Time *inst = new Time(
            tv->year,
            tv->month,
            tv->day,
            tv->hour,
            tv->minute,
            tv->second,
            tv->millisec);

        return (mxidaf_Time *)inst;
    }
    catch(...)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            "unexpected exception caught");

        return NULL;
    }
}

extern "C" mxidaf_Time *
    mxidaf_Time_new_system_time()
{
    try
    {
        Time *inst = new Time(Time::now());

        return (mxidaf_Time *)inst;
    }
    catch(...)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            "unexpected exception caught");

        return NULL;
    }
}

extern "C" void
    mxidaf_Time_delete(
        mxidaf_Time *self)
{
    if (!self)
        return;

    Time *inst = (Time *)self;

    delete inst;
}

extern "C" mxidaf_Time_Value *
    mxidaf_Time_get(
        const mxidaf_Time *self,
        mxidaf_Time_Value *tv)
{
    assert(self && tv);

    const Time *inst = (const Time *)self;

    try
    {
        tv->year = inst->year();
        tv->month = inst->month();
        tv->day = inst->day();
        tv->hour = inst->hour();
        tv->minute = inst->minute();
        tv->second = inst->second();
        tv->millisec = inst->millisec();

        return tv;
    }
    catch(...)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            "unexpected exception caught");

        return NULL;
    }
}

extern "C" mxidaf_Time *
    mxidaf_Time_clone(
        const mxidaf_Time *from)
{
    assert(from);

    const Time *from_inst = (const Time *)from;

    try
    {
        Time *to = new Time(*from_inst);

        return (mxidaf_Time *)to;
    }
    catch(...)
    {
        mxidaf_Service_set_last_error(
            mxidaf_Service_instance(),
            MXIDAF_ERROR_INTERNAL,
            "unexpected exception caught");

        return NULL;
    }
}

extern "C" int
    mxidaf_Time_compare(
        const mxidaf_Time *lhs_,
        const mxidaf_Time *rhs_)
{
    const Time *lhs = (const Time *)lhs_;
    const Time *rhs = (const Time *)rhs_;

    return lhs->compare(*rhs);
}
