#ifndef _DATABASE_FIXTURE_HPP_
#define _DATABASE_FIXTURE_HPP_

#include <unistd.h>

#include <string>

#include <mxhtag/Database.hpp>

struct Fixture
{
    Fixture()
        : dbpath("/dev/shm/mxhtag.sqlite3")
    {
        if (0 != unlink(this->dbpath.c_str()))
        {
            // do nothing
        }
    }

    virtual ~Fixture()
    {
        if (0 != unlink(this->dbpath.c_str()))
        {
            // do nothing
        }
    }

    const std::string    dbpath;
};

struct CreateTablesFixture
    : public Fixture
{
    using Fixture::dbpath;

    CreateTablesFixture()
    {
        mxhtag::Database::create_tables(dbpath);
    }
};

struct DatabaseFixture
    : public CreateTablesFixture
{
    using CreateTablesFixture::dbpath;

    DatabaseFixture()
    {
        using boost::lexical_cast;
        using boost::posix_time::ptime;

        using std::string;

        mxhtag::Database database(dbpath);

        const unsigned int readings_per_tag = 10;
        const unsigned int equ_count = 3;
        const unsigned int tags_per_equ = 3;

        const ptime base_time = boost::posix_time::time_from_string("2010-01-01 00:00:00");

        for(unsigned int reading = 0; reading < readings_per_tag; ++reading)
        {
            const ptime reading_time = base_time + boost::posix_time::seconds(reading);

            std::vector<mxtag::Tag> reading_list;
            for(unsigned int equ = 0; equ < equ_count; ++equ)
            {
                for(unsigned int tag = 0; tag < tags_per_equ; ++tag)
                {
                    reading_list.push_back(mxtag::Tag(
                        "equipment" + lexical_cast<string>(equ),
                        "e" + lexical_cast<string>(equ) + "tag" + lexical_cast<string>(tag),
                        reading_time,
                        mxtag::Value("test-value"),
                        "test-unit"));
                }
            }

            database.insert_reading_list(reading_list);
        }
    }
};


#endif
