#include <boost/test/unit_test.hpp>

#include <mxidaf/Cache.h>

#include <boost/range.hpp>
#include <boost/scope_exit.hpp>

#include <mxidaf/Service.h>

#include "DatabaseFixture.hpp"

using boost::size;

using std::string;

namespace
{

struct TestFixture
    : public DatabaseFixture
{
    TestFixture()
    {
        mxidaf_Service *service = mxidaf_Service_instance();
        cache = mxidaf_Service_get_cache(
            service);
    }

    ~TestFixture()
    {
        mxidaf_Cache_delete(
            cache);
    }

    mxidaf_Cache *cache;
};

}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Cache_delete_ShouldPass, TestFixture)
{
    // arrange
    mxidaf_Cache *inst = mxidaf_Service_get_cache(
        mxidaf_Service_instance());

    // act
    mxidaf_Cache_delete(
        inst);

    // assert
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Cache_get_equipment_list_ShouldPass, TestFixture)
{
    // arrange
    mxidaf_Equipment *equipment_list[10];

    // act
    const int len = mxidaf_Cache_get_equipment_list(
        cache,
        &equipment_list[0],
        size(equipment_list));

    BOOST_SCOPE_EXIT((&len)(&equipment_list))
    {
        for (int ax = 0; ax < len; ++ax)
        {
            mxidaf_Equipment_delete(equipment_list[ax]);
            equipment_list[ax] = NULL;
        }
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_CHECK_EQUAL(3, len);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Stroage_get_equipment_list_WithZeroLen_ShouldPass, TestFixture)
{
    // arrange

    // act
    const int len = mxidaf_Cache_get_equipment_list(
        cache,
        0,
        0);

    // assert
    BOOST_CHECK_EQUAL(3, len);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Cache_get_equipment_ShouldPass, TestFixture)
{
    // arrange

    // act
    mxidaf_Equipment *equipment = mxidaf_Cache_get_equipment(
        cache,
        "equipment1");

    BOOST_SCOPE_EXIT((&equipment))
    {
        mxidaf_Equipment_delete(equipment);
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_CHECK(NULL != equipment);
    BOOST_CHECK_EQUAL(string("equipment1"), mxidaf_Equipment_get_name(equipment));
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Cache_get_equipment_WithNotExistEquipment_ShouldFail, TestFixture)
{
    // arrange

    // act
    mxidaf_Equipment *equipment = mxidaf_Cache_get_equipment(
        cache,
        "not-exist-equipment");

    const int errc = mxidaf_Service_get_last_error(
        mxidaf_Service_instance());

    // assert
    BOOST_CHECK(NULL == equipment);
    BOOST_CHECK_EQUAL(MXIDAF_ERROR_NOT_FOUND, errc);
}
