#include <boost/test/unit_test.hpp>

#include <mxidaf/Service.h>

#include <string>

#include <boost/scope_exit.hpp>

#include <mxhtag/Database.hpp>

using std::string;

namespace
{

struct TestFixture
{
    TestFixture()
        : default_db_path("/dev/shm/mxhtag.sqlite3")
    {
    }

    const string default_db_path;
};

}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Service_instance_ShouldPass, TestFixture)
{
    // arrange

    // act
    mxidaf_Service *inst = mxidaf_Service_instance();

    // assert
    BOOST_CHECK(NULL != inst);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Service_get_cache_ShouldPass, TestFixture)
{
    // arrange
    if (0 != unlink(default_db_path.c_str()))
    {
        // ignored
    }

    mxhtag::Database::create_tables(default_db_path);
    BOOST_SCOPE_EXIT((&default_db_path))
    {
        if (0 != unlink(default_db_path.c_str()))
        {
            // ignored
        }
    }
    BOOST_SCOPE_EXIT_END

    mxidaf_Service *service = mxidaf_Service_instance();

    // act
    mxidaf_Cache *cache = mxidaf_Service_get_cache(
        service);

    BOOST_SCOPE_EXIT((&cache))
    {
        if (cache)
            mxidaf_Cache_delete(cache);

        cache = NULL;
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_CHECK(NULL != cache);
}

BOOST_FIXTURE_TEST_CASE(tset_mxidaf_Service_get_cache_WhenDbNotExist_ShouldFail, TestFixture)
{
    // arrange
    if (0 != unlink(default_db_path.c_str()))
    {
        // ignored
    }

    mxidaf_Service *service = mxidaf_Service_instance();

    // act
    mxidaf_Cache *cache = mxidaf_Service_get_cache(
        service);

    BOOST_SCOPE_EXIT((&cache))
    {
        if (cache)
            mxidaf_Cache_delete(cache);

        cache = NULL;
    }
    BOOST_SCOPE_EXIT_END

    const mxidaf_Error error = mxidaf_Service_get_last_error(service);

    // assert
    BOOST_REQUIRE(NULL == cache);
    BOOST_CHECK_EQUAL(MXIDAF_ERROR_NOT_FOUND, error);
}
