#include <boost/test/unit_test.hpp>

#include <mxidaf/Stream.h>

#include <vector>

#include <boost/foreach.hpp>
#include <boost/range.hpp>
#include <boost/scope_exit.hpp>

#include <mxidaf/Service.h>

#include "DatabaseFixture.hpp"

using boost::size;

using std::string;
using std::vector;

namespace
{

struct TestFixture
    : public DatabaseFixture
{
    TestFixture()
    {
        mxidaf_Service *service = mxidaf_Service_instance();

        cache = mxidaf_Service_get_cache(
            service);

        equipment = mxidaf_Cache_get_equipment(
            cache,
            "equipment1");

        stream = mxidaf_Equipment_get_stream(
            equipment,
            "e1tag0");
    }

    ~TestFixture()
    {
        mxidaf_Stream_delete(stream);
        mxidaf_Equipment_delete(equipment);
        mxidaf_Cache_delete(cache);
    }

    mxidaf_Cache *cache;
    mxidaf_Equipment *equipment;
    mxidaf_Stream *stream;
};

}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Stream_destruct_ShouldPass, TestFixture)
{
    // arrange
    mxidaf_Stream *inst = mxidaf_Equipment_get_stream(
        equipment,
        "e1tag1");

    // act
    mxidaf_Stream_delete(
        inst);

    // assert
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Stream_equipment_name_ShouldPass, TestFixture)
{
    // arrange

    // act
    const char *equipment_name = mxidaf_Stream_equipment_name(
        stream);

    // assert
    BOOST_CHECK_EQUAL(string("equipment1"), equipment_name);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Stream_tag_name_ShouldPass, TestFixture)
{
    // arrange

    // act
    const char *tag_name = mxidaf_Stream_tag_name(
        stream);

    // assert
    BOOST_CHECK_EQUAL(string("e1tag0"), tag_name);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Stream_get_last_tag_list_ShouldPass, TestFixture)
{
    // arrange
    mxidaf_Tag *tag_list[20];

    // act
    const int len = mxidaf_Stream_get_last_tag_list(
        stream,
        &tag_list[0],
        size(tag_list));

    BOOST_SCOPE_EXIT((&len)(&tag_list))
    {
        for(int ax = 0; ax < len; ++ax)
        {
            mxidaf_Tag_delete(tag_list[ax]);
            tag_list[ax] = NULL;
        }
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_CHECK_EQUAL(10, len);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Stream_get_last_tag_list_WithZeroLen_ShouldPass, TestFixture)
{
    // arrange

    // act
    const int len = mxidaf_Stream_get_last_tag_list(
        stream,
        0,
        0);

    // assert
    BOOST_CHECK_EQUAL(10, len);
}
