#include <boost/test/unit_test.hpp>

#include <mxidaf/Time.h>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/scope_exit.hpp>

using boost::gregorian::date;
using boost::posix_time::microsec_clock;
using boost::posix_time::ptime;
using boost::posix_time::time_duration;
using boost::posix_time::milliseconds;

namespace
{

struct TestFixture
{
};

}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Time_new_ShouldPass, TestFixture)
{
    // arrange
    const mxidaf_Time_Value ctv = {
        year: 2016,
        month: 2,
        day: 19,
        hour: 16,
        minute: 19,
        second: 4,
        millisec: 123};

    // act
    mxidaf_Time *inst = mxidaf_Time_new(
        &ctv);

    BOOST_SCOPE_EXIT((&inst))
    {
        if (inst)
        {
            mxidaf_Time_delete(inst);

            inst = NULL;
        }
    }
    BOOST_SCOPE_EXIT_END

    // assert
    mxidaf_Time_Value tv;
    mxidaf_Time_get(
        inst,
        &tv);

    BOOST_CHECK_EQUAL(2016, tv.year);
    BOOST_CHECK_EQUAL(2, tv.month);
    BOOST_CHECK_EQUAL(19, tv.day);
    BOOST_CHECK_EQUAL(16, tv.hour);
    BOOST_CHECK_EQUAL(19, tv.minute);
    BOOST_CHECK_EQUAL(4, tv.second);
    BOOST_CHECK_EQUAL(123, tv.millisec);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Time_new_system_time_ShouldPass, TestFixture)
{
    // arrange
    const ptime now = microsec_clock::universal_time();

    // act
    mxidaf_Time *inst = mxidaf_Time_new_system_time();

    BOOST_SCOPE_EXIT((&inst))
    {
        if (inst)
        {
            mxidaf_Time_delete(inst);

            inst = NULL;
        }
    }
    BOOST_SCOPE_EXIT_END

    // assert
    mxidaf_Time_Value tv;
    mxidaf_Time_get(
        inst,
        &tv);

    const ptime t(
        date(tv.year, tv.month, tv.day),
        time_duration(tv.hour, tv.minute, tv.second) + milliseconds(tv.millisec));

    const time_duration td = t - now;

    BOOST_CHECK(td.total_milliseconds() < 1000);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Time_clone_ShouldPass, TestFixture)
{
    // arrange
    const mxidaf_Time_Value ctv = {
        year: 2016,
        month: 2,
        day: 19,
        hour: 16,
        minute: 19,
        second: 4,
        millisec: 123};
    mxidaf_Time *from = mxidaf_Time_new(
        &ctv);

    BOOST_SCOPE_EXIT((&from))
    {
        if (from)
        {
            mxidaf_Time_delete(from);

            from = NULL;
        }
    }
    BOOST_SCOPE_EXIT_END

    // act
    mxidaf_Time *to = mxidaf_Time_clone(
        from);

    BOOST_SCOPE_EXIT((&to))
    {
        if (to)
        {
            mxidaf_Time_delete(to);

            to = NULL;
        }
    }

    BOOST_SCOPE_EXIT_END
    // assert
    BOOST_CHECK(to != from);

    mxidaf_Time_Value tv;
    mxidaf_Time_get(
        to,
        &tv);

    BOOST_CHECK_EQUAL(2016, tv.year);
    BOOST_CHECK_EQUAL(2, tv.month);
    BOOST_CHECK_EQUAL(19, tv.day);
    BOOST_CHECK_EQUAL(16, tv.hour);
    BOOST_CHECK_EQUAL(19, tv.minute);
    BOOST_CHECK_EQUAL(4, tv.second);
    BOOST_CHECK_EQUAL(123, tv.millisec);
}
