#include <boost/test/unit_test.hpp>

#include <mxidaf/Value.h>

#include <string>

#include <boost/scope_exit.hpp>

using std::string;

namespace
{

struct TestFixture
{
};

}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Value_new_int_ShouldPass, TestFixture)
{
    // arrange

    // act
    mxidaf_Value *v = mxidaf_Value_new_int(
        -1234567890ll);

    BOOST_SCOPE_EXIT((&v))
    {
        mxidaf_Value_delete(v);
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_REQUIRE(v);
    BOOST_REQUIRE(mxidaf_Value_is_int(v));
    BOOST_CHECK_EQUAL(-1234567890ll, mxidaf_Value_as_int(v));
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Value_new_uint_ShouldPass, TestFixture)
{
    // arrange

    // act
    mxidaf_Value *v = mxidaf_Value_new_uint(
        9876543210ull);

    BOOST_SCOPE_EXIT((&v))
    {
        mxidaf_Value_delete(v);
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_REQUIRE(v);
    BOOST_REQUIRE(mxidaf_Value_is_uint(v));
    BOOST_CHECK_EQUAL(9876543210ull, mxidaf_Value_as_uint(v));
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Value_new_float_ShouldPass, TestFixture)
{
    // arrange

    // act
    mxidaf_Value *v = mxidaf_Value_new_float(
        -3.14159265);

    BOOST_SCOPE_EXIT((&v))
    {
        mxidaf_Value_delete(v);
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_REQUIRE(mxidaf_Value_is_float(v));
    BOOST_CHECK_CLOSE(-3.14159265, mxidaf_Value_as_float(v), 0.001);
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Value_new_string_ShouldPass, TestFixture)
{
    // arrange

    // act
    mxidaf_Value *v = mxidaf_Value_new_string(
        "hello world");

    BOOST_SCOPE_EXIT((&v))
    {
        mxidaf_Value_delete(v);
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_REQUIRE(v);
    BOOST_REQUIRE(mxidaf_Value_is_string(v));
    BOOST_CHECK(string("hello world") == mxidaf_Value_as_string(v));
}

BOOST_FIXTURE_TEST_CASE(test_mxidaf_Value_clone_ShouldPass, TestFixture)
{
    // arrange
    mxidaf_Value *from = mxidaf_Value_new_int(
        -1234567890ll);

    BOOST_SCOPE_EXIT((&from))
    {
        mxidaf_Value_delete(from);
    }
    BOOST_SCOPE_EXIT_END

    // act
    mxidaf_Value *to = mxidaf_Value_clone(
        from);

    BOOST_SCOPE_EXIT((&to))
    {
        mxidaf_Value_delete(to);
    }
    BOOST_SCOPE_EXIT_END

    // assert
    BOOST_REQUIRE(to);
    BOOST_REQUIRE(mxidaf_Value_is_int(to));
    BOOST_CHECK_EQUAL(-1234567890ll, mxidaf_Value_as_int(to));
}
