#include <assert.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>

#include <mxidaf/Service.h>

void
    import(mxidaf_TagImporter *tag_importer);

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 2)
    {
        printf("usage: <iteration>\n");
        return 1;
    }

    const int iteration = atoi(argv[1]);
    if (iteration <= 0)
    {
        printf("usage: <iteration>\n");
        return 1;
    }

    mxidaf_Service *service = mxidaf_Service_instance();
    mxidaf_TagImporter *tag_importer = mxidaf_Service_get_tag_importer(
        service);
    assert(tag_importer);

    int i;
    for(i = 0; i < iteration; ++i)
    {
        import(tag_importer);
    }

    return 0;
}

void
    import(
        mxidaf_TagImporter *tag_importer)
{
    assert(tag_importer);

    mxidaf_Time *at = mxidaf_Time_new_system_time();
    assert(at);

    mxidaf_Value *volt = mxidaf_Value_new_float(5.010);
    assert(volt);

    mxidaf_Tag *volt_tag = mxidaf_Tag_new(
        volt,
        at,
        "v");
    assert(volt_tag);

    mxidaf_Value_delete(volt);
    mxidaf_Time_delete(at);

    int rc = mxidaf_TagImporter_import(
        tag_importer,
        "electrical-probe",
        "voltage",
        &volt_tag,
        1);
    assert(0 == rc);

    mxidaf_Tag_delete(volt_tag);
}
