#include <assert.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>

#include <mxidaf/Service.h>

void
    iteration();

int
    main()
{
    int i;
    for(i = 0;; ++i)
    {
        printf("iteration: %d\n", i);
        iteration();
    }

    return 0;
}

void
    iteration()
{
    mxidaf_Service *service = mxidaf_Service_instance();

    // query //////////////////////////////////////////////////////////////////
    mxidaf_Cache *cache = mxidaf_Service_get_cache(
        service);
    if (!cache)
    {
        printf("!cache: %s\n", mxidaf_Service_get_last_error_msg(service));
        return;
    }

    mxidaf_Equipment *equipment = mxidaf_Cache_get_equipment(
        cache,
        "electrical-probe");
    assert(equipment);

    mxidaf_Stream *stream = mxidaf_Equipment_get_stream(
        equipment,
        "voltage");
    assert(stream);

    mxidaf_Tag *tag_list[3] = {0};
    const int len = mxidaf_Stream_get_last_tag_list(
        stream,
        tag_list,
        3);

    int ax;
    for(ax = 0; ax < len; ++ax)
    {
        mxidaf_Value *value = NULL;
        mxidaf_Time *at = NULL;
        mxidaf_String *unit = NULL;

        int rc = mxidaf_Tag_get(
                tag_list[ax],
                &value,
                &at,
                &unit);
        assert(0 == rc);

        mxidaf_Time_Value tv;
        mxidaf_Time_get(
            at,
            &tv);

        mxidaf_Value_delete(value);
        mxidaf_Time_delete(at);
        mxidaf_String_delete(unit);

        mxidaf_Tag_delete(tag_list[ax]);
    }

    mxidaf_Stream_delete(stream);
    mxidaf_Equipment_delete(equipment);
    mxidaf_Cache_delete(cache);

    // import /////////////////////////////////////////////////////////////////
    mxidaf_Time *at = mxidaf_Time_new_system_time();
    assert(at);

    mxidaf_Value *volt = mxidaf_Value_new_float(5.010);
    assert(volt);

    mxidaf_Tag *volt_tag = mxidaf_Tag_new(
        volt,
        at,
        "v");
    assert(volt_tag);

    mxidaf_Value_delete(volt);
    mxidaf_Time_delete(at);

    mxidaf_TagImporter *tag_importer = mxidaf_Service_get_tag_importer(
        service);
    assert(tag_importer);

    int rc = mxidaf_TagImporter_import(
        tag_importer,
        "electrical-probe",     /* equipment name */
        "voltage",              /* tag name */
        &volt_tag,
        1);
    assert(0 == rc);

    mxidaf_TagImporter_delete(tag_importer);
    mxidaf_Tag_delete(volt_tag);
}
