#include <mxidafpp/Service.hpp>

#include <boost/make_shared.hpp>

#include <mxidafpp/impl/CacheImpl.hpp>
#include <mxidafpp/impl/TagImporterImpl.hpp>

#include <mxutil/log/Log.hpp>
#include <mxutil/log/SyslogLogger.hpp>

using boost::filesystem::path;
using boost::make_shared;
using boost::shared_ptr;

using mxhtag::Database;

using mxidafpp::impl::CacheImpl;
using mxidafpp::impl::TagImporterImpl;

using mxutil::log::Log;
using mxutil::log::Logger;
using mxutil::log::SyslogLogger;

using std::string;

namespace
{

static const path DEFAULT_DBPATH = "/dev/shm/mxhtag.sqlite3";

void
    turn_off_logger()
{
    shared_ptr<Logger> logger = make_shared<SyslogLogger>("libmxidafpp");
    logger->set_level(Logger::EROR);
    Log::get().set_logger(logger);
}

}

namespace mxidafpp
{

Service &
    Service::instance()
{
    static Service inst;

    return inst;
}

struct Service::Impl
{
};

Service::Service()
    : pimpl(new Impl())
{
    turn_off_logger();
}

Service::~Service()
{
}

const Cache
    Service::get_cache()
{
    try
    {
        return Cache(make_shared<CacheImpl>(make_shared<Database>(DEFAULT_DBPATH.string())));
    }
    catch(mxhtag::OpenError &e)
    {
        BOOST_THROW_EXCEPTION(
            NotFoundError() << ErrInternalMsg(diagnostic_information(e))
        );
    }
    catch(mxhtag::VersionError &e)
    {
        BOOST_THROW_EXCEPTION(
            VersionError() << ErrInternalMsg(diagnostic_information(e))
        );
    }
    catch(mxhtag::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error() << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

const Cache
    Service::get_cache(const path &p)
{
    try
    {
        return Cache(make_shared<CacheImpl>(make_shared<Database>(p.string())));
    }
    catch(mxhtag::OpenError &e)
    {
        BOOST_THROW_EXCEPTION(
            NotFoundError() << ErrInternalMsg(diagnostic_information(e))
        );
    }
    catch(mxhtag::VersionError &e)
    {
        BOOST_THROW_EXCEPTION(
            VersionError() << ErrInternalMsg(diagnostic_information(e))
        );
    }
    catch(mxhtag::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error() << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

const TagImporter
    Service::get_tag_importer()
{
    return TagImporter(make_shared<TagImporterImpl>(
        "127.0.0.1",
        1883));
}

const TagImporter
    Service::get_tag_importer(
        const string &host,
        int port)
{
    return TagImporter(make_shared<TagImporterImpl>(
        host,
        port));
}

}
