#include <mxidafpp/Tag.hpp>

#include <mxtag/validate.hpp>

using mxtag::is_valid_unit;

using std::string;

namespace mxidafpp
{

struct Tag::Impl
{
    Impl(
        const Value &value_,
        const Time &at_,
        const string &unit_)
        : value(value_)
        , at(at_)
        , unit(unit_)
    {
    }

    const Value     value;
    const Time      at;
    const string    unit;
};

Tag::Tag(
    const Value &value,
    const Time &at,
    const string &unit)
    : pimpl(new Impl(
        value,
        at,
        unit))
{
    if (!is_valid_unit(pimpl->unit))
        BOOST_THROW_EXCEPTION(
            ArgumentError()
                << ErrMsg("illegal unit: " + pimpl->unit)
        );
}

Tag::~Tag()
{
}

const Time &
    Tag::at() const
{
    return pimpl->at;
}

const Value &
    Tag::value() const
{
    return pimpl->value;
}

const string &
    Tag::unit() const
{
    return pimpl->unit;
}

bool
    operator ==(
        const Tag &lhs,
        const Tag &rhs)
{
    if (&lhs == &rhs)
        return true;

    return
        lhs.at() == rhs.at() &&
        lhs.value() == rhs.value() &&
        lhs.unit() == rhs.unit();
}

bool
    operator !=(
        const Tag &lhs,
        const Tag &rhs)
{
    return !(lhs == rhs);
}

}
