#include <mxidafpp/Time.hpp>

#include <mxtag/tag_form.hpp>

#include <mxidafpp/impl/utility.hpp>

using boost::gregorian::date;
using boost::posix_time::microsec_clock;
using boost::posix_time::milliseconds;
using boost::posix_time::ptime;
using boost::posix_time::time_duration;

using mxtag::form_iso8601_time;

using mxidafpp::impl::precision_to_millisec;

using std::ostream;

namespace
{

const ptime
    ptime_from_raw(
        int year,
        int month,
        int day,
        int hour,
        int minute,
        int second,
        int millisec)
{
    return ptime(
        date(
            year,
            month,
            day),
        time_duration(
            hour,
            minute,
            second) + milliseconds(millisec)
    );
}

}

namespace mxidafpp
{

const Time
    Time::now()
{
    const ptime now = microsec_clock::universal_time();

    return Time(
        now.date().year(),
        now.date().month(),
        now.date().day(),
        now.time_of_day().hours(),
        now.time_of_day().minutes(),
        now.time_of_day().seconds(),
        now.time_of_day().total_milliseconds() % 1000);
}

struct Time::Impl
{
    Impl(
        const ptime &p_)
        : p(p_)
    {
    }

    const ptime p;
};

Time::Time(
    int year,
    int month,
    int day,
    int hour,
    int minute,
    int second,
    int millisec)
    : pimpl(new Impl(
        ptime_from_raw(
            year,
            month,
            day,
            hour,
            minute,
            second,
            millisec
        )
    ))
{
}

Time::Time(
    const ptime &p)
    : pimpl(new Impl(precision_to_millisec(p)))
{
}

Time::~Time()
{
}

int
    Time::year() const
{
    return pimpl->p.date().year();
}

int
    Time::month() const
{
    return pimpl->p.date().month();
}

int
    Time::day() const
{
    return pimpl->p.date().day();
}

int
    Time::hour() const
{
    return pimpl->p.time_of_day().hours();
}

int
    Time::minute() const
{
    return pimpl->p.time_of_day().minutes();
}

int
    Time::second() const
{
    return pimpl->p.time_of_day().seconds();
}

int
    Time::millisec() const
{
    return pimpl->p.time_of_day().total_milliseconds() % 1000;
}

int
    Time::compare(
        const Time &rhs) const
{
    return
        pimpl->p > rhs.pimpl->p ? 1 :
        pimpl->p < rhs.pimpl->p ? -1 :
        0;
}

const ptime
    Time::to_ptime() const
{
    return pimpl->p;
}

const std::string
    Time::to_string() const
{
    return form_iso8601_time(pimpl->p);
}

ostream &
    operator <<(
        ostream &os,
        const Time &rhs)
{
    os << rhs.to_string();

    return os;
}

bool
    operator ==(
        const Time &lhs,
        const Time &rhs)
{
    return lhs.compare(rhs) == 0;
}

bool
    operator !=(
        const Time &lhs,
        const Time &rhs)
{
    return lhs.compare(rhs) != 0;
}

bool
    operator < (
        const Time &lhs,
        const Time &rhs)
{
    return lhs.compare(rhs) < 0;
}

bool
    operator <=(
        const Time &lhs,
        const Time &rhs)
{
    return lhs.compare(rhs) <= 0;
}

bool
    operator > (
        const Time &lhs,
        const Time &rhs)
{
    return lhs.compare(rhs) > 0;
}
bool
    operator >=(
        const Time &lhs,
        const Time &rhs)
{
    return lhs.compare(rhs) >= 0;
}

}
