#include <mxidafpp/Value.hpp>

#include <sstream>

#include <mxtag/Value.hpp>

using std::ostream;
using std::string;
using std::stringstream;

typedef
    mxtag::Value
    MxValue;

namespace mxidafpp
{

struct Value::Impl
{
    Impl(const MxValue &mxvalue_)
        : mxvalue(mxvalue_)
    {
    }

    const MxValue   mxvalue;
};

Value::Value(
    int64_t i)
    : pimpl(new Impl(MxValue(i)))
{
}

Value::Value(
    uint64_t u)
    : pimpl(new Impl(MxValue(u)))
{
}

Value::Value(
    double d)
    : pimpl(new Impl(MxValue(d)))
{
}

Value::Value(
    const string &s)
    : pimpl(new Impl(MxValue(s)))
{
}

Value::~Value()
{
}

bool
    Value::is_int() const
{
    return pimpl->mxvalue.is_int();
}

int64_t
    Value::as_int() const
{
    return pimpl->mxvalue.as_int();
}

bool
    Value::is_uint() const
{
    return pimpl->mxvalue.is_uint();
}

uint64_t
    Value::as_uint() const
{
    return pimpl->mxvalue.as_uint();
}

bool
    Value::is_float() const
{
    return pimpl->mxvalue.is_float();
}

double
    Value::as_float() const
{
    return pimpl->mxvalue.as_float();
}

bool
    Value::is_string() const
{
    return pimpl->mxvalue.is_string();
}

const string &
    Value::as_string() const
{
    return pimpl->mxvalue.as_string();
}

bool
    Value::equal(const Value &rhs) const
{
    return pimpl->mxvalue == rhs.pimpl->mxvalue;
}

const string
    Value::to_string() const
{
    stringstream ss;
    ss << pimpl->mxvalue;

    return ss.str();
}

bool
    operator ==(
        const Value &lhs,
        const Value &rhs)
{
    return lhs.equal(rhs);
}

bool
    operator !=(
        const Value &lhs,
        const Value &rhs)
{
    return !(lhs == rhs);
}

ostream &
    operator <<(
        ostream &os,
        const Value &rhs)
{
    os << rhs.to_string();
    return os;
}

}
