#include <mxidafpp/impl/CacheImpl.hpp>

#include <boost/assert.hpp>
#include <boost/foreach.hpp>
#include <boost/make_shared.hpp>

#include <mxidafpp/impl/EquipmentImpl.hpp>

using boost::make_shared;
using boost::shared_ptr;

using mxhtag::Database;

using std::string;
using std::vector;

namespace mxidafpp
{
namespace impl
{

struct CacheImpl::Impl
{
    Impl(
        shared_ptr<Database> database_)
        : database(database_)
    {
    }

    const shared_ptr<Database>  database;
};

CacheImpl::CacheImpl(
    shared_ptr<Database> database)
    : pimpl(new Impl(database))
{
    BOOST_VERIFY(pimpl->database);
}

CacheImpl::~CacheImpl()
{
}

const vector<Equipment>
    CacheImpl::get_equipment_list()
{
    try
    {
        const vector<string> equ_name_list = pimpl->database->get_equipment_name_list();

        vector<Equipment> equ_list;
        BOOST_FOREACH(const string &equ_name, equ_name_list)
        {
            equ_list.push_back(Equipment(make_shared<EquipmentImpl>(
                pimpl->database,
                equ_name)));
        }

        return equ_list;
    }
    catch(mxhtag::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error() << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

const Equipment
    CacheImpl::get_equipment(
        const string &equ_name)
{
    try
    {
        if (!pimpl->database->is_equipment_exist(equ_name))
        {
            BOOST_THROW_EXCEPTION(
                NotFoundError() << ErrMsg(equ_name)
            );
        }

        return Equipment(make_shared<EquipmentImpl>(
            pimpl->database,
            equ_name));
    }
    catch(mxhtag::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error() << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

}
}
