#include <mxidafpp/impl/EquipmentImpl.hpp>

#include <boost/assert.hpp>
#include <boost/foreach.hpp>
#include <boost/make_shared.hpp>

#include <mxidafpp/impl/StreamImpl.hpp>

using boost::make_shared;
using boost::shared_ptr;

using mxhtag::Database;

using std::string;
using std::vector;

namespace mxidafpp
{
namespace impl
{

struct EquipmentImpl::Impl
{
    Impl(
        shared_ptr<Database> database_,
        const string &name_)
        : database(database_)
        , name(name_)
    {
    }

    const shared_ptr<Database>  database;
    const string                name;
};

EquipmentImpl::EquipmentImpl(
    shared_ptr<Database> database,
    const string &name)
    : pimpl(new Impl(
        database,
        name))
{
    BOOST_VERIFY(pimpl->database);
}

EquipmentImpl::~EquipmentImpl()
{
}

const string &
    EquipmentImpl::name() const
{
    return pimpl->name;
}

bool
    EquipmentImpl::equal(
        const EquipmentImpl &rhs) const
{
    return
        pimpl->database == rhs.pimpl->database &&
        pimpl->name == rhs.pimpl->name;
}

const vector<Stream>
    EquipmentImpl::get_stream_list()
{
    try
    {
        const vector<string> tag_name_list = pimpl->database->get_tag_name_list_by_equipment(pimpl->name);

        vector<Stream> stream_list;
        BOOST_FOREACH(const string &tag_name, tag_name_list)
        {
            stream_list.push_back(Stream(make_shared<StreamImpl>(
                pimpl->database,
                pimpl->name,
                tag_name)));
        }

        return stream_list;
    }
    catch(mxhtag::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error() << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

const Stream
    EquipmentImpl::get_stream(const string &name)
{
    try
    {
        if (!pimpl->database->is_equipment_tag_exist(pimpl->name, name))
        {
            BOOST_THROW_EXCEPTION(
                NotFoundError() << ErrMsg(name)
            );
        }

        return Stream(make_shared<StreamImpl>(
            pimpl->database,
            pimpl->name,
            name));
    }
    catch(mxhtag::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error() << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

}
}
