#include <mxidafpp/impl/StreamImpl.hpp>

#include <mxidafpp/impl/Tag.hpp>

using boost::shared_ptr;

using mxidafpp::Tag;
using mxidafpp::Value;
using mxidafpp::impl::mxidafpp_from_mxtag;

using mxhtag::Database;

using std::string;
using std::vector;

namespace mxidafpp
{
namespace impl
{

struct StreamImpl::Impl
{
    Impl(
        shared_ptr<Database> database_,
        const string &equ_name_,
        const string &tag_name_)
        : database(database_)
        , equ_name(equ_name_)
        , tag_name(tag_name_)
    {
    }

    const shared_ptr<Database>  database;
    const string                equ_name;
    const string                tag_name;
};

StreamImpl::StreamImpl(
    shared_ptr<Database> database,
    const string &equ_name,
    const string &tag_name)
    : pimpl(new Impl(
        database,
        equ_name,
        tag_name))
{
    BOOST_VERIFY(pimpl->database);
}

StreamImpl::~StreamImpl()
{
}

const string &
    StreamImpl::equipment_name() const
{
    return pimpl->equ_name;
}

const string &
    StreamImpl::tag_name() const
{
    return pimpl->tag_name;
}

bool
    StreamImpl::equal(
        const StreamImpl &rhs) const
{
    return
        pimpl->database == rhs.pimpl->database &&
        pimpl->equ_name == rhs.pimpl->equ_name &&
        pimpl->tag_name == rhs.pimpl->tag_name;
}

const vector<Tag>
    StreamImpl::get_last_tag_list(size_t count)
{
    try
    {
        vector<mxtag::Tag> mxtag_list;
        if (count == 0)
        {
            mxtag_list = pimpl->database->get_tag_list_by_equipment_tag(
                pimpl->equ_name,
                pimpl->tag_name);
        }
        else
        {
            mxtag_list = pimpl->database->get_last_tag_list_by_equipment_tag(
                pimpl->equ_name,
                pimpl->tag_name,
                count);
        }

        vector<Tag> tag_list;
        transform(
            mxtag_list.begin(),
            mxtag_list.end(),
            back_inserter(tag_list),
            &mxidafpp_from_mxtag);

        return tag_list;
    }
    catch(mxhtag::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error() << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

}
}
