#include <mxidafpp/impl/TagImporterImpl.hpp>

#include <sys/types.h>
#include <unistd.h>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/make_shared.hpp>
#include <boost/shared_ptr.hpp>

#include <mxmqtt/Worker.hpp>

#include <mxtag/mqtt/ProtobufSerializer.hpp>
#include <mxtag/validate.hpp>

#include <mxidafpp/impl/Tag.hpp>

using boost::lexical_cast;
using boost::make_shared;
using boost::shared_ptr;

using mxmqtt::Message;
using mxmqtt::Worker;

using mxtag::is_valid_equipment_name;
using mxtag::is_valid_tag_name;
using mxtag::mqtt::ProtobufSerializer;
using mxtag::mqtt::Serializer;

using std::pair;
using std::string;
using std::vector;

namespace mxidafpp
{
namespace impl
{

struct TagImporterImpl::Impl
{
    Impl(
        const string &host,
        int port)
        : worker(
            host,
            port,
            "mxidafpp-" + lexical_cast<string>(getpid()),
            true)
        , serializer(make_shared<ProtobufSerializer>())
    {
    }

    Worker  worker;

    const shared_ptr<Serializer>    serializer;
};

TagImporterImpl::TagImporterImpl(
    const string &host,
    int port)
    : pimpl(new Impl(
        host,
        port))
{
}

TagImporterImpl::~TagImporterImpl()
{
}

void
    TagImporterImpl::import(
        const string &equipment_name,
        const string &tag_name,
        const vector<mxidafpp::Tag> &tag_list)
{
    if (!is_valid_equipment_name(equipment_name))
        BOOST_THROW_EXCEPTION(
            ArgumentError()
                << ErrMsg("illegal equipment name: " + equipment_name)
        );

    if (!is_valid_tag_name(tag_name))
        BOOST_THROW_EXCEPTION(
            ArgumentError()
                << ErrMsg("illegal tag name: " + tag_name)
        );

    try
    {
        BOOST_FOREACH(const mxidafpp::Tag &t, tag_list)
        {
            const pair<string, vector<uint8_t> > mqtt =
                pimpl->serializer->from_tag(
                    mxtag_from_mxidafpp(
                        t,
                        equipment_name,
                        tag_name
                    )
                );

            pimpl->worker.publish(Message(
                mqtt.first,
                mqtt.second,
                2,
                false
            ));
        }
    }
    catch(mxmqtt::Error &e)
    {
        BOOST_THROW_EXCEPTION(
            Error()
                << ErrInternalMsg(diagnostic_information(e))
        );
    }
}

}
}
