#include <boost/test/unit_test.hpp>

#include <mxidafpp/Cache.hpp>

#include <mxidafpp/Service.hpp>

#include "DatabaseFixture.hpp"

using mxidafpp::Service;
using mxidafpp::Cache;

using std::vector;

namespace
{

struct TestFixture
    : public DatabaseFixture
{
    TestFixture()
        : cache(Service::instance().get_cache())
    {
    }

    Cache cache;
};

}

namespace mxidafpp
{

BOOST_FIXTURE_TEST_CASE(test_Cache_get_equipment_list_ShouldPass, TestFixture)
{
    // arrange

    // act
    const vector<Equipment> equipment_list =
        cache.get_equipment_list();

    // assert
    BOOST_CHECK_EQUAL(3, equipment_list.size());
}

BOOST_FIXTURE_TEST_CASE(test_Cache_get_equipment_ShouldPass, TestFixture)
{
    // arrange

    // act & assert
    const Equipment equipment = cache.get_equipment("equipment1");
}

BOOST_FIXTURE_TEST_CASE(test_Cache_get_equipment_WithNotExistEquipment_ShouldRaiseNotFoundError, TestFixture)
{
    // arrange

    // act & assert
    BOOST_CHECK_THROW(cache.get_equipment("not-exist-equipment"), NotFoundError);
}

}
