#include <boost/test/unit_test.hpp>

#include <mxidafpp/Equipment.hpp>

#include <mxidafpp/Service.hpp>

#include "DatabaseFixture.hpp"

using mxidafpp::Equipment;
using mxidafpp::Service;
using mxidafpp::Cache;

using std::string;
using std::vector;

namespace
{

struct TestFixture
    : public DatabaseFixture
{
    TestFixture()
        : cache(Service::instance().get_cache())
        , equipment(cache.get_equipment("equipment1"))
    {
    }

    Cache cache;
    Equipment equipment;
};

}

namespace mxidafpp
{

BOOST_FIXTURE_TEST_CASE(test_Equipment_operator_eq_ShouldPass, TestFixture)
{
    // arrange
    Equipment rhs = cache.get_equipment("equipment1");

    // act
    const bool result = equipment == rhs;

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_Equipment_operator_ne_ShouldPass, TestFixture)
{
    // arrange
    Equipment rhs = cache.get_equipment("equipment2");

    // act
    const bool result = equipment != rhs;

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_Equipment_name_ShouldPass, TestFixture)
{
    // arrange

    // act
    const string name = equipment.name();

    // assert
    BOOST_CHECK_EQUAL("equipment1", name);
}

BOOST_FIXTURE_TEST_CASE(test_Equipment_get_stream_list_ShouldPass, TestFixture)
{
    // arrange

    // act
    const vector<Stream> stream_list = equipment.get_stream_list();

    // assert
    BOOST_CHECK_EQUAL(3, stream_list.size());
}

BOOST_FIXTURE_TEST_CASE(test_Equipment_get_stream_ShouldPass, TestFixture)
{
    // arrange

    // act & assert
    const Stream stream = equipment.get_stream("e1tag2");
}

BOOST_FIXTURE_TEST_CASE(test_Equipment_get_stream_WithNotExistTag_ShouldRaiseNotFoundError, TestFixture)
{
    // arrange

    // act & assert
    BOOST_CHECK_THROW(equipment.get_stream("tag-not-exist"), NotFoundError);
}

}
