#include <boost/test/unit_test.hpp>

#include <mxidafpp/Stream.hpp>

#include <mxidafpp/Service.hpp>

#include "DatabaseFixture.hpp"

using boost::posix_time::time_from_string;

using mxidafpp::Service;
using mxidafpp::Cache;
using mxidafpp::Stream;

using mxidafpp::Tag;
using mxidafpp::Time;
using mxidafpp::Value;

using std::string;
using std::vector;

namespace
{

struct TestFixture
    : public DatabaseFixture
{
    TestFixture()
        : cache(Service::instance().get_cache())
        , stream(cache.get_equipment("equipment1").get_stream("e1tag0"))
    {
    }

    Cache cache;
    const Stream stream;
};

}

namespace mxidafpp
{

BOOST_FIXTURE_TEST_CASE(test_Stream_operator_eq_ShouldPass, TestFixture)
{
    // arrange
    Stream rhs = cache.get_equipment("equipment1").get_stream("e1tag0");

    // act
    const bool result = stream == rhs;

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_Stream_operator_ne_ShouldPass, TestFixture)
{
    // arrange
    Stream rhs = cache.get_equipment("equipment1").get_stream("e1tag1");

    // act
    const bool result = stream != rhs;

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_Stream_equipment_name_ShouldPass, TestFixture)
{
    // arrange

    // act
    const string equipment_name = stream.equipment_name();

    // assert
    BOOST_CHECK_EQUAL("equipment1", equipment_name);
}

BOOST_FIXTURE_TEST_CASE(test_Stream_tag_name_ShouldPass, TestFixture)
{
    // arrange

    // act
    const string tag_name = stream.tag_name();

    // assert
    BOOST_CHECK_EQUAL("e1tag0", tag_name);
}

BOOST_FIXTURE_TEST_CASE(test_Stream_get_last_tag_list_ShouldPass, TestFixture)
{
    // arrange

    // act
    const vector<Tag> tag_list = stream.get_last_tag_list(5);

    // assert
    BOOST_REQUIRE_EQUAL(5, tag_list.size());
    BOOST_CHECK(tag_list[0] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 9, 0), "test-unit"));
    BOOST_CHECK(tag_list[1] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 8, 0), "test-unit"));
    BOOST_CHECK(tag_list[2] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 7, 0), "test-unit"));
    BOOST_CHECK(tag_list[3] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 6, 0), "test-unit"));
    BOOST_CHECK(tag_list[4] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 5, 0), "test-unit"));
}

BOOST_FIXTURE_TEST_CASE(test_Stream_get_last_tag_list_WithZeroCount_ShouldPass, TestFixture)
{
    // arrange

    // act
    const vector<Tag> tag_list = stream.get_last_tag_list(0);

    // assert
    BOOST_REQUIRE_EQUAL(10, tag_list.size());
    BOOST_CHECK(tag_list[0] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 9, 0), "test-unit"));
    BOOST_CHECK(tag_list[1] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 8, 0), "test-unit"));
    BOOST_CHECK(tag_list[2] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 7, 0), "test-unit"));
    BOOST_CHECK(tag_list[3] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 6, 0), "test-unit"));
    BOOST_CHECK(tag_list[4] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 5, 0), "test-unit"));
    BOOST_CHECK(tag_list[5] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 4, 0), "test-unit"));
    BOOST_CHECK(tag_list[6] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 3, 0), "test-unit"));
    BOOST_CHECK(tag_list[7] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 2, 0), "test-unit"));
    BOOST_CHECK(tag_list[8] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 1, 0), "test-unit"));
    BOOST_CHECK(tag_list[9] == Tag(Value("test-value"), Time(2010, 1, 1, 0, 0, 0, 0), "test-unit"));
}

}
