#include <boost/test/unit_test.hpp>

#include <mxidafpp/Time.hpp>

#include <sstream>

using boost::gregorian::date;
using boost::posix_time::microsec_clock;
using boost::posix_time::milliseconds;
using boost::posix_time::ptime;
using boost::posix_time::time_duration;

using std::string;
using std::stringstream;

namespace
{

struct TestFixture
{
};

}

namespace mxidafpp
{

BOOST_FIXTURE_TEST_CASE(test_Time_now_ShouldPass, TestFixture)
{
    // arrange
    const ptime ptime_now = microsec_clock::universal_time();

    // act
    const Time now = Time::now();

    // assert
    const ptime result(
        date(
            now.year(),
            now.month(),
            now.day()),
        time_duration(
            now.hour(),
            now.minute(),
            now.second()) + milliseconds(now.millisec()));

    const time_duration diff = result - ptime_now;
    BOOST_CHECK(diff.total_seconds() < 2);
}

BOOST_FIXTURE_TEST_CASE(test_Time_to_string_ShouldPass, TestFixture)
{
    // arrange
    const Time t(2016, 3, 22, 9, 41, 39, 123);

    // act
    const string s = t.to_string();

    // assert
    BOOST_CHECK_EQUAL("2016-03-22T09:41:39.123Z", s);
}

BOOST_FIXTURE_TEST_CASE(test_Time_operator_putto_ShouldPass, TestFixture)
{
    // arrange
    const Time t(2016, 3, 22, 9, 41, 39, 123);

    stringstream ss;

    // act
    ss << t;

    // assert
    BOOST_CHECK_EQUAL("2016-03-22T09:41:39.123Z", ss.str());
}

BOOST_FIXTURE_TEST_CASE(test_Time_operator_eq_ShouldPass, TestFixture)
{
    // arrange
    const Time lhs(2016, 3, 22, 9, 41, 39, 123);
    const Time rhs(2016, 3, 22, 9, 41, 39, 123);

    // act
    const bool should_be_true = lhs == rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_Time_operator_ne_ShouldPass, TestFixture)
{
    // arrange
    const Time lhs(2016, 3, 22, 9, 41, 39, 123);
    const Time rhs(2016, 3, 22, 9, 41, 39, 122);

    // act
    const bool should_be_true = lhs != rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_Time_operator_lt_ShouldPass, TestFixture)
{
    // arrange
    const Time lhs(2016, 3, 22, 9, 41, 39, 122);
    const Time rhs(2016, 3, 22, 9, 41, 39, 123);

    // act
    const bool should_be_true = lhs < rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_Time_operator_le_ShouldPass, TestFixture)
{
    // arrange
    const Time lhs(2016, 3, 22, 9, 41, 39, 123);
    const Time rhs(2016, 3, 22, 9, 41, 39, 123);

    // act
    const bool should_be_true = lhs <= rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_Time_operator_gt_ShouldPass, TestFixture)
{
    // arrange
    const Time lhs(2016, 3, 22, 9, 41, 39, 123);
    const Time rhs(2016, 3, 22, 9, 41, 39, 122);

    // act
    const bool should_be_true = lhs > rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_Time_operator_ge_ShouldPass, TestFixture)
{
    // arrange
    const Time lhs(2016, 3, 22, 9, 41, 39, 123);
    const Time rhs(2016, 3, 22, 9, 41, 39, 123);

    // act
    const bool should_be_true = lhs >= rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

}
