#ifndef _MXMODBUS_MODBUS_CLIENT_HPP_
#define _MXMODBUS_MODBUS_CLIENT_HPP_

#include <vector>

#include <boost/cstdint.hpp>
#include <boost/noncopyable.hpp>

#include <mxmodbus/error.hpp>
#include <mxmodbus/types.hpp>

namespace mxmodbus
{

class ModbusClient // interface
    : private boost::noncopyable
{
protected:
    ModbusClient();

public:
    virtual ~ModbusClient();

    // function-code 0x01
    const std::vector<bool>
        read_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    // function-code 0x02
    const std::vector<bool>
        read_discrete_inputs(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    // function-code 0x03
    const std::vector<uint16_t>
        read_holding_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    // function-code 0x04
    const std::vector<uint16_t>
        read_input_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    // function-code 0x05
    void
        write_single_coil(
            mxmodbus::device_t,
            mxmodbus::address_t,
            bool value); // throws mxmodbus::Error

    // function-code 0x06
    void
        write_single_register(
            mxmodbus::device_t,
            mxmodbus::address_t,
            uint16_t value); // throws mxmodbus::Error

    // function-code 0x0F
    void
        write_multiple_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<bool> &value); // throws mxmodbus::Error

    // function-code 0x10
    void
        write_multiple_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<uint16_t> &value); // throws mxmodbus::Error

private:
    virtual const std::vector<bool>
        do_read_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity) = 0;

    virtual const std::vector<bool>
        do_read_discrete_inputs(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity) = 0;

    virtual const std::vector<uint16_t>
        do_read_holding_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity) = 0;

    virtual const std::vector<uint16_t>
        do_read_input_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity) = 0;

    virtual void
        do_write_single_coil(
            mxmodbus::device_t,
            mxmodbus::address_t,
            bool value) = 0;

    virtual void
        do_write_single_register(
            mxmodbus::device_t,
            mxmodbus::address_t,
            uint16_t value) = 0;

    virtual void
        do_write_multiple_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<bool> &value) = 0;

    virtual void
        do_write_multiple_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<uint16_t> &value) = 0;
};

}

#endif
