#ifndef _MXMODBUS_MODBUS_CLIENT_PROXY_HPP_
#define _MXMODBUS_MODBUS_CLIENT_PROXY_HPP_

#include <boost/function.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <mxmodbus/ModbusClient.hpp>

namespace mxmodbus
{

class ModbusClientProxy
    : public mxmodbus::ModbusClient
{
public:
    typedef
        boost::function<
            boost::shared_ptr<mxmodbus::ModbusClient>
                ()
        > CreateModbusClientCallback;

public:
    explicit ModbusClientProxy(const CreateModbusClientCallback &);
    virtual ~ModbusClientProxy();

    using mxmodbus::ModbusClient::read_coils;
    using mxmodbus::ModbusClient::read_discrete_inputs;
    using mxmodbus::ModbusClient::read_holding_registers;
    using mxmodbus::ModbusClient::read_input_registers;
    using mxmodbus::ModbusClient::write_single_coil;
    using mxmodbus::ModbusClient::write_single_register;
    using mxmodbus::ModbusClient::write_multiple_coils;
    using mxmodbus::ModbusClient::write_multiple_registers;

private:
    virtual const std::vector<bool>
        do_read_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual const std::vector<bool>
        do_read_discrete_inputs(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual const std::vector<uint16_t>
        do_read_holding_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual const std::vector<uint16_t>
        do_read_input_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual void
        do_write_single_coil(
            mxmodbus::device_t,
            mxmodbus::address_t,
            bool value);

    virtual void
        do_write_single_register(
            mxmodbus::device_t,
            mxmodbus::address_t,
            uint16_t value);

    virtual void
        do_write_multiple_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<bool> &value);

    virtual void
        do_write_multiple_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<uint16_t> &value);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
