#ifndef _MXMODBUS_MODBUS_RTU_CLIENT_HPP_
#define _MXMODBUS_MODBUS_RTU_CLIENT_HPP_

#include <boost/scoped_ptr.hpp>

#include <mxmodbus/ModbusClient.hpp>

namespace mxmodbus
{

class ModbusRtuClient
    : public mxmodbus::ModbusClient
{
public:
    typedef enum
    {
        BAUD_9600 = 9600,
        BAUD_19200 = 19200,
        BAUD_38400 = 38400,
        BAUD_57600 = 57600,
        BAUD_115200 = 115200
    } baud_t;

    typedef enum
    {
        NONE = 'N',
        EVEN = 'E',
        ODD = 'O'
    } parity_t;

    typedef enum
    {
        STOP_1 = 1,
        STOP_2 = 2
    } stop_bit_t;

public:
    ModbusRtuClient(
        const std::string &path,
        baud_t baud,
        parity_t parity,
        stop_bit_t stop_bit,
        int interchar_timeout_ms,
        int default_response_timeout_ms); // throws mxmodbus::Error

    virtual ~ModbusRtuClient();

    using mxmodbus::ModbusClient::read_coils;
    using mxmodbus::ModbusClient::read_discrete_inputs;
    using mxmodbus::ModbusClient::read_holding_registers;
    using mxmodbus::ModbusClient::read_input_registers;
    using mxmodbus::ModbusClient::write_single_coil;
    using mxmodbus::ModbusClient::write_single_register;
    using mxmodbus::ModbusClient::write_multiple_coils;
    using mxmodbus::ModbusClient::write_multiple_registers;

private:
    virtual const std::vector<bool>
        do_read_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual const std::vector<bool>
        do_read_discrete_inputs(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual const std::vector<uint16_t>
        do_read_holding_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual const std::vector<uint16_t>
        do_read_input_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity);

    virtual void
        do_write_single_coil(
            mxmodbus::device_t,
            mxmodbus::address_t,
            bool value);

    virtual void
        do_write_single_register(
            mxmodbus::device_t,
            mxmodbus::address_t,
            uint16_t value);

    virtual void
        do_write_multiple_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<bool> &value);

    virtual void
        do_write_multiple_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<uint16_t> &value);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
