#ifndef _MXMODBUS_MODBUS_CLIENT_IMPL_HPP_
#define _MXMODBUS_MODBUS_CLIENT_IMPL_HPP_

#include <vector>

#include <boost/scoped_ptr.hpp>

#include <modbus/modbus.h>

#include <mxmodbus/error.hpp>

namespace mxmodbus
{
namespace priv
{

class ModbusClientImpl
{
public:
    explicit ModbusClientImpl(modbus_t *);

    ~ModbusClientImpl();

    const std::vector<bool>
        read_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    const std::vector<bool>
        read_discrete_inputs(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    const std::vector<uint16_t>
        read_holding_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    const std::vector<uint16_t>
        read_input_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity); // throws mxmodbus::Error

    void
        write_single_coil(
            mxmodbus::device_t,
            mxmodbus::address_t,
            bool value); // throws mxmodbus::Error

    void
        write_single_register(
            mxmodbus::device_t,
            mxmodbus::address_t,
            uint16_t value); // throws mxmodbus::Error

    void
        write_multiple_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<bool> &value); // throws mxmodbus::Error

    void
        write_multiple_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            const std::vector<uint16_t> &value); // throws mxmodbus::Error

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}
}

#endif
