#ifndef _MXMODBUS_TCP_MODBUS_TCP_PROTOCOL_HPP_
#define _MXMODBUS_TCP_MODBUS_TCP_PROTOCOL_HPP_

#include <vector>

#include <boost/cstdint.hpp>

#include <mxmodbus/types.hpp>

namespace mxmodbus
{
namespace tcp
{

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_read_coils_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        uint16_t &quantity);

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_read_discrete_inputs_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        uint16_t &quantity);

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_read_holding_registers_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        uint16_t &quantity);

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_read_input_registers_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        uint16_t &quantity);

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_write_single_coil_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        bool &value);

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_write_single_register_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        uint16_t &value);

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_write_multiple_coils_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        std::vector<bool> &value);

// return
//   -1 on failure
//   0 on insufficient packet length
//   >0 as number of bytes consumed
int
    parse_write_multiple_registers_request(
        const std::vector<uint8_t> &packet,
        uint16_t &id,
        mxmodbus::device_t &device,
        mxmodbus::address_t &address,
        std::vector<uint16_t> &value);

const std::vector<uint8_t>
    form_read_coils_response(
        uint16_t id,
        mxmodbus::device_t device,
        const std::vector<bool> &status);

const std::vector<uint8_t>
    form_read_coils_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

const std::vector<uint8_t>
    form_read_discrete_inputs_response(
        uint16_t id,
        mxmodbus::device_t device,
        const std::vector<bool> &status);

const std::vector<uint8_t>
    form_read_discrete_inputs_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

const std::vector<uint8_t>
    form_read_holding_registers_response(
        uint16_t id,
        mxmodbus::device_t device,
        const std::vector<uint16_t> &value);

const std::vector<uint8_t>
    form_read_holding_registers_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

const std::vector<uint8_t>
    form_read_input_registers_response(
        uint16_t id,
        mxmodbus::device_t device,
        const std::vector<uint16_t> &registers);

const std::vector<uint8_t>
    form_read_input_registers_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

const std::vector<uint8_t>
    form_write_single_coil_response(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::address_t address,
        bool value);

const std::vector<uint8_t>
    form_write_single_coil_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

const std::vector<uint8_t>
    form_write_single_register_response(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::address_t address,
        uint16_t value);

const std::vector<uint8_t>
    form_write_single_register_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

const std::vector<uint8_t>
    form_write_multiple_coils_response(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::address_t address,
        uint16_t quantity);

const std::vector<uint8_t>
    form_write_multiple_coils_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

const std::vector<uint8_t>
    form_write_multiple_registers_response(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::address_t address,
        uint16_t quantity);

const std::vector<uint8_t>
    form_write_multiple_registers_exception(
        uint16_t id,
        mxmodbus::device_t device,
        mxmodbus::exception_t exception);

}
}

#endif
