#include <mxmodbus/ModbusClient.hpp>

#include <mxutil/log.hpp>

namespace mxmodbus
{

using std::vector;

ModbusClient::ModbusClient()
{
}

ModbusClient::~ModbusClient()
{
}

const vector<bool>
    ModbusClient::read_coils(
        device_t d,
        address_t a,
        size_t q)
{
    try
    {
        return this->do_read_coils(d, a, q);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

const vector<bool>
    ModbusClient::read_discrete_inputs(
        device_t d,
        address_t a,
        size_t q)
{
    try
    {
        return this->do_read_discrete_inputs(d, a, q);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

const vector<uint16_t>
    ModbusClient::read_holding_registers(
        device_t d,
        address_t a,
        size_t q)
{
    try
    {
        return this->do_read_holding_registers(d, a, q);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

const vector<uint16_t>
    ModbusClient::read_input_registers(
        device_t d,
        address_t a,
        size_t q)
{
    try
    {
        return this->do_read_input_registers(d, a, q);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

void
    ModbusClient::write_single_coil(
        device_t d,
        address_t a,
        bool v)
{
    try
    {
        return this->do_write_single_coil(d, a, v);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

void
    ModbusClient::write_single_register(
        device_t d,
        address_t a,
        uint16_t v)
{
    try
    {
        return this->do_write_single_register(d, a, v);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

void
    ModbusClient::write_multiple_coils(
        device_t d,
        address_t a,
        const vector<bool> &v)
{
    try
    {
        return this->do_write_multiple_coils(d, a, v);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

void
    ModbusClient::write_multiple_registers(
        device_t d,
        address_t a,
        const vector<uint16_t> &v)
{
    try
    {
        return this->do_write_multiple_registers(d, a, v);
    }
    catch(Error &)
    {
        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(std::exception &e)
    {
        MXLOG_WARN(e.what());
        BOOST_THROW_EXCEPTION(Error());
    }
    catch(...)
    {
        MXLOG_WARN("unexpected exception caught");
        BOOST_THROW_EXCEPTION(Error());
    }
}

}
