#include <mxmodbus/ModbusClientProxy.hpp>

#include <mxutil/log.hpp>

using boost::shared_ptr;

using std::string;
using std::vector;

namespace mxmodbus
{

struct ModbusClientProxy::Impl
{
    Impl(const CreateModbusClientCallback &create_modbus_client_)
        : create_modbus_client(create_modbus_client_)
    {
    }

    const CreateModbusClientCallback    create_modbus_client;

    shared_ptr<ModbusClient>    modbus_client;

    ModbusClient &
        get_modbus_client()
    {
        if (!this->modbus_client)
        {
            this->modbus_client = this->create_modbus_client(); // throws mxmodbus::Error

            BOOST_ASSERT(this->modbus_client);
        }

        return *this->modbus_client;
    }
};

ModbusClientProxy::ModbusClientProxy(
    const CreateModbusClientCallback &create_modbus_client)
    : pimpl(new Impl(create_modbus_client))
{
}

ModbusClientProxy::~ModbusClientProxy()
{
}

const vector<bool>
    ModbusClientProxy::do_read_coils(
        device_t device,
        address_t addr,
        size_t quantity)
{
    try
    {
        return pimpl->get_modbus_client().read_coils(device, addr, quantity);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

const vector<bool>
    ModbusClientProxy::do_read_discrete_inputs(
        device_t device,
        address_t addr,
        size_t quantity)
{
    try
    {
        return pimpl->get_modbus_client().read_discrete_inputs(device, addr, quantity);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

const vector<uint16_t>
    ModbusClientProxy::do_read_holding_registers(
        device_t device,
        address_t addr,
        size_t quantity)
{
    try
    {
        return pimpl->get_modbus_client().read_holding_registers(device, addr, quantity);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

const vector<uint16_t>
    ModbusClientProxy::do_read_input_registers(
        device_t device,
        address_t addr,
        size_t quantity)
{
    try
    {
        return pimpl->get_modbus_client().read_input_registers(device, addr, quantity);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

void
    ModbusClientProxy::do_write_single_coil(
        device_t device,
        address_t addr,
        bool value)
{
    try
    {
        return pimpl->get_modbus_client().write_single_coil(device, addr, value);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

void
    ModbusClientProxy::do_write_single_register(
        device_t device,
        address_t addr,
        uint16_t value)
{
    try
    {
        return pimpl->get_modbus_client().write_single_register(device, addr, value);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

void
    ModbusClientProxy::do_write_multiple_coils(
        device_t device,
        address_t addr,
        const vector<bool> &value)
{
    try
    {
        return pimpl->get_modbus_client().write_multiple_coils(device, addr, value);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

void
    ModbusClientProxy::do_write_multiple_registers(
        device_t device,
        address_t addr,
        const vector<uint16_t> &value)
{
    try
    {
        return pimpl->get_modbus_client().write_multiple_registers(device, addr, value);
    }
    catch(Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));
        pimpl->modbus_client.reset();

        throw;
    }
}

}
