// This is a simple test program to test whether
// mxmodbus::ModbusTcpClient operate correctly.

#include <iostream>
#include <vector>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#include <mxmodbus/ModbusTcpClient.hpp>

using boost::diagnostic_information;
using boost::lexical_cast;

using mxmodbus::ModbusTcpClient;

using std::cerr;
using std::cout;
using std::endl;
using std::string;
using std::vector;

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 3)
    {
        cout << "usage: <ip> <port>" << endl;
        return 1;
    }

    const string host = argv[1];
    const int port = lexical_cast<int>(argv[2]);

    try
    {
        cout << "connecting to " << host << ":" << port << endl;
        ModbusTcpClient modbus(host, port, 1000);

        cout << "read holding registers" << endl;
        const vector<uint16_t> ao_list = modbus.read_holding_registers(0x00, 0x0400, 4);

        BOOST_FOREACH(uint16_t ao, ao_list)
        {
            cout << ao << endl;
        }
    }
    catch(mxmodbus::Error &e)
    {
        cerr << diagnostic_information(e) << endl;
        return 1;
    }

    return 0;
}

