// This is a simple test program to test whether
// mxmodbus::ModbusTcpClient operate correctly.

#include <iostream>
#include <vector>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#include <mxmodbus/ModbusTcpClient.hpp>

using boost::diagnostic_information;
using boost::lexical_cast;

using mxmodbus::ModbusTcpClient;

using std::cerr;
using std::cout;
using std::endl;
using std::string;
using std::vector;

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 3)
    {
        cout << "usage: <ip> <port>" << endl;
        return 1;
    }

    const string host = argv[1];
    const int port = lexical_cast<int>(argv[2]);

    try
    {
        cout << "connecting to " << host << ":" << port << endl;
        ModbusTcpClient modbus(host, port, 1000);

        cout << "reading coils" << endl;
        const vector<bool> di_list = modbus.read_coils(0x00, 0x0000, 6);

        cout << "reading input registers" << endl;
        const vector<uint16_t> ai = modbus.read_input_registers(0x00, 0x0130, 2);

        for(size_t ax = 0; ax < di_list.size(); ++ax)
            cout << "di" << ax << ": " << di_list[ax] << endl;

        uint32_t v = ai[1];
        v <<= 16;
        v += ai[0];
        cout << "ai0: " << *reinterpret_cast<float *>(&v) << endl;
    }
    catch(mxmodbus::Error &e)
    {
        cerr << diagnostic_information(e) << endl;
        return 1;
    }

    return 0;
}
