#include <boost/test/unit_test.hpp>

#include <boost/range.hpp>

#include <mxmodbus/tcp/modbus_tcp_protocol.hpp>

namespace mxmodbus
{
namespace tcp
{

using boost::size;

using std::vector;

namespace
{

class TestFixture
{
};
}

BOOST_FIXTURE_TEST_CASE(test_parse_read_coils_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x01, 0x00, 0x13, 0x00, 0x11};

    // act
    uint16_t id = 0;
    uint8_t device = 0;
    uint16_t address = 0;
    uint16_t quantity = 0;
    const int result = parse_read_coils_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        quantity);

    // assert
    BOOST_CHECK_EQUAL(12, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x0013, address);
    BOOST_CHECK_EQUAL(0x0011, quantity);
}

BOOST_FIXTURE_TEST_CASE(test_parse_read_discrete_inputs_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x02, 0x00, 0xC4, 0x00, 0x16};

    // act
    uint16_t id = 0;
    uint8_t device = 0;
    uint16_t address = 0;
    uint16_t quantity = 0;
    const int result = parse_read_discrete_inputs_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        quantity);

    // assert
    BOOST_CHECK_EQUAL(12, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x00C4, address);
    BOOST_CHECK_EQUAL(0x0016, quantity);
}

BOOST_FIXTURE_TEST_CASE(test_parse_read_holding_registers_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x03, 0x00, 0x6B, 0x00, 0x03};

    // act
    uint16_t id = 0;
    uint8_t device = 0;
    uint16_t address = 0;
    uint16_t quantity = 0;
    const int result = parse_read_holding_registers_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        quantity);

    // assert
    BOOST_CHECK_EQUAL(12, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x006B, address);
    BOOST_CHECK_EQUAL(0x0003, quantity);
}

BOOST_FIXTURE_TEST_CASE(test_parse_read_input_registers_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x04, 0x00, 0x08, 0x00, 0x01};

    // act
    uint16_t id = 0;
    uint8_t device = 0;
    uint16_t address = 0;
    uint16_t quantity = 0;
    const int result = parse_read_input_registers_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        quantity);

    // assert
    BOOST_CHECK_EQUAL(12, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x0008, address);
    BOOST_CHECK_EQUAL(0x0001, quantity);
}

BOOST_FIXTURE_TEST_CASE(test_parse_write_single_coil_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x05, 0x00, 0xAC, 0xFF, 0x00};

    // act
    uint16_t id = 0;
    uint8_t device = 0;
    uint16_t address = 0;
    bool value = false;
    const int result = parse_write_single_coil_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        value);

    // assert
    BOOST_CHECK_EQUAL(12, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x00AC, address);
    BOOST_CHECK_EQUAL(true, value);
}

BOOST_FIXTURE_TEST_CASE(test_parse_write_single_register_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x06, 0x00, 0x01, 0x00, 0x03};

    // act
    uint16_t id = 0;
    uint8_t device = 0;
    uint16_t address = 0;
    uint16_t value = 0x0000;
    const int result = parse_write_single_register_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        value);

    // assert
    BOOST_CHECK_EQUAL(12, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x0001, address);
    BOOST_CHECK_EQUAL(0x0003, value);
}

BOOST_FIXTURE_TEST_CASE(test_parse_write_multiple_coils_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x09, 0x01, 0x0F, 0x00, 0x13, 0x00, 0x0A, 0x02, 0xCD, 0x01};

    // act
    uint16_t id =0;
    uint8_t device = 0;
    uint16_t address = 0;
    vector<bool> value;
    const int result = parse_write_multiple_coils_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        value);

    // assert
    BOOST_CHECK_EQUAL(15, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x0013, address);
    const bool value_[] = {true, false, true, true, false, false, true, true, true, false};
    BOOST_CHECK_EQUAL_COLLECTIONS(
        value_, value_ + size(value_),
        value.begin(), value.end());
}

BOOST_FIXTURE_TEST_CASE(test_parse_write_multiple_coils_request_with_quantity_0x0100_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {
        0x00, 0x01, 0x00, 0x00, 0x00, 0x27,
        0x01,
        0x0F, 0x00, 0x13, 0x01, 0x00, 0x20,
        0x55, 0xAA, 0x00, 0xFF, 0x50, 0x05, 0xA0, 0x0A, 0xF0, 0x0F, 0x5A, 0xA5, 0xFA, 0xAF, 0x5F, 0xF5,
        0x55, 0xAA, 0x00, 0xFF, 0x50, 0x05, 0xA0, 0x0A, 0xF0, 0x0F, 0x5A, 0xA5, 0xFA, 0xAF, 0x5F, 0xF5
    };

    // act
    uint16_t id =0;
    uint8_t device = 0;
    uint16_t address = 0;
    vector<bool> value;
    const int result = parse_write_multiple_coils_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        value);

    // assert
    BOOST_CHECK_EQUAL(45, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x0013, address);
}

BOOST_FIXTURE_TEST_CASE(test_parse_write_multiple_registers_request_ShouldReturnLength, TestFixture)
{
    // arrange
    const uint8_t request[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x0B, 0x01, 0x10, 0x00, 0x01, 0x00, 0x02, 0x04, 0x00, 0x0A, 0x01, 0x02};

    // act
    uint16_t id = 0;
    uint8_t device = 0;
    uint16_t address = 0;
    vector<uint16_t> value;
    const int result = parse_write_multiple_registers_request(
        vector<uint8_t>(request, request + size(request)),
        id,
        device,
        address,
        value);

    // assert
    BOOST_CHECK_EQUAL(17, result);
    BOOST_CHECK_EQUAL(0x0001, id);
    BOOST_CHECK_EQUAL(0x01, device);
    BOOST_CHECK_EQUAL(0x0001, address);
    const uint16_t value_[] = {0x000A, 0x0102};
    BOOST_CHECK_EQUAL_COLLECTIONS(
        value_, value_ + size(value_),
        value.begin(), value.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_read_coils_response_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const bool coil_status[] = {
        true, false, true, true, false, false, true, true,
        true, true, false, true, false, true, true, false,
        true, false, true};

    // act
    const vector<uint8_t> result = form_read_coils_response(
        id,
        device,
        vector<bool>(coil_status, coil_status + size(coil_status)));

    // assert
    const uint8_t response[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x01, 0x03, 0xCD, 0x6B, 0x05};
    BOOST_CHECK_EQUAL_COLLECTIONS(response, response + size(response), result.begin(), result.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_read_discrete_inputs_response_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const bool input_status[] = {
        false, false, true, true, false, true, false, true,
        true, true, false, true, true, false, true, true,
        true, false, true, false, true, true};

    // act
    const vector<uint8_t> result = form_read_discrete_inputs_response(
        id,
        device,
        vector<bool>(input_status, input_status + size(input_status)));

    // assert
    const uint8_t response[] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01, 0x02, 0x03, 0xAC, 0xDB, 0x35};
    BOOST_CHECK_EQUAL_COLLECTIONS(response, response + size(response), result.begin(), result.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_read_holding_registers_response_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const uint16_t holding_register[] = {0x022B, 0x0000, 0x0064};

    // act
    const vector<uint8_t> result = form_read_holding_registers_response(
        id,
        device,
        vector<uint16_t>(holding_register, holding_register + size(holding_register)));

    // assert
    const uint8_t response[] = {
        0x00, 0x01, 0x00, 0x00, 0x00, 0x09, 0x01,
        0x03, 0x06, 0x02, 0x2B, 0x00, 0x00, 0x00, 0x64};
    BOOST_CHECK_EQUAL_COLLECTIONS(response, response + size(response), result.begin(), result.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_read_input_registers_response_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const uint16_t input_register[] = {0x000A};

    // act
    const vector<uint8_t> result = form_read_input_registers_response(
        id,
        device,
        vector<uint16_t>(input_register, input_register + size(input_register)));

    // assert
    const uint8_t response[] = {
        0x00, 0x01, 0x00, 0x00, 0x00, 0x05, 0x01,
        0x04, 0x02, 0x00, 0x0A};
    BOOST_CHECK_EQUAL_COLLECTIONS(response, response + size(response), result.begin(), result.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_write_single_coil_response_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const uint16_t address = 0x00AC;
    const bool value = true;

    // act
    const vector<uint8_t> result = form_write_single_coil_response(
        id,
        device,
        address,
        value);

    // assert
    const uint8_t response[] = {
        0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01,
        0x05, 0x00, 0xAC, 0xFF, 0x00};
    BOOST_CHECK_EQUAL_COLLECTIONS(
        response, response + size(response),
        result.begin(), result.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_write_single_register_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const uint16_t address = 0x0001;
    const uint16_t value = 0x0003;

    // act
    const vector<uint8_t> result = form_write_single_register_response(
        id,
        device,
        address,
        value);

    // assert
    const uint8_t response[] = {
        0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01,
        0x06, 0x00, 0x01, 0x00, 0x03};
    BOOST_CHECK_EQUAL_COLLECTIONS(
        response, response + size(response),
        result.begin(), result.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_write_multiple_coils_response_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const uint16_t address = 0x0013;
    const uint16_t quantity = 0x000A;

    // act
    const vector<uint8_t> result = form_write_multiple_coils_response(
        id,
        device,
        address,
        quantity);

    // assert
    const uint8_t response[] = {
        0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01,
        0x0F, 0x00, 0x13, 0x00, 0x0A};
    BOOST_CHECK_EQUAL_COLLECTIONS(
        response, response + size(response),
        result.begin(), result.end());
}

BOOST_FIXTURE_TEST_CASE(test_form_write_multiple_registers_response_ShouldReturnPacket, TestFixture)
{
    // arrange
    const uint16_t id = 0x0001;
    const uint8_t device = 0x01;
    const uint16_t address = 0x0001;
    const uint16_t quantity = 0x0002;

    // act
    const vector<uint8_t> result = form_write_multiple_registers_response(
        id,
        device,
        address,
        quantity);

    // assert
    const uint8_t response[] = {
        0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01,
        0x10, 0x00, 0x01, 0x00, 0x02};
    BOOST_CHECK_EQUAL_COLLECTIONS(
        response, response + size(response),
        result.begin(), result.end());
}

}
}
