// This is a simple test program to test whether
// mxmodbus::ModbusTcpClient write function operate correctly.

#include <iostream>
#include <vector>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/range.hpp>

#include <mxmodbus/ModbusTcpClient.hpp>

using boost::diagnostic_information;
using boost::lexical_cast;
using boost::size;

using mxmodbus::ModbusTcpClient;

using std::cerr;
using std::cout;
using std::endl;
using std::string;
using std::vector;

void
    read_coils(ModbusTcpClient &modbus)
{
        cout << "read coils" << endl;
        const vector<bool> co_list = modbus.read_coils(0x00, 0x00, 5);
        BOOST_FOREACH(bool co, co_list)
        {
            cout << co << endl;
        }
}

void
    read_registers(ModbusTcpClient &modbus)
{
    cout << "read registers" << endl;
    const vector<uint16_t> hr_list = modbus.read_holding_registers(0x00, 0x00, 5);
    BOOST_FOREACH(uint16_t hr, hr_list)
    {
        cout << hr << endl;
    }
}

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 3)
    {
        cout << "usage: <ip> <port>" << endl;
        return 1;
    }

    const string host = argv[1];
    const int port = lexical_cast<int>(argv[2]);

    try
    {
        cout << "connecting to " << host << ":" << port << endl;
        ModbusTcpClient modbus(host, port, 1000);

        cout << "initialize coil data" << endl;
        bool co_data[] = {false, false, false, false, false};
        modbus.write_multiple_coils(
            0x00,
            0x00,
            vector<bool>(co_data, co_data + size(co_data))
        );

        read_coils(modbus);

        cout << "write single coil" << endl;
        modbus.write_single_coil(0x00, 0x00, true);

        cout << "write multiple coils" << endl;
        bool co_write_data[] = {true, true};
        modbus.write_multiple_coils(
            0x00,
            0x02,
            vector<bool>(co_write_data, co_write_data + size(co_write_data))
        );

        read_coils(modbus);

        cout << "initialize holding register data" << endl;
        uint16_t hr_data[] = {0x0000, 0x0000, 0x0000, 0x0000, 0x0000};
        modbus.write_multiple_registers(
            0x00,
            0x00,
            vector<uint16_t>(hr_data, hr_data + size(hr_data))
        );

        read_registers(modbus);

        cout << "write single register" << endl;
        modbus.write_single_register(0x00, 0x00, 0x0001);

        cout << "write multiple registers" << endl;
        uint16_t hr_write_data[] = {0x0002, 0x0003};
        modbus.write_multiple_registers(
            0x00,
            0x02,
            vector<uint16_t>(hr_write_data, hr_write_data + size(hr_write_data))
        );

        read_registers(modbus);
    }
    catch(mxmodbus::Error &e)
    {
        cerr << diagnostic_information(e) << endl;
        return 1;
    }

    return 0;
}
