#ifndef _MXMQTT_CLIENT_HPP_
#define _MXMQTT_CLIENT_HPP_

#include <string>
#include <vector>

#include <boost/function.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxmqtt/error.hpp>
#include <mxmqtt/Message.hpp>
#include <mxmqtt/types.hpp>

namespace mxmqtt
{

class Client
    : private boost::noncopyable
{
public:
    typedef
        boost::function<
            void
                (const mxmqtt::Message &)
        >
        OnMessageCallback;

public:
    /**
     * @brief Constructor.
     */
    Client(
        const std::string &host,
        int port,
        const std::string &client_id,
        bool clean_session = true);
    Client(
        const std::string &host,
        int port,
        const std::string &client_id,
        const OnMessageCallback &omc,
        const std::vector<std::string> &subscribe_topic_list,
        bool clean_session = true);
    ~Client();

    /**
     * @brief Primary operation loop.
     *
     * Can be called in another thread.
     */
    void
        loop(
            int timeout_ms = 100);  // throws mxmqtt::IllegalStateError

    /**
     * @brief Object validity checker.
     *
     * Can be used to check the validity of the object.
     */
    bool
        is_valid() const;

    bool
        is_connected() const;

    mxmqtt::message_id_t
        publish(
            const mxmqtt::Message &msg);    // throws mxmqtt::IllegalStateError

    bool
        is_publish_complete(
            mxmqtt::message_id_t) const;    // throws mxmqtt::IllegalStateError

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
