#ifndef _MXMQTT_MOSQUITTO_CLIENT_HPP_
#define _MXMQTT_MOSQUITTO_CLIENT_HPP_

#include <string>
#include <vector>

#include <boost/function.hpp>
#include <boost/scoped_ptr.hpp>

#include <mosquittopp.h>

#include <mxmqtt/error.hpp>
#include <mxmqtt/Message.hpp>
#include <mxmqtt/types.hpp>

namespace mxmqtt
{

class MosquittoClient
    : public mosqpp::mosquittopp
{
public:
    typedef
        boost::function<
            void
                (const mxmqtt::Message &)
        >
        OnMessageCallback;

public:
    MosquittoClient(
        const std::string &host,
        int port,
        const std::string &client_id,
        const OnMessageCallback &omc,
        const std::vector<std::string> &subscribe_topic_list,
        bool clean_session);
    virtual ~MosquittoClient();

    void
        start();    // throws mxmqtt::MosquittoError, mxmqtt:IllegalStateError

    void
        run_once(
            int timeout_ms);

    bool
        is_valid() const;

    bool
        is_connected() const;

    /**
     * @brief Publish message.
     *
     * @return
     *      Returns message-id.
     */
    mxmqtt::message_id_t
        publish_message(
            const mxmqtt::Message &);   // throws mxmqtt::MosquittoError, mxmqtt::NotConnectedError, mxmqtt:IllegalStateError

    bool
        is_publish_complete(
            mxmqtt::message_id_t) const;

private:
    virtual void
        on_connect(
            int rc);
    virtual void
        on_subscribe(
            int mid,
            int qos_count,
            const int *granted_qos);
    virtual void
        on_disconnect(
            int rc);
    virtual void
        on_message(
            const struct mosquitto_message *message);
    virtual void
        on_publish(
            int mid);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
