#ifndef _MXMQTT_WORKER_HPP_
#define _MXMQTT_WORKER_HPP_

#include <string>
#include <vector>

#include <boost/function.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxmqtt/error.hpp>
#include <mxmqtt/Message.hpp>
#include <mxmqtt/types.hpp>

namespace mxmqtt
{

class Worker
    : private boost::noncopyable
{
public:
    typedef
        boost::function<
            void
                (const mxmqtt::Message &)
        >
        OnMessageCallback;

public:
    Worker(
        const std::string &host,
        int port,
        const std::string &client_id,
        bool clean_session = true);
    /**
     * @brief Constructor
     *
     * @note
     *      omc would be called in another thread.
     */
    Worker(
        const std::string &host,
        int port,
        const std::string &client_id,
        const OnMessageCallback &omc,
        const std::vector<std::string> &subscribe_topic_list,
        bool clean_session = true);
    ~Worker();

    void
        publish(
            const mxmqtt::Message &msg,
            int timeout_ms = 3000);     // throws mxmqtt::Error

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
