#include <mxmqtt/Message.hpp>

using std::string;
using std::vector;

namespace mxmqtt
{

struct Message::Impl
{
    Impl(
        const string &topic_,
        const vector<uint8_t> &payload_,
        int qos_,
        bool retain_)
        : topic(topic_)
        , payload(payload_)
        , qos(qos_)
        , retain(retain_)
    {
    }

    const string            topic;
    const vector<uint8_t>   payload;
    const int               qos;
    const bool              retain;
};

Message::Message(
    const string &topic,
    const vector<uint8_t> &payload,
    int qos,
    bool retain)
    : pimpl(new Impl(
        topic,
        payload,
        qos,
        retain))
{
}

Message::~Message()
{
}

const string &
    Message::topic() const
{
    return pimpl->topic;
}

const vector<uint8_t> &
    Message::payload() const
{
    return pimpl->payload;
}

int
    Message::qos() const
{
    return pimpl->qos;
}

bool
    Message::retain() const
{
    return pimpl->retain;
}

bool
    Message::operator ==(const Message &rhs) const
{
    if (this == &rhs)
        return true;

    if (pimpl == rhs.pimpl)
        return true;

    return
        this->topic() == rhs.topic() &&
        this->payload() == rhs.payload() &&
        this->qos() == rhs.qos() &&
        this->retain() == rhs.retain();
}

bool
    Message::operator !=(const Message &rhs) const
{
    return !(*this == rhs);
}

}
