// This is a simple test program to test whether
// mxmqtt::Client::publish operate correctly.

#include <iostream>
#include <vector>

#include <boost/lexical_cast.hpp>

#include <mxmqtt/Client.hpp>

using boost::lexical_cast;

using mxmqtt::Client;
using mxmqtt::Message;
using mxmqtt::message_id_t;

using std::cout;
using std::endl;
using std::string;
using std::vector;

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 3)
    {
        cout << "usage: <ip> <port>\n";
        return 1;
    }

    const string host = argv[1];
    const int port = lexical_cast<int>(argv[2]);

    Client client(
        host,
        port,
        "local-publisher",
        true);

    while(!client.is_connected())
        client.loop();

    const string topic = "/this/is/topic";
    const string content = "This is message.";
    const int qos = 2;
    const bool retain = false;

    const message_id_t msg_id = client.publish(Message(topic, vector<uint8_t>(content.begin(), content.end()), qos, retain));
    cout << "Publish Topic: " << topic << endl;
    cout << "Publish Content: " << content << endl;
    cout << "Publish QoS: " << qos << endl;
    cout << "Publish Retain: " << retain << endl;

    while(!client.is_publish_complete(msg_id))
        client.loop();

    return 0;
}
