#include <mxsl/mxsl.hpp>

#include <cstdlib>
#include <vector>

#include <sys/ioctl.h>
#include <net/if.h>
#include <unistd.h>
#include <netinet/in.h>

#include <boost/foreach.hpp>

#include <mxutil/log.hpp>

#include <mxsl/error.hpp>

using mxsl::Error;

using std::string;
using std::vector;

namespace
{

void
    do_validate();

}

namespace mxsl
{

void
    validate()
{
#ifndef MXSL_DISABLED
    do_validate();
#endif
}

}

namespace
{

const vector<vector<uint8_t> >
    get_mac_list()
{
    struct ifreq ifr;
    struct ifconf ifc;

    char buf[1024];

    int sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
    if (sock == -1)
        BOOST_THROW_EXCEPTION(Error());

    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    if (-1 == ioctl(sock, SIOCGIFCONF, &ifc))
    {
        close(sock);
        BOOST_THROW_EXCEPTION(Error());
    }

    struct ifreq *it = ifc.ifc_req;
    const struct ifreq* const end = it + (ifc.ifc_len / sizeof(struct ifreq));

    vector<vector<uint8_t> > mac_list;
    for (; it != end; ++it)
    {
        const string iface_name(it->ifr_name, it->ifr_name + sizeof(ifr.ifr_name));
        if (0 != iface_name.find("eth"))
            continue;

        strncpy(ifr.ifr_name, it->ifr_name, sizeof(ifr.ifr_name));
        if (0 != ioctl(sock, SIOCGIFFLAGS, &ifr))
            BOOST_THROW_EXCEPTION(Error());

        // skip loopback
        if (ifr.ifr_flags & IFF_LOOPBACK)
            continue;

        if (0 != ioctl(sock, SIOCGIFHWADDR, &ifr))
            BOOST_THROW_EXCEPTION(Error());

        const vector<uint8_t> mac(ifr.ifr_hwaddr.sa_data, ifr.ifr_hwaddr.sa_data + 6);

        mac_list.push_back(mac);
    }

    close(sock);

    return mac_list;
}

void
    validate_mac(const vector<uint8_t> &mac)
{
    if (mac.size() != 6)
        BOOST_THROW_EXCEPTION(Error());

    // avoid 00:90:e8 sequence appear directly in binary
    uint32_t v = mac[0];
    v <<= 8;
    v += mac[1];
    v <<= 8;
    v += mac[2];

    v *= 6547;
    v += 3727;

    if (v != 242871239)
        BOOST_THROW_EXCEPTION(Error());
}

void
    do_validate()
{
    try
    {
        const vector<vector<uint8_t> > mac_list = get_mac_list();

        BOOST_FOREACH(const vector<uint8_t> &mac, mac_list)
        {
            validate_mac(mac);
        }
    }
    catch(Error &e)
    {
#ifdef MXSL_DEBUG_MSG_ENABLED
        MXLOG_WARN(diagnostic_information(e));
#endif
        exit(1);
    }
}

}
