#ifndef _MXTAG_VALUE_HPP_
#define _MXTAG_VALUE_HPP_

#include <iosfwd>
#include <string>

#include <boost/cstdint.hpp>
#include <boost/shared_ptr.hpp>

namespace mxtag
{

class Value // value, no-cmp, immutable
{
public:
    // allow implicit conversion
    Value(int64_t i);
    Value(uint64_t u);
    Value(double d);
    Value(const std::string &s);

    //Value(const Value &);

    ~Value();

    bool
        is_int() const;
    int64_t
        as_int() const;

    bool
        is_uint() const;
    uint64_t
        as_uint() const;

    bool
        is_float() const;
    double
        as_float() const;

    bool
        is_string() const;
    const std::string &
        as_string() const;

    //Value &
    //    operator =(const Value &);

private:
    struct Impl;
    boost::shared_ptr<Impl> pimpl;
};

bool
    operator ==(
        const Value &,
        const Value &);

bool
    operator !=(
        const Value &,
        const Value &);

std::ostream &
    operator <<(
        std::ostream &,
        const Value &);

}

#endif
