#ifndef _MXTAG_LOG_SERIALIZER_HPP_
#define _MXTAG_LOG_SERIALIZER_HPP_

#include <map>
#include <set>
#include <string>
#include <vector>

#include <boost/cstdint.hpp>
#include <boost/noncopyable.hpp>

#include <mxtag/error.hpp>
#include <mxtag/Tag.hpp>

namespace mxtag
{
namespace log
{

class Serializer // interface
    : private boost::noncopyable
{
protected:
    Serializer();

public:
    virtual ~Serializer();

    const std::vector<uint8_t>
        from_tag(const mxtag::Tag &) const;
    const mxtag::Tag
        to_tag(const std::vector<uint8_t> &) const; // throws mxtag::ParseError

    const std::vector<uint8_t>
        from_tag_list(const std::map<std::string, std::set<std::string> > &) const;
    const std::map<std::string, std::set<std::string> >
        to_tag_list(const std::vector<uint8_t> &) const; // throws mxtag::ParseError

private:
    virtual const std::vector<uint8_t>
        do_from_tag(const mxtag::Tag &) const = 0;
    virtual const mxtag::Tag
        do_to_tag(const std::vector<uint8_t> &) const = 0;
    virtual const std::vector<uint8_t>
        do_from_tag_list(const std::map<std::string, std::set<std::string> > &) const = 0;
    virtual const std::map<std::string, std::set<std::string> >
        do_to_tag_list(const std::vector<uint8_t> &) const = 0;
};

}
}

#endif
