#include <mxtag/Tag.hpp>

#include <mxtag/validate.hpp>

using boost::posix_time::ptime;

using std::string;

namespace mxtag
{

struct Tag::Impl
{
    Impl(
        const string &equipment_,
        const string &tag_,
        const ptime &at_,
        const Value &value_,
        const string &unit_)
        : equipment(equipment_)
        , tag(tag_)
        , at(at_)
        , value(value_)
        , unit(unit_)
    {
    }

    const string    equipment;
    const string    tag;
    const ptime     at;
    const Value     value;
    const string    unit;
};

Tag::Tag(
    const string &equipment,
    const string &tag,
    const ptime &at,
    const Value &value,
    const string &unit)
    : pimpl(new Impl(
        equipment,
        tag,
        at,
        value,
        unit))
{
    if (!is_valid_equipment_name(equipment))
    {
        BOOST_THROW_EXCEPTION(
            ArgumentError()
                << EInfoMsg("illegal equipment: " + equipment)
        );
    }

    if (!is_valid_tag_name(tag))
    {
        BOOST_THROW_EXCEPTION(
            ArgumentError()
                << EInfoMsg("illegal tag: " + tag)
        );
    }

    if (!is_valid_unit(unit))
    {
        BOOST_THROW_EXCEPTION(
            ArgumentError()
                << EInfoMsg("illegal unit: " + unit)
        );
    }
}

Tag::~Tag()
{
}

const string &
    Tag::equipment() const
{
    return pimpl->equipment;
}

const string &
    Tag::tag() const
{
    return pimpl->tag;
}

const ptime &
    Tag::at() const
{
    return pimpl->at;
}

const Value &
    Tag::value() const
{
    return pimpl->value;
}

const string &
    Tag::unit() const
{
    return pimpl->unit;
}

bool
    operator ==(
        const Tag &lhs,
        const Tag &rhs)
{
    return
        lhs.equipment() == rhs.equipment() &&
        lhs.tag() == rhs.tag() &&
        lhs.at() == rhs.at() &&
        lhs.value() == rhs.value() &&
        lhs.unit() == rhs.unit();
}

bool
    operator !=(
        const Tag &lhs,
        const Tag &rhs)
{
    return !(lhs == rhs);
}

}
