#include <mxtag/Value.hpp>

#include <iostream>

#include <boost/make_shared.hpp>
#include <boost/shared_ptr.hpp>

using boost::make_shared;
using boost::shared_ptr;

using std::ostream;
using std::string;

namespace mxtag
{

struct Value::Impl
{
    Impl()
    {
    }

    shared_ptr<int64_t>     iv;
    shared_ptr<uint64_t>    uv;
    shared_ptr<double>      fv;
    shared_ptr<string>      sv;
};

Value::Value(int64_t iv)
    : pimpl(new Impl())
{
    pimpl->iv = make_shared<int64_t>(iv);
}

Value::Value(uint64_t uv)
    : pimpl(new Impl())
{
    pimpl->uv = make_shared<uint64_t>(uv);
}

Value::Value(double fv)
    : pimpl(new Impl())
{
    pimpl->fv = make_shared<double>(fv);
}

Value::Value(const string &sv)
    : pimpl(new Impl())
{
    pimpl->sv = make_shared<string>(sv);
}

Value::~Value()
{
}

bool
    Value::is_int() const
{
    return pimpl->iv;
}

int64_t
    Value::as_int() const
{
    BOOST_ASSERT(pimpl->iv);
    return *pimpl->iv;
}

bool
    Value::is_uint() const
{
    return pimpl->uv;
}

uint64_t
    Value::as_uint() const
{
    BOOST_ASSERT(pimpl->uv);
    return *pimpl->uv;
}

bool
    Value::is_float() const
{
    return pimpl->fv;
}

double
    Value::as_float() const
{
    BOOST_ASSERT(pimpl->fv);
    return *pimpl->fv;
}

bool
    Value::is_string() const
{
    return pimpl->sv;
}

const string &
    Value::as_string() const
{
    BOOST_ASSERT(pimpl->sv);
    return *pimpl->sv;
}

bool
    operator ==(
        const Value &lhs,
        const Value &rhs)
{
    return
        (lhs.is_int() && rhs.is_int() && lhs.as_int() == rhs.as_int())
        || (lhs.is_uint() && rhs.is_uint() && lhs.as_uint() == rhs.as_uint())
        || (lhs.is_float() && rhs.is_float() && lhs.as_float() == rhs.as_float())
        || (lhs.is_string() && rhs.is_string() && lhs.as_string() == rhs.as_string());
}

bool
    operator !=(
        const Value &lhs,
        const Value &rhs)
{
    return !(lhs == rhs);
}

ostream &
    operator <<(
        ostream &os,
        const Value &rhs)
{
    if (rhs.is_int())
        os << rhs.as_int();
    else if (rhs.is_uint())
        os << rhs.as_uint();
    else if (rhs.is_float())
        os << rhs.as_float();
    else if (rhs.is_string())
        os << rhs.as_string();
    else
        BOOST_VERIFY(!"unexpected value type");

    return os;
}

}
