#include <mxtag/log/EnvelopedIStream.hpp>

#include <mxtag/priv/vlq.hpp>

using mxtag::priv::vlq_decode;

using std::istream;
using std::streamsize;
using std::vector;

namespace
{

const size_t MAX_ENVELOP_SIZE = 64 * 1024;

}

namespace mxtag
{
namespace log
{

struct EnvelopedIStream::Impl
{
    Impl(istream &is_)
        : is(is_)
    {
    }

    istream &   is;
};

EnvelopedIStream::EnvelopedIStream(
    istream &is)
    : pimpl(new Impl(is))
{
}

EnvelopedIStream::~EnvelopedIStream()
{
}

const vector<uint8_t>
    EnvelopedIStream::read()
{
    // read vlq
    vector<uint8_t> vlq_buf;
    while(vlq_buf.empty() || (vlq_buf.back() & 0x80) == 0x80)
    {
        char c;
        pimpl->is.read(&c, 1);

        if (pimpl->is.gcount() != 1)
        {
            if (vlq_buf.empty())
                BOOST_THROW_EXCEPTION(Eof());
            else
                BOOST_THROW_EXCEPTION(ParseError());
        }

        vlq_buf.push_back(c);
    }

    const uint64_t len = vlq_decode(vlq_buf);
    if (len > MAX_ENVELOP_SIZE)
        BOOST_THROW_EXCEPTION(ParseError());

    // read actual data
    vector<uint8_t> buf(len);
    pimpl->is.read(reinterpret_cast<char *>(&buf.front()), buf.size());
    if (pimpl->is.gcount() != static_cast<streamsize>(buf.size()))
        BOOST_THROW_EXCEPTION(ParseError());

    return buf;
}

EnvelopedIStream &
    operator >>(
        EnvelopedIStream &is,
        vector<uint8_t> &rhs)
{
    rhs = is.read();
    return is;
}

}
}
