#include <mxtag/log/EnvelopedOStream.hpp>

#include <mxutil/log.hpp>

#include <mxtag/log/internal_log.hpp>

using std::ostream;
using std::vector;

namespace mxtag
{
namespace log
{

struct EnvelopedOStream::Impl
{
    Impl(
        ostream &os_,
        size_t buffer_size_)
        : os(os_)
        , buffer_size(buffer_size_)
        , bytes_written(0)
    {
    }

    ostream &       os;
    const size_t    buffer_size;

    vector<uint8_t> buffer;
    size_t          bytes_written;

    void
        write(const vector<uint8_t> &buf)
    {
        this->buffer.insert(
            this->buffer.end(),
            buf.begin(),
            buf.end());

        this->bytes_written += buf.size();

        if (this->buffer.size() >= buffer_size)
        {
            this->os.write(reinterpret_cast<const char *>(&this->buffer.front()), this->buffer.size());

            if (this->os.bad())
            {
                MXLOG_WARN("ostream bad");
                BOOST_THROW_EXCEPTION(SystemError());
            }

            this->buffer.clear();
        }
    }
};

EnvelopedOStream::EnvelopedOStream(
    ostream &os,
    size_t buffer_size)
    : pimpl(new Impl(
        os,
        buffer_size))
{
}

EnvelopedOStream::~EnvelopedOStream()
{
    if (!pimpl->buffer.empty())
    {
        pimpl->os.write(reinterpret_cast<const char *>(&pimpl->buffer.front()), pimpl->buffer.size());
        if (pimpl->os.bad())
            MXLOG_WARN("write file failure");

        pimpl->buffer.clear();
    }
}

size_t
    EnvelopedOStream::bytes_written() const
{
    return pimpl->bytes_written;
}

void
    EnvelopedOStream::write(const vector<uint8_t> &v)
{
    pimpl->write(envelope_encode(v));
}

EnvelopedOStream &
    operator <<(
        EnvelopedOStream &eos,
        const vector<uint8_t> &rhs)
{
    eos.write(rhs);

    return eos;
}

}
}
