#include <mxtag/log/ProtobufSerializer.hpp>

#include <boost/foreach.hpp>

#include <tag.pb.h>

#include <mxtag/priv/conversion.hpp>

using mxtag::priv::at_ms_from_ptime;
using mxtag::priv::ptime_from_at_ms;
using mxtag::priv::value_from_pb_value;
using mxtag::priv::value_to_pb_value;

using std::map;
using std::set;
using std::string;
using std::vector;

typedef
    mxtag::pb::Tag
    PbTag;

typedef
    mxtag::pb::Value
    PbValue;

typedef
    mxtag::pb::TagNameList
    PbTagNameList;

typedef
    mxtag::pb::TagName
    PbTagName;

namespace mxtag
{
namespace log
{

ProtobufSerializer::ProtobufSerializer()
{
}

ProtobufSerializer::~ProtobufSerializer()
{
}

const vector<uint8_t>
    ProtobufSerializer::do_from_tag(const Tag &t) const
{
    PbTag tag;
    tag.set_equipment(t.equipment());
    tag.set_tag(t.tag());
    tag.set_at_ms(at_ms_from_ptime(t.at()));

    value_to_pb_value(t.value(), *tag.mutable_value());

    if (!t.unit().empty())
        tag.set_unit(t.unit());

    vector<uint8_t> rtn(tag.ByteSize());
    tag.SerializeToArray(&rtn.front(), rtn.size());

    return rtn;
}

const mxtag::Tag
    ProtobufSerializer::do_to_tag(const vector<uint8_t> &v) const
{
    PbTag tag;
    tag.ParseFromArray(&v.front(), v.size());

    if (!tag.IsInitialized())
        BOOST_THROW_EXCEPTION(ParseError());

    if (!tag.has_equipment() ||
        !tag.has_tag() ||
        !tag.has_at_ms() ||
        !tag.has_value())
        BOOST_THROW_EXCEPTION(ParseError());

    const Value value = value_from_pb_value(tag.value());

    string unit;
    if (tag.has_unit())
        unit = tag.unit();

    return Tag(
        tag.equipment(),
        tag.tag(),
        ptime_from_at_ms(tag.at_ms()),
        value_from_pb_value(tag.value()),
        unit);
}

const vector<uint8_t>
    ProtobufSerializer::do_from_tag_list(const map<string, set<string> > &tag_list) const
{
    PbTagNameList tnlist;

    typedef map<string, set<string> > eq_tag_list_t;
    BOOST_FOREACH(const eq_tag_list_t::value_type &et, tag_list)
    {
        PbTagName *tn = tnlist.add_tag_name();
        BOOST_ASSERT(tn);

        tn->set_equipment(et.first);

        BOOST_FOREACH(const string &t, et.second)
        {
            tn->add_tag(t);
        }
    }

    vector<uint8_t> pb_buf(tnlist.ByteSize());
    BOOST_VERIFY(
        tnlist.SerializeToArray(&pb_buf.front(), pb_buf.size())
    );

    return pb_buf;
}

const map<string, set<string> >
    ProtobufSerializer::do_to_tag_list(const vector<uint8_t> &v) const
{
    PbTagNameList tnlist;
    tnlist.ParseFromArray(&v.front(), v.size());

    if (!tnlist.IsInitialized())
        BOOST_THROW_EXCEPTION(ParseError());

    map<string, set<string> > tag_list;
    // walkthrough tnlist and generate tag_list
    for(int ax = 0; ax < tnlist.tag_name_size(); ++ax)
    {
        const PbTagName &tn = tnlist.tag_name(ax);
        if (!tn.has_equipment())
            BOOST_THROW_EXCEPTION(ParseError());

        const string &equipment = tn.equipment();

        for(int bx = 0; bx < tn.tag_size(); ++bx)
        {
            const string &tag = tn.tag(bx);
            tag_list[equipment].insert(tag);
        }
    }

    return tag_list;
}

}
}
