#include <mxtag/mqtt/ProtobufSerializer.hpp>

#include <mxtag/priv/conversion.hpp>
#include <mxtag/priv/mqtt.hpp>

#include <tag.pb.h>

using mxtag::priv::at_ms_from_ptime;
using mxtag::priv::parse_topic;
using mxtag::priv::ptime_from_at_ms;
using mxtag::priv::value_from_pb_value;
using mxtag::priv::value_to_pb_value;

using std::make_pair;
using std::pair;
using std::string;
using std::vector;

typedef
    mxtag::pb::Tag
    PbTag;

typedef
    mxtag::pb::Value
    PbValue;

namespace mxtag
{
namespace mqtt
{

ProtobufSerializer::ProtobufSerializer()
{
}

ProtobufSerializer::~ProtobufSerializer()
{
}

const pair<string, vector<uint8_t> >
    ProtobufSerializer::do_from_tag(const Tag &t) const
{
    PbTag tag;
    tag.set_at_ms(at_ms_from_ptime(t.at()));

    value_to_pb_value(t.value(), *tag.mutable_value());

    if (!t.unit().empty())
        tag.set_unit(t.unit());

    vector<uint8_t> payload(tag.ByteSize());
    tag.SerializeToArray(&payload.front(), payload.size());

    return make_pair<string, vector<uint8_t> >(
        "/equs/" + t.equipment() + "/tags/" + t.tag(),
        payload);
}

const Tag
    ProtobufSerializer::do_to_tag(
        const string &topic,
        const vector<uint8_t> &payload) const
{
    string equipment;
    string tag;
    if (!parse_topic(topic, equipment, tag))
        BOOST_THROW_EXCEPTION(ParseError());

    PbTag t;
    t.ParseFromArray(&payload.front(), payload.size());
    if (!t.IsInitialized())
        BOOST_THROW_EXCEPTION(ParseError());

    if (!t.has_at_ms() ||
        !t.has_value())
        BOOST_THROW_EXCEPTION(ParseError());

    return Tag(
        equipment,
        tag,
        ptime_from_at_ms(t.at_ms()),
        value_from_pb_value(t.value()),
        t.has_unit() ? t.unit() : "");
}

}
}
