#include <mxtag/mqtt/tag_parse.hpp>

#include <boost/lexical_cast.hpp>
#include <boost/regex.hpp>

#include <rapidjson/document.h>

#include <mxtag/tag_parse.hpp>

#include <mxutil/log.hpp>

using boost::lexical_cast;
using boost::posix_time::ptime;
using boost::regex;
using boost::regex_match;
using boost::smatch;

using mxtag::parse_iso8601_time;

using rapidjson::Document;
using rapidjson::Value;

using std::string;

namespace mxtag
{
namespace mqtt
{

bool
    parse_topic(
        const string &topic,
        string &equipment_id,
        string &tag_id)
{
    static const regex e(
        "^/equs/([\\w\\d](?:[\\w\\d\\- ]*[\\w\\d])*)/tags/([\\w\\d](?:[\\w\\d\\- ]*[\\w\\d])*)$"
    );

    smatch what;
    if (!regex_match(
            topic,
            what,
            e))
    {
        return false;
    }

    equipment_id = what[1].str();
    tag_id = what[2].str();

    return true;
}

bool
    parse_payload(
        const string &payload,
        ptime &at,
        string &value,
        string &unit)
{
    Document doc;
    doc.Parse(payload.c_str());
    if (!doc.IsObject())
    {
        MXLOG_WARN("!doc.IsObject()");
        return false;
    }

    if (!parse_iso8601_time(
            doc["at"].GetString(),
            at))
    {
        MXLOG_WARN(string("parse_iso8601_time failed: ") + doc["at"].GetString());
        return false;
    }

    Value &value_ = doc["value"];
    if (value_.IsString())
    {
        value = string("\"") + value_.GetString() + '\"';
    }
    else if (value_.IsInt64())
    {
        value = lexical_cast<string>(value_.GetInt64());
    }
    else if (value_.IsDouble())
    {
        value = lexical_cast<string>(value_.GetDouble());
    }
    else
    {
        MXLOG_WARN("unexpected datatype");
        return false;
    }

    if (doc.HasMember("unit"))
    {
        unit = doc["unit"].GetString();
    }
    else
    {
        unit = "";
    }

    return true;
}

}
}
