#include <mxtag/priv/conversion.hpp>

using boost::posix_time::from_time_t;
using boost::posix_time::ptime;
using boost::posix_time::time_duration;

typedef
    mxtag::pb::Tag
    PbTag;

typedef
    mxtag::pb::Value
    PbValue;

namespace
{

uint64_t
    power(
        uint64_t v,
        uint64_t exp)
{
    uint64_t rtn = 1;
    for(unsigned ax = 0; ax < exp; ++ax)
        rtn *= v;

    return rtn;
}

}

namespace mxtag
{
namespace priv
{

uint64_t
    at_ms_from_ptime(const ptime &t)
{
    static const ptime BASE = from_time_t(0);
    const time_duration diff = t - BASE;

    uint64_t at_ms = 0;
    at_ms += diff.hours();
    at_ms *= 60;
    at_ms += diff.minutes();
    at_ms *= 60;
    at_ms += diff.seconds();

    const int frac_diff = 3 - static_cast<int>(time_duration::num_fractional_digits());

    uint64_t ms = 0;
    if (frac_diff >= 0)
        ms = diff.fractional_seconds() * power(10, frac_diff);
    else
        ms = diff.fractional_seconds() / power(10, abs(frac_diff));

    at_ms *= 1000;
    at_ms += ms;

    return at_ms;
}

const ptime
    ptime_from_at_ms(uint64_t at_ms)
{
    static const ptime BASE = from_time_t(0);

    const int frac_diff = 3 - static_cast<int>(time_duration::num_fractional_digits());
    const uint64_t ms = at_ms % 1000;

    long fractional_seconds = 0;
    if (frac_diff >= 0)
        fractional_seconds = ms / power(10, frac_diff);
    else
        fractional_seconds = ms * power(10, abs(frac_diff));

    const long s = (at_ms / 1000) % 60;
    const long m = (at_ms / 1000 / 60) % 60;
    const long h = at_ms / 1000 / 60 / 60;

    return BASE + time_duration(h, m, s, fractional_seconds);
}

void
    value_to_pb_value(
        const Value &value,
        PbValue &pb_value)
{
    pb_value.Clear();
    if (value.is_int())
        pb_value.set_int_value(value.as_int());
    else if (value.is_uint())
        pb_value.set_uint_value(value.as_uint());
    else if (value.is_float())
        pb_value.set_float_value(value.as_float());
    else if (value.is_string())
        pb_value.set_str_value(value.as_string());
    else
        BOOST_VERIFY(!"unexpected value");
}

const Value
    value_from_pb_value(
        const PbValue &pb_value)   // throws ParseError
{
    if (pb_value.has_int_value())
        return Value(pb_value.int_value());
    else if (pb_value.has_uint_value())
        return Value(pb_value.uint_value());
    else if (pb_value.has_float_value())
        return Value(pb_value.float_value());
    else if (pb_value.has_str_value())
        return Value(pb_value.str_value());
    else
        BOOST_THROW_EXCEPTION(ParseError());
}

}
}
