#include <mxtag/priv/vlq.hpp>

using std::vector;

namespace mxtag
{
namespace priv
{

// VLQ:
//   LSB first.
//   bit-7 uses as 'has-more'.

const vector<uint8_t>
    vlq_encode(uint64_t v)
{
    vector<uint8_t> rtn;

    for(;;)
    {
        rtn.push_back(
            (v & 0x7Fu)
            | ((v >> 7) == 0 ? 0x00 : 0x80));

        v >>= 7;

        if (v == 0)
            break;
    }

    return rtn;
}

uint64_t
    vlq_decode(const vector<uint8_t> &v)
{
    // 64 / 7 = 9.xx -> 10
    if (v.size() == 0 ||
        v.size() > (64 + 6) / 7)
        BOOST_THROW_EXCEPTION(ArgumentError());

    // check only the last byte has bit-7 == 0
    {
        if ((v.back() & 0x80u) != 0x00)
            BOOST_THROW_EXCEPTION(ArgumentError());

        for(size_t ax = 0; ax < v.size() - 1; ++ax)
        {
            if ((v[ax] & 0x80u) == 0x00)
                BOOST_THROW_EXCEPTION(ArgumentError());
        }
    }

    uint64_t rtn = 0u;
    for(size_t ax = 0; ax < v.size(); ++ax)
    {
        const uint64_t to_merge = v[ax] & 0x7F;

        rtn |= to_merge << (7 * ax);
    }

    return rtn;
}

}
}
