#include <boost/test/unit_test.hpp>

#include <mxtag/mqtt/ProtobufSerializer.hpp>

using boost::posix_time::time_from_string;

using std::pair;
using std::string;
using std::vector;

namespace
{

struct TestFixture
{
    mxtag::mqtt::ProtobufSerializer serializer;
};

}

namespace mxtag
{

BOOST_FIXTURE_TEST_CASE(test_ProtobufSerializer_from_tag_ShouldReturnCorrectTopic, TestFixture)
{
    // arrange
    const Tag t(
        "Test_Equipment",
        "Test_Tag_0",
        time_from_string("1970-01-01 12:34:56.789"),
        Value(static_cast<int64_t>(-123456)),
        "fake-unit");

    // act
    const pair<string, vector<uint8_t> > r = serializer.from_tag(t);

    // assert
    BOOST_CHECK_EQUAL("/equs/Test_Equipment/tags/Test_Tag_0", r.first);
}

BOOST_FIXTURE_TEST_CASE(test_ProtobufSerializer_tag_Conversion_ShouldPass, TestFixture)
{
    // arrange
    const Tag t(
        "Test_Equipment",
        "Test_Tag_0",
        time_from_string("1970-01-01 12:34:56.789"),
        Value(static_cast<int64_t>(-123456)),
        "fake-unit");

    // act
    const pair<string, vector<uint8_t> > p = serializer.from_tag(t);
    const Tag r = serializer.to_tag(p.first, p.second);

    // assert
    BOOST_CHECK_EQUAL("Test_Equipment", r.equipment());
    BOOST_CHECK_EQUAL("Test_Tag_0", r.tag());
    BOOST_CHECK_EQUAL(time_from_string("1970-01-01 12:34:56.789"), r.at());
    BOOST_CHECK(Value(static_cast<int64_t>(-123456)) == r.value());
    BOOST_CHECK_EQUAL("fake-unit", r.unit());
}

}
