#include <boost/test/unit_test.hpp>

#include <mxtag/priv/vlq.hpp>

#include <boost/range.hpp>

using boost::size;

using std::vector;

namespace
{

class TestFixture
{
};

}

namespace mxtag
{
namespace priv
{

BOOST_FIXTURE_TEST_CASE(test_vlq_encode_WithOneByteValue_ShouldPass, TestFixture)
{
    // arrange
    const uint64_t value = 0x77;

    // act
    const vector<uint8_t> vlq = vlq_encode(value);

    // assert
    BOOST_REQUIRE_EQUAL(1, vlq.size());
    BOOST_CHECK_EQUAL(0x77u, vlq[0]);
}

BOOST_FIXTURE_TEST_CASE(test_vlq_encode_WithTwoByteValue_ShouldPass, TestFixture)
{
    // arrange
    const uint64_t value = 0xF7;

    // act
    const vector<uint8_t> vlq = vlq_encode(value);

    // assert
    BOOST_REQUIRE_EQUAL(2, vlq.size());
    BOOST_CHECK_EQUAL(0x77u | 0x80u, vlq[0]);
    BOOST_CHECK_EQUAL(0x01u, vlq[1]);
}

BOOST_FIXTURE_TEST_CASE(test_vlq_encode_WithTenByteValue_ShouldPass, TestFixture)
{
    // arrange
    const uint64_t value = 0xFEDCBA9876543210ull;

    // act
    const vector<uint8_t> vlq = vlq_encode(value);

    // assert
    BOOST_REQUIRE_EQUAL(10, vlq.size());
    BOOST_CHECK_EQUAL(0x10u | 0x80u, vlq[0]);
    BOOST_CHECK_EQUAL(0x64u | 0x80u, vlq[1]);
    BOOST_CHECK_EQUAL(0x50u | 0x80u, vlq[2]);
    BOOST_CHECK_EQUAL(0x32u | 0x80u, vlq[3]);
    BOOST_CHECK_EQUAL(0x07u | 0x80u, vlq[4]);
    BOOST_CHECK_EQUAL(0x53u | 0x80u, vlq[5]);
    BOOST_CHECK_EQUAL(0x2Eu | 0x80u, vlq[6]);
    BOOST_CHECK_EQUAL(0x6Eu | 0x80u, vlq[7]);
    BOOST_CHECK_EQUAL(0x7Eu | 0x80u, vlq[8]);
    BOOST_CHECK_EQUAL(0x01u, vlq[9]);
}

BOOST_FIXTURE_TEST_CASE(test_vlq_decode_WithOneByteValue_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t VALUE[] = {
        0x77
    };

    // act
    const uint64_t v = vlq_decode(vector<uint8_t>(&VALUE[0], &VALUE[0] + size(VALUE)));

    // assert
    BOOST_CHECK_EQUAL(0x77ull, v);
}

BOOST_FIXTURE_TEST_CASE(test_vlq_decode_WithTwoByteValue_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t VALUE[] = {
        0x77 | 0x80, 0x01
    };

    // act
    const uint64_t v = vlq_decode(vector<uint8_t>(&VALUE[0], &VALUE[0] + size(VALUE)));

    // assert
    BOOST_CHECK_EQUAL(0xF7ull, v);
}

BOOST_FIXTURE_TEST_CASE(test_vlq_decode_WithTenByteValue_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t VALUE[] = {
        0x10 | 0x80, 0x64 | 0x80, 0x50 | 0x80, 0x32 | 0x80,
        0x07 | 0x80, 0x53 | 0x80, 0x2E | 0x80, 0x6E | 0x80,
        0x7E | 0x80, 0x01
    };

    // act
    const uint64_t v = vlq_decode(vector<uint8_t>(&VALUE[0], &VALUE[0] + size(VALUE)));

    // assert
    BOOST_CHECK_EQUAL(0xFEDCBA9876543210ull, v);
}

}
}
