#include <boost/test/unit_test.hpp>

#include <mxtag/Tag.hpp>

using boost::posix_time::time_from_string;

namespace
{

struct TestFixture
{
};

}

namespace mxtag
{

BOOST_FIXTURE_TEST_CASE(test_Tag_ctor_WithIllegalEquipmentName_ShouldRaiseError, TestFixture)
{
    // arrange

    // act & assert
    BOOST_CHECK_THROW(
        Tag(
            "",
            "tag-name",
            time_from_string("2010-01-01 00:00:00"),
            Value("value"),
            "unit"),
        ArgumentError);
}

BOOST_FIXTURE_TEST_CASE(test_Tag_ctor_WithIllegalTagName_ShouldRaiseError, TestFixture)
{
    // arrange

    // act & assert
    BOOST_CHECK_THROW(
        Tag(
            "equipment-name",
            "",
            time_from_string("2010-01-01 00:00:00"),
            Value("value"),
            "unit"),
        ArgumentError);
}

BOOST_FIXTURE_TEST_CASE(test_Tag_ctor_WithIllegalUnit_ShouldRaiseError, TestFixture)
{
    // arrange

    // act & assert
    BOOST_CHECK_THROW(
        Tag(
            "equipment-name",
            "tag-name",
            time_from_string("2010-01-01 00:00:00"),
            Value("value"),
            "illegal.unit"),
        ArgumentError);
}

BOOST_FIXTURE_TEST_CASE(test_Tag_operator_eq_ShouldPass, TestFixture)
{
    // arrange
    const Tag lhs(
        "equipment-name",
        "tag-name",
        time_from_string("2010-01-01 00:00:00"),
        Value("value"),
        "unit");
    const Tag rhs(
        "equipment-name",
        "tag-name",
        time_from_string("2010-01-01 00:00:00"),
        Value("value"),
        "unit");

    // act
    const bool should_be_true = lhs == rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_Tag_operator_ne_ShouldPass, TestFixture)
{
    // arrange
    const Tag lhs(
        "equipment-name",
        "tag-name",
        time_from_string("2010-01-01 00:00:00"),
        Value("value"),
        "unit");
    const Tag rhs(
        "equipment-name",
        "tag-name",
        time_from_string("2010-01-01 00:00:01"),
        Value("value"),
        "unit");

    // act
    const bool should_be_true = lhs != rhs;

    // assert
    BOOST_CHECK(should_be_true);
}

}
