#include <boost/test/unit_test.hpp>

#include <mxtag/Value.hpp>

#include <sstream>

using std::stringstream;

namespace
{

class TestFixture
{
};

}

namespace mxtag
{

BOOST_FIXTURE_TEST_CASE(test_Value_ctor_WithInt_ShouldBehaveCorrectly, TestFixture)
{
    // arrange

    // act
    const Value v(static_cast<int64_t>(12345));

    // assert
    BOOST_REQUIRE(v.is_int());
    BOOST_CHECK_EQUAL(12345, v.as_int());
}

BOOST_FIXTURE_TEST_CASE(test_Value_ctor_WithUInt_ShouldBehaveCorrectly, TestFixture)
{
    // arrange

    // act
    const Value v(static_cast<uint64_t>(1234567890ull));

    // assert
    BOOST_REQUIRE(v.is_uint());
    BOOST_CHECK_EQUAL(1234567890ull, v.as_uint());
}

BOOST_FIXTURE_TEST_CASE(test_Value_ctor_WithFloat_ShouldBehaveCorrectly, TestFixture)
{
    // arrange

    // act
    const Value v(1.2345);

    // assert
    BOOST_REQUIRE(v.is_float());
    BOOST_CHECK_CLOSE(1.2345, v.as_float(), 0.001);
}

BOOST_FIXTURE_TEST_CASE(test_Value_ctor_WithString_ShouldBehaveCorrectly, TestFixture)
{
    // arrange

    // act
    const Value v("the-value");

    // assert
    BOOST_REQUIRE(v.is_string());
    BOOST_CHECK_EQUAL("the-value", v.as_string());
}

BOOST_FIXTURE_TEST_CASE(test_Value_operator_eq_WithUInt_ShouldPass, TestFixture)
{
    // arrange
    const Value lhs(static_cast<uint64_t>(1234567890ull));
    const Value rhs(static_cast<uint64_t>(1234567890ull));

    // act
    const bool result = lhs == rhs;

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_Value_operator_ne_ShouldPass, TestFixture)
{
    // arrange
    const Value lhs(static_cast<uint64_t>(1234567890ull));
    const Value rhs(static_cast<int64_t>(1234567890ll));

    // act
    const bool result = lhs != rhs;

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_Value_operator_putto_WithFloat_ShouldPass, TestFixture)
{
    // arrange
    const Value rhs(3.5);

    stringstream ss;

    // act
    ss << rhs;

    // assert
    BOOST_CHECK_EQUAL("3.5", ss.str());
}

BOOST_FIXTURE_TEST_CASE(test_Value_operator_putto_WithInt_ShouldPass, TestFixture)
{
    // arrange
    const Value rhs(static_cast<int64_t>(-123));

    stringstream ss;

    // act
    ss << rhs;

    // assert
    BOOST_CHECK_EQUAL("-123", ss.str());
}

BOOST_FIXTURE_TEST_CASE(test_Value_operator_putto_WithUInt_ShouldPass, TestFixture)
{
    // arrange
    const Value rhs(static_cast<uint64_t>(123));

    stringstream ss;

    // act
    ss << rhs;

    // assert
    BOOST_CHECK_EQUAL("123", ss.str());
}

BOOST_FIXTURE_TEST_CASE(test_Value_operator_putto_WithString_ShouldPass, TestFixture)
{
    // arrange
    const Value rhs("this-is-a-value");

    stringstream ss;

    // act
    ss << rhs;

    // assert
    BOOST_CHECK_EQUAL("this-is-a-value", ss.str());
}

}
