#include <boost/test/unit_test.hpp>

#include <mxtag/validate.hpp>

using std::string;

namespace
{

struct TestFixture
{
};

}

namespace mxtag
{

BOOST_FIXTURE_TEST_CASE(test_is_valid_equipment_name_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "valid_equipment-name_0123456789";

    // act
    const bool should_be_true = is_valid_equipment_name(
        SUT);

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_equipment_name_WithTooShortName_ShouldReturnFalse, TestFixture)
{
    // arrange
    const string SUT = "ne";

    // act
    const bool should_be_false = is_valid_equipment_name(
        SUT);

    // assert
    BOOST_CHECK(!should_be_false);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_equipment_name_WithTooLongName_ShouldReturnFalse, TestFixture)
{
    // arrange
    const string SUT =
        "this-is-a-very-long-equipment-name-that-should-fail"
        "0123456789-01234567890-123456789-0123456789-0123456789"
        "0123456789-01234567890-123456789-0123456789-0123456789"
        "0123456789-01234567890-123456789-0123456789-0123456789"
        "0123456789-01234567890-123456789-0123456789-0123456789"
        "0123456789-01234567890-123456789-0123456789-0123456789"
        ;

    // act
    const bool should_be_false = is_valid_equipment_name(
        SUT);

    // assert
    BOOST_CHECK(!should_be_false);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_equipment_name_WithIllegalChar_ShouldReturnFalse, TestFixture)
{
    // arrange
    const string SUT = "illegal.char";

    // act
    const bool should_be_false = is_valid_equipment_name(
        SUT);

    // assert
    BOOST_CHECK(!should_be_false);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_tag_name_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "valid_tag-name_0123456789";

    // act
    const bool should_be_true = is_valid_tag_name(
        SUT);

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_tag_name_WithIllegalChar_ShouldReturnFalse, TestFixture)
{
    // arrange
    const string SUT = "illegal.tag-name";

    // act
    const bool should_be_false = is_valid_tag_name(
        SUT);

    // assert
    BOOST_CHECK(!should_be_false);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_unit_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "valid_unit_0123456789";

    // act
    const bool should_be_true = is_valid_unit(
        SUT);

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_unit_WithEmptyString_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "";

    // act
    const bool should_be_true = is_valid_unit(
        SUT);

    // assert
    BOOST_CHECK(should_be_true);
}

BOOST_FIXTURE_TEST_CASE(test_is_valid_unit_WithIllegalChar_ShouldReturnFalse, TestFixture)
{
    // arrange
    const string SUT = "illegal.unit";

    // act
    const bool should_be_false = is_valid_unit(
        SUT);

    // assert
    BOOST_CHECK(!should_be_false);
}

}
