#include <iostream>
#include <string>
#include <utility>
#include <vector>

#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/timer.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

#include <mxtag/Tag.hpp>
#include <mxtag/tag_form.hpp>
#include <mxtag/Value.hpp>
#include <mxtag/log/JsonSerializer.hpp>
#include <mxtag/log/ProtobufSerializer.hpp>
#include <mxtag/log/Serializer.hpp>

using boost::lexical_cast;
using boost::shared_ptr;
using boost::timer;
using boost::posix_time::time_from_string;

using mxtag::form_iso8601_time;
using mxtag::Tag;
using mxtag::Value;
using mxtag::log::JsonSerializer;
using mxtag::log::ProtobufSerializer;
using mxtag::log::Serializer;

using std::cerr;
using std::endl;
using std::string;
using std::vector;

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 3)
    {
        cerr << "usage: <format> <loop-count>" << endl;
        return 1;
    }

    shared_ptr<Serializer> serializer;
    const string format = lexical_cast<string>(argv[1]);
    if (format == "json")
        serializer.reset(new JsonSerializer);
    else if (format == "protobuf")
        serializer.reset(new ProtobufSerializer);
    else
    {
        cerr << "<format> error." << endl;
        return 1;
    }

    const int loop_count = lexical_cast<int>(argv[2]);

    const Tag t_int(
        "Equipment_int",
        "Tag_int",
        time_from_string("1970-01-01 12:34:56.789"),
        Value(static_cast<int64_t>(-123456)),
        "unit_int");
    const vector<uint8_t> internal_int = serializer->from_tag(t_int);

    timer t;

    for(int ax = 0; ax < loop_count; ++ax)
    {
        const Tag t = serializer->to_tag(internal_int);
    }

    cerr << t.elapsed() << endl;

    return 0;
}
