#include <iostream>
#include <map>
#include <set>
#include <string>

#include <boost/logic/tribool.hpp>
#include <boost/make_shared.hpp>
#include <boost/shared_ptr.hpp>

#include <mxtag/mqtt/TagSubscriber.hpp>
#include <mxtag/tag_form.hpp>

#include <mxutil/log.hpp>
#include <mxutil/log/Log.hpp>
#include <mxutil/log/StderrLogger.hpp>

#include <stag/Option.hpp>

using boost::logic::tribool;
using boost::make_shared;
using boost::shared_ptr;

using mxtag::form_iso8601_time;
using mxtag::mqtt::TagSubscriber;
using mxtag::Tag;

using mxutil::log::Log;
using mxutil::log::Logger;
using mxutil::log::StderrLogger;

using stag::Option;

using std::cout;
using std::endl;
using std::map;
using std::set;
using std::string;

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[]);

void
    process_tag(const Tag &v);

}

int
    main(
        int argc,
        const char *argv[])
{
    const Option opt = get_option(argc, argv);

    shared_ptr<Logger> logger(new StderrLogger());
    logger->set_level(Logger::WARN);
    if (opt.verbose)
        logger->set_level(Logger::DBUG);

    logger->enable_line_of_file();
    Log::get().set_logger(logger);

    map<string, set<string> > eq_tag_list;
    eq_tag_list["+"].insert("+");
    shared_ptr<TagSubscriber> tag_subscriber = make_shared<TagSubscriber>(
        opt.mqtt_broker_host,
        opt.mqtt_broker_port,
        "subscriber",
        true,
        eq_tag_list,
        &process_tag);

    while(true)
    {
        tag_subscriber->loop();
        usleep(100 * 1000);
    }

    return 0;
}

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[])
{
    Option opt;
    const tribool result = opt.parse(argc, argv);
    if (result)
    {
        return opt;
    }
    else if(!result)
    {
        exit(1);
    }
    else
    {
        exit(0);
    }
}


void
    process_tag(const Tag &t)
{
    cout
        << "equ=" << t.equipment()
        << ",tag=" << t.tag()
        << ",value=" << t.value();

    if (!t.unit().empty())
        cout << ",unit=" << t.unit();

    cout
        << ",at=" << form_iso8601_time(t.at())
        << endl;
}

}
