#include <stag/Option.hpp>

#include <iostream>

#include <boost/program_options.hpp>

#include <config.h>

using boost::logic::indeterminate;
using boost::logic::tribool;
using boost::program_options::command_line_style::default_style;
using boost::program_options::command_line_style::allow_guessing;
using boost::program_options::options_description;
using boost::program_options::value;
using boost::program_options::variables_map;

using std::cout;
using std::endl;
using std::string;

namespace stag
{

Option::Option()
    : mqtt_broker_host("127.0.0.1")
    , mqtt_broker_port(1883)
    , verbose(false)
{
}

tribool
    Option::parse(
        int argc,
        const char *argv[])
{
    options_description desc("subscribe-tag v" VERSION);
    desc.add_options()
        ("mqtt-broker-host", value<string>(), "Change MQTT broker address.")
        ("mqtt-broker-port", value<int>(), "Change MQTT broker port.")
        ("verbose", "Print log messages to stderr.")
        ("help", "print this manual.");

    variables_map vm;
    try
    {
        store(
            parse_command_line(argc, argv, desc, default_style & (~allow_guessing)),
            vm
        );
    }
    catch(boost::program_options::error &e)
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("help"))
    {
        cout << desc << endl;
        return indeterminate;
    }

    if (vm.count("mqtt-broker-host"))
        this->mqtt_broker_host = vm["mqtt-broker-host"].as<string>();

    if (vm.count("mqtt-broker-port"))
        this->mqtt_broker_port = vm["mqtt-broker-port"].as<int>();

    if (vm.count("verbose"))
        this->verbose = true;

    return true;
}

}
