#include <iostream>

#include <boost/lexical_cast.hpp>
#include <boost/range.hpp>

#include <mxtag/mqtt/TagSubscriber.hpp>

using boost::lexical_cast;
using boost::size;

using mxtag::mqtt::TagSubscriber;

using mxtag::Tag;

using std::cerr;
using std::cout;
using std::endl;
using std::map;
using std::set;
using std::string;
using std::vector;

void
    process_tag(const Tag &v)
{
    cout
        << "equ = " << v.equipment()
        << ", tag = " << v.tag()
        << ", unit = " << v.unit()
        << ", value = " << v.value()
        << ", at = " << v.at()
        << endl;
}

int
    main(
        int argc,
        const char *argv[])
{
    if (argc != 3)
    {
        cerr << "usage: <host> <port>" << endl;
        return 1;
    }

    const string host = argv[1];
    const int port = lexical_cast<int>(argv[2]);

    map<string, set<string> > eq_tag_list;
    const string TAG_LIST[] = {
        "di0",
        "di1",
        "di2",
        "ai0"
    };
    eq_tag_list["SYS-FRONT-DOOR"] = set<string>(TAG_LIST, TAG_LIST + size(TAG_LIST));

    TagSubscriber tagsub(
        host,
        port,
        "subscriber",
        true,
        eq_tag_list,
        &process_tag);

    for(int ax = 0; ax < 600; ++ax)
    {
        usleep(100 * 1000);
        tagsub.loop();
    }

    return 0;
}
