#include <mxutil/PidFile.hpp>

#include <sys/file.h>

#include <boost/lexical_cast.hpp>

using boost::lexical_cast;
using boost::shared_ptr;

using std::string;

namespace mxutil
{

shared_ptr<PidFile>
    PidFile::get(const string &path)
{
    return shared_ptr<PidFile>(new PidFile(path));
}

struct PidFile::Impl
{
    Impl()
        : fd(0)
    {
    }

    int     fd;
};

PidFile::PidFile(const string &path)
    : pimpl(new Impl())
{
    int fd = open(path.c_str(), O_RDWR | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
    if (-1 == fd)
    {
        BOOST_THROW_EXCEPTION(LockError());
    }

    if (0 != flock(fd, LOCK_EX | LOCK_NB))
    {
        close(fd);

        BOOST_THROW_EXCEPTION(LockError());
    }

    // write pid
    const string s = lexical_cast<string>(getpid());
    if (static_cast<int>(s.size()) != write(fd, s.c_str(), s.size()))
    {
        close(fd);

        BOOST_THROW_EXCEPTION(LockError());
    }

    pimpl->fd = fd;
}

PidFile::~PidFile()
{
    if (pimpl->fd == 0)
        return;

    if (-1 == lseek(pimpl->fd, SEEK_SET, 0))
    {
    }

    if (0 != ftruncate(pimpl->fd, 0))
    {
    }

    if (0 != flock(pimpl->fd, LOCK_UN))
    {
    }

    close(pimpl->fd);
    pimpl->fd = 0;
}

}

