#include <mxutil/log/Log.hpp>

#include <sstream>

#include <boost/make_shared.hpp>
#include <boost/shared_ptr.hpp>

#include <mxutil/log/Logger.hpp>
#include <mxutil/log/StderrLogger.hpp>

using boost::make_shared;
using boost::shared_ptr;

using mxutil::log::Logger;

using std::string;
using std::stringstream;

namespace mxutil
{
namespace log
{

shared_ptr<Log> Log::instance;

Log &
    Log::get()
{
    if (!instance)
        instance.reset(new Log());

    return *instance;
}

struct Log::Impl
{
    Impl()
    {
    }

    boost::shared_ptr<mxutil::log::Logger> logger;
};

Log::Log()
    : pimpl(new Impl())
{
}

Log::~Log()
{
}

void
    Log::set_logger(
        shared_ptr<Logger> logger)
{
    pimpl->logger = logger;
}

void
    Log::log(
        Logger::level_t lvl,
        const char *file,
        int line,
        const string &msg)
{
    if (!pimpl->logger)
    {
        pimpl->logger = make_shared<StderrLogger>();
        pimpl->logger->set_level(Logger::DBUG);
    }

    pimpl->logger->log(lvl, file, line, msg);
}

}
}
