#include <mxutil/log/Logger.hpp>

#include <sstream>

using std::string;
using std::stringstream;

namespace mxutil
{
namespace log
{

struct Logger::Impl
{
    Impl()
        : level(WARN)
        , enable_line_of_file(false)
    {
    }

    level_t level;

    bool    enable_line_of_file;
};

Logger::Logger()
    : pimpl(new Impl())
{
}

Logger::~Logger()
{
}

void
    Logger::enable_line_of_file(bool enable)
{
    pimpl->enable_line_of_file = enable;
}

void
    Logger::set_level(level_t lvl)
{
    BOOST_ASSERT(lvl >= DBUG && lvl <= EROR);

    pimpl->level = lvl;

    this->do_set_level(lvl);
}

void
    Logger::log(
        level_t lvl,
        const char *file,
        int line,
        const string &msg)
{
    BOOST_ASSERT(lvl >= DBUG && lvl <= EROR);

    if (lvl < pimpl->level)
        return;

    stringstream ss;
    if (pimpl->enable_line_of_file)
        ss << file << '(' << line << "): ";

    ss << msg;

    this->do_log(lvl, ss.str());
}

}
}
