#include <mxutil/log/SyslogLogger.hpp>

#include <syslog.h>

#include <boost/range.hpp>

using boost::size;

using mxutil::log::Logger;

using std::string;

namespace
{

const int LOG_LEVEL[] = {
    LOG_DEBUG,
    LOG_INFO,
    LOG_WARNING,
    LOG_ERR
};

}

namespace mxutil
{
namespace log
{

struct SyslogLogger::Impl
{
    Impl(const string &name_)
        : name(name_)
    {
    }

    const string    name;
};

SyslogLogger::SyslogLogger(const string &name)
    : pimpl(new Impl(name))
{
    openlog(pimpl->name.c_str(), LOG_PID, LOG_USER);
}

SyslogLogger::~SyslogLogger()
{
    BOOST_STATIC_ASSERT(
        DBUG == 0 &&
        INFO == 1 &&
        WARN == 2 &&
        EROR == 3);
}

void
    SyslogLogger::do_log(
        level_t level,
        const string &str)
{
    BOOST_ASSERT(level < size(LOG_LEVEL));

    syslog(LOG_LEVEL[level], "%s", str.c_str());
}

}
}
