#include <mxutil/sched/Scheduler.hpp>

#include <boost/chrono/chrono.hpp>

#include <mxutil/log.hpp>

using boost::chrono::nanoseconds;
using boost::chrono::steady_clock;
using boost::posix_time::microsec_clock;
using boost::posix_time::not_a_date_time;
using boost::posix_time::ptime;
using boost::posix_time::time_duration;

namespace mxutil
{
namespace sched
{

struct Scheduler::Impl
{
    Impl(const FireCallback &fire_callback_)
        : fire_callback(fire_callback_)
        , last_check(not_a_date_time)
    {
    }

    const FireCallback  fire_callback;

    // system clock change detection
    ptime                       last_check;
    steady_clock::time_point    last_check_timepoint;
};

Scheduler::Scheduler(const FireCallback &fire_callback)
    : pimpl(new Impl(fire_callback))
{
}

Scheduler::~Scheduler()
{
}

void
    Scheduler::check_and_fire()
{
    const ptime now = microsec_clock::universal_time();
    const steady_clock::time_point now_timepoint = steady_clock::now();

    if (pimpl->last_check == not_a_date_time)
    {
        this->reset_timeout(now);
    }
    else
    {
        // detect whether system has changed since last invocation

        const time_duration diff = now - pimpl->last_check;
        const nanoseconds diff_timepoint = now_timepoint - pimpl->last_check_timepoint;

        if (diff.is_negative() ||
            abs(diff.total_nanoseconds() - diff_timepoint.count()) > 1000000000ll) // difference > 1 second
        {
            MXLOG_WARN("clock skew detected, recalculate next fire time");

            this->reset_timeout(now);
        }
    }

    pimpl->last_check = now;
    pimpl->last_check_timepoint = now_timepoint;

    if (this->is_timeout(now))
    {
        this->reset_timeout(now);

        pimpl->fire_callback();
    }
}

}
}
