#include <boost/test/unit_test.hpp>

#include <mxutil/PidFile.hpp>

namespace mxutil
{

using boost::shared_ptr;

using std::string;

namespace
{

struct TestFixture
{
    TestFixture()
        : pidfile_path("/dev/shm/test_pid")
    {
    }

    virtual ~TestFixture()
    {
        if (0 != unlink(this->pidfile_path.c_str()))
        {
            // do nothing
        }
    }

    const string    pidfile_path;
};

}

BOOST_FIXTURE_TEST_CASE(test_PidFile_get_ShouldPass, TestFixture)
{
    // arrange

    // act
    shared_ptr<PidFile> pidfile = PidFile::get(pidfile_path);

    // assert
    // nothing to assert
}

BOOST_FIXTURE_TEST_CASE(test_PidFile_get_WhenAlreadyLocked_ShouldThrowLockError, TestFixture)
{
    // arrange
    const shared_ptr<PidFile> pidfile = PidFile::get(pidfile_path);

    // act & assert
    BOOST_CHECK_THROW(PidFile::get(pidfile_path), LockError);
}

}
