# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.

# Checks for existence of coverage tools:
#  * gcov
#  * lcov
#  * genhtml
#  * gcovr
# 
# Sets ac_cv_check_gcov to yes if tooling is present
# and reports the executables to the variables LCOV, GCOVR and GENHTML.
AC_DEFUN([AC_TDD_GCOV],
[
# check whether --enable-gcov is given ########################################
  AC_ARG_ENABLE(gcov,
  AS_HELP_STRING([--enable-gcov],
  		 [enable coverage testing with gcov]),
  [use_gcov=yes], [use_gcov=no])

  AM_CONDITIONAL(HAVE_GCOV, test "x$use_gcov" = "xyes")
###############################################################################

  if test "x$use_gcov" = "xyes"; then

# check utility existence
  AC_CHECK_PROG(GCOV, gcov, gcov)
  if test "x$GCOV" = "x"; then
    AC_MSG_ERROR([gcov not found])
  fi

  AC_CHECK_PROG(LCOV, lcov, lcov)
  if test "x$LCOV" = "x"; then
    AC_MSG_ERROR([lcov not found])
  fi

  AC_CHECK_PROG(GENHTML, genhtml, genhtml)
  if test "x$GENHTML" = "x"; then
    AC_MSG_ERROR([genhtml not found])
  fi

  AC_CHECK_PROG(SED, sed, sed)
  if test "x$SED" = "x"; then
    AC_MSG_ERROR([sed not found])
  fi
###############################################################################

# remove all optimization flags from CFLAGS ###################################
  changequote({,})
  CFLAGS=`echo "$CFLAGS" | $SED -e 's/-O[0-9]*/-O0/g'`
  CXXFLAGS=`echo "$CXXFLAGS" | $SED -e 's/-O[0-9]*/-O0/g'`
  changequote([,])
###############################################################################

# add the special gcc flags ###################################################
  COVERAGE_CFLAGS="-g -fprofile-arcs -ftest-coverage"
  COVERAGE_CXXFLAGS="-g -fprofile-arcs -ftest-coverage"
  COVERAGE_LDFLAGS="-fprofile-arcs"
###############################################################################

  fi # test "x$use_gcov" = "xyes"

]) # AC_TDD_GCOV
