#ifndef _MXLOGARCHIVER_PROFILE_CONFIGURATION_HPP_
#define _MXLOGARCHIVER_PROFILE_CONFIGURATION_HPP_

#include <string>
#include <vector>

#include <boost/cstdint.hpp>

#include <mxlogarchiver/error.hpp>

namespace mxlogarchiver
{

struct ProfileConfiguration
{
    static const ProfileConfiguration
        parse(const std::string &content); // throws mxlogarchiver::ParseError

    struct Archive
    {
        std::string                     directory;
        boost::shared_ptr<uint64_t>     directory_size_limit_byte;
        unsigned int                    threshold_byte_per_file;
        boost::shared_ptr<unsigned int> period_sec;
        boost::shared_ptr<unsigned int> archives_to_keep;

        Archive(
            const std::string &directory_,
            boost::shared_ptr<uint64_t> directory_size_limit_byte_,
            unsigned int threshold_byte_per_file_,
            boost::shared_ptr<unsigned int> period_sec_,
            boost::shared_ptr<unsigned int> archives_to_keep_)
            : directory(directory_)
            , directory_size_limit_byte(directory_size_limit_byte_)
            , threshold_byte_per_file(threshold_byte_per_file_)
            , period_sec(period_sec_)
            , archives_to_keep(archives_to_keep_)
        {
        }
    };

    Archive archive;

    struct Equipment
    {
        struct Tag
        {
            std::string tag;
            bool        log_on_change;

            Tag(
                const std::string &tag_,
                bool log_on_change_)
                : tag(tag_)
                , log_on_change(log_on_change_)
            {
            }
        };

        std::string equipment;
        std::vector<Tag> tag_list;

        Equipment(
            const std::string &equipment_,
            const std::vector<Tag> &tag_list_)
            : equipment(equipment_)
            , tag_list(tag_list_)
        {
        }
    };

    std::vector<Equipment> equipment_list;

    ProfileConfiguration(
        const Archive & archive_,
        const std::vector<Equipment> &equipment_list_)
        : archive(archive_)
        , equipment_list(equipment_list_)
    {
    }
};

}

#endif
