#include <mxlogarchiver/App.hpp>

#include <boost/assert.hpp>
#include <boost/foreach.hpp>

using boost::shared_ptr;

using mxtag::mqtt::TagSubscriber;

using mxutil::sched::Scheduler;

using std::vector;

namespace mxlogarchiver
{

struct App::Impl
{
    Impl(
        shared_ptr<TagSubscriber> tag_subscriber_,
        const vector<shared_ptr<Scheduler> > &scheduler_list_)
        : tag_subscriber(tag_subscriber_)
        , scheduler_list(scheduler_list_)
    {
    }

    const shared_ptr<TagSubscriber>         tag_subscriber;
    const vector<shared_ptr<Scheduler> >    scheduler_list;
};

App::App(
    shared_ptr<TagSubscriber> subscriber,
    const vector<shared_ptr<Scheduler> > &scheduler_list)
    : pimpl(new Impl(
        subscriber,
        scheduler_list))
{
    BOOST_ASSERT(pimpl->tag_subscriber);
}

App::~App()
{
}

void
    App::start()
{
}

void
    App::loop()
{
    pimpl->tag_subscriber->loop();

    BOOST_FOREACH(shared_ptr<Scheduler> s, pimpl->scheduler_list)
    {
        BOOST_ASSERT(s);
        s->check_and_fire();
    }
}

}
