#include <mxlogarchiver/LogOnChangeDecorator.hpp>

#include <boost/foreach.hpp>
#include <boost/make_shared.hpp>
#include <boost/tuple/tuple.hpp>
#include <boost/tuple/tuple_comparison.hpp>

using boost::make_shared;
using boost::make_tuple;
using boost::shared_ptr;
using boost::tuple;

using mxtag::Tag;
using mxtag::Value;

using std::map;
using std::set;
using std::string;

typedef
    tuple<string, string>
    EquTagType;

namespace mxlogarchiver
{

struct LogOnChangeDecorator::Impl
{
    Impl()
    {
    }

    map<EquTagType, shared_ptr<Value> > tag_value_cache;
};

LogOnChangeDecorator::LogOnChangeDecorator(
    const map<string, set<string> > &tag_list,
    shared_ptr<LogStream> inner)
    : LogStreamDecorator(inner)
    , pimpl(new Impl())
{
    // initialize tag_value_cache
    for(map<string, set<string> >::const_iterator
        iter = tag_list.begin();
        iter != tag_list.end();
        ++iter)
    {
        const string &e = iter->first;
        BOOST_FOREACH(const string &t, iter->second)
        {
            pimpl->tag_value_cache.insert(make_pair(
                make_tuple(e, t),
                shared_ptr<Value>()
            ));
        }
    }
}

LogOnChangeDecorator::~LogOnChangeDecorator()
{
}

void
    LogOnChangeDecorator::do_write(const Tag &t)
{
    do
    {
        const map<EquTagType, shared_ptr<Value> >::iterator iter =
            pimpl->tag_value_cache.find(make_tuple(t.equipment(), t.tag()));

        if (iter == pimpl->tag_value_cache.end())
            break;

        if (!iter->second)
        {
            // received the first time
            iter->second = make_shared<Value>(t.value());
            break;
        }

        if (*iter->second != t.value())
        {
            // value changed
            *iter->second = t.value();
            break;
        }

        // duplicated value, filter out
        return;
    }
    while(false);

    this->inner().write(t);
}

}
