#include <mxlogarchiver/LogStreamDecorator.hpp>

using boost::filesystem::path;
using boost::shared_ptr;

using mxtag::Tag;

namespace mxlogarchiver
{

struct LogStreamDecorator::Impl
{
    Impl(shared_ptr<LogStream> inner_)
        : inner(inner_)
    {
    }

    const shared_ptr<LogStream> inner;
};

LogStreamDecorator::LogStreamDecorator(
    shared_ptr<LogStream> inner)
    : pimpl(new Impl(inner))
{
    BOOST_ASSERT(pimpl->inner);
}

LogStreamDecorator::~LogStreamDecorator()
{
}

const path &
    LogStreamDecorator::do_path() const
{
    return pimpl->inner->path();
}

size_t
    LogStreamDecorator::do_bytes_written() const
{
    return pimpl->inner->bytes_written();
}

void
    LogStreamDecorator::do_write(const Tag &t)
{
    pimpl->inner->write(t);
}

LogStream &
    LogStreamDecorator::inner()
{
    return *pimpl->inner;
}

const LogStream &
    LogStreamDecorator::inner() const
{
    return *pimpl->inner;
}

}
