#include <mxlogarchiver/LogStreamImpl.hpp>

#include <fstream>

#include <boost/make_shared.hpp>

#include <mxutil/log.hpp>

#include <mxtag/log/EnvelopedOStream.hpp>
#include <mxtag/log/ProtobufSerializer.hpp>

using boost::make_shared;
using boost::shared_ptr;

using mxtag::Tag;
using mxtag::log::EnvelopedOStream;
using mxtag::log::ProtobufSerializer;
using mxtag::log::Serializer;

using std::ios_base;
using std::map;
using std::ofstream;
using std::ostream;
using std::set;
using std::string;
using std::vector;

typedef
    boost::filesystem::path
    fspath;

namespace
{
    const size_t    OUTPUT_BUFFER_SIZE = 4096;
}

namespace mxlogarchiver
{

struct LogStreamImpl::Impl
{
    Impl(
        const fspath &p_,
        const map<string, set<string> > &tag_list_)
        : serializer(new ProtobufSerializer())
        , p(p_)
        , header(this->serializer->from_tag_list(tag_list_))
    {
    }

    const shared_ptr<Serializer>  serializer;

    const fspath            p;
    const vector<uint8_t>   header;

    shared_ptr<ostream>             os;
    shared_ptr<EnvelopedOStream>    eos;
};

LogStreamImpl::LogStreamImpl(
    const fspath &p,
    const map<string, set<string> > &tag_list)
    : pimpl(new Impl(
        p,
        tag_list))
{
}

LogStreamImpl::~LogStreamImpl()
{
}

const fspath &
    LogStreamImpl::do_path() const
{
    return pimpl->p;
}

size_t
    LogStreamImpl::do_bytes_written() const
{
    if (pimpl->eos)
        return pimpl->eos->bytes_written();
    else
        return 0;
}

void
    LogStreamImpl::do_write(const Tag &t)
{
    // create eos if not exist
    if (!pimpl->eos)
    {
        if (!is_directory(pimpl->p.parent_path()))
        {
            boost::system::error_code ec;
            create_directories(pimpl->p.parent_path(), ec);
            if (ec)
            {
                MXLOG_EROR("create directory failed: " + pimpl->p.parent_path().string());
                BOOST_THROW_EXCEPTION(SystemError());
            }
        }

        pimpl->os = make_shared<ofstream>(pimpl->p.string().c_str(), ios_base::binary);
        if (!(*pimpl->os))
        {
            MXLOG_EROR("open file failed: " + pimpl->p.string());
            pimpl->os.reset();
            BOOST_THROW_EXCEPTION(SystemError());
        }

        pimpl->eos.reset(new EnvelopedOStream(
            *pimpl->os,
            OUTPUT_BUFFER_SIZE));

        pimpl->eos->write(pimpl->header);
    }

    try
    {
        pimpl->eos->write(pimpl->serializer->from_tag(t));
    }
    catch(mxtag::SystemError &e)
    {
        MXLOG_EROR(diagnostic_information(e));

        BOOST_THROW_EXCEPTION(SystemError());
    }
}

}
