#include <mxlogarchiver/TagFilteringDecorator.hpp>

#include <boost/foreach.hpp>
#include <boost/tuple/tuple.hpp>
#include <boost/tuple/tuple_comparison.hpp>

using boost::make_tuple;
using boost::shared_ptr;
using boost::tuple;

using mxtag::Tag;

using std::map;
using std::set;
using std::string;

typedef
    tuple<string, string>
    EquTagType;

namespace mxlogarchiver
{

struct TagFilteringDecorator::Impl
{
    Impl()
    {
    }

    set<EquTagType>     tag_list;
};

TagFilteringDecorator::TagFilteringDecorator(
    const map<string, set<string> > &tag_list,
    shared_ptr<LogStream> inner)
    : LogStreamDecorator(inner)
    , pimpl(new Impl())
{
    // initialize tag_list
    for(map<string, set<string> >::const_iterator
        iter = tag_list.begin();
        iter != tag_list.end();
        ++iter)
    {
        const string &e = iter->first;
        BOOST_FOREACH(const string &t, iter->second)
        {
            pimpl->tag_list.insert(make_tuple(e, t));
        }
    }
}

TagFilteringDecorator::~TagFilteringDecorator()
{
}

void
    TagFilteringDecorator::do_write(const Tag &t)
{
    const set<EquTagType>::const_iterator iter = pimpl->tag_list.find(make_tuple(t.equipment(), t.tag()));
    if (iter == pimpl->tag_list.end()) // not found
        return;

    this->inner().write(t);
}

}
