#include <string>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/make_shared.hpp>

#include <mxtag/Tag.hpp>

#include <mxlogarchiver/ArchiveRemoveWorker.hpp>
#include <mxlogarchiver/LogArchiver.hpp>

using boost::filesystem::path;
using boost::make_shared;
using boost::posix_time::microsec_clock;
using boost::shared_ptr;

using mxlogarchiver::ArchiveDirectory;
using mxlogarchiver::ArchiveRemoveWorker;
using mxlogarchiver::LogArchiver;
using mxlogarchiver::LogStreamFactory;

using mxtag::Tag;
using mxtag::Value;

using std::string;

int
    main(
        int,
        char *[])
{
    const string profile_name = "mxcs";
    const string directory = "/tmp/logs/mxcs";
    const path temporary_directory = "/tmp/mxcs";
    const unsigned int threshold_byte = 1024;
    const unsigned int archive_count = 2;
    const unsigned int directory_size_limit_byte = 1024 * 1024 * 1024;

    const shared_ptr<ArchiveDirectory> archive_directory = make_shared<ArchiveDirectory>(
        directory);

    archive_directory->set_directory_size_limit_byte(directory_size_limit_byte);
    archive_directory->set_file_count_limit(archive_count);

    const shared_ptr<LogStreamFactory> factory = make_shared<LogStreamFactory>(
        temporary_directory / profile_name);

    factory->add_tag("My_ioLogik-E2242", "di0", false);
    factory->add_tag("My_ioLogik-E2242", "ai0", true);

    const shared_ptr<ArchiveRemoveWorker> worker = make_shared<ArchiveRemoveWorker>();

    LogArchiver archiver(
        factory,
        archive_directory,
        threshold_byte,
        worker);

    {
        Tag t(
            "My ioLogik-E2242",
            "di0",
            microsec_clock::universal_time(),
            Value(static_cast<int64_t>(1)));

        archiver.write(t);
    }

    {
        Tag t(
            "My other ioLogik-E2242",
            "ai0",
            microsec_clock::universal_time(),
            Value(1.12),
            "V");

        archiver.write(t);
    }

    return 0;
}
