#ifndef _MXLOGCONVERTER_FORMATTER_HPP_
#define _MXLOGCONVERTER_FORMATTER_HPP_

#include <map>
#include <ostream>
#include <set>
#include <string>
#include <vector>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxlogconverter/Extractor.hpp>

#include <mxtag/Tag.hpp>
#include <mxtag/Value.hpp>

namespace mxlogconverter
{

class Formatter
    : public mxlogconverter::Extractor::Handler
{
public:
    struct Tag
    {
        Tag(
            const std::string &tag_,
            const mxtag::Value &value_,
            const std::string &unit_ = "")
            : tag(tag_)
            , value(value_)
            , unit(unit_)
        {
        }

        std::string     tag;
        mxtag::Value    value;
        std::string     unit;
    };

protected:
    Formatter(
        std::ostream &,
        int window_size);

public:
    virtual ~Formatter();

    virtual void
        on_equ_tag_list(const std::map<std::string, std::set<std::string> > &);

    virtual void
        on_tag(const mxtag::Tag &);

    virtual void
        on_eof();

protected:
    const std::map<std::string, std::set<std::string> > &
        equ_tag_list() const;

private:
    virtual void
        print_header(std::ostream &){}

    virtual void
        print_record(
            std::ostream &,
            const boost::posix_time::ptime &,
            const std::map<std::string, std::vector<Tag> > &){}  // (equipment, vector<Tag>)

    virtual void
        print_footer(std::ostream &){}

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
