#include <mxlogconverter/App.hpp>

#include <boost/iostreams/filtering_stream.hpp>
#include <boost/iostreams/filter/gzip.hpp>
#include <boost/shared_ptr.hpp>

#include <mxutil/error.hpp>
#include <mxutil/log.hpp>

using boost::iostreams::filtering_istream;
using boost::iostreams::gzip_decompressor;
using boost::shared_ptr;

using mxlogconverter::Extractor;

using mxutil::ArgumentError;

using std::istream;
using std::string;

namespace mxlogconverter
{

struct App::Impl
{
    Impl(
        istream &input_stream_,
        shared_ptr<Extractor::Handler> handler_)
        : input_stream(input_stream_)
        , handler(handler_)
    {
    }

    istream &                               input_stream;
    const shared_ptr<Extractor::Handler>    handler;
};

App::App(
    istream &input_stream,
    shared_ptr<Extractor::Handler> handler)
    : pimpl(new Impl(
        input_stream,
        handler))
{
}

App::~App()
{
}

void
    App::run()
{
    filtering_istream in;
    in.push(gzip_decompressor());
    in.push(pimpl->input_stream);

    Extractor ext;
    ext.parse(in, *pimpl->handler);
}

}
