#include <mxlogconverter/Extractor.hpp>

#include <vector>

#include <mxtag/log/EnvelopedIStream.hpp>
#include <mxtag/log/ProtobufSerializer.hpp>

#include <mxutil/log.hpp>

using boost::scoped_ptr;

using mxlogconverter::Extractor;

using mxtag::Eof;
using mxtag::Tag;
using mxtag::log::EnvelopedIStream;
using mxtag::log::ProtobufSerializer;
using mxtag::log::Serializer;

using std::istream;
using std::map;
using std::set;
using std::string;
using std::vector;

namespace mxlogconverter
{

struct Extractor::Impl
{
    Impl()
        : serializer(new ProtobufSerializer())
    {
    }

    scoped_ptr<Serializer>  serializer;
};

Extractor::Extractor()
    : pimpl(new Impl())
{
}

Extractor::~Extractor()
{
}

void
    Extractor::parse(
        istream &input_stream,
        Handler &handler)
{
    EnvelopedIStream eis(input_stream);
    vector<uint8_t> content;

    eis >> content;
    const map<string, set<string> > tag_list = pimpl->serializer->to_tag_list(content);
    handler.on_equ_tag_list(tag_list);

    while(true)
    {
        try
        {
            eis >> content;
            const Tag tag = pimpl->serializer->to_tag(content);
            handler.on_tag(tag);
        }
        catch(Eof)
        {
            break;
        }
    }

    handler.on_eof();
}

}
