#include <mxlogconverter/JsonFormatter.hpp>

#include <sstream>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#include <mxutil/log.hpp>

#include <mxtag/tag_form.hpp>

#include <mxlogconverter/error.hpp>
#include <mxlogconverter/Formatter.hpp>

using boost::lexical_cast;
using boost::posix_time::ptime;

using mxlogconverter::ArgumentError;
using mxlogconverter::Formatter;

using mxtag::form_iso8601_time;
using mxtag::Value;

using std::map;
using std::ostream;
using std::string;
using std::stringstream;
using std::vector;

namespace
{

const string
    string_from_value(const Value &v)
{
    if (v.is_string())
        return string("\"") + v.as_string() + '"';
    else if (v.is_int())
        return lexical_cast<string>(v.as_int());
    else if (v.is_uint())
        return lexical_cast<string>(v.as_uint());
    else if (v.is_float())
        return lexical_cast<string>(v.as_float());
    else
        BOOST_THROW_EXCEPTION(ArgumentError());
}

}

namespace mxlogconverter
{

struct JsonFormatter::Impl
{
    Impl()
        : is_first_record(true)
    {
    }

    bool is_first_record;
};

JsonFormatter::JsonFormatter(
    ostream &output_stream,
    int window_size)
    : Formatter(output_stream, window_size)
    , pimpl(new Impl())
{
}

JsonFormatter::~JsonFormatter()
{
}

void
    JsonFormatter::print_header(ostream &output_stream)
{
    output_stream << "[";
}

void
    JsonFormatter::print_record(
        ostream &output_stream,
        const ptime &at,
        const map<string, vector<Formatter::Tag> > &record_list)
{
    typedef map<string, vector<Formatter::Tag> > record_t;
    BOOST_FOREACH(const record_t::value_type &r, record_list)
    {
        const string &equipment = r.first;
        const vector<Formatter::Tag> &tag_list = r.second;

        stringstream ss;

        if (!pimpl->is_first_record)
            ss << ',';

        ss << '{'
            << "\"equ\":\"" << equipment << "\","
            << "\"at\":\"" << form_iso8601_time(at) << "\","
            << "\"tagList\":[";

        bool is_first_tag = true;
        int  output_count = 0;
        BOOST_FOREACH(const Formatter::Tag &t, tag_list)
        {
            string value;
            try
            {
                value = string_from_value(t.value);
            }
            catch(ArgumentError &e)
            {
                MXLOG_EROR(diagnostic_information(e));
                continue;
            }

            if (!is_first_tag)
                ss << ',';

            ss << '{'
                << "\"tag\":\"" << t.tag << "\",";
            if (!t.unit.empty())
                ss << "\"unit\":\"" << t.unit << "\",";
            ss << "\"value\":" << value
                << "}";

            is_first_tag = false;
            output_count++;
        }

        ss << ']'
            << "}";

        if (output_count)
            output_stream << ss.str();

        pimpl->is_first_record = false;
    }
}

void
    JsonFormatter::print_footer(ostream &output_stream)
{
    output_stream << "]";
}

}
