#include <mxlogconverter/Option.hpp>

#include <algorithm>

#include <boost/program_options.hpp>
#include <boost/range.hpp>

using boost::size;
using boost::logic::tribool;
using boost::logic::indeterminate;
using boost::program_options::options_description;
using boost::program_options::value;
using boost::program_options::variables_map;
using boost::program_options::command_line_style::default_style;
using boost::program_options::command_line_style::allow_guessing;

using std::cout;
using std::endl;
using std::find;
using std::string;
using std::vector;

namespace
{

const vector<string>
    get_format_list()
{
    string format_list[] = {"csv", "json", "xml"};
    return vector<string>(format_list, format_list + size(format_list));
}

}

namespace mxlogconverter
{

Option::Option()
    : dump(false)
    , window_size(128)
{
}

tribool
    Option::parse_args(
        int argc,
        const char *argv[])
{
    options_description desc("Allowed options");
    desc.add_options()
        ("dump", "Dump log in human readable format.")
        ("format", value<string>(), "Convert log to indicated format. (REQUIRED: csv, json, xml)")
        ("window-size", value<int>(), "Window Size. (Default: 128)")
        ("help", "Print this manual.");

    variables_map vm;
    try
    {
        store(
            parse_command_line(argc, argv, desc, default_style & (~allow_guessing)),
            vm
        );
    }
    catch(boost::program_options::error &e)
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("help"))
    {
        cout << desc << endl;
        return indeterminate;
    }

    if (vm.count("dump"))
    {
        this->dump = true;
    }

    if (vm.count("format"))
    {
        string format = vm["format"].as<string>();
        vector<string> format_list = get_format_list();
        if (find(format_list.begin(), format_list.end(), format) == format_list.end())
        {
            cout << desc << endl;
            return false;
        }
        this->format = format;
    }

    if (vm.count("window-size"))
        this->window_size = vm["window-size"].as<int>();

    if (vm.count("dump") + vm.count("format") != 1)
    {
        cout << desc << endl;
        return false;
    }

    return true;
}

}
