#include <boost/test/unit_test.hpp>

#include <map>
#include <set>
#include <sstream>

#include <boost/range.hpp>

#include <mxlogconverter/CsvFormatter.hpp>

#include <mxtag/Tag.hpp>
#include <mxtag/Value.hpp>

using boost::posix_time::time_from_string;
using boost::size;

using mxlogconverter::Extractor;
using mxlogconverter::CsvFormatter;

using mxtag::Tag;
using mxtag::Value;

using std::map;
using std::set;
using std::string;
using std::stringstream;
using std::vector;

namespace
{

class TestFixture
{
};

}

namespace mxlogconverter
{

BOOST_FIXTURE_TEST_CASE(test_CsvFormatter_on_equ_tag_list_ShouldReturnHeader, TestFixture)
{
    // arrange
    const string tags[] = {"ai0", "di0"};
    map<string, set<string> > equ_tag_list;
    equ_tag_list["My_ioLogik-E2242"] = set<string>(tags, tags + size(tags));

    stringstream ss;
    CsvFormatter csv_formatter(ss, 1);

    // act
    csv_formatter.on_equ_tag_list(equ_tag_list);

    // assert
    BOOST_CHECK_EQUAL(
        "at,\"ai0@My_ioLogik-E2242\",\"di0@My_ioLogik-E2242\"\n",
        ss.str()
    );
}

BOOST_FIXTURE_TEST_CASE(test_CsvFormatter_on_tag_ShouldReturnRecord, TestFixture)
{
    // arrange
    const string tags[] = {"ai0", "di0"};
    map<string, set<string> > equ_tag_list;
    equ_tag_list["My_ioLogik-E2242"] = set<string>(tags, tags + size(tags));

    stringstream ss;
    CsvFormatter csv_formatter(ss, 1);
    csv_formatter.on_equ_tag_list(equ_tag_list);
    ss.str("");
    ss.clear();

    const Tag tag(
        "My_ioLogik-E2242",
        "ai0",
        time_from_string("2015-08-31 12:01:05.123"),
        Value(1.23),
        "V");

    // act
    csv_formatter.on_tag(tag);
    csv_formatter.on_eof();

    // assert
    BOOST_CHECK_EQUAL(
        "\"2015-08-31T12:01:05.123Z\",\"1.23(V)\",\n",
        ss.str()
    );
}

BOOST_FIXTURE_TEST_CASE(test_CsvFormatter_on_tag_WithUInt64Value_ShouldReturnRecord, TestFixture)
{
    // arrange
    const string tags[] = {"ai0", "di0"};
    map<string, set<string> > equ_tag_list;
    equ_tag_list["My_ioLogik-E2242"] = set<string>(tags, tags + size(tags));

    stringstream ss;
    CsvFormatter csv_formatter(ss, 1);
    csv_formatter.on_equ_tag_list(equ_tag_list);
    ss.str("");
    ss.clear();

    const Tag tag(
        "My_ioLogik-E2242",
        "ai0",
        time_from_string("2015-08-31 12:01:05.123"),
        Value(static_cast<uint64_t>(0xFFFFFFFFFFFFFFFFull)),
        "V");

    // act
    csv_formatter.on_tag(tag);
    csv_formatter.on_eof();

    // assert
    BOOST_CHECK_EQUAL(
        "\"2015-08-31T12:01:05.123Z\",\"18446744073709551615(V)\",\n",
        ss.str()
    );
}

}
