#include <boost/test/unit_test.hpp>

#include <map>
#include <set>
#include <sstream>

#include <boost/range.hpp>

#include <mxlogconverter/JsonFormatter.hpp>

#include <mxtag/Tag.hpp>
#include <mxtag/Value.hpp>

using boost::posix_time::time_from_string;
using boost::size;

using mxlogconverter::Extractor;
using mxlogconverter::JsonFormatter;

using mxtag::Tag;
using mxtag::Value;

using std::map;
using std::set;
using std::string;
using std::stringstream;
using std::vector;

namespace
{

class TestFixture
{
};

}

namespace mxlogconverter
{

BOOST_FIXTURE_TEST_CASE(test_JsonFormatter_ShouldPass, TestFixture)
{
    // arrange
    const string tags[] = {"ai0", "di0"};
    map<string, set<string> > equ_tag_list;
    equ_tag_list["My_ioLogik-E2242"] = set<string>(tags, tags + size(tags));

    stringstream ss;
    JsonFormatter json_formatter(ss, 1);

    // act
    json_formatter.on_equ_tag_list(equ_tag_list);
    json_formatter.on_tag(Tag(
        "My_ioLogik-E2242",
        "ai0",
        time_from_string("2015-08-31 12:01:05.123"),
        Value(1.23),
        "V"));
    json_formatter.on_eof();

    // assert
    BOOST_CHECK_EQUAL(
        "["
            "{"
                "\"equ\":\"My_ioLogik-E2242\","
                "\"at\":\"2015-08-31T12:01:05.123Z\","
                "\"tagList\":["
                    "{"
                        "\"tag\":\"ai0\","
                        "\"unit\":\"V\","
                        "\"value\":1.23"
                    "}"
                "]"
            "}"
        "]",
        ss.str());
}

BOOST_FIXTURE_TEST_CASE(test_JsonFormatter_WithSameAt_ShouldPass, TestFixture)
{
    // arrange
    const string tags[] = {"ai0", "di0"};
    map<string, set<string> > equ_tag_list;
    equ_tag_list["My_ioLogik-E2242"] = set<string>(tags, tags + size(tags));
    equ_tag_list["My_ioLogik-E1260"] = set<string>(tags, tags + size(tags));

    stringstream ss;
    JsonFormatter json_formatter(ss, 10);

    // act
    json_formatter.on_equ_tag_list(equ_tag_list);
    json_formatter.on_tag(Tag(
        "My_ioLogik-E2242",
        "ai0",
        time_from_string("2015-08-31 12:01:05.123"),
        Value(1.23),
        "V"));
    json_formatter.on_tag(Tag(
        "My_ioLogik-E1260",
        "ai0",
        time_from_string("2015-08-31 12:01:05.123"),
        Value(1.23),
        "V"));
    json_formatter.on_eof();

    // assert
    BOOST_CHECK_EQUAL(
        "["
            "{"
                "\"equ\":\"My_ioLogik-E1260\","
                "\"at\":\"2015-08-31T12:01:05.123Z\","
                "\"tagList\":["
                    "{"
                        "\"tag\":\"ai0\","
                        "\"unit\":\"V\","
                        "\"value\":1.23"
                    "}"
                "]"
            "},"
            "{"
                "\"equ\":\"My_ioLogik-E2242\","
                "\"at\":\"2015-08-31T12:01:05.123Z\","
                "\"tagList\":["
                    "{"
                        "\"tag\":\"ai0\","
                        "\"unit\":\"V\","
                        "\"value\":1.23"
                    "}"
                "]"
            "}"
        "]",
        ss.str());
}

}
