#ifndef _MXLOGUPLOADER_BUILDER_HPP_
#define _MXLOGUPLOADER_BUILDER_HPP_

#include <boost/filesystem.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <mxloguploader/App.hpp>
#include <mxloguploader/AppClear.hpp>
#include <mxloguploader/AppTest.hpp>
#include <mxloguploader/error.hpp>

namespace mxloguploader
{

class Builder
    : private boost::noncopyable
{
public:
    Builder();
    ~Builder();

    void
        set_configuration(const boost::filesystem::path &p);

    void
        set_profile(const std::string &);   // throws mxloguploader::ParseError

    void
        set_file(const boost::filesystem::path &p);

    void
        set_file_count(unsigned int file_count);

    boost::shared_ptr<mxloguploader::App>
        build() const;

    boost::shared_ptr<mxloguploader::AppTest>
        build_test() const;

    boost::shared_ptr<mxloguploader::AppClear>
        build_clear() const;

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
