#ifndef _MXLOGUPLOADER_PROFILE_CONFIGURATION_HPP_
#define _MXLOGUPLOADER_PROFILE_CONFIGURATION_HPP_

#include <string>

#include <boost/shared_ptr.hpp>

#include <mxloguploader/error.hpp>
#include <mxloguploader/LogConverter.hpp>

namespace mxloguploader
{

struct ProfileConfiguration
{
    struct Method
    {
        struct HttpPost
        {
            struct Authentication
            {
                std::string username;
                std::string password;
            };

            std::string                         url;
            boost::shared_ptr<Authentication>   authentication;

            bool                                bypass_ssl;
        };

        boost::shared_ptr<HttpPost> http_post;
    };

    struct Retry
    {
        unsigned int count;
        unsigned int period_sec;
    };

    struct Content
    {
        mxloguploader::LogConverter::format_t   format;
        mxloguploader::LogConverter::compress_t compress;
    };

    Method      method;
    Retry       retry;
    std::string log_directory;
    Content     content;

    static const ProfileConfiguration
        parse(const std::string &content); // throws mxloguploader::ParseError
};

}

#endif
