#ifndef _MXLOGUPLOADER_STATUS_HPP_
#define _MXLOGUPLOADER_STATUS_HPP_

#include <string>

#include <boost/cstdint.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/make_shared.hpp>
#include <boost/shared_ptr.hpp>

#include <mxloguploader/error.hpp>

namespace mxloguploader
{

struct Status
{
    struct LastAttempt
    {
        std::string                 name;
        boost::posix_time::ptime    at;
        std::string                 status;

        LastAttempt(
            const std::string &name_,
            const boost::posix_time::ptime &at_,
            const std::string &status_)
            : name(name_)
            , at(at_)
            , status(status_)
        {
        }
    };

    struct LastSuccess
    {
        std::string                 name;
        boost::posix_time::ptime    at;

        LastSuccess(
            const std::string &name_,
            const boost::posix_time::ptime &at_)
            : name(name_)
            , at(at_)
        {
        }
    };

    boost::shared_ptr<LastAttempt>  last_attempt;
    boost::shared_ptr<LastSuccess>  last_success;
    int64_t                         file_upload_count;

public:
    static const Status
        parse(const std::string &);  // throws mxloguploader::ParseError

    Status()
        : last_attempt()
        , last_success()
        , file_upload_count(0)
    {
    }

    Status(const Status &rhs)
        : file_upload_count(rhs.file_upload_count)
    {
        if (rhs.last_attempt)
            this->last_attempt = boost::make_shared<LastAttempt>(*rhs.last_attempt);

        if (rhs.last_success)
            this->last_success = boost::make_shared<LastSuccess>(*rhs.last_success);
    }

    Status &
        operator =(const Status &rhs)
    {
        if (this == &rhs)
            return *this;

        this->file_upload_count = rhs.file_upload_count;

        if (rhs.last_attempt)
            this->last_attempt = boost::make_shared<LastAttempt>(*rhs.last_attempt);

        if (rhs.last_success)
            this->last_success = boost::make_shared<LastSuccess>(*rhs.last_success);

        return *this;
    }

    const std::string
        str() const;
};

}

#endif
