#include <boost/filesystem.hpp>

#include <mxutil/log.hpp>
#include <mxutil/PidFile.hpp>

#include <mxsl/mxsl.hpp>

#include <mxloguploader/Option.hpp>
#include <mxloguploader/Builder.hpp>

using boost::logic::tribool;
using boost::filesystem::path;
using boost::shared_ptr;

using mxutil::PidFile;

using mxloguploader::App;
using mxloguploader::AppClear;
using mxloguploader::AppTest;
using mxloguploader::Builder;
using mxloguploader::Option;

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[]);

}

int
    main(
        int argc,
        const char *argv[])
{
    try
    {
        const Option opt = get_option(argc, argv);

        mxsl::validate();

        shared_ptr<PidFile> pidfile;
        if (opt.profile)
            pidfile = PidFile::get("/var/run/mxloguploader." + *opt.profile + ".pid");

        Builder builder;
        builder.set_configuration("/etc/mxloguploader/configuration");

        if (opt.profile)
            builder.set_profile(*opt.profile);

        if (opt.file)
            builder.set_file(*opt.file);

        builder.set_file_count(opt.file_count);

        if (opt.upload)
        {
            shared_ptr<App> app = builder.build();
            app->run();
        }
        else if (opt.clear)
        {
            shared_ptr<AppClear> app = builder.build_clear();

            app->run();
        }
        else if (opt.upload_test)
        {
            shared_ptr<AppTest> app = builder.build_test();

            app->run();
        }
    }
    catch(mxutil::LockError &e)
    {
        MXLOG_EROR(diagnostic_information(e));
        exit(3);
    }
    catch(mxloguploader::Error &e)
    {
        MXLOG_EROR(diagnostic_information(e));
        exit(4);
    }
    catch(boost::exception &e)
    {
        MXLOG_EROR(diagnostic_information(e));
        exit(1);
    }
    catch(std::exception &e)
    {
        MXLOG_EROR(e.what());
        exit(1);
    }
    catch(...)
    {
        MXLOG_EROR("unexpected exception caught");
        exit(1);
    }

    return 0;
}

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[])
{
    Option opt;
    const tribool parsed = opt.parse_args(argc, argv);
    if (parsed)
        return opt;
    else if (!parsed)
        exit(2);
    else
        exit(0);
}

}
