#include <mxloguploader/AppClear.hpp>

#include <mxutil/PidFile.hpp>

using boost::filesystem::path;
using boost::shared_ptr;

using mxutil::PidFile;

namespace mxloguploader
{

struct AppClear::Impl
{
    Impl(
        const path &working_directory_,
        shared_ptr<StatusPublisher> publisher_)
        : working_directory(working_directory_)
        , publisher(publisher_)
    {
    }

    const path                  working_directory;
    shared_ptr<StatusPublisher> publisher;

    shared_ptr<PidFile>         pidfile;
};

AppClear::AppClear(
    const path &working_directory,
    shared_ptr<StatusPublisher> publisher)
    : pimpl(new Impl(
        working_directory,
        publisher))
{
    BOOST_ASSERT(pimpl->publisher);

    if (!is_directory(pimpl->working_directory))
        create_directories(working_directory);

    pimpl->pidfile =
        PidFile::get((pimpl->working_directory / "lock").string());
}

AppClear::~AppClear()
{
}

void
    AppClear::run()
{
    // close pidfile before performing clear
    pimpl->pidfile.reset();

    if (exists(pimpl->working_directory))
        remove_all(pimpl->working_directory);

    pimpl->publisher->clear();
}

}
