#include <mxloguploader/AppTest.hpp>

#include <fstream>
#include <iomanip>

#include <boost/date_time/posix_time/posix_time.hpp>

#include <mxutil/log.hpp>

#include <mxtag/tag_form.hpp>

using boost::shared_ptr;
using boost::filesystem::path;
using boost::filesystem::temp_directory_path;
using boost::filesystem::unique_path;
using boost::posix_time::ptime;
using boost::posix_time::second_clock;
using boost::system::error_code;

using mxtag::form_iso8601_time;

using std::endl;
using std::ofstream;
using std::setfill;
using std::setw;
using std::string;
using std::stringstream;

namespace
{

const string
    generate_external_filename(const ptime &time)
{
    stringstream ss;
    ss << "upload-test."
        << setw(4) << time.date().year()
        << setw(2) << setfill('0') << static_cast<int>(time.date().month())
        << setw(2) << setfill('0') << time.date().day() << '-'
        << setw(2) << setfill('0') << time.time_of_day().hours()
        << setw(2) << setfill('0') << time.time_of_day().minutes()
        << setw(2) << setfill('0') << time.time_of_day().seconds()
        << ".txt";

    return ss.str();
}

}

namespace mxloguploader
{

struct AppTest::Impl
{
    Impl(shared_ptr<Uploader> uploader_)
        : uploader(uploader_)
        , upload_file_path(
            temp_directory_path()
            / unique_path(string("upload-test") + "-%%%%%%%%"))
    {
    }

    const shared_ptr<Uploader>  uploader;
    const path                  upload_file_path;

    void
        create_upload_file(const ptime &now)
    {
        ofstream outfile(this->upload_file_path.string().c_str());
        if (!outfile)
        {
            MXLOG_EROR("open file failed: " + this->upload_file_path.string());
            BOOST_THROW_EXCEPTION(Error());
        }

        outfile
            << "Upload Test" << '\n'
            << "Time: " << form_iso8601_time(now) << endl;
    }
};

AppTest::AppTest(shared_ptr<Uploader> uploader)
    : pimpl(new Impl(
        uploader))
{
    BOOST_ASSERT(pimpl->uploader);
}

AppTest::~AppTest()
{
    error_code ec;
    remove(pimpl->upload_file_path, ec);
    if (ec)
    {
        // ignored
    }
}

void
    AppTest::run()
{
    // create upload file
    const ptime now = second_clock::universal_time();
    pimpl->create_upload_file(now);

    // do upload
    const string filename = generate_external_filename(now);
    pimpl->uploader->upload(pimpl->upload_file_path, &filename);
}

}
