#include <mxloguploader/HttpPostUploader.hpp>

#include <cstdlib>
#include <sstream>

#include <boost/make_shared.hpp>

#include <config.h>

using boost::filesystem::path;
using boost::make_shared;
using boost::shared_ptr;
using boost::tuple;

using std::string;
using std::stringstream;

namespace
{

typedef
    tuple<
        string,     // username
        string      // password
    > Authentication;

}

namespace mxloguploader
{

struct HttpPostUploader::Impl
{
    Impl(
        const string &url_,
        bool bypass_ssl_)
        : url(url_)
        , bypass_ssl(bypass_ssl_)
    {
    }

    const string    url;
    const bool      bypass_ssl;

    shared_ptr<Authentication>  authentication;
};

HttpPostUploader::HttpPostUploader(
    const string &url,
    bool bypass_ssl,
    unsigned int retry_count,
    unsigned int retry_period_sec)
    : Uploader(
        retry_count,
        retry_period_sec)
    , pimpl(new Impl(
        url,
        bypass_ssl))
{
}

HttpPostUploader::~HttpPostUploader()
{
}

void
    HttpPostUploader::set_authentication(
        const string &username,
        const string &password)
{
    // username should not contain ':'
    const size_t pos = username.find(':');
    BOOST_VERIFY(pos == string::npos);

    pimpl->authentication = make_shared<Authentication>(username, password);
}

void
    HttpPostUploader::do_upload(
        const string &filename,
        const path &p)
{
    stringstream cmd;
    cmd << "curl" << " -f"
        << " --max-time 60"
        << " --user-agent mxloguploader/" VERSION
        << " -F " << '"' << "file=@" << p.string() << ";filename=" << filename << '"';

    if (pimpl->authentication)
    {
        const string &username = pimpl->authentication->get<0>();
        const string &password = pimpl->authentication->get<1>();

        cmd << " --user " << username << ':' << password;
    }

    if (pimpl->bypass_ssl)
        cmd << " -k";

    cmd << " " << pimpl->url;
    cmd << " 2>&1 >/dev/null";

    const int status = system(cmd.str().c_str());
    if (status == -1)
    {
        BOOST_THROW_EXCEPTION(SystemError());
    }
    else if (WEXITSTATUS(status) != 0)
    {
        BOOST_THROW_EXCEPTION(UploadError());
    }
}

}
