#include <mxloguploader/Option.hpp>

#include <boost/make_shared.hpp>
#include <boost/program_options.hpp>

#include <config.h>

using boost::logic::tribool;
using boost::logic::indeterminate;
using boost::make_shared;
using boost::program_options::options_description;
using boost::program_options::value;
using boost::program_options::variables_map;
using boost::program_options::command_line_style::default_style;
using boost::program_options::command_line_style::allow_guessing;

using std::cout;
using std::endl;
using std::string;

namespace mxloguploader
{

tribool
    Option::parse_args(
        int argc,
        const char *argv[])
{
    options_description desc("Allowed options");
    desc.add_options()
        ("profile,p", value<string>(), "Profile to operate on.")
        ("file,f", value<string>(), "File path to profile configuration. Can only be used with --upload-test.")
        ("upload", "[COMMAND] Upload logs.")
        ("upload-test", "[COMMAND] Upload a test file.")
        ("file-count", value<unsigned int>(), "Set maximum number of logs to upload.")
        ("clear", "[COMMAND] Clear all local bookings of the given Profile.")
        ("help", "Print this manual.")
        ("version", "Print version.");

    variables_map vm;
    try
    {
        store(
            parse_command_line(argc, argv, desc, default_style & (~allow_guessing)),
            vm
        );
    }
    catch(boost::program_options::error &e)
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("help"))
    {
        cout << desc << endl;
        return indeterminate;
    }

    if (vm.count("version"))
    {
        cout << "mxloguploader v" << VERSION << endl;
        return indeterminate;
    }

    if (vm.count("profile") + vm.count("file") != 1)
    {
        cout << desc << endl;
        return false;
    }

    if (1 != (vm.count("upload") + vm.count("clear") + vm.count("upload-test")))
    {
        cout << desc << endl;
        return false;
    }

    if (1 == vm.count("file")
        && 1 != vm.count("upload-test"))
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("profile"))
        this->profile = make_shared<string>(vm["profile"].as<string>());

    if (vm.count("file"))
        this->file = make_shared<string>(vm["file"].as<string>());

    if (vm.count("upload"))
        this->upload = true;

    if (vm.count("upload-test"))
        this->upload_test = true;

    if (vm.count("clear"))
        this->clear = true;

    if (vm.count("file-count"))
        this->file_count = vm["file-count"].as<unsigned int>();

    return true;
}

}
