#include <mxloguploader/StatusPublisher.hpp>

#include <boost/lexical_cast.hpp>

#include <mxmqtt/Worker.hpp>

using boost::lexical_cast;

using mxmqtt::Message;
using mxmqtt::Worker;

using std::string;
using std::vector;

namespace mxloguploader
{

struct StatusPublisher::Impl
{
    Impl(
        const string &profile,
        const string &host,
        int port)
        : topic("/system/log-uploader/" + profile + "/status")
        , publisher(
            host,
            port,
            "log-uploader-" + lexical_cast<string>(getpid()))
    {
    }

    const string    topic;
    Worker          publisher;
};

StatusPublisher::StatusPublisher(
    const string &profile,
    const string &host,
    int port)
    : pimpl(new Impl(
        profile,
        host,
        port))
{
}

StatusPublisher::~StatusPublisher()
{
}

void
    StatusPublisher::publish(const Status &s)
{
    const string payload = s.str();

    pimpl->publisher.publish(Message(
        pimpl->topic,
        vector<uint8_t>(payload.begin(), payload.end()),
        1,
        true));
}

void
    StatusPublisher::clear()
{
    pimpl->publisher.publish(Message(
        pimpl->topic,
        vector<uint8_t>(),
        1,
        true));
}

}
