#include <mxloguploader/Uploader.hpp>

#include <mxutil/log.hpp>

using boost::filesystem::path;
using boost::shared_ptr;

using std::string;

namespace mxloguploader
{

struct Uploader::Impl
{
    Impl(
        unsigned int retry_count_,
        unsigned int retry_period_sec_)
        : retry_count(retry_count_)
        , retry_period_sec(retry_period_sec_)
    {
    }

    const unsigned int                  retry_count;
    const unsigned int                  retry_period_sec;
};

Uploader::Uploader(
    unsigned int retry_count,
    unsigned int retry_period_sec)
    : pimpl(new Impl(
        retry_count,
        retry_period_sec))
{
}

Uploader::~Uploader()
{
}

void
    Uploader::upload(
        const path &external_log,
        const string *change_filename)
{
    const string external_filename =
        change_filename ? *change_filename : external_log.filename().string();

    try
    {
        for(unsigned int retry = 0;
            retry <= pimpl->retry_count;
            ++retry)
        {
            try
            {
                this->do_upload(
                    external_filename,
                    external_log);

                break;
            }
            catch(mxloguploader::Error &e)
            {
                if (retry == pimpl->retry_count)
                {
                    throw;
                }
                else
                {
                    sleep(pimpl->retry_period_sec);
                    continue;
                }
            }
        }
    }
    catch(mxloguploader::Error &e)
    {
        MXLOG_WARN(diagnostic_information(e));

        throw;
    }
    catch(boost::exception &e)
    {
        MXLOG_EROR(diagnostic_information(e));

        BOOST_THROW_EXCEPTION(mxloguploader::Error());
    }
    catch(std::exception &e)
    {
        MXLOG_EROR("unexpected exception caught: " + string(e.what()));

        BOOST_THROW_EXCEPTION(mxloguploader::Error());
    }
    catch(...)
    {
        MXLOG_EROR("unexpected exception caught");

        BOOST_THROW_EXCEPTION(mxloguploader::Error());
    }
}

}
