#include <boost/test/unit_test.hpp>

#include <boost/range.hpp>

#include <mxloguploader/Option.hpp>

using boost::logic::tribool;
using boost::size;

using mxloguploader::Option;

namespace
{

struct TestFixture
{
    Option opt;
};

}

namespace mxloguploader
{

BOOST_FIXTURE_TEST_CASE(test_Option_parse_args_WithUpload_ShouldPass, TestFixture)
{
    // arrange
    const char *argv[] = {
        "dummy",
        "--profile", "mxcs",
        "--upload",
        "--file-count", "40"
    };

    // act
    const tribool result = opt.parse_args(size(argv), argv);

    // assert
    BOOST_CHECK_EQUAL(true, result);
    BOOST_REQUIRE(opt.profile);
    BOOST_CHECK_EQUAL("mxcs", *opt.profile);
    BOOST_CHECK_EQUAL(true, opt.upload);
    BOOST_CHECK_EQUAL(false, opt.clear);
    BOOST_CHECK_EQUAL(40, opt.file_count);
}

BOOST_FIXTURE_TEST_CASE(test_Option_parse_args_WithClear_ShouldPass, TestFixture)
{
    // arrange
    const char *argv[] = {
        "dummy",
        "--profile", "mxcs",
        "--clear"
    };

    // act
    const tribool result = opt.parse_args(size(argv), argv);

    // assert
    BOOST_CHECK_EQUAL(true, result);
    BOOST_REQUIRE(opt.profile);
    BOOST_CHECK_EQUAL("mxcs", *opt.profile);
    BOOST_CHECK_EQUAL(false, opt.upload);
    BOOST_CHECK_EQUAL(true, opt.clear);
}

}
