#include <boost/test/unit_test.hpp>

#include <mxloguploader/ProfileConfiguration.hpp>

using std::string;

namespace
{

class TestFixture
{
};

}

namespace mxloguploader
{

BOOST_FIXTURE_TEST_CASE(test_ProfileConfiguration_parse_ShouldPass, TestFixture)
{
    // arrange
    const string SUT =
        "# /etc/mxloguploader/conf.d/mxcs.conf\n"
        "method = {\n"
        "    http-post = {\n"
        "        url = \"https://mxcloud.moxa.com/upload\"\n"
        "        authentication = {\n"
        "            username = \"moxa\"\n"
        "            password = \"mahaha\"\n"
        "        }\n"
        "        bypass-ssl = false\n"
        "    }\n"
        "}\n"
        "\n"
        "retry = {\n"
        "    count = 3\n"
        "    period-sec = 10\n"
        "}\n"
        "log-directory = \"/var/mxc/logs\"\n"
        "\n"
        "content = {\n"
        "    format = \"json\"\n"
        "    compress = \"gzip\"\n"
        "}\n";

    // act
    const ProfileConfiguration result = ProfileConfiguration::parse(SUT);

    // assert
    BOOST_REQUIRE(result.method.http_post);
    BOOST_CHECK_EQUAL("https://mxcloud.moxa.com/upload", result.method.http_post->url);
    BOOST_REQUIRE(result.method.http_post->authentication);
    BOOST_CHECK_EQUAL("moxa", result.method.http_post->authentication->username);
    BOOST_CHECK_EQUAL("mahaha", result.method.http_post->authentication->password);
    BOOST_CHECK_EQUAL(false, result.method.http_post->bypass_ssl);

    BOOST_CHECK_EQUAL(3, result.retry.count);
    BOOST_CHECK_EQUAL(10, result.retry.period_sec);

    BOOST_CHECK_EQUAL("/var/mxc/logs", result.log_directory);

    BOOST_CHECK_EQUAL(LogConverter::FORMAT_JSON, result.content.format);
    BOOST_CHECK_EQUAL(LogConverter::COMPRESS_GZIP, result.content.compress);
}

BOOST_FIXTURE_TEST_CASE(test_ProfileConfiguration_parse_WithXmlFormat_ShouldPass, TestFixture)
{
    // arrange
    const string SUT =
        "# /etc/mxloguploader/conf.d/mxcs.conf\n"
        "method = {\n"
        "    http-post = {\n"
        "        url = \"https://mxcloud.moxa.com/upload\"\n"
        "        authentication = {\n"
        "            username = \"moxa\"\n"
        "            password = \"mahaha\"\n"
        "        }\n"
        "        bypass-ssl = false\n"
        "    }\n"
        "}\n"
        "\n"
        "retry = {\n"
        "    count = 3\n"
        "    period-sec = 10\n"
        "}\n"
        "log-directory = \"/var/mxc/logs\"\n"
        "\n"
        "content = {\n"
        "    format = \"xml\"\n"
        "    compress = \"gzip\"\n"
        "}\n";

    // act
    const ProfileConfiguration result = ProfileConfiguration::parse(SUT);

    // assert
    BOOST_CHECK_EQUAL(LogConverter::FORMAT_XML, result.content.format);
}

}
