#include <boost/test/unit_test.hpp>

#include <boost/make_shared.hpp>

#include <mxloguploader/Status.hpp>

using boost::make_shared;
using boost::posix_time::time_from_string;

using std::string;

namespace
{

class TestFixture
{
};

}

namespace mxloguploader
{

BOOST_FIXTURE_TEST_CASE(test_Status_parse_ShouldPass, TestFixture)
{
    // arrange
    const string SUT =
        "{\n"
            "\"lastAttempt\": {\n"
                "\"name\": \"/var/mxc/logs/20150706-153807.log.gz\",\n"
                "\"at\": \"2015-07-06T15:38:46Z\",\n"
                "\"status\": \"remote server error\"\n"
            "},\n"
            "\"lastSuccess\": {\n"
                "\"name\": \"/var/mxc/logs/20150706-123456.log.gz\",\n"
                "\"at\": \"2015-07-06T13:46:22Z\"\n"
            "},\n"
            "\"fileUploadCount\": 47\n"
        "}\n";

    // act
    const Status result = Status::parse(SUT);

    // assert
    BOOST_REQUIRE(result.last_attempt);
    BOOST_CHECK_EQUAL("/var/mxc/logs/20150706-153807.log.gz", result.last_attempt->name);
    BOOST_CHECK_EQUAL(time_from_string("2015-07-06 15:38:46"), result.last_attempt->at);
    BOOST_CHECK_EQUAL("remote server error", result.last_attempt->status);

    BOOST_REQUIRE(result.last_success);
    BOOST_CHECK_EQUAL("/var/mxc/logs/20150706-123456.log.gz", result.last_success->name);
    BOOST_CHECK_EQUAL(time_from_string("2015-07-06 13:46:22"), result.last_success->at);

    BOOST_CHECK_EQUAL(47, result.file_upload_count);
}

BOOST_FIXTURE_TEST_CASE(test_Status_str_ShouldPass, TestFixture)
{
    // arrange
    Status status;
    status.last_attempt = make_shared<Status::LastAttempt>(
        "/var/mxc/logs/20150706-153807.log.gz",
        time_from_string("2015-07-06 15:38:46"),
        "remote server error");
    status.last_success = make_shared<Status::LastSuccess>(
        "/var/mxc/logs/20150706-123456.log.gz",
        time_from_string("2015-07-06 13:46:22"));
    status.file_upload_count = 47;

    // act
    const string result = status.str();

    std::cerr << result << std::endl;

    // assert
    const Status check = Status::parse(result);
    BOOST_REQUIRE(check.last_attempt);
    BOOST_CHECK_EQUAL("/var/mxc/logs/20150706-153807.log.gz", check.last_attempt->name);
    BOOST_CHECK_EQUAL(time_from_string("2015-07-06 15:38:46"), check.last_attempt->at);
    BOOST_CHECK_EQUAL("remote server error", check.last_attempt->status);

    BOOST_REQUIRE(check.last_success);
    BOOST_CHECK_EQUAL("/var/mxc/logs/20150706-123456.log.gz", check.last_success->name);
    BOOST_CHECK_EQUAL(time_from_string("2015-07-06 13:46:22"), check.last_success->at);

    BOOST_CHECK_EQUAL(47, check.file_upload_count);
}

}
