#ifndef _MXMODBUSGATEWAY_CONFIGURATION_HPP_
#define _MXMODBUSGATEWAY_CONFIGURATION_HPP_

#include <string>
#include <vector>

#include <mxmodbus/ModbusRtuClient.hpp>

#include <mxmodbusgateway/error.hpp>

namespace mxmodbusgateway
{

struct Configuration
{
    static const Configuration
        parse(const std::string &content); // throws mxmodbusgateway::ParseError

    struct Port
    {
        struct Tcp
        {
            std::string host;
            int service;

            Tcp(
                const std::string &host_,
                int service_)
                : host(host_)
                , service(service_)
            {
            }
        };

        Tcp tcp;

        struct Rtu
        {
            struct SerialConfig
            {
                mxmodbus::ModbusRtuClient::baud_t       baud;
                mxmodbus::ModbusRtuClient::parity_t     parity;
                mxmodbus::ModbusRtuClient::stop_bit_t   stopbit;

                SerialConfig(
                    mxmodbus::ModbusRtuClient::baud_t       baud_,
                    mxmodbus::ModbusRtuClient::parity_t     parity_,
                    mxmodbus::ModbusRtuClient::stop_bit_t   stopbit_)
                    : baud(baud_)
                    , parity(parity_)
                    , stopbit(stopbit_)
                {
                }
            };

            std::string     path;
            SerialConfig    serial_config;
            unsigned int    interframe_delay_ms;
            unsigned int    interchar_timeout_ms;
            unsigned int    request_timeout_ms;

            Rtu(
                const std::string &path_,
                const SerialConfig &serial_config_,
                unsigned int interframe_delay_ms_,
                unsigned int interchar_timeout_ms_,
                unsigned int request_timeout_ms_)
                : path(path_)
                , serial_config(serial_config_)
                , interframe_delay_ms(interframe_delay_ms_)
                , interchar_timeout_ms(interchar_timeout_ms_)
                , request_timeout_ms(request_timeout_ms_)
            {
            }
        };

        Rtu rtu;

        Port(
            const Tcp &tcp_,
            const Rtu &rtu_)
            : tcp(tcp_)
            , rtu(rtu_)
        {
        }
    };

    std::vector<Port> port_list;

    Configuration(
        const std::vector<Port> &port_list_)
        : port_list(port_list_)
    {
    }
};

}

#endif
