#ifndef _MXMODBUSGATEWAY_SERVER_HPP_
#define _MXMODBUSGATEWAY_SERVER_HPP_

#include <boost/asio.hpp>
#include <boost/enable_shared_from_this.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxutil/async.hpp>

#include <mxmodbusgateway/worker/ModbusWorker.hpp>

namespace mxmodbusgateway
{

class Server
    : public boost::enable_shared_from_this<Server>
    , private boost::noncopyable
{
public:
    Server(
        boost::asio::io_service &,
        const std::string &host,
        int port,
        boost::shared_ptr<mxmodbusgateway::worker::ModbusWorker>);
    ~Server();

    void
        async_run(mxutil::CompleteCallback);

    void
        cancel();

private:
    void
        on_resolve_complete_(
            const boost::system::error_code &ec,
            boost::asio::ip::tcp::resolver::iterator);

    void
        on_accept_(
            const boost::system::error_code &ec,
            boost::shared_ptr<boost::asio::ip::tcp::acceptor>,
            boost::shared_ptr<boost::asio::ip::tcp::socket>);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
