#ifndef _MXMODBUSGATEWAY_SESSION_HPP_
#define _MXMODBUSGATEWAY_SESSION_HPP_

#include <boost/asio.hpp>
#include <boost/enable_shared_from_this.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <mxutil/async.hpp>

#include <mxmodbusgateway/Command.hpp>
#include <mxmodbusgateway/worker/ModbusWorker.hpp>

namespace mxmodbusgateway
{

class Session
    : public boost::enable_shared_from_this<Session>
    , private boost::noncopyable
{
public:
    Session(
        boost::asio::io_service &,
        boost::shared_ptr<boost::asio::ip::tcp::socket>,
        boost::shared_ptr<mxmodbusgateway::worker::ModbusWorker>);
    ~Session();

    void
        async_run(mxutil::CompleteCallback);
    void
        cancel();

private:
    void
        execute_next_command_();

private:
    void
        on_read_complete_(
            const boost::system::error_code &ec,
            std::size_t bytes_transferred);

    void
        on_command_complete_(
            const boost::system::error_code &ec,
            boost::shared_ptr<mxmodbusgateway::Command>);

    void
        on_write_complete_(
            const boost::system::error_code &ec);

    void
        on_input_timeout_(
            const boost::system::error_code &ec);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
