#ifndef _MXMODBUSGATEWAY_WRITE_MULTIPLE_COILS_COMMAND_HPP_
#define _MXMODBUSGATEWAY_WRITE_MULTIPLE_COILS_COMMAND_HPP_

#include <vector>

#include <boost/asio.hpp>
#include <boost/cstdint.hpp>
#include <boost/enable_shared_from_this.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxmodbusgateway/Command.hpp>
#include <mxmodbusgateway/error.hpp>
#include <mxmodbusgateway/worker/ModbusWorker.hpp>

namespace mxmodbusgateway
{

class WriteMultipleCoilsCommand
    : public mxmodbusgateway::Command
    , public boost::enable_shared_from_this<WriteMultipleCoilsCommand>
{
public:
    static boost::shared_ptr<WriteMultipleCoilsCommand>
        parse(
            std::vector<uint8_t> &,
            boost::asio::io_service &);  // throws mxmodbusgateway::ParseError

private:
    WriteMultipleCoilsCommand(
        boost::asio::io_service &,
        uint16_t packet_id,
        mxmodbus::device_t,
        mxmodbus::address_t,
        std::vector<bool> value);

public:
    virtual ~WriteMultipleCoilsCommand();

    virtual void
        async_execute(
            mxmodbusgateway::worker::ModbusWorker &,
            std::vector<uint8_t> &out,
            const mxutil::CompleteCallback &);

private:
    void
        on_read_complete_(
            const boost::system::error_code &,
            std::vector<uint8_t> *out,
            const mxutil::CompleteCallback &);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
