#ifndef _MXMODBUSGATEWAY_WORKER_MODBUS_WORKER_HPP_
#define _MXMODBUSGATEWAY_WORKER_MODBUS_WORKER_HPP_

#include <vector>

#include <boost/asio.hpp>
#include <boost/cstdint.hpp>
#include <boost/function.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <mxutil/async.hpp>

#include <mxmodbus/ModbusClient.hpp>

#include <mxmodbusgateway/errc.hpp>

namespace mxmodbusgateway
{
namespace worker
{

class ModbusWorker
    : private boost::noncopyable
{
    typedef
        boost::function<
            boost::shared_ptr<mxmodbus::ModbusClient>
                ()
        >
        CreateModbusClientFunc;

public:
    ModbusWorker(
        boost::asio::io_service &,
        const CreateModbusClientFunc &,
        unsigned int interframe_delay_ms);
    ~ModbusWorker();

    void
        async_read_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity,
            std::vector<bool> &,
            const mxutil::CompleteCallback &);

    void
        async_read_discrete_inputs(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity,
            std::vector<bool> &,
            const mxutil::CompleteCallback &);

    void
        async_read_holding_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity,
            std::vector<uint16_t> &,
            const mxutil::CompleteCallback &);

    void
        async_read_input_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::size_t quantity,
            std::vector<uint16_t> &,
            const mxutil::CompleteCallback &);

    void
        async_write_single_coil(
            mxmodbus::device_t,
            mxmodbus::address_t,
            bool value,
            const mxutil::CompleteCallback &);

    void
        async_write_single_register(
            mxmodbus::device_t,
            mxmodbus::address_t,
            uint16_t value,
            const mxutil::CompleteCallback &);

    void
        async_write_multiple_coils(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::vector<bool> value,
            const mxutil::CompleteCallback &);

    void
        async_write_multiple_registers(
            mxmodbus::device_t,
            mxmodbus::address_t,
            std::vector<uint16_t> value,
            const mxutil::CompleteCallback &);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}
}

#endif
