#ifndef _MXMODBUSGATEWAY_WORKER_READ_COMMAND_HPP_
#define _MXMODBUSGATEWAY_WORKER_READ_COMMAND_HPP_

#include <boost/scoped_ptr.hpp>

#include <mxmodbus/types.hpp>

#include <mxmodbusgateway/worker/Command.hpp>

namespace mxmodbusgateway
{
namespace worker
{

template <
    typename DATA,
    const DATA (mxmodbus::ModbusClient::*READ_FUNC)(
        mxmodbus::device_t,
        mxmodbus::address_t,
        std::size_t
    )
>
class ReadCommand
    : public mxmodbusgateway::worker::Command
{
public:
    ReadCommand(
        mxmodbus::device_t device,
        mxmodbus::address_t address,
        std::size_t quantity,
        DATA &out)
        : device_(device)
        , address_(address)
        , quantity_(quantity)
        , out_(out)
    {
    }

    virtual ~ReadCommand(){}

private:
    virtual void
        do_execute(mxmodbus::ModbusClient &mc)
    {
        this->out_ = (mc.*READ_FUNC)(this->device_, this->address_, this->quantity_);
    }

private:
    const mxmodbus::device_t    device_;
    const mxmodbus::address_t   address_;
    const std::size_t           quantity_;
    DATA &                      out_;
};

}
}

#endif
