#include <mxutil/PidFile.hpp>
#include <mxutil/Signal.hpp>
#include <mxutil/log.hpp>
#include <mxutil/log/Log.hpp>
#include <mxutil/log/StderrLogger.hpp>
#include <mxutil/log/SyslogLogger.hpp>

#include <mxsl/mxsl.hpp>

#include <mxmodbusgateway/Builder.hpp>
#include <mxmodbusgateway/Option.hpp>

using boost::logic::tribool;
using boost::shared_ptr;

using mxutil::PidFile;
using mxutil::Signal;
using mxutil::log::Log;
using mxutil::log::Logger;
using mxutil::log::StderrLogger;
using mxutil::log::SyslogLogger;

using mxmodbusgateway::App;
using mxmodbusgateway::Builder;
using mxmodbusgateway::Option;

using std::cerr;
using std::endl;

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[]);

shared_ptr<App>
    get_app();

volatile bool stop = false;
App * volatile app_ptr = NULL;
void
    on_term(int)
{
    stop = true;

    if (app_ptr != NULL)
        app_ptr->stop();
}

}

int
    main(
        int argc,
        const char *argv[])
{
    try
    {
        const Option opt = get_option(argc, argv);

        shared_ptr<Logger> logger(new StderrLogger());
        logger->set_level(Logger::DBUG);
        logger->enable_line_of_file();
        if (!opt.foreground)
        {
            const int result = daemon(0, 0);
            if (0 != result)
            {
                cerr << strerror(errno) << endl;
                return 1;
            }

            logger.reset(new SyslogLogger("mxmodbusgateway"));
            logger->set_level(Logger::WARN);
        }
        Log::get().set_logger(logger);

        const shared_ptr<App> app = get_app();

        Signal::instance().sigterm(&on_term);

        shared_ptr<PidFile> pidfile;
        if (!opt.foreground)
            pidfile = PidFile::get("/var/run/mxmodbusgateway.pid");

        mxsl::validate();

        app->start();

        app_ptr = &*app;

        app->loop();

        app_ptr = NULL;

        return 0;
    }
    catch(boost::exception &e)
    {
        MXLOG_EROR(diagnostic_information(e));
    }
    catch(std::exception &e)
    {
        MXLOG_EROR(e.what());
    }

    return 1;
}

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[])
{
    Option opt;
    const tribool parsed = opt.parse_args(argc, argv);
    if (parsed)
        return opt;
    else if (!parsed)
        exit(2);
    else
        exit(0);
}

shared_ptr<App>
    get_app()
{
    Builder builder;
    builder.set_configuration("etc/mxmodbusgateway/configuration");

    return builder.build();
}

}
