#include <mxmodbusgateway/App.hpp>

#include <boost/foreach.hpp>

#include <mxutil/log.hpp>

using boost::asio::io_service;
using boost::shared_ptr;
using boost::system::error_code;

using std::vector;

namespace
{

void
    dummy_callback(const error_code &e)
{
    MXLOG_EROR(e.message());
    exit(1);
}

}

namespace mxmodbusgateway
{

struct App::Impl
{
    Impl(
        shared_ptr<io_service> iosvc_,
        const vector<shared_ptr<Server> > &server_list_)
        : iosvc(iosvc_)
        , server_list(server_list_)
    {
    }

    const shared_ptr<io_service>        iosvc;
    const vector<shared_ptr<Server> >   server_list;
};

App::App(
    shared_ptr<io_service> iosvc,
    const vector<shared_ptr<Server> > &server_list)
    : pimpl(new Impl(
        iosvc,
        server_list))
{
    BOOST_ASSERT(pimpl->iosvc);
}

App::~App()
{
}

void
    App::start()
{
    BOOST_FOREACH(shared_ptr<Server> server, pimpl->server_list)
    {
        server->async_run(&dummy_callback);
    }
}

void
    App::stop()
{
    pimpl->iosvc->stop();
}

void
    App::loop()
{
    pimpl->iosvc->run();
}

}
