#include <mxmodbusgateway/CommandFactory.hpp>

#include <boost/assert.hpp>

#include <mxmodbusgateway/ReadCoilsCommand.hpp>
#include <mxmodbusgateway/ReadDiscreteInputsCommand.hpp>
#include <mxmodbusgateway/ReadHoldingRegistersCommand.hpp>
#include <mxmodbusgateway/ReadInputRegistersCommand.hpp>
#include <mxmodbusgateway/WriteMultipleCoilsCommand.hpp>
#include <mxmodbusgateway/WriteMultipleRegistersCommand.hpp>
#include <mxmodbusgateway/WriteSingleCoilCommand.hpp>
#include <mxmodbusgateway/WriteSingleRegisterCommand.hpp>

using boost::asio::io_service;
using boost::shared_ptr;

using std::vector;

namespace mxmodbusgateway
{

struct CommandFactory::Impl
{
    Impl()
    {
    }

    vector<uint8_t>     buf;
};

CommandFactory::CommandFactory()
    : pimpl(new Impl())
{
}

CommandFactory::~CommandFactory()
{
}

void
    CommandFactory::append_raw(const vector<uint8_t> &data)
{
    pimpl->buf.insert(pimpl->buf.begin(), data.begin(), data.end());
}

shared_ptr<Command>
    CommandFactory::extract_command(io_service &iosvc)
{
    shared_ptr<Command> cmd;
    int pecount = 0;

    try
    {
        cmd = ReadCoilsCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch(ParseError &)
    {
        ++pecount;
    }

    try
    {
        cmd = ReadDiscreteInputsCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch(ParseError &)
    {
        ++pecount;
    }

    try
    {
        cmd = ReadHoldingRegistersCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch (ParseError &)
    {
        ++pecount;
    }

    try
    {
        cmd = ReadInputRegistersCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch (ParseError &)
    {
        ++pecount;
    }

    try
    {
        cmd = WriteSingleCoilCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch(ParseError &)
    {
        ++pecount;
    }

    try
    {
        cmd = WriteSingleRegisterCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch(ParseError &)
    {
        ++pecount;
    }

    try
    {
        cmd = WriteMultipleCoilsCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch(ParseError &)
    {
        ++pecount;
    }

    try
    {
        cmd = WriteMultipleRegistersCommand::parse(pimpl->buf, iosvc);
        if (cmd)
            return cmd;
    }
    catch(ParseError &)
    {
        ++pecount;
    }

    if (8 == pecount)
        BOOST_THROW_EXCEPTION(ParseError());
    else
        return cmd;
}

}
