#include <mxmodbusgateway/Option.hpp>

#include <boost/program_options.hpp>

#include <config.h>

using boost::logic::tribool;
using boost::logic::indeterminate;
using boost::program_options::options_description;
using boost::program_options::value;
using boost::program_options::variables_map;
using boost::program_options::command_line_style::default_style;
using boost::program_options::command_line_style::allow_guessing;

using std::cout;
using std::endl;

namespace mxmodbusgateway
{

Option::Option()
    : foreground(false)
{
}

tribool
    Option::parse_args(
        int argc,
        const char *argv[])
{
    options_description desc("Allowed options");
    desc.add_options()
        ("foreground,f", "Do not daemonize.")
        ("help", "Print this manual.")
        ("version", "Print version.");

    variables_map vm;
    try
    {
        store(
            parse_command_line(argc, argv, desc, default_style & (~allow_guessing)),
            vm
        );
    }
    catch(boost::program_options::error &e)
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("help"))
    {
        cout << desc << endl;
        return indeterminate;
    }

    if (vm.count("version"))
    {
        cout << "mxmodbusgateway v" << VERSION << endl;
        return indeterminate;
    }

    if (vm.count("f") || vm.count("foreground"))
        this->foreground = true;

    return true;
}

}
