#include <mxmodbusgateway/errc.hpp>

#include <boost/lexical_cast.hpp>
#include <boost/range.hpp>

namespace mxmodbusgateway
{
namespace errc
{

using boost::lexical_cast;
using boost::size;
using boost::system::error_category;
using boost::system::error_code;

using mxmodbus::exception_t;
using mxmodbus::exception_code;

using std::string;

static const string MESSAGE_LIST[] = {
    "success",
    "unknown error",
    "parse error",
    "timeout error",
    "modbus error"
};

class Category
    : public error_category
{
public:
    Category(){}
    virtual ~Category(){}

    virtual const char *
        name() const
    {
        static const char NAME[] = "mxmodbusgateway::errc::Category";

        return NAME;
    }

    virtual string
        message(int ev) const
    {
        if (ev >= 0 &&
            static_cast<unsigned int>(ev) < size(MESSAGE_LIST))
            return MESSAGE_LIST[ev];

        if (ev >= modbus_exception_first &&
            ev <= modbus_exception_last)
            return "modbus-exception: " + lexical_cast<string>(ev - modbus_exception_base);

        return MESSAGE_LIST[unknown_error];
    }
};

const error_category &
    category()
{
    static Category c;
    return c;
}

exception_t
    modbus_exception_from_error_code(const error_code &ec)
{
    if (&ec.category() != &category())
        return exception_code::GATEWAY_TARGET_DEVICE_FAILED_TO_RESPOND;

    if (ec.value() < modbus_exception_first ||
        ec.value() > modbus_exception_last)
        return exception_code::GATEWAY_TARGET_DEVICE_FAILED_TO_RESPOND;

    return ec.value() - modbus_exception_base;
}

}
}
