#ifndef _MXMODBUSIO_BUILDER_HPP_
#define _MXMODBUSIO_BUILDER_HPP_

#include <string>
#include <vector>

#include <boost/cstdint.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <mxmodbusio/App.hpp>
#include <mxmodbusio/error.hpp>

namespace mxmodbusio
{

class Builder
    : private boost::noncopyable
{
public:
    Builder();
    ~Builder();

    void
        set_modbus(
            const std::string &host,
            int port,
            uint8_t device_id,
            int request_timeout_ms); // throws mxmodbusio::SystemExit

    void
        add_read_request(
            int addr,
            int quantity); // throws mxmodbusio::SystemExit

    void
        add_write_request(
            int addr,
            const std::vector<uint16_t> &data,
            bool single); // throws mxmodbusio::SystemExit

    boost::shared_ptr<mxmodbusio::App>
        build() const;

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
