#ifndef _MXMODBUSIO_COMMAND_FACTORY_HPP_
#define _MXMODBUSIO_COMMAND_FACTORY_HPP_

#include <vector>

#include <boost/cstdint.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxmodbusio/Command.hpp>
#include <mxmodbusio/error.hpp>

namespace mxmodbusio
{

class CommandFactory
    : private boost::noncopyable
{
public:
    static CommandFactory &
        instance();

private:
    CommandFactory();
    ~CommandFactory();

public:
    boost::shared_ptr<mxmodbusio::Command>
        create_read_command(
            int_least32_t reference,
            std::size_t quantity) const;    // throws mxmodbusio::ArgumentError

    boost::shared_ptr<mxmodbusio::Command>
        create_write_command(
            int_least32_t reference,
            const std::vector<uint16_t> &data) const;   // throws mxmodbusio::ArgumentError

    const std::vector<boost::shared_ptr<mxmodbusio::Command> >
        create_write_single_command(
            int_least32_t reference,
            const std::vector<uint16_t> &data) const;   // throws mxmodbusio::ArgumentError

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
