#ifndef _MXMODBUSIO_READ_COMMAND_HPP_
#define _MXMODBUSIO_READ_COMMAND_HPP_

#include <mxmodbus/types.hpp>

#include <mxmodbusio/Command.hpp>
#include <mxmodbusio/impl/utility.hpp>

namespace mxmodbusio
{

template <
    typename DATA,
    const DATA (mxmodbus::ModbusClient::*READ_FUNC)(
        mxmodbus::device_t,
        mxmodbus::address_t,
        std::size_t
    )
>
class ReadCommand
    : public mxmodbusio::Command
{
public:
    ReadCommand(
        mxmodbus::address_t address,
        std::size_t quantity)
        : address_(address)
        , quantity_(quantity)
    {
    }

    virtual ~ReadCommand()
    {
    }

private:
    virtual void
        do_execute(
            mxmodbus::ModbusClient &mc,
            mxmodbus::device_t device,
            std::ostream &os)
    {
        DATA data = (mc.*READ_FUNC)(
            device,
            this->address_,
            this->quantity_);

        mxmodbusio::impl::print_data(os, data);
    }

private:
    const mxmodbus::address_t       address_;
    const std::size_t               quantity_;
};

}

#endif
