#ifndef _MXMODBUSIO_WRITE_COMMAND_HPP_
#define _MXMODBUSIO_WRITE_COMMAND_HPP_

#include <ostream>

#include <boost/type_traits.hpp>

#include <mxmodbus/ModbusClient.hpp>
#include <mxmodbus/types.hpp>

#include <mxmodbusio/Command.hpp>

namespace mxmodbusio
{

template <
    typename VALUE,
    void (mxmodbus::ModbusClient::*WRITE_FUNC)(
        mxmodbus::device_t,
        mxmodbus::address_t,
        VALUE
    )
>
class WriteCommand
    : public mxmodbusio::Command
{
public:
    WriteCommand(
        mxmodbus::address_t address,
        VALUE value)
        : address_(address)
        , value_(value)
    {
    }

    virtual ~WriteCommand()
    {
    }

public:
    virtual void
        do_execute(
            mxmodbus::ModbusClient &mc,
            mxmodbus::device_t device,
            std::ostream &os)
    {
        (mc.*WRITE_FUNC)(
            device,
            this->address_,
            this->value_);

        os << std::endl;
    }

private:
    const mxmodbus::address_t                               address_;
    const typename boost::remove_reference<VALUE>::type     value_;
};

}

#endif
