#ifndef _MXMODBUSIO_ERROR_HPP_
#define _MXMODBUSIO_ERROR_HPP_

#include <exception>

#include <boost/exception/all.hpp>

#include <mxmodbusio/exit_status.hpp>

namespace mxmodbusio
{

struct Error
    : virtual boost::exception
    , virtual std::exception
{
    virtual const char *
        what() const throw()
    {
        static const char WHAT[] = "mxmodbusio::Error";
        return WHAT;
    }
};

struct ArgumentError
    : virtual Error
{
    virtual const char *
        what() const throw()
    {
        static const char WHAT[] = "mxmodbusio::ArgumentError";
        return WHAT;
    }
};

struct ParseError
    : virtual Error
{
    virtual const char *
        what() const throw()
    {
        static const char WHAT[] = "mxmodbusio::ParseError";
        return WHAT;
    }
};

struct SystemExit
    : virtual Error
{
    virtual const char *
        what() const throw()
    {
        static const char WHAT[] = "mxmodbusio::SystemExit";
        return WHAT;
    }
};

typedef
    boost::error_info<struct tag_exit_status, exit_status_t>
    EInfoExitStatus;

typedef
    boost::error_info<struct tag_err_msg, std::string>
    ErrMsg;

}

#endif
