#include <mxmodbusio/App.hpp>

#include <iostream>
#include <ostream>

#include <boost/foreach.hpp>

#include <mxutil/log.hpp>

#include <mxmodbus/error.hpp>

using boost::get_error_info;
using boost::shared_ptr;

using mxmodbus::device_t;
using mxmodbus::Error;
using mxmodbus::EInfoException;
using mxmodbus::exception_t;
using mxmodbus::ModbusClient;
using mxmodbus::ProtocolError;

using mxmodbusio::Command;
using mxmodbusio::EInfoExitStatus;
using mxmodbusio::exit_status;
using mxmodbusio::SystemExit;

using std::cout;
using std::ostream;
using std::vector;

namespace mxmodbusio
{

struct App::Impl
{
    Impl(
        shared_ptr<ModbusClient> modbus_client_,
        device_t device_,
        shared_ptr<CommandSource> cmd_source_)
        : modbus_client(modbus_client_)
        , device(device_)
        , cmd_source(cmd_source_)
    {
    }

    const shared_ptr<ModbusClient>  modbus_client;
    const device_t                  device;
    const shared_ptr<CommandSource> cmd_source;
};

App::App(
    shared_ptr<ModbusClient> modbus_client,
    device_t device,
    shared_ptr<CommandSource> cmd_source)
    : pimpl(new Impl(
        modbus_client,
        device,
        cmd_source))
{
    BOOST_VERIFY(pimpl->modbus_client);
    BOOST_VERIFY(pimpl->cmd_source);
}

App::~App()
{
}

void
    App::run()
{
    try
    {
        while(shared_ptr<Command> c = pimpl->cmd_source->next())
        {
            c->execute(
                *pimpl->modbus_client,
                pimpl->device,
                cout); // throws mxmodbus::ProtocolError, mxmodbus::Error
        }
    }
    catch(ProtocolError &e) {
        MXLOG_EROR(diagnostic_information(e));

        const exception_t *ec = get_error_info<EInfoException>(e);
        if (ec)
            BOOST_THROW_EXCEPTION(SystemExit() << EInfoExitStatus(8 + *ec));

        BOOST_THROW_EXCEPTION(SystemExit() << EInfoExitStatus(exit_status::UNKNOWN_ERROR));
    }
    catch(Error &e)
    {
        MXLOG_EROR(diagnostic_information(e));

        BOOST_THROW_EXCEPTION(SystemExit() << EInfoExitStatus(exit_status::IO_ERROR));
    }
}

}
