#include <mxmodbusio/CommandPrompt.hpp>

#include <boost/algorithm/string.hpp>

#include <mxmodbusio/CommandFactory.hpp>
#include <mxmodbusio/ReadCommand.hpp>
#include <mxmodbusio/WriteCommand.hpp>
#include <mxmodbusio/impl/parse_command.hpp>

using boost::shared_ptr;
using boost::to_lower_copy;

using mxmodbus::address_t;

using mxmodbusio::Command;
using mxmodbusio::CommandFactory;
using mxmodbusio::impl::parse_read_command;
using mxmodbusio::impl::parse_swrite_command;
using mxmodbusio::impl::parse_write_command;

using std::istream;
using std::string;
using std::vector;

namespace
{

shared_ptr<Command>
    try_read_command(
        const string &line)
{
    int_least32_t reference;
    size_t quantity;

    if (!parse_read_command(
            line,
            reference,
            quantity))
        return shared_ptr<Command>();

    return CommandFactory::instance().create_read_command(
        reference,
        quantity);
}

shared_ptr<Command>
    try_write_command(
        const string &line)
{
    int_least32_t reference;
    vector<uint16_t> data;

    if (!parse_write_command(
            line,
            reference,
            data))
        return shared_ptr<Command>();

    return CommandFactory::instance().create_write_command(
        reference,
        data);
}

shared_ptr<Command>
    try_swrite_command(
        const string &line)
{
    int_least32_t reference;
    vector<uint16_t> data(1);

    if (!parse_swrite_command(
            line,
            reference,
            data.front()))
        return shared_ptr<Command>();

    const vector<shared_ptr<Command> > cmd_list = CommandFactory::instance().create_write_single_command(
        reference,
        data);

    BOOST_VERIFY(cmd_list.size() == 1);

    return cmd_list.front();
}

}

namespace mxmodbusio
{

struct CommandPrompt::Impl
{
    Impl(
        shared_ptr<istream> is_)
        : is(is_)
    {
    }

    shared_ptr<istream> is;
};

CommandPrompt::CommandPrompt(
    shared_ptr<istream> is)
    : pimpl(new Impl(
        is))
{
}

CommandPrompt::~CommandPrompt()
{
}

shared_ptr<Command>
    CommandPrompt::do_next()
{
    shared_ptr<Command> rtn;

    string line;
    while(getline(*pimpl->is, line))
    {
        const string cmd = to_lower_copy(line);
        if (cmd == "quit")
            break;

        rtn = try_read_command(cmd);
        if (rtn)
            break;

        rtn = try_write_command(cmd);
        if (rtn)
            break;

        rtn = try_swrite_command(cmd);
        if (rtn)
            break;

        BOOST_THROW_EXCEPTION(
            ParseError()
                << ErrMsg("illegal command: " + line)
        );
    }

    return rtn;
}

}
