#include <boost/test/unit_test.hpp>

#include <mxmodbusio/impl/parse_command.hpp>

#include <boost/range.hpp>

using boost::size;

using std::string;
using std::vector;

namespace
{

struct TestFixture
{
};

}

namespace mxmodbusio
{
namespace impl
{

BOOST_FIXTURE_TEST_CASE(test_parse_read_command_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "read 30 40";

    // act
    int_least32_t reference = 0;
    size_t quantity = 0;
    const bool should_be_true = parse_read_command(
        SUT,
        reference,
        quantity);

    // assert
    BOOST_CHECK(should_be_true);
    BOOST_CHECK_EQUAL(30, reference);
    BOOST_CHECK_EQUAL(40, quantity);
}

BOOST_FIXTURE_TEST_CASE(test_parse_read_command_WithHex_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "read 0x30 0x40";

    // act
    int_least32_t reference = 0;
    size_t quantity = 0;
    const bool should_be_true = parse_read_command(
        SUT,
        reference,
        quantity);

    // assert
    BOOST_CHECK(should_be_true);
    BOOST_CHECK_EQUAL(0x30, reference);
    BOOST_CHECK_EQUAL(0x40, quantity);
}

BOOST_FIXTURE_TEST_CASE(test_parse_read_command_WithLargeReference_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "read 0x3ffff 1";

    // act
    int_least32_t reference = 0;
    size_t quantity = 0;
    const bool should_be_true = parse_read_command(
        SUT,
        reference,
        quantity);

    // assert
    BOOST_CHECK(should_be_true);
    BOOST_CHECK_EQUAL(0x3FFFF, reference);
    BOOST_CHECK_EQUAL(1, quantity);
}

BOOST_FIXTURE_TEST_CASE(test_parse_data_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "0:1:0faa:1234";

    // act
    vector<uint16_t> data;
    const bool should_be_true = parse_data(
        SUT,
        data);

    // assert
    BOOST_CHECK(should_be_true);

    const uint16_t DATA[] = {
        0,
        1,
        0x0FAA,
        0x1234
    };

    BOOST_CHECK(data == vector<uint16_t>(&DATA[0], &DATA[0] + size(DATA)));
}

BOOST_FIXTURE_TEST_CASE(test_parse_write_command_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "write 0x33 66:99:aa:bb";

    // act
    int_least32_t reference = 0;
    vector<uint16_t> data;
    const bool should_be_true = parse_write_command(
        SUT,
        reference,
        data);

    // assert
    BOOST_CHECK(should_be_true);
    BOOST_CHECK_EQUAL(0x33, reference);

    const uint16_t DATA[] = {
        0x66,
        0x99,
        0xAA,
        0xBB
    };

    BOOST_CHECK(data == vector<uint16_t>(&DATA[0], &DATA[0] + size(DATA)));
}

BOOST_FIXTURE_TEST_CASE(test_parse_swrite_command_ShouldPass, TestFixture)
{
    // arrange
    const string SUT = "swrite 0x33 66";

    // act
    int_least32_t reference = 0;
    uint16_t data;
    const bool should_be_true = parse_swrite_command(
        SUT,
        reference,
        data);

    // assert
    BOOST_CHECK(should_be_true);
    BOOST_CHECK_EQUAL(0x33, reference);
    BOOST_CHECK_EQUAL(0x66, data);
}

}
}
