#ifndef _MXMODBUSCLIENT_BUILDER_HPP_
#define _MXMODBUSCLIENT_BUILDER_HPP_

#include <string>

#include <boost/filesystem.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <mxmodbusclient/App.hpp>

#include <mxmodbusclient/error.hpp>

namespace mxmodbusclient
{

class Builder
    : private boost::noncopyable
{
public:
    Builder();
    ~Builder();

    void
        set_equipment_template_path(
            const boost::filesystem::path &);   // throws mxmodbusclient::SystemExit
    void
        set_modbus_host_port(
            const std::string &host,
            int port);   // throws mxmodbusclient::SystemExit

    void
        set_modbus_request_timeout_ms(int timeout_ms);

    void
        set_device_id(uint8_t device_id);

    void
        set_dry_run(bool dry_run);

    boost::shared_ptr<mxmodbusclient::App>
        build() const; // throws mxmodbusclient::SystemExit

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}

#endif
