#ifndef _MXMODBUSMASTER_CONFIGURATION_HPP_
#define _MXMODBUSMASTER_CONFIGURATION_HPP_

#include <string>
#include <vector>

#include <boost/cstdint.hpp>

#include <mxmodbusmaster/error.hpp>

namespace mxmodbusmaster
{

struct Configuration
{
    static const Configuration
        parse(const std::string &content); // throws mxmodbusmaster::ParseError

    struct Polling
    {
        std::string     host;
        int             service;
        unsigned int    polling_period_ms;
        unsigned int    request_timeout_ms;

        struct Device
        {
            uint8_t         device_id;
            std::string     template_id;
            std::string     equipment_name;

            Device(
                uint8_t device_id_,
                const std::string &template_id_,
                const std::string &equipment_name_)
                : device_id(device_id_)
                , template_id(template_id_)
                , equipment_name(equipment_name_)
            {
            }
        };

        std::vector<Device> device_list;

        Polling(
            const std::string &host_,
            int service_,
            unsigned int polling_period_ms_,
            unsigned int request_timeout_ms_,
            const std::vector<Device> &device_list_)
            : host(host_)
            , service(service_)
            , polling_period_ms(polling_period_ms_)
            , request_timeout_ms(request_timeout_ms_)
            , device_list(device_list_)
        {
        }
    };

    std::vector<Polling>    polling_list;
    std::string             equipment_status_path;

    Configuration(
        const std::vector<Polling> polling_list_,
        const std::string &equipment_status_path_)
        : polling_list(polling_list_)
        , equipment_status_path(equipment_status_path_)
    {
    }
};

}

#endif
