#ifndef _MXMODBUSMASTER_EQUIPMENT_TEMPLATE_HPP_
#define _MXMODBUSMASTER_EQUIPMENT_TEMPLATE_HPP_

#include <string>
#include <vector>

#include <boost/cstdint.hpp>
#include <boost/smart_ptr.hpp>

#include <mxmodbusmaster/error.hpp>
#include <mxmodbusmaster/modbus/function.hpp>
#include <mxmodbusmaster/modbus/trivial/byte_order.hpp>
#include <mxmodbusmaster/modbus/trivial/tag_type.hpp>

namespace mxmodbusmaster
{

struct EquipmentTemplate
{
    static const EquipmentTemplate
        parse(const std::string &); // throws mxmodbusmaster::ParseError

    std::string     id;

    struct Request
    {
        mxmodbusmaster::modbus::function_t  function;
        uint16_t                            address;
        uint16_t                            quantity;

        Request(
            mxmodbusmaster::modbus::function_t function_,
            uint16_t address_,
            uint16_t quantity_)
            : function(function_)
            , address(address_)
            , quantity(quantity_)
        {
        }
    };

    std::vector<Request>    request_list;

    struct Tag
    {
        std::string                                                         id;
        uint32_t                                                            reference;
        unsigned int                                                        quantity;
        mxmodbusmaster::modbus::trivial::tag_type_t                         type;
        boost::shared_ptr<std::string>                                      unit;
        boost::shared_ptr<std::vector<uint16_t> >                           invalid_value;
        boost::shared_ptr<mxmodbusmaster::modbus::trivial::byte_order_t>    byte_order;
        boost::shared_ptr<double>                                           slope;
        boost::shared_ptr<double>                                           offset;


        Tag(
            const std::string &id_,
            uint32_t reference_,
            unsigned int quantity_,
            mxmodbusmaster::modbus::trivial::tag_type_t type_)
            : id(id_)
            , reference(reference_)
            , quantity(quantity_)
            , type(type_)
        {
        }
    };

    std::vector<Tag>    tag_list;

    EquipmentTemplate(
        const std::string &id_,
        const std::vector<Request> &request_list_,
        const std::vector<Tag> &tag_list_)
        : id(id_)
        , request_list(request_list_)
        , tag_list(tag_list_)
    {
    }
};

}

#endif /* _MXMODBUSMASTER_EQUIPMENT_TEMPLATE_HPP_ */
