#ifndef _MXMODBUSMASTER_MODBUS_EQUIPMENT_HPP_
#define _MXMODBUSMASTER_MODBUS_EQUIPMENT_HPP_

#include <string>

#include <boost/function.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxmodbus/ModbusClient.hpp>

#include <mxmodbusmaster/modbus/EquipmentStatus.hpp>
#include <mxmodbusmaster/modbus/Reader.hpp>
#include <mxmodbusmaster/reading/Reading.hpp>

namespace mxmodbusmaster
{
namespace modbus
{

class Equipment
    : private boost::noncopyable
{
public:
    typedef
        uint8_t
        device_t;

    typedef
        boost::function<
            void
                (
                    const std::string &equipment_id,
                    const mxmodbusmaster::reading::Reading &
                )
        >
        PublishReadingFunction;

    typedef
        boost::function<
            void
                (
                    const mxmodbusmaster::modbus::EquipmentStatus &
                )
        >
        PublishStatusFunction;

public:
    Equipment(
        const std::string &id,
        device_t,
        boost::shared_ptr<mxmodbusmaster::modbus::Reader>);
    ~Equipment();

    const mxmodbusmaster::modbus::EquipmentStatus
        equipment_status() const;

    void
        read_and_publish(
            mxmodbus::ModbusClient &,
            const PublishReadingFunction &,
            const PublishStatusFunction &) const; // throws mxmodbusmaster::RequestError

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}
}

#endif
