#ifndef _MXMODBUSMASTER_MODBUS_EQUIPMENT_STATUS_HPP_
#define _MXMODBUSMASTER_MODBUS_EQUIPMENT_STATUS_HPP_

#include <string>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/optional.hpp>
#include <boost/shared_ptr.hpp>

namespace mxmodbusmaster
{
namespace modbus
{

class EquipmentStatus // value, no-cmp, immutable
{
public:
    class LastPoll;

public:
    EquipmentStatus(
        const std::string &equipment);
    EquipmentStatus(
        const std::string &equipment,
        const LastPoll &last_poll,
        int_least64_t total_request_count,
        int_least64_t total_request_success,
        int_least64_t total_reading_count,
        const boost::posix_time::ptime &last_success_at = boost::posix_time::not_a_date_time);
    //Equipment(const Equipment &);
    ~EquipmentStatus();

    const std::string &
        equipment() const;

    boost::optional<LastPoll>
        last_poll() const;

    boost::optional<boost::posix_time::ptime>
        last_success_at() const;

    int_least64_t
        total_request_count() const;

    int_least64_t
        total_request_success() const;

    int_least64_t
        total_reading_count() const;

    //Equipment &
    //  operator =(const Equipment &);

    const EquipmentStatus
        accumulate(const LastPoll &) const;

private:
    struct Impl;
    boost::shared_ptr<Impl> pimpl;
};

const std::pair<std::string, std::vector<uint8_t> >
    mqtt_from_equipment_status(const EquipmentStatus &);
const std::string
    json_from_equipment_status_list(
        const std::vector<EquipmentStatus> &,
        const boost::posix_time::ptime &at = boost::posix_time::microsec_clock::universal_time());

class EquipmentStatus::LastPoll // value, no-cmp, immutable
{
public:
    LastPoll(
        const boost::posix_time::ptime &at,
        const boost::posix_time::time_duration &elapsed,
        int_least64_t request_count,
        int_least64_t request_success,
        int_least64_t reading_count);
    //LastPoll(const LastPoll &);
    ~LastPoll();

    const boost::posix_time::ptime &
        at() const;
    const boost::posix_time::time_duration &
        elapsed() const;
    int_least64_t
        request_count() const;
    int_least64_t
        request_success() const;
    int_least64_t
        reading_count() const;

    //LastPoll &
    //  operator =(const LastPoll &);

private:
    struct Impl;
    boost::shared_ptr<Impl> pimpl;
};

}
}

#endif
