#ifndef _MXMODBUSMASTER_MODBUS_MASTER_HPP_
#define _MXMODBUSMASTER_MODBUS_MASTER_HPP_

#include <string>
#include <vector>

#include <boost/function.hpp>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <mxmodbusmaster/modbus/Equipment.hpp>
#include <mxmodbusmaster/reading/Reading.hpp>

namespace mxmodbusmaster
{
namespace modbus
{

class Master
    : private boost::noncopyable
{
public:
    typedef
        boost::function<
            void
                (
                    const std::string &equipment_id,
                    const mxmodbusmaster::reading::Reading &
                )
        >
        PublishReadingFunction;

    typedef
        boost::function<
            void
                (
                    const mxmodbusmaster::modbus::EquipmentStatus &
                )
        >
        PublishStatusFunction;

public:
    Master(
        const std::string &host,
        int service,
        unsigned int request_timeout_ms,
        const std::vector<boost::shared_ptr<mxmodbusmaster::modbus::Equipment> > &);
    ~Master();

    const std::vector<boost::shared_ptr<mxmodbusmaster::modbus::Equipment> > &
        equipment_list() const;
    void
        read_and_publish(
            const PublishReadingFunction &,
            const PublishStatusFunction &);

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}
}

#endif
