#ifndef _MXMODBUSMASTER_MODBUS_SPARSE_VECTOR_HPP_
#define _MXMODBUSMASTER_MODBUS_SPARSE_VECTOR_HPP_

#include <map>
#include <stdexcept>

#include <mxmodbusmaster/error.hpp>

namespace mxmodbusmaster
{
namespace modbus
{

template <typename T>
class SparseVector
{
public:
    typedef
        std::map<std::size_t, T>
        content_t;

public:
    explicit SparseVector(std::size_t size)
        : size_(size)
    {
    }

    SparseVector(const SparseVector &rhs)
        : size_(rhs.size_)
        , content_(rhs.content_)
    {
    }

    ~SparseVector()
    {
    }

    bool
        has_value(std::size_t index) const
    {
        if (index >= this->size_)
            BOOST_THROW_EXCEPTION(mxmodbusmaster::NotFoundError());

        const typename content_t::const_iterator iter = this->content_.find(index);
        return iter != this->content_.end();
    }

    const T &
        get(std::size_t index) const
    {
        if (index >= this->size_)
            BOOST_THROW_EXCEPTION(mxmodbusmaster::NotFoundError());

        const typename content_t::const_iterator iter = this->content_.find(index);
        if (iter == this->content_.end())
            BOOST_THROW_EXCEPTION(mxmodbusmaster::NotFoundError());

        return iter->second;
    }

    void
        set(std::size_t index,
            const T &value)
    {
        if (index >= this->size_)
            BOOST_THROW_EXCEPTION(mxmodbusmaster::NotFoundError());

        this->content_[index] = value;
    }

    SparseVector &
        operator =(const SparseVector &rhs)
    {
        if (this == &rhs)
            return *this;

        this->size_ = rhs.size_;
        this->content_ = rhs.content_;

        return *this;
    }

    bool
        operator ==(const SparseVector &rhs) const
    {
        if (this == &rhs)
            return true;

        if (this->size_ != rhs.size_)
            return false;

        if (this->content_ != rhs.content_)
            return false;

        return true;
    }

private:
    std::size_t size_;
    content_t   content_;
};

}
}

#endif
