#ifndef _MXMODBUSMASTER_MODBUS_TRIVIAL_TAG_HPP_
#define _MXMODBUSMASTER_MODBUS_TRIVIAL_TAG_HPP_

#include <vector>

#include <boost/cstdint.hpp>
#include <boost/date_time/posix_time/posix_time_types.hpp>
#include <boost/noncopyable.hpp>
#include <boost/optional.hpp>
#include <boost/scoped_ptr.hpp>

#include <mxmodbusmaster/error.hpp>
#include <mxmodbusmaster/reading/Reading.hpp>
#include <mxmodbusmaster/modbus/Reference.hpp>
#include <mxmodbusmaster/modbus/trivial/byte_order.hpp>
#include <mxmodbusmaster/modbus/trivial/tag_type.hpp>

namespace mxmodbusmaster
{
namespace modbus
{
namespace trivial
{

class Tag
    : private boost::noncopyable
{
public:
    Tag(const std::string &id,
        uint32_t address,
        std::size_t quantity,
        mxmodbusmaster::modbus::trivial::tag_type_t);
    ~Tag();

    void
        set_unit(const std::string &);
    void
        set_invalid_value(const std::vector<uint16_t> &);   // throws mxmodbusmaster::ArgumentError
    void
        set_byte_order(mxmodbusmaster::modbus::trivial::byte_order_t);
    void
        set_slope(double);
    void
        set_offset(double);

    const std::string &
        id() const;

    // returns boost::optional to support invalid-value
    boost::optional<mxmodbusmaster::reading::Reading>
        get_reading(const mxmodbusmaster::modbus::Reference &,
                    const boost::posix_time::ptime &now) const; // throws mxmodbusmaster::NotFoundError

private:
    struct Impl;
    boost::scoped_ptr<Impl> pimpl;
};

}
}
}

#endif
