#include <mxmodbusclient/App.hpp>

#include <boost/bind.hpp>

using boost::bind;
using boost::shared_ptr;

using mxmodbusclient::ReadingCollector;

using mxmodbusmaster::modbus::Master;

namespace mxmodbusclient
{

struct App::Impl
{
    Impl(
        shared_ptr<Master> master_,
        shared_ptr<ReadingCollector> reading_collector_)
        : master(master_)
        , reading_collector(reading_collector_)
    {
    }

    const shared_ptr<Master>                master;
    const shared_ptr<ReadingCollector>      reading_collector;
};

App::App(
    shared_ptr<Master> master,
    shared_ptr<ReadingCollector> reading_collector)
    : pimpl(new Impl(
        master,
        reading_collector
    ))
{
    BOOST_ASSERT(pimpl->master && pimpl->reading_collector);
}

App::~App()
{
}

void
    App::run()
{
    pimpl->master->read_and_publish(
        bind(
            &ReadingCollector::collect_reading,
            pimpl->reading_collector,
            _1,
            _2),
        bind(
            &ReadingCollector::collect_status,
            pimpl->reading_collector,
            _1)
    );

    pimpl->reading_collector->print_reading(std::cout);
}

}
