#include <mxmodbusclient/ReadingCollector.hpp>

#include <set>
#include <sstream>
#include <utility>

#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>

using boost::shared_ptr;

using mxmodbusclient::EInfoExitStatus;
using mxmodbusclient::exit_status;
using mxmodbusclient::SystemExit;

using mxmodbusmaster::EquipmentTemplate;
using mxmodbusmaster::modbus::EquipmentStatus;
using mxmodbusmaster::reading::Reading;

using std::endl;
using std::map;
using std::ostream;
using std::pair;
using std::string;
using std::stringstream;
using std::vector;

namespace mxmodbusclient
{

struct ReadingCollector::Impl
{
    Impl(
        const string &equipment_id_,
        const vector<EquipmentTemplate::Tag> &tag_list_)
        : equipment_id(equipment_id_)
        , tag_list(tag_list_)
    {
    }

    const string                            equipment_id;
    const vector<EquipmentTemplate::Tag>    tag_list;

    map<string, Reading>                    reading_list;
    shared_ptr<EquipmentStatus>             equipment_status;
};

ReadingCollector::ReadingCollector(
    const string &equipment_id,
    const vector<EquipmentTemplate::Tag> &tag_list)
    : pimpl(new Impl(equipment_id, tag_list))
{
}

ReadingCollector::~ReadingCollector()
{
}

void
    ReadingCollector::collect_reading(
        const string &equipment_id,
        const Reading &reading)
{
    BOOST_ASSERT(equipment_id == pimpl->equipment_id);

    pimpl->reading_list.insert(
        pair<string, Reading>(reading.tag_id(), reading));
}

void
    ReadingCollector::collect_status(
        const EquipmentStatus &status)
{
    BOOST_ASSERT(status.equipment() == pimpl->equipment_id);

    pimpl->equipment_status.reset(new EquipmentStatus(status));
}

void
    ReadingCollector::print_reading(ostream &os)
{
    if (!pimpl->equipment_status)
        BOOST_THROW_EXCEPTION(SystemExit() << EInfoExitStatus(exit_status::IO_ERROR));

    BOOST_ASSERT(pimpl->equipment_status->last_poll());

    os << "elapsed-ms: " << pimpl->equipment_status->last_poll()->elapsed().total_milliseconds() << endl;

    int error_count = 0;
    BOOST_FOREACH(EquipmentTemplate::Tag tag, pimpl->tag_list)
    {
        const map<string, Reading>::const_iterator iter = pimpl->reading_list.find(tag.id);
        if (iter == pimpl->reading_list.end())
        {
            os << tag.id << ": " << "error" << endl;
            error_count++;
            continue;
        }

        os << tag.id << ": " << iter->second.value().to_json_value() << endl;
    }

    if (error_count)
        BOOST_THROW_EXCEPTION(SystemExit() << EInfoExitStatus(exit_status::MODBUS_ERROR));
}

}
