#include <mxmodbusmaster/App.hpp>

#include <cstdio>
#include <fstream>

#include <boost/foreach.hpp>

#include <mxutil/log.hpp>

using boost::filesystem::path;
using boost::shared_ptr;
using boost::system::error_code;

using mxutil::sched::Scheduler;

using mxmodbusmaster::modbus::Equipment;
using mxmodbusmaster::modbus::EquipmentStatus;

using std::ios_base;
using std::ofstream;
using std::string;
using std::vector;

namespace
{

void
    create_directory_if_not_exist(const path &p)
{
    error_code ec;

    if (!is_directory(p, ec))
    {
        create_directories(p, ec);
    }
}

}

namespace mxmodbusmaster
{

struct App::Impl
{
    Impl(
        const vector<shared_ptr<Equipment> > &equipment_list_,
        const path &status_file_,
        const vector<shared_ptr<Scheduler> > &scheduler_list_)
        : equipment_list(equipment_list_)
        , status_file(status_file_)
        , scheduler_list(scheduler_list_)
    {
    }

    const vector<shared_ptr<Equipment> >    equipment_list;
    const path                              status_file;
    const vector<shared_ptr<Scheduler> >    scheduler_list;
};

App::App(
    const vector<shared_ptr<Equipment> > &equipment_list,
    const path &status_file,
    const vector<shared_ptr<Scheduler> > &scheduler_list)
    : pimpl(new Impl(
        equipment_list,
        status_file,
        scheduler_list))
{
}

App::~App()
{
}

void
    App::loop()
{
    // TODO check whether publisher is connected

    BOOST_FOREACH(shared_ptr<Scheduler> s, pimpl->scheduler_list)
    {
        BOOST_ASSERT(s);
        s->check_and_fire();
    }
}

void
    App::export_equipment_status()
{
    vector<EquipmentStatus> equipment_status_list;
    BOOST_FOREACH(shared_ptr<Equipment> equipment, pimpl->equipment_list)
    {
        equipment_status_list.push_back(equipment->equipment_status());
    }

    const string json = json_from_equipment_status_list(equipment_status_list);

    // create directory
    create_directory_if_not_exist(pimpl->status_file.parent_path());

    // write to temp-file
    const path tmp = pimpl->status_file.string() + ".part";
    ofstream outfile(tmp.c_str(), ios_base::binary);
    if (!outfile)
    {
        MXLOG_WARN("open file failed: " + tmp.string());
        return;
    }

    outfile.write(json.c_str(), json.size());
    outfile.close();

    // rename to actual file
    if (0 != rename(tmp.string().c_str(), pimpl->status_file.string().c_str()))
    {
        MXLOG_WARN("rename file failed: from " + tmp.string() + ", to " + pimpl->status_file.string());
        return;
    }
}

}
