#include <mxmodbusmaster/Option.hpp>

#include <boost/program_options.hpp>

#include <config.h>

namespace mxmodbusmaster
{

using boost::logic::tribool;
using boost::logic::indeterminate;
using boost::program_options::options_description;
using boost::program_options::value;
using boost::program_options::variables_map;
using boost::program_options::command_line_style::default_style;
using boost::program_options::command_line_style::allow_guessing;

using std::cout;
using std::endl;
using std::string;

Option::Option()
    : foreground(false)
    , mqtt_broker_host("127.0.0.1")
    , mqtt_broker_port(1883)
{
}

tribool
    Option::parse_args(
        int argc,
        const char *argv[])
{
    options_description desc("Allowed options");
    desc.add_options()
        ("foreground,f", "Do not daemonize.")
        ("mqtt-broker-host", value<string>(), "Change MQTT broker address.")
        ("mqtt-broker-port", value<int>(), "Change MQTT borker port.")
        ("help", "Print this manual.")
        ("version", "Print version.");

    variables_map vm;
    try
    {
        store(
            parse_command_line(argc, argv, desc, default_style & (~allow_guessing)),
            vm
        );
    }
    catch(boost::program_options::error &e)
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("help"))
    {
        cout << desc << endl;
        return indeterminate;
    }

    if (vm.count("version"))
    {
        cout << "mxmodbusmaster v" << VERSION << endl;
        return indeterminate;
    }

    if (vm.count("f") || vm.count("foreground"))
        this->foreground = true;

    if (vm.count("mqtt-broker-host"))
        this->mqtt_broker_host = vm["mqtt-broker-host"].as<string>();

    if (vm.count("mqtt-broker-port"))
        this->mqtt_broker_port = vm["mqtt-broker-port"].as<int>();

    return true;
}

}
