#include <mxmodbusmaster/modbus/Master.hpp>

#include <boost/bind.hpp>
#include <boost/foreach.hpp>

#include <mxutil/log.hpp>

#include <mxmodbus/ModbusClientProxy.hpp>
#include <mxmodbus/ModbusTcpClient.hpp>

namespace mxmodbusmaster
{
namespace modbus
{

using boost::bind;
using boost::scoped_ptr;
using boost::shared_ptr;

using mxmodbus::ModbusClientProxy;
using mxmodbus::ModbusTcpClient;

using std::string;
using std::vector;

namespace
{
    static const int DEFAULT_REQUEST_TIMEOUT_MS = 30 * 1000;
}

struct Master::Impl
{
    Impl(
        const string &host_,
        int service_,
        unsigned int request_timeout_ms_,
        const vector<shared_ptr<Equipment> > &device_list_)
        : host(host_)
        , service(service_)
        , request_timeout_ms(request_timeout_ms_)
        , device_list(device_list_)
    {
    }

    const string                            host;
    const int                               service;
    const unsigned int                      request_timeout_ms;
    const vector<shared_ptr<Equipment> >    device_list;

    scoped_ptr<ModbusClientProxy>           modbus_client;

    shared_ptr<ModbusTcpClient>
        create_modbus_tcp_client() const
    {
        return shared_ptr<ModbusTcpClient>(new ModbusTcpClient(
            host,
            service,
            request_timeout_ms
        ));
    }
};

Master::Master(
    const string &host,
    int service,
    unsigned int request_timeout_ms,
    const vector<shared_ptr<Equipment> > &device_list)
    : pimpl(new Impl(host, service, request_timeout_ms, device_list))
{
    pimpl->modbus_client.reset(new ModbusClientProxy(bind(
        &Impl::create_modbus_tcp_client,
        &*pimpl)));
}

Master::~Master()
{
}

void
    Master::read_and_publish(
        const PublishReadingFunction &publish_reading,
        const PublishStatusFunction &publish_status)
{
    BOOST_FOREACH(shared_ptr<Equipment> device, pimpl->device_list)
    {
        try
        {
            BOOST_ASSERT(device);
            device->read_and_publish(
                *pimpl->modbus_client,
                publish_reading,
                publish_status); // throws RequestError
        }
        catch(RequestError &e)
        {
            MXLOG_WARN(diagnostic_information(e));

            continue;
        }
    }
}

}
}
