#include <mxmodbusmaster/modbus/ReadDiscreteInputsCommand.hpp>

#include <boost/foreach.hpp>

namespace mxmodbusmaster
{
namespace modbus
{

using mxmodbus::ModbusClient;

using std::vector;

namespace
{

const vector<uint16_t>
    word_vector_from_bool_vector(const vector<bool> &from)
{
    vector<uint16_t> to;
    BOOST_FOREACH(bool b, from)
    {
        to.push_back(b);
    }

    return to;
}

}

struct ReadDiscreteInputsCommand::Impl
{
    Impl(
        address_t addr_,
        size_t quantity_)
        : addr(addr_)
        , quantity(quantity_)
    {
    }

    const address_t addr;
    const size_t    quantity;
};

ReadDiscreteInputsCommand::ReadDiscreteInputsCommand(
    address_t addr,
    size_t quantity)
    : pimpl(new Impl(addr, quantity))
{
}

ReadDiscreteInputsCommand::~ReadDiscreteInputsCommand()
{
}

void
    ReadDiscreteInputsCommand::run(
        device_t device,
        ModbusClient &modbus,
        Reference &ref) const
{
    const vector<bool> buf = modbus.read_discrete_inputs(device, pimpl->addr, pimpl->quantity);
    ref.set(Reference::DISCRETE_INPUT_BASE + pimpl->addr, word_vector_from_bool_vector(buf));
}

}
}
