#include <mxmodbusmaster/modbus/trivial/byte_order.hpp>

#include <map>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/range.hpp>

#include <mxutil/log.hpp>

namespace mxmodbusmaster
{
namespace modbus
{
namespace trivial
{

using boost::lexical_cast;
using boost::size;

using std::map;
using std::string;
using std::vector;

namespace
{

class ReorderProfile
{
private:
    typedef
        map<
            byte_order_t,
            vector<size_t>
        >
        profile_t;

public:
    ReorderProfile()
    {
        const size_t BA[] = {1, 0};
        this->profile_[BO_BA] = vector<size_t>(BA, BA + size(BA));

        const size_t CDAB[] = {2, 3, 0, 1};
        this->profile_[BO_CDAB] = vector<size_t>(CDAB, CDAB + size(CDAB));

        const size_t DCBA[] = {3, 2, 1, 0};
        this->profile_[BO_DCBA] = vector<size_t>(DCBA, DCBA + size(DCBA));
    }

    const vector<size_t>
        get_profile(byte_order_t bo) const
    {
        profile_t::const_iterator iter = this->profile_.find(bo);
        if (iter == profile_.end())
        {
            MXLOG_EROR("profile not found: " + lexical_cast<string>(static_cast<int>(bo)));

            BOOST_THROW_EXCEPTION(ArgumentError());
        }

        return iter->second;
    }

private:
    profile_t profile_;
};

}

const vector<uint8_t>
    reorder(
        byte_order_t bo,
        const vector<uint8_t> &from)
{
    static const ReorderProfile profiles;

    if (bo == BO_NC)
        return from;

    const vector<size_t> profile = profiles.get_profile(bo);
    vector<uint8_t> buf = from;
    if (profile.size() != buf.size())
    {
        MXLOG_WARN("insufficient length: expecting "
            + lexical_cast<string>(profile.size()) + ", but "
            + lexical_cast<string>(buf.size()));
        buf.resize(profile.size(), 0x00);
    }

    vector<uint8_t> rtn;
    BOOST_FOREACH(size_t index, profile)
    {
        rtn.push_back(buf.at(index));
    }

    return rtn;
}

}
}
}
