#include <mxmodbusmaster/modbus/trivial/tag_convert.hpp>

#include <algorithm>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#include <mxutil/log.hpp>

namespace mxmodbusmaster
{
namespace modbus
{
namespace trivial
{

using boost::lexical_cast;

using std::string;
using std::vector;

namespace
{

float
    interpret_float_from_uint32(uint32_t buf)
{
    union
    {
        float cast_float;
        uint32_t cast_buf;
    };

    cast_buf = buf;

    return cast_float;
}

double
    interpret_double_from_uint64(uint64_t buf)
{
    union
    {
        double cast_double;
        uint64_t cast_buf;
    };
    cast_buf = buf;

    return cast_double;
}

}

uint16_t
    uint16_from_binary(const vector<uint8_t> &buf)
{
    if (buf.size() != 2)
    {
        MXLOG_WARN("unexpected length: expecting 2, but " + lexical_cast<string>(buf.size()));
    }

    uint16_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return lbuf;
}

uint32_t
    uint32_from_binary(const vector<uint8_t> &buf)
{
    if (buf.size() != 4)
    {
        MXLOG_WARN("unexpected length: expecting 4, but " + lexical_cast<string>(buf.size()));
    }

    uint32_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return lbuf;
}

uint64_t
    uint64_from_binary(const vector<uint8_t> &buf)
{
    if (buf.size() != 8)
    {
        MXLOG_WARN("unexpected length: expecting 8, but " + lexical_cast<string>(buf.size()));
    }

    uint64_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return lbuf;
}

int16_t
    int16_from_binary(const vector<uint8_t> &buf)
{
    if (buf.size() != 2)
    {
        MXLOG_WARN("unexpected length: expecting 2, but " + lexical_cast<string>(buf.size()));
    }

    uint16_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return*reinterpret_cast<int16_t *>(&lbuf);
}

int32_t
    int32_from_binary(const vector<uint8_t> &buf)
{
    if (buf.size() != 4)
    {
        MXLOG_WARN("unexpected length: expecting 4, but " + lexical_cast<string>(buf.size()));
    }

    uint32_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return *reinterpret_cast<int32_t *>(&lbuf);
}

int64_t
    int64_from_binary(const vector<uint8_t> &buf)
{
    if (buf.size() != 8)
    {
        MXLOG_WARN("unexpected length: expecting 8, but " + lexical_cast<string>(buf.size()));
    }

    uint64_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return*reinterpret_cast<int64_t *>(&lbuf);
}

float
    float32_from_binary(const vector<uint8_t> &buf)
{
    BOOST_STATIC_ASSERT(sizeof(float) == 4);

    if (buf.size() != 4)
    {
        MXLOG_WARN("unexpected length: expecting 4, but " + lexical_cast<string>(buf.size()));
    }

    uint32_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return interpret_float_from_uint32(lbuf);
}

double
    float64_from_binary(const vector<uint8_t> &buf)
{
    BOOST_STATIC_ASSERT(sizeof(double) == 8);

    if (buf.size() != 8)
    {
        MXLOG_WARN("unexpected length: expecting 8, but " + lexical_cast<string>(buf.size()));
    }

    uint64_t lbuf = 0;
    BOOST_FOREACH(uint8_t b, buf)
    {
        lbuf <<= 8;
        lbuf += b;
    }

    return interpret_double_from_uint64(lbuf);
}

const string
    string_from_binary(const vector<uint8_t> &buf)
{
    const vector<uint8_t>::const_iterator iter = find(buf.begin(), buf.end(), 0);
    return string(buf.begin(), iter);
}

}
}
}
