#include <mxmodbusmaster/reading/IntValue.hpp>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/make_shared.hpp>

#include <mxmodbusmaster/reading/FloatValue.hpp>

namespace mxmodbusmaster
{
namespace reading
{

using boost::lexical_cast;
using boost::make_shared;
using boost::shared_ptr;

using std::string;
using std::stringstream;

struct IntValue::Impl
{
    Impl(int64_t v_)
        : v(v_)
    {
    }

    const int64_t   v;
};

IntValue::IntValue(int64_t v)
    : pimpl(new Impl(v))
{
}

IntValue::~IntValue()
{
}

shared_ptr<ValueBase>
    IntValue::multiply(double m) const
{
    return make_shared<FloatValue>(static_cast<double>(pimpl->v) * m);
}

shared_ptr<ValueBase>
    IntValue::add(double a) const
{
    return make_shared<FloatValue>(static_cast<double>(pimpl->v) + a);
}

const string
    IntValue::to_json_value() const
{
    return lexical_cast<string>(pimpl->v);
}

const mxtag::Value
    IntValue::to_mxtag_value() const
{
    return pimpl->v;
}

}
}
