#include <mxmodbusmaster/reading/Reading.hpp>

#include <boost/make_shared.hpp>

namespace mxmodbusmaster
{
namespace reading
{

using boost::make_shared;
using boost::posix_time::ptime;
using boost::shared_ptr;

using std::string;

struct Reading::Impl
{
    Impl(
        const string &tag_id_,
        const ptime &timestamp_,
        const Value &value_,
        shared_ptr<string> unit_)
        : tag_id(tag_id_)
        , timestamp(timestamp_)
        , value(value_)
        , unit(unit_)
    {
    }

    const string                    tag_id;
    const ptime                     timestamp;
    const Value                     value;
    const shared_ptr<const string>  unit;
};

Reading::Reading(
    const string &tag_id,
    const ptime &timestamp,
    const Value &value)
    : pimpl(new Impl(
        tag_id,
        timestamp,
        value,
        shared_ptr<string>()))
{
}

Reading::Reading(
    const string &tag_id,
    const ptime &timestamp,
    const Value &value,
    const string &unit)
    : pimpl(new Impl(
        tag_id,
        timestamp,
        value,
        make_shared<string>(unit)))
{
}

Reading::~Reading()
{
}

const string &
    Reading::tag_id() const
{
    return pimpl->tag_id;
}

const ptime &
    Reading::timestamp() const
{
    return pimpl->timestamp;
}

const Value &
    Reading::value() const
{
    return pimpl->value;
}

const string *
    Reading::unit() const
{
    if (!pimpl->unit)
        return NULL;
    else
        return &*pimpl->unit;
}

}
}
