#include <mxmodbusmaster/reading/UIntValue.hpp>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/make_shared.hpp>

#include <mxmodbusmaster/reading/FloatValue.hpp>

namespace mxmodbusmaster
{
namespace reading
{

using boost::lexical_cast;
using boost::make_shared;
using boost::shared_ptr;

using std::string;

struct UIntValue::Impl
{
    Impl(uint64_t v_)
        : v(v_)
    {
    }

    const uint64_t   v;
};

UIntValue::UIntValue(uint64_t v)
    : pimpl(new Impl(v))
{
}

UIntValue::~UIntValue()
{
}

shared_ptr<ValueBase>
    UIntValue::multiply(double m) const
{
    return make_shared<FloatValue>(static_cast<double>(pimpl->v) * m);
}

shared_ptr<ValueBase>
    UIntValue::add(double a) const
{
    return make_shared<FloatValue>(static_cast<double>(pimpl->v) + a);
}

const string
    UIntValue::to_json_value() const
{
    return lexical_cast<string>(pimpl->v);
}

const mxtag::Value
    UIntValue::to_mxtag_value() const
{
    return pimpl->v;
}

}
}
