#include <mxmodbusmaster/reading/Value.hpp>

#include <boost/assert.hpp>
#include <boost/make_shared.hpp>

#include <mxmodbusmaster/reading/FloatValue.hpp>
#include <mxmodbusmaster/reading/IntValue.hpp>
#include <mxmodbusmaster/reading/UIntValue.hpp>
#include <mxmodbusmaster/reading/StringValue.hpp>

namespace mxmodbusmaster
{
namespace reading
{

using boost::shared_ptr;
using boost::make_shared;

using std::string;

const Value
    Value::get(uint64_t v)
{
    return Value(make_shared<UIntValue>(v));
}

const Value
    Value::get(int64_t v)
{
    return Value(make_shared<IntValue>(v));
}

const Value
    Value::get(double v)
{
    return Value(make_shared<FloatValue>(v));
}

const Value
    Value::get(const string &v)
{
    return Value(make_shared<StringValue>(v));
}

Value::Value(shared_ptr<ValueBase> internal)
    : internal_(internal)
{
    BOOST_ASSERT(this->internal_);
}

Value::~Value()
{
}

const Value
    Value::multiply(double m) const
{
    return Value(this->internal_->multiply(m));
}

const Value
    Value::add(double a) const
{
    return Value(this->internal_->add(a));
}

const string
    Value::to_json_value() const
{
    return this->internal_->to_json_value();
}

const mxtag::Value
    Value::to_mxtag_value() const
{
    return this->internal_->to_mxtag_value();
}

}
}
