#include <boost/test/unit_test.hpp>

#include <mxmodbusmaster/modbus/SparseVector.hpp>

using mxmodbusmaster::modbus::SparseVector;

namespace
{

struct TestFixture
{
    TestFixture()
        : sparse_vector(10)
    {
    }

    SparseVector<int>   sparse_vector;
};

}

namespace mxmodbusmaster
{
namespace modbus
{

BOOST_FIXTURE_TEST_CASE(test_SparseVector_has_value_WhenNotExist_ShouldReturnFalse, TestFixture)
{
    // arrange

    // act
    const bool result = sparse_vector.has_value(3);

    // assert
    BOOST_CHECK_EQUAL(false, result);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_has_value_WhenExist_ShouldReturnTrue, TestFixture)
{
    // arrange
    sparse_vector.set(3, 10);

    // act
    const bool result = sparse_vector.has_value(3);

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_get_WhenExist_ShouldPass, TestFixture)
{
    // arrange
    sparse_vector.set(3, 10);

    // act
    const int value = sparse_vector.get(3);

    // assert
    BOOST_CHECK_EQUAL(10, value);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_get_WhenNotExist_ShouldThrowNotFoundError, TestFixture)
{
    // arrange

    // act & assert
    BOOST_CHECK_THROW(sparse_vector.get(3), NotFoundError);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_get_WithIndexOutOfRange_ShouldThrowOutOfRange, TestFixture)
{
    // arrange

    // act && assert
    BOOST_CHECK_THROW(sparse_vector.get(10), NotFoundError);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_set_WithIndexOutOfRange_ShouldThrowOutOfRange, TestFixture)
{
    // arrange

    // act && assert
    BOOST_CHECK_THROW(sparse_vector.set(10, -1), NotFoundError);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_operator_equ_ShouldPass, TestFixture)
{
    // arrange
    sparse_vector.set(3, 10);
    sparse_vector.set(6, 20);

    SparseVector<int> sparse_vector2(10);
    sparse_vector2.set(6, 20);
    sparse_vector2.set(3, 10);

    // act
    const bool result = sparse_vector == sparse_vector2;

    // assert
    BOOST_CHECK_EQUAL(true, result);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_copy_ctor_ShouldPass, TestFixture)
{
    // arrange
    sparse_vector.set(3, 10);
    sparse_vector.set(6, 20);

    // act
    const SparseVector<int> lhs = sparse_vector;

    // assert
    BOOST_CHECK(sparse_vector == lhs);
}

BOOST_FIXTURE_TEST_CASE(test_SparseVector_operator_assignment_ShouldPass, TestFixture)
{
    // arrange
    sparse_vector.set(3, 10);
    sparse_vector.set(6, 20);

    // act
    SparseVector<int> lhs(1);
    lhs = sparse_vector;

    // assert
    BOOST_CHECK(sparse_vector == lhs);
}

}
}
