#include <boost/test/unit_test.hpp>

#include <mxmodbusmaster/modbus/trivial/Tag.hpp>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/range.hpp>

namespace mxmodbusmaster
{
namespace modbus
{
namespace trivial
{

using boost::lexical_cast;
using boost::optional;
using boost::posix_time::time_from_string;
using boost::size;

using mxmodbusmaster::reading::Reading;

using std::vector;

namespace
{

struct TestFixture
{
    TestFixture()
    {
        ref.set(0x40000, 0x0123);
        ref.set(0x40001, 0x4567);
    }

    Reference   ref;
};

}

BOOST_FIXTURE_TEST_CASE(test_Tag_get_reading_ShouldPass, TestFixture)
{
    // arrange
    Tag tag("tag-id", 0x40000, 2, TAGT_UINT32);
    tag.set_unit("meters");
    tag.set_byte_order(BO_DCBA);
    tag.set_slope(0.123);
    tag.set_offset(0.456);

    // act
    optional<Reading> reading = tag.get_reading(ref, time_from_string("2000-01-01 00:00:00"));

    // assert
    BOOST_REQUIRE(reading);
    BOOST_CHECK_CLOSE(213107856.195, lexical_cast<double>(reading->value().to_json_value()), 0.0001);
}

BOOST_FIXTURE_TEST_CASE(test_Tag_get_reading_WithInvalidValue_ShouldReturnNone, TestFixture)
{
    // arrange
    const uint16_t INVALID_VALUE[] = {0x0123, 0x4567};
    Tag tag("tag-id", 0x40000, 2, TAGT_UINT32);
    tag.set_invalid_value(vector<uint16_t>(INVALID_VALUE, INVALID_VALUE + size(INVALID_VALUE)));

    // act
    optional<Reading> reading = tag.get_reading(ref, time_from_string("2000-01-01 00:00:00"));

    // assert
    BOOST_CHECK(!reading);
}

BOOST_FIXTURE_TEST_CASE(test_Tag_get_reading_WithoutUnitShouldPass, TestFixture)
{
    // arrange
    Tag tag("tag-id", 0x40000, 2, TAGT_UINT32);

    // act
    optional<Reading> reading = tag.get_reading(ref, time_from_string("2000-01-01 00:00:00"));

    // assert
    BOOST_CHECK(NULL == reading->unit());
}

}
}
}
