#include <boost/test/unit_test.hpp>

#include <mxmodbusmaster/modbus/trivial/byte_order.hpp>

#include <string>

#include <boost/range.hpp>

namespace mxmodbusmaster
{
namespace modbus
{
namespace trivial
{

using boost::size;

using std::string;
using std::vector;

namespace
{

class TestFixture
{
};

}

BOOST_FIXTURE_TEST_CASE(test_reorder_WithNC_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t ABCD[] = {'A', 'B', 'C', 'D'};

    // act
    const vector<uint8_t> result = reorder(
        BO_NC,
        vector<uint8_t>(ABCD, ABCD + size(ABCD))
    );

    // assert
    BOOST_CHECK_EQUAL("ABCD", string(result.begin(), result.end()));
}

BOOST_FIXTURE_TEST_CASE(test_reorder_WithBA_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t AB[] = {'A', 'B'};

    // act
    const vector<uint8_t> result = reorder(
        BO_BA,
        vector<uint8_t>(AB, AB + size(AB))
    );

    // assert
    BOOST_CHECK_EQUAL("BA", string(result.begin(), result.end()));
}

BOOST_FIXTURE_TEST_CASE(test_reorder_WithCDAB_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t ABCD[] = {'A', 'B', 'C', 'D'};

    // act
    const vector<uint8_t> result = reorder(
        BO_CDAB,
        vector<uint8_t>(ABCD, ABCD + size(ABCD))
    );

    // assert
    BOOST_CHECK_EQUAL("CDAB", string(result.begin(), result.end()));
}

BOOST_FIXTURE_TEST_CASE(test_reorder_WithDCBA_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t ABCD[] = {'A', 'B', 'C', 'D'};

    // act
    const vector<uint8_t> result = reorder(
        BO_DCBA,
        vector<uint8_t>(ABCD, ABCD + size(ABCD))
    );

    // assert
    BOOST_CHECK_EQUAL("DCBA", string(result.begin(), result.end()));
}

}
}
}
