#include <boost/test/unit_test.hpp>

#include <mxmodbusmaster/modbus/trivial/tag_convert.hpp>

#include <boost/range.hpp>

namespace mxmodbusmaster
{
namespace modbus
{
namespace trivial
{

using boost::size;

using std::string;
using std::vector;

namespace
{

class TestFixture
{
};

}

BOOST_FIXTURE_TEST_CASE(test_uint16_from_binary_ShouldPass, TestFixture)
{
    // arraneg
    const uint8_t FROM[] = {0x12, 0x34};

    // act
    const uint16_t result = uint16_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_EQUAL(0x1234, result);
}

BOOST_FIXTURE_TEST_CASE(test_uint32_from_binary_ShouldPass, TestFixture)
{
    // arraneg
    const uint8_t FROM[] = {0x12, 0x34, 0x56, 0x78};

    // act
    const uint32_t result = uint32_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_EQUAL(0x12345678, result);
}

BOOST_FIXTURE_TEST_CASE(test_uint64_from_binary_ShouldPass, TestFixture)
{
    // arraneg
    const uint8_t FROM[] = {0x12, 0x34, 0x56, 0x78, 0xAB, 0xCD, 0xEF, 0x01};

    // act
    const uint64_t result = uint64_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_EQUAL(0x12345678ABCDEF01ull, result);
}

BOOST_FIXTURE_TEST_CASE(test_int16_from_binary_ShouldPass, TestFixture)
{
    // arraneg
    const int16_t from = -1000;
    const uint8_t FROM[] = {
        (from >> 8) & 0xFF,
        (from >> 0) & 0xFF
    };

    // act
    const int16_t result = int16_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_EQUAL(-1000, result);
}

BOOST_FIXTURE_TEST_CASE(test_int32_from_binary_ShouldPass, TestFixture)
{
    // arraneg
    const int32_t from = -1000000;
    const uint8_t FROM[] = {
        (from >> 24) & 0xFF,
        (from >> 16) & 0xFF,
        (from >> 8) & 0xFF,
        (from >> 0) & 0xFF
    };

    // act
    const int32_t result = int32_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_EQUAL(-1000000, result);
}

BOOST_FIXTURE_TEST_CASE(test_int64_from_binary_ShouldPass, TestFixture)
{
    // arraneg
    const int64_t from = -1000000000000ll;
    const uint8_t FROM[] = {
        (from >> 56) & 0xFF,
        (from >> 48) & 0xFF,
        (from >> 40) & 0xFF,
        (from >> 32) & 0xFF,
        (from >> 24) & 0xFF,
        (from >> 16) & 0xFF,
        (from >> 8) & 0xFF,
        (from >> 0) & 0xFF
    };

    // act
    const int64_t result = int64_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_EQUAL(-1000000000000ll, result);
}

BOOST_FIXTURE_TEST_CASE(test_float32_from_binary_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t FROM[] = {0x40, 0x49, 0x0F, 0xDB};

    // act
    const float result = float32_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_CLOSE(3.14159265, result, 0.0001);
}

BOOST_FIXTURE_TEST_CASE(test_float64_from_binary_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t FROM[] = {0xC0, 0x09, 0x21, 0xFB, 0x53, 0xC8, 0xD4, 0xF1};

    // act
    const double result = float64_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_CLOSE(-3.14159265, result, 0x0001);
}

BOOST_FIXTURE_TEST_CASE(test_string_from_binary_ShouldPass, TestFixture)
{
    // arrange
    const uint8_t FROM[] = {'A', 'B', 'C', '1', '2', '3'};

    // act
    const string result = string_from_binary(vector<uint8_t>(FROM, FROM + size(FROM)));

    // assert
    BOOST_CHECK_EQUAL("ABC123", result);
}

}
}
}
