#include <boost/test/unit_test.hpp>

#include <mxmodbusmaster/reading/Reading.hpp>

#include <boost/date_time/posix_time/posix_time.hpp>

namespace mxmodbusmaster
{
namespace reading
{

using boost::posix_time::time_from_string;

using std::string;

namespace
{

class TestFixture
{
};

}

BOOST_FIXTURE_TEST_CASE(test_Reading_ctor_ShouldPass, TestFixture)
{
    // arrange

    // act
    const Reading reading(
        "tag-id",
        time_from_string("2000-01-01 00:00:00.000"),
        Value::get("hello-world"));

    // assert
    BOOST_CHECK_EQUAL("tag-id", reading.tag_id());
    BOOST_CHECK_EQUAL(time_from_string("2000-01-01 00:00:00.000"), reading.timestamp());
    BOOST_CHECK_EQUAL("\"hello-world\"", reading.value().to_json_value());
    BOOST_CHECK(NULL == reading.unit());
}

BOOST_FIXTURE_TEST_CASE(test_Reading_ctor_WithUnit_ShouldPass, TestFixture)
{
    // arrange

    // act
    const Reading reading(
        "tag-id",
        time_from_string("2000-01-01 00:00:00.000"),
        Value::get("hello-world"),
        "count");

    // assert
    BOOST_CHECK_EQUAL("tag-id", reading.tag_id());
    BOOST_CHECK_EQUAL(time_from_string("2000-01-01 00:00:00.000"), reading.timestamp());
    BOOST_CHECK_EQUAL("\"hello-world\"", reading.value().to_json_value());
    BOOST_REQUIRE(NULL != reading.unit());
    BOOST_CHECK_EQUAL(string("count"), *reading.unit());
}

}
}
