#include <boost/test/unit_test.hpp>

#include <mxmodbusmaster/reading/Value.hpp>

#include <boost/lexical_cast.hpp>

namespace mxmodbusmaster
{
namespace reading
{

using boost::lexical_cast;

namespace
{

class TestFixture
{
};

}

BOOST_FIXTURE_TEST_CASE(test_Value_get_WithUInt_ShouldPass, TestFixture)
{
    // arrange

    // act
    const Value v = Value::get(static_cast<uint64_t>(123));

    // assert
    BOOST_CHECK_EQUAL(123, lexical_cast<int>(v.to_json_value()));
}

BOOST_FIXTURE_TEST_CASE(test_Value_get_WithInt_ShouldPass, TestFixture)
{
    // arrange

    // act
    const Value v = Value::get(static_cast<int64_t>(-123));

    // assert
    BOOST_CHECK_EQUAL(-123, lexical_cast<int>(v.to_json_value()));
}

BOOST_FIXTURE_TEST_CASE(test_Value_get_WithDouble_ShouldPass, TestFixture)
{
    // arrange

    // act
    const Value v = Value::get(static_cast<double>(123.456));

    // assert
    BOOST_CHECK_CLOSE(123.456, lexical_cast<double>(v.to_json_value()), 0.0001);
}

BOOST_FIXTURE_TEST_CASE(test_Value_get_WithString_ShouldPass, TestFixture)
{
    // arrange

    // act
    const Value v = Value::get("hello-world");

    // assert
    BOOST_CHECK_EQUAL("\"hello-world\"", v.to_json_value());
}

BOOST_FIXTURE_TEST_CASE(test_Value_multiply_ShouldPass, TestFixture)
{
    // arrange
    const Value v = Value::get(static_cast<int64_t>(3));

    // act
    const Value result = v.multiply(1.5);

    // assert
    BOOST_CHECK_CLOSE(4.5, lexical_cast<double>(result.to_json_value()), 0.0001);
}

BOOST_FIXTURE_TEST_CASE(test_Value_add_ShouldPass, TestFixture)
{
    // arrange
    const Value v = Value::get(static_cast<int64_t>(3));

    // act
    const Value result = v.add(1.5);

    // assert
    BOOST_CHECK_CLOSE(4.5, lexical_cast<double>(result.to_json_value()), 0.0001);
}

}
}
