#include <boost/test/unit_test.hpp>

#include <mxmodbusmaster/Option.hpp>

#include <boost/range.hpp>

using boost::logic::indeterminate;
using boost::logic::tribool;
using boost::size;

namespace
{

class TestFixture
{
};

}

namespace mxmodbusmaster
{

BOOST_FIXTURE_TEST_CASE(test_Option_ctor_ShouldSetDefault, TestFixture)
{
    // arrange

    // act
    Option opt;

    // assert
    BOOST_CHECK_EQUAL(false, opt.foreground);
    BOOST_CHECK_EQUAL("127.0.0.1", opt.mqtt_broker_host);
    BOOST_CHECK_EQUAL(1883, opt.mqtt_broker_port);
}

BOOST_FIXTURE_TEST_CASE(test_Option_parse_args_ShouldPass, TestFixture)
{
    // arrange
    const char *args[] = {
        "command-line",
        "-f",
        "--mqtt-broker-host", "192.168.31.78",
        "--mqtt-broker-port", "1884"
    };

    // act
    Option opt;
    const tribool result = opt.parse_args(size(args), args);

    // assert
    BOOST_CHECK_EQUAL(true, result);
    BOOST_CHECK_EQUAL(true, opt.foreground);
    BOOST_CHECK_EQUAL("192.168.31.78", opt.mqtt_broker_host);
    BOOST_CHECK_EQUAL(1884, opt.mqtt_broker_port);
}

BOOST_FIXTURE_TEST_CASE(test_Option_parse_args_WithHelp_ShouldPass, TestFixture)
{
    // arrange
    const char *args[] = {
        "command-line",
        "--help"
    };

    // act
    Option opt;
    const tribool result = opt.parse_args(size(args), args);

    // assert
    BOOST_CHECK(indeterminate(result));
}

BOOST_FIXTURE_TEST_CASE(test_Option_parse_args_WithIllegalArgument_ShouldPass, TestFixture)
{
    // arrange
    const char *args[] = {
        "command-line",
        "--some-thing-unexpected"
    };

    // act
    Option opt;
    const tribool result = opt.parse_args(size(args), args);

    // assert
    BOOST_CHECK_EQUAL(false, result);
}

}
