#include <boost/test/unit_test.hpp>

#include <mxserialframer/Option.hpp>

#include <boost/range.hpp>

using boost::asio::serial_port_base;
using boost::logic::tribool;
using boost::size;

namespace
{

class TestFixture
{
};

}

namespace mxserialframer
{

BOOST_FIXTURE_TEST_CASE(test_Option_ShouldPass, TestFixture)
{
    // arrange
    const char *args[] = {
        "dummy",
        "--device=/dev/ttyS0",
        "--config=115200-8-n-2",
        "--noinput-timeout-ms=30",
        "--overflow-size=40"
    };

    Option opt;

    // act
    const tribool result = opt.parse_args(size(args), args);

    // assert
    BOOST_CHECK(result);

    BOOST_CHECK_EQUAL("/dev/ttyS0", opt.device);
    BOOST_CHECK_EQUAL(115200, opt.config.baud.value());
    BOOST_CHECK_EQUAL(8, opt.config.char_size.value());
    BOOST_CHECK_EQUAL(serial_port_base::parity::none, opt.config.parity.value());
    BOOST_CHECK_EQUAL(serial_port_base::stop_bits::two, opt.config.stop_bits.value());

    BOOST_REQUIRE(opt.noinput_timeout_ms);
    BOOST_CHECK_EQUAL(30, *opt.noinput_timeout_ms);

    BOOST_REQUIRE(opt.overflow_size);
    BOOST_CHECK_EQUAL(40, *opt.overflow_size);

    BOOST_CHECK_EQUAL(false, opt.verbose);
}

BOOST_FIXTURE_TEST_CASE(test_Option_WithVerbose_ShouldPass, TestFixture)
{
    // arrange
    const char *args[] = {
        "dummy",
        "--device=/dev/ttyS0",
        "--config=115200-8-n-2",
        "--verbose"
    };

    Option opt;

    // act
    const tribool result = opt.parse_args(size(args), args);

    // assert
    BOOST_CHECK(result);

    BOOST_CHECK_EQUAL(true, opt.verbose);
}

}
