#include <mxtagcollector/Configuration.hpp>

#include <boost/lexical_cast.hpp>

#include <libconfig.h++>

#include <mxutil/log.hpp>

using boost::lexical_cast;

using libconfig::Config;
using libconfig::ParseException;
using libconfig::Setting;
using libconfig::SettingNotFoundException;

using mxutil::ParseError;

using std::string;

namespace mxtagcollector
{

const Configuration
    Configuration::parse(const string &str)
{
    Config config;

    try
    {
        config.readString(str.c_str());

        const unsigned int sqlite3_busy_retry_timeout_ms = config.lookup("sqlite3-busy-retry-timeout-ms");
        if (sqlite3_busy_retry_timeout_ms > 60000)
        {
            MXLOG_EROR("illegal configuration: sqlite3-busy-retry-timeout-ms: "
                + lexical_cast<string>(sqlite3_busy_retry_timeout_ms));
            BOOST_THROW_EXCEPTION(ParseError());
        }

        const unsigned int sqlite3_busy_wait_ms = config.lookup("sqlite3-busy-wait-ms");
        if (sqlite3_busy_wait_ms > 1000)
        {
            MXLOG_EROR("illegal configuration: sqlite3-busy-wait-ms: "
                + lexical_cast<string>(sqlite3_busy_wait_ms));
            BOOST_THROW_EXCEPTION(ParseError());
        }

        const unsigned int sqlite3_commit_period_ms = config.lookup("sqlite3-commit-period-ms");
        if (sqlite3_commit_period_ms > 60 * 1000)
        {
            MXLOG_EROR("illegal configuration: sqlite3-commit-period-ms: "
                + lexical_cast<string>(sqlite3_commit_period_ms));
            BOOST_THROW_EXCEPTION(ParseError());
        }

        return Configuration(
            sqlite3_busy_retry_timeout_ms,
            sqlite3_busy_wait_ms,
            sqlite3_commit_period_ms);
    }
    catch(ParseException &e)
    {
        MXLOG_EROR(e.what());
        BOOST_THROW_EXCEPTION(ParseError());
    }
    catch(SettingNotFoundException &e)
    {
        MXLOG_EROR(e.what());
        BOOST_THROW_EXCEPTION(ParseError());
    }
}

}
