#include <boost/foreach.hpp>

#include <mxsl/mxsl.hpp>

#include <mxidafpp/Service.hpp>

#include <mxtaghistory/Option.hpp>

using boost::logic::tribool;
using boost::tuple;

using mxidafpp::Cache;
using mxidafpp::Equipment;
using mxidafpp::Service;
using mxidafpp::Stream;
using mxidafpp::Tag;

using mxtaghistory::Option;

using std::cerr;
using std::cout;
using std::endl;
using std::string;
using std::vector;

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[]);

typedef
    tuple<
        Stream,
        vector<Tag>
    > StreamTagList;

}

int
    main(
        int argc,
        const char *argv[])
{
    const Option opt = get_option(
        argc,
        argv);

    mxsl::validate();

    try
    {
        Service &svc = Service::instance();
        Cache cache = svc.get_cache(opt.mxhtag_file);

        vector<Equipment> equipment_list;
        if (opt.equipment)
        {
            equipment_list.push_back(cache.get_equipment(*opt.equipment));
        }
        else
        {
            equipment_list = cache.get_equipment_list();
        }

        vector<StreamTagList> stream_tag_list;
        if (opt.tag_list.empty())
        {
            BOOST_FOREACH(Equipment &e, equipment_list)
            {
                const vector<Stream> append = e.get_stream_list();

                BOOST_FOREACH(const Stream &s, append)
                {
                    stream_tag_list.push_back(StreamTagList(s, vector<Tag>()));
                }
            }
        }
        else
        {
            BOOST_ASSERT(equipment_list.size() == 1);

            BOOST_FOREACH(const string &t, opt.tag_list)
            {
                stream_tag_list.push_back(StreamTagList(
                    equipment_list.front().get_stream(t),
                    vector<Tag>()
                ));
            }
        }

        // fill tag_list
        const size_t count = opt.last ? *opt.last : 0;
        BOOST_FOREACH(StreamTagList &st, stream_tag_list)
        {
            const vector<Tag> append = st.get<0>().get_last_tag_list(count);

            st.get<1>().insert(
                st.get<1>().end(),
                append.begin(),
                append.end());
        }

        // dump
        cout << "\"equipment\",\"tag\",\"at\",\"value\",\"unit\"" << endl;
        BOOST_FOREACH(const StreamTagList &st, stream_tag_list)
        {
            const string &equipment = st.get<0>().equipment_name();
            const string &tag = st.get<0>().tag_name();

            BOOST_FOREACH(const Tag &t, st.get<1>())
            {
                cout
                    << '\"' << equipment << '\"'
                    << ",\"" << tag << '\"'
                    << ",\"" << t.at() << '\"'
                    << ",\"" << t.value() << '\"'
                    << ",\"" << t.unit() << '\"' << endl;
            }
        }

        return 0;
    }
    catch(mxidafpp::NotFoundError &e)
    {
        cerr << "not found error" << endl;

        cerr << "[traceback]" << endl;
        cerr << diagnostic_information(e) << endl;

        return 3;
    }
    catch(mxidafpp::Error &e)
    {
        cerr << "db access failure" << endl;

        cerr << "[traceback]" << endl;
        cerr << diagnostic_information(e) << endl;

        return 3;
    }
    catch(boost::exception &e)
    {
        cerr << "unknown error" << endl;

        cerr << "[traceback]" << endl;
        cerr << diagnostic_information(e) << endl;
        return 1;
    }
}

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[])
{
    Option opt;
    const tribool parsed = opt.parse_args(argc, argv);
    if (parsed)
        return opt;
    else if (!parsed)
        exit(2);
    else
        exit(0);
}

}
