#include <mxtaghistory/Option.hpp>

#include <iostream>

#include <boost/make_shared.hpp>
#include <boost/program_options.hpp>

#include <config.h>

using boost::logic::indeterminate;
using boost::logic::tribool;
using boost::make_shared;
using boost::program_options::options_description;
using boost::program_options::value;
using boost::program_options::variables_map;
using boost::program_options::command_line_style::default_style;
using boost::program_options::command_line_style::allow_guessing;

using std::cout;
using std::endl;
using std::string;
using std::vector;

namespace mxtaghistory
{

Option::Option()
    : mxhtag_file("/dev/shm/mxhtag.sqlite3")
{
}

tribool
    Option::parse_args(
        int argc,
        const char *argv[])
{
    options_description desc("Options");
    desc.add_options()
        ("mxhtag-file,f", value<string>(), "Change MXHTAG file path.")
        ("last", value<int>(), "Dump last few values of each Tag.")
        ("equipment", value<string>(), "Equipment name.")
        ("tag", value<vector<string> >()->composing(),
            "Tag name.\n"
            "Can be given multiple times.\n"
            "Must be used with --equipment.")
        ("help", "Print this manual.")
        ("version", "Print version.");

    variables_map vm;
    try
    {
        store(
            parse_command_line(argc, argv, desc, default_style & (~allow_guessing)),
            vm
        );
    }
    catch(boost::program_options::error &e)
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("help"))
    {
        cout << desc << endl;
        return indeterminate;
    }

    if (vm.count("version"))
    {
        cout << "mxtaghistory v" << VERSION << endl;
        return indeterminate;
    }

    if (vm.count("mxhtag-file"))
    {
        this->mxhtag_file = vm["mxhtag-file"].as<string>();
    }

    if (vm.count("last"))
    {
        this->last = make_shared<int>(vm["last"].as<int>());
    }

    if (vm.count("equipment"))
    {
        this->equipment = make_shared<string>(vm["equipment"].as<string>());
    }

    if (vm.count("tag"))
    {
        this->tag_list = vm["tag"].as<vector<string> >();
    }

    if (!this->tag_list.empty())
    {
        if (!this->equipment)
        {
            cout << desc << endl;
            return false;
        }
    }

    return true;
}

}
