#include <mxtagimport/Option.hpp>

#include <boost/make_shared.hpp>

#include <mxutil/log/Log.hpp>
#include <mxutil/log/StderrLogger.hpp>

#include <mxsl/mxsl.hpp>

#include <mxidafpp/Service.hpp>

using boost::logic::tribool;
using boost::make_shared;
using boost::shared_ptr;

using mxidafpp::Service;
using mxidafpp::Tag;
using mxidafpp::TagImporter;
using mxidafpp::Time;

using mxtagimport::Option;

using mxutil::log::Log;
using mxutil::log::Logger;
using mxutil::log::StderrLogger;

using std::cerr;
using std::endl;
using std::vector;

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[]);

}

int
    main(
        int argc,
        const char *argv[])
{
    const Option opt = get_option(
        argc,
        argv);

    shared_ptr<Logger> logger = make_shared<StderrLogger>();
    logger->set_level(Logger::WARN);
    Log::get().set_logger(logger);

    mxsl::validate();

    try
    {
        Service &svc = Service::instance();

        TagImporter tag_importer = svc.get_tag_importer(opt.mqtt_broker_host, opt.mqtt_broker_port);

        shared_ptr<Tag> tag;
        if (opt.unit)
            tag = make_shared<Tag>(*opt.value, Time(opt.at), *opt.unit);
        else
            tag = make_shared<Tag>(*opt.value, Time(opt.at));

        tag_importer.import(
            opt.equipment,
            opt.tag,
            vector<Tag>(1, *tag));

        return 0;
    }
    catch(mxidafpp::Error &e)
    {
        cerr << "error" << endl;
        cerr << "[traceback]" << endl;
        cerr << diagnostic_information(e) << endl;

        return 4;
    }
    catch(boost::exception &e)
    {
        cerr << "error" << endl;
        cerr << "[traceback]" << endl;
        cerr << diagnostic_information(e) << endl;

        return 1;
    }
}

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[])
{
    Option opt;
    const tribool parsed = opt.parse_args(argc, argv);
    if (parsed)
        return opt;
    else if (!parsed)
        exit(2);
    else
        exit(0);
}

}
