#include <boost/bind.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/make_shared.hpp>

#include <mxutil/log.hpp>
#include <mxutil/sched/DailyScheduler.hpp>

#include <ptag/Option.hpp>
#include <ptag/publish_reading.hpp>

using boost::bind;
using boost::lexical_cast;
using boost::logic::tribool;
using boost::make_shared;
using boost::shared_ptr;
using boost::posix_time::milliseconds;
using boost::posix_time::ptime;
using boost::posix_time::microsec_clock;

using mxutil::sched::DailyScheduler;
using mxutil::sched::Scheduler;

using mxmqtt::Worker;

using ptag::Option;
using ptag::publish_reading;

using std::string;

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[]);

void
    publish_all_readings(
        shared_ptr<Worker>,
        int equ_count,
        int tag_count);

}

int
    main(
        int argc,
        const char *argv[])
{
    const Option opt = get_option(argc, argv);
    shared_ptr<Worker> publisher = make_shared<Worker>(
        opt.mqtt_broker_host,
        opt.mqtt_broker_port,
        "publish-tag");

    shared_ptr<Scheduler> sched = make_shared<DailyScheduler>(
        milliseconds(opt.period_ms),
        bind(
            &publish_all_readings,
            publisher,
            opt.equ_count,
            opt.tag_count
        )
    );

    while(true)
    {
        sched->check_and_fire();

        usleep(1 * 1000);
    }

    return 0;
}

namespace
{

const Option
    get_option(
        int argc,
        const char *argv[])
{
    Option opt;
    const tribool result = opt.parse(argc, argv);
    if (result)
    {
        return opt;
    }
    else if(!result)
    {
        exit(1);
    }
    else
    {
        exit(0);
    }
}

void
    publish_all_readings(
        shared_ptr<Worker> publisher,
        int equ_count,
        int tag_count)
{
    MXLOG_WARN("publish tags");

    const ptime now = microsec_clock::universal_time();
    for(int equ = 0; equ < equ_count; ++equ)
    {
        const string equ_name = "equipment-" + lexical_cast<string>(equ);
        for(int tag = 0; tag < tag_count; ++tag)
        {
            const string tag_name = "tag-" + lexical_cast<string>(tag);

            publish_reading(*publisher, equ_name, tag_name, now);
        }
    }
}

}
