#include <ptag/Option.hpp>

#include <iostream>

#include <boost/program_options.hpp>

using boost::logic::indeterminate;
using boost::logic::tribool;
using boost::program_options::command_line_style::default_style;
using boost::program_options::command_line_style::allow_guessing;
using boost::program_options::options_description;
using boost::program_options::value;
using boost::program_options::variables_map;

using std::cout;
using std::endl;
using std::string;

namespace ptag
{

Option::Option()
    : equ_count(-1)
    , tag_count(-1)
    , period_ms(-1)
    , mqtt_broker_host("127.0.0.1")
    , mqtt_broker_port(1883)
{
}

tribool
    Option::parse(
        int argc,
        const char *argv[])
{
    options_description desc("Allowed options");
    desc.add_options()
        ("equ-count", value<int>(), "Equipment count.")
        ("tag-count", value<int>(), "Tag count per equipment.")
        ("period-ms", value<int>(), "Publish period.")
        ("mqtt-broker-host", value<string>(), "Change MQTT broker address.")
        ("mqtt-broker-port", value<int>(), "Change MQTT broker port.")
        ("help", "print this manual.");

    variables_map vm;
    try
    {
        store(
            parse_command_line(argc, argv, desc, default_style & (~allow_guessing)),
            vm
        );

    }
    catch(boost::program_options::error &e)
    {
        cout << desc << endl;
        return false;
    }

    if (vm.count("help"))
    {
        cout << desc << endl;
        return indeterminate;
    }

    if (!vm.count("equ-count") ||
        !vm.count("tag-count") ||
        !vm.count("period-ms"))
    {
        cout << desc << endl;
        return false;
    }

    this->equ_count = vm["equ-count"].as<int>();
    this->tag_count = vm["tag-count"].as<int>();
    this->period_ms = vm["period-ms"].as<int>();

    if (vm.count("mqtt-broker-host"))
        this->mqtt_broker_host = vm["mqtt-broker-host"].as<string>();

    if (vm.count("mqtt-broker-port"))
        this->mqtt_broker_port = vm["mqtt-broker-port"].as<int>();

    return true;
}

}
