# API Endpoint with Files
In this section, we will demonstrate how to upload/download files with some API Endpoints.

(Examples: `/system/export`, `/network/openvpn/config`)

## Overview
There are three components in the upload/download files scenario:
1. Client (users' program)
2. API Endpoint (Bundle)
3. File Server (HTTP Server)

In the following section we will use import/export of system configuration files as examples to demonstrate how to call API endpoints with files.

## Using Built-In API Helpers (recommended)
ThingsPro has a built-in file server and API helpers.

### Download (Exporting System Configuration)
1. The **Client** sends a request `HTTP GET /helper/download?resource=/system/export`
2. The **Client** receives a `HTTP 302 Redirect` to the download URL.

![Sequence_DownloadHelper](assets/images/Sequence_DownloadHelper.png)

### Upload (Importing System Configuration)
1. The **Client** sends the request `HTTP POST /helper/upload?resource=/system/import`
The request's Content-Type of the request must be a `multipart/form-data` as follows:
	- `file`: File field
	- `jsonData`: Additional JSON string for target API endpoint.

2. The **Client** will receive a response from the API endpoint.

![Sequence_UploadHelper](assets/images/Sequence_UploadHelper.png)

## Using APIs

### Download (Exporting System Configuration)
1. The **Client** sends the request to download system configuration by `HTTP POST /api/v1/system/export`. Please refer to ThingsPro API reference to understand the POST data.
2. **API Endpoint (Bundle)** sends HTTP POST request to File Server.
3. **File Server** responds download link(s) to API Endpoint.
4. **API Endpoint (Bundle)** gets response from file server and respond to client.
5. The **Client** gets response from bundle and download system configuration file from URL provided by preceding request.

![Sequence_Direct_Download](assets/images/Sequence_Direct_Download.png)

### Upload (Importing System Configuration)
1. The **Client** sends the request to upload system configuration file by `HTTP POST /api/v1/files/upload` (a built-in local file server)
2. **File Server** receives files and responds download link(s) to client.
3. The **Client** gets response from file server and send HTTP PUT data to API Endpoint by `HTTP PUT /api/v1/system/import` with given URL from the response of preceding action.
4. **API Endpoint (Bundle)** takes the system configuration file from the URL and overwrite original settings.
5. The **Client** gets response from bundle and ThingsPro Gateway will reboot automatically.

![Sequence_Direct_Upload.png](assets/images/Sequence_Direct_Upload.png)
