# Using the CURL Command

This section demonstrates the use of the `curl` command to interact with the *RESTful API* and request resources. The response is in JSON format.

The `curl` utility is a command line tool available on Unix, Linux, Mac OS X, Windows, and many other platforms. This utility provides easy direct access to the HTTP protocol (among others) from the command-line and is therefore an ideal way to interact with ThingsPro via the HTTP RESTful API.

## Getting a Resource

Usage: For simple `GET` requests, just supply the URL of the request. The `-X GET` command specifies the `GET` option, which is the default operation of the `curl` command. Use `--insecure` to skip certificate validation.
```
$ curl --insecure [-X GET] https://localhost/api/v1/<resource>
```

### Example for GET

The following example requests the first port mapping rule:
```
$ curl --insecure https://localhost/api/v1/network/portmapping/1
```

The response of the port mapping resource in JSON format is given below:
```json
{
  "id": 1,
  "enable": true,
  "service": "http",
  "portStart": 80,
  "portEnd": 80,
  "destIp": "192.168.5.123",
  "protocol": "tcp"
}
```

Some resources provide a *collection* of content as in the following case:

```
$ curl --insecure https://localhost/api/v1/network/portmapping
```

```json
[
  {
    "id": 1,
    "enable": true,
    "service": "http",
    "portStart": 80,
    "portEnd": 80,
    "destIp": "192.168.5.123",
    "protocol": "tcp"
  },
  {
    "id": 2,
    "enable": false,
    "service": "test",
    "portStart": 12340,
    "portEnd": 12345,
    "destIp": "192.168.5.123",
    "protocol": "both"
  }
]
```

## Creating/Updating a Resource

Usage: Specify a header using the `-H` option and provide data using the `-d` option.
```
$ curl --insecure -H 'Content-Type: application/json' \
       -X <POST|PUT> https://localhost/api/v1/<resource> \
       -d '<DATA>'
```

### Example for POST

The following command creates a port mapping rule using the `curl` command:
```
$ curl --insecure -H 'Content-Type: application/json' \
       -X POST https://localhost/api/v1/network/portmapping \
       -d '{ "enable": true,
             "service": "http",
             "portStart": 8080,
             "portEnd": 8080,
             "destIp": "192.168.5.123",
             "protocol": "tcp"}'
```

After the create (*POST*) command is processed, an `id` is assigned and included in the JSON response.
```json
{
  "id": 2,
  "enable": true,
  "service": "http",
  "portStart": 8080,
  "portEnd": 8080,
  "destIp": "192.168.5.123",
  "protocol": "tcp"
}
```

### Example for PUT

The following command requests an update to the second port mapping rule:
```
$ curl --insecure -H 'Content-Type: application/json' \
       -X PUT https://localhost/api/v1/network/portmapping/2 \
       -d '{ "id": 2,
             "enable": true,
             "service": "http",
             "portStart": 8000,
             "portEnd": 8000,
             "destIp": "192.168.5.123",
             "protocol": "tcp"}'
```

The response to the PUT request is as shown below:
```json
{
  "id": 2,
  "enable": true,
  "service": "http",
  "portStart": 8000,
  "portEnd": 8000,
  "destIp": "192.168.5.123",
  "protocol": "tcp"
}
```

## Deleting a Resource
Usage: Specify the URL of the resource that you want to delete:
```
$ curl --insecure -X DELETE https://localhost/api/v1/<resource>
```

### Example for DELETE

To delete a port mapping rule with `id = 2`, use the following command:
```
$ curl --insecure -X DELETE https://localhost/api/v1/network/portmapping/2
```
