# Using Python Call to RESTful API

Developers who want to call the *RESTful API* using a *Python* program can use the `requests` library.

## `Requests: HTTP for Humans`
> Requests is an elegant and simple HTTP library for Python, built for human beings. 
> In order to get more information about Python Requests library, please refer
> to its main page on
> http://docs.python-requests.org/

### Installing the Requests Library

Logon to ThingsPro Gateway via a console port or Ethernet ssh daemon. Type 
below commands to install Python Requests library: 

```pip install requests```

### Getting a Resource

Get current system status as follows:

Create a file: `get_system_status.py` with execution permission.
```py
#!/usr/bin/env python

import requests

r = requests.get('https://localhost/api/v1/system/status', verify=False)
data = r.json()

print (data)
```
`data` would be the output of response.

### Updating a Resource with JSON Payload

Enable and set fixed DNS to `8.8.8.8` (primary), `168.96.1.1` (secondary).

Create a file `set_dns.py` with execution permission.
```py
#!/usr/bin/env python

import requests

dns = {
    "enableFixed": True,
    "fixedDns": ["8.8.8.8", "168.95.1.1"]
}

r = requests.put(
        'https://localhost/api/v1/network/dns',
        json=dns,
        verify=False
    )

resp_data = r.json()

print (resp_data)
```
