#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import re
import logging

from sanji.core import Sanji
from sanji.core import Route
from sanji.model import Model

from voluptuous import Schema
from voluptuous import Required
from voluptuous import REMOVE_EXTRA
from voluptuous import Any
from voluptuous import All
from voluptuous import Length, message, truth, Invalid

_logger = logging.getLogger("sanji.customequipment.index")
NAME_PATTERN = re.compile(ur'^[A-Za-z0-9_-]+$')


@message('not a vaild name', cls=Invalid)
@truth
def IsValidName(v):
    return NAME_PATTERN.match(v)


class Index(Sanji):

    CUSTOM_EQUIPMENT_TAG_SCHEMA = Schema({
        Required("name"): All(Any(unicode, str), Length(1, 255)),
        Required("description"): All(Any(unicode, str), Length(0, 65535))
    }, extra=REMOVE_EXTRA)

    CUSTOM_EQUIPMENT_SCHEMA = Schema({
        Required("equipmentName"): All(
            Any(unicode, str), IsValidName(), Length(min=3, max=255)),
        Required("equipmentTags"): [CUSTOM_EQUIPMENT_TAG_SCHEMA]
    }, extra=REMOVE_EXTRA)

    def init(self, *args, **kwargs):
        path_root = os.path.abspath(os.path.dirname(__file__))
        self.customequipments = Model(name="customequipment", path=path_root)

    @Route(methods="get", resource="/mxc/custom/equipments")
    def get_all_customequipments(self, message, response):
        return response(data=self.customequipments.getAll())

    @Route(methods="post", resource="/mxc/custom/equipments",
           schema=CUSTOM_EQUIPMENT_SCHEMA)
    def post_customequipments(self, message, response):
        eq_names = [
            eq["equipmentName"]
            for eq in self.customequipments.getAll()
        ]

        if message.data["equipmentName"] in eq_names:
            return response(
                code=400,
                data={
                        "message": "{} is already exist!"
                        .format(message.data["equipmentName"])
                    }
                )

        resp = self.customequipments.add(message.data)
        return response(data=resp)

    @Route(methods="put", resource="/mxc/custom/equipments/:id",
           schema=CUSTOM_EQUIPMENT_SCHEMA)
    def update_customequipments(self, message, response):
        resp = self.customequipments.update(
            id=int(message.param["id"]), newObj=message.data)
        return response(data=resp)

    @Route(methods="delete", resource="/mxc/custom/equipments/:id")
    def delete_customequipments(self, message, response):
        self.customequipments.remove(id=int(message.param["id"]))
        return response(data={"message": "success"})

    @Route(methods="delete", resource="/mxc/custom/equipments")
    def delete_batch_customequipments(self, message, response):
        for tid in message.data:
            self.customequipments.remove(id=int(tid))
        return response(data={"message": "success"})

if __name__ == "__main__":
    from sanji.connection.mqtt import Mqtt
    FORMAT = "%(asctime)s - %(levelname)s - %(lineno)s - %(message)s"
    logging.basicConfig(level=0, format=FORMAT)
    _logger = logging.getLogger("sanji.customequipment")
    logging.getLogger("sh").setLevel(logging.WARN)
    index = Index(connection=Mqtt())
    index.start()
