#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import sys
import unittest
import glob
import simplejson as json
from mock import Mock
from mock import patch
from sanji.connection.mockup import Mockup
from sanji.message import Message

try:
    sys.path.append(os.path.dirname(os.path.realpath(__file__)) + "/../")
    from index import Index
    from logprofile import LogUploader
    from logprofile import LogProfile
except ImportError as e:
    print "Please check the python PATH for import test module. (%s)" \
        % __file__
    print (e)
    exit(1)


class TestIndexClass(unittest.TestCase):

    @patch("logprofile.CronTab")
    @patch("logprofile.restart_mxlogarchiver")
    def setUp(self, CronTab, restart_mxlogarchiver):
        LogProfile.LP_CONFIG_DIR = "/tmp"
        LogUploader.UL_CONFIG_DIR = "/tmp"
        root_path = os.path.abspath(os.path.dirname(__file__) + "/../")
        files = glob.glob(os.path.join(root_path, "data/*.json")) + \
            glob.glob(os.path.join(root_path, "data/*.backup"))

        for prevConfig in files:
            try:
                os.unlink(prevConfig)
            except:
                pass

        with open(os.path.join(
             root_path, "data", "logprofiles.json.factory")) as fp:
            self.logprofile = json.load(fp)
        self.logprofile.append({
            "name": "log-profile-1",
            "archiver": {
                "directory": "/var/mxc/upload",
                "thresholdBytePerFile": 5242800,
                "archivesToKeep": 24
            },
            "uploader": {
                "httpPost": {
                    "url": "http://host/post",
                    "enableAuth": False,
                    "authentication": {
                        "username": "uploader",
                        "password": "password"
                    },
                    "bypassSsl": True,
                    "cronExpr": "*/5 * * * *",
                    "format": "json",
                    "compress": "gzip",
                    "enableCompress": True,
                    "retryCount": 3,
                    "retryPeriodSec": 3
                }
            },
            "equipmentList": [
                {
                    "equipmentName": "MyioLogik-E2242",
                    "equipmentTags": [{
                        "name": "di0",
                        "logOnChange": False
                    }, {
                        "name": "di1",
                        "logOnChange": False
                    }, {
                        "name": "ai0",
                        "logOnChange": False
                    }]
                },
                {
                    "equipmentName": "MyotherioLogik-E2242",
                    "equipmentTags": [{
                        "name": "di0",
                        "logOnChange": False
                    }, {
                        "name": "di1",
                        "logOnChange": False
                    }, {
                        "name": "ai0",
                        "logOnChange": False
                    }]
                }
            ]
        })
        self.index = Index(connection=Mockup())
        self.index.logprofiles.add(self.logprofile[0])

    def tearDown(self):
        self.index.stop()
        self.index = None

    @patch("logprofile.CronTab")
    @patch("logprofile.restart_mxlogarchiver")
    def test_get_logprofile_with_non_exist_equipmentName(
         self, CronTab, restart_mxlogarchiver):
        """Get logprofiles with query non-exist equipmentName,
           should return 0 row"""
        resp = Mock()
        message = Message({
            "query": {
                "equipmentName": "1234"
            }
        })

        def resp(data, code=200):
            self.assertEqual(len(data), 0)

        self.index.get_all_logprofile(
            message=message, response=resp, test=True)

    @patch("logprofile.CronTab")
    @patch("logprofile.restart_mxlogarchiver")
    def test_get_logprofile_with_equipmentName(
         self, CronTab, restart_mxlogarchiver):
        """Get logprofiles with query equipmentName, should return 1 row"""
        message = Message({
            "query": {
                "equipmentName": "MyioLogik-E2242"
            }
        })

        def resp(data, code=200):
            self.assertEqual(len(data), 1)

        self.index.get_all_logprofile(
            message=message, response=resp, test=True)

if __name__ == "__main__":
    unittest.main()
