#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import logging
from sanji.core import Sanji
from sanji.core import Route
from relay import Relays
_logger = logging.getLogger("sanji.relay.index")


class Index(Sanji):

    def init(self, *args, **kwargs):
        path_root = os.path.abspath(os.path.dirname(__file__))
        self.relays = Relays(name="relays", path=path_root)
        try:
            self.relays.install_all()
        except Exception as e:
            _logger.error(str(e))

    @Route(methods="get", resource="/relay")
    def get(self, message, response):
        return response(data=self.relays.getAll())

    @Route(methods="get", resource="/relay/:id")
    def get_id(self, message, response):
        relay = self.relays.get(id=int(message.param["id"]))
        if relay is None:
            return response(code=404)
        return response(data=relay)

    @Route(methods="put", resource="/relay/:id")
    def put(self, message, response):
        updatedRelay = self.relays.update(
            id=int(message.param["id"]),
            newObj=message.data
        )

        if updatedRelay is None:
            return response(code=404)
        return response(data=updatedRelay)


if __name__ == "__main__":
    from sanji.connection.mqtt import Mqtt
    FORMAT = "%(asctime)s - %(levelname)s - %(lineno)s - %(message)s"
    logging.basicConfig(level=0, format=FORMAT)
    _logger = logging.getLogger("sanji.relay")
    logging.getLogger("sh").setLevel(logging.WARN)
    index = Index(connection=Mqtt())
    index.start()
