#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import logging
import os

from sanji.core import Sanji
from sanji.core import Route
from sanji.connection.mqtt import Mqtt
from service import SService


class Index(Sanji):
    _logger = logging.getLogger("sanji.services.index")

    def init(self, *args, **kwargs):
        path_root = os.path.abspath(os.path.dirname(__file__))
        self.service = SService(name="service", path=path_root)

    @Route(methods="get", resource="/system/services")
    def get_all(self, message, response):
        return response(data=self.service.get_all())

    @Route(methods="get", resource="/system/services/:id")
    def get(self, message, response):
        id = 0
        name = None
        if isinstance(message.param["id"], int) or \
                message.param["id"].isdigit():
            id = int(message.param["id"])
        else:
            name = message.param["id"]

        data = self.service.get(id=id, name=name)
        if data is None:
            return response(code=404)
        return response(data=data)

    @Route(methods="put", resource="/system/services/:id")
    def put(self, message, response):
        id = 0
        name = None
        if isinstance(message.param["id"], int) or \
                message.param["id"].isdigit():
            id = int(message.param["id"])
        else:
            name = message.param["id"]

        data = self.service.update(
            id=id, name=name, newObj=message.data)
        if data is None:
            return response(code=404)
        return response(data=data)


if __name__ == "__main__":
    FORMAT = '%(asctime)s - %(levelname)s - %(lineno)s - %(message)s'
    logging.basicConfig(level=0, format=FORMAT)
    logging.getLogger("sh").setLevel(logging.WARN)
    index = Index(connection=Mqtt())
    index.start()

    '''
    path_root = os.path.abspath(os.path.dirname(__file__))
    service = SService(name="service", path=path_root)
    print service.getAll()
    print service.get(id=1)
    print service.get(id=2)
    print service.get(name="ssh")
    print service.get(name="rsyslog")

    print "update:"
    print service.update(id=1, newObj={"name": "ssh", "enable": False})
    print service.update(
        name="rsyslog", newObj={"name": "rsyslog", "enable": True})
    '''
