#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import os
import sys
import unittest
import glob
from mock import Mock
from mock import patch
from sanji.connection.mockup import Mockup
from sanji.message import Message

try:
    sys.path.append(os.path.dirname(os.path.realpath(__file__)) + "/../")
    from index import Index
except ImportError as e:
    print "Please check the python PATH for import test module. (%s)" \
        % __file__
    print (e)
    exit(1)


class TestIndexClass(unittest.TestCase):

    @patch("service.sh")
    def setUp(self, mock_sh):
        self.root_path = os.path.abspath(os.path.dirname(__file__) + "/../")
        jsons = glob.glob(os.path.join(self.root_path, "data/*.json"))
        backups = glob.glob(os.path.join(self.root_path, "data/*.backup"))
        for file in jsons + backups:
            os.unlink(file)
        files = glob.glob(os.path.join(self.root_path, "data/*.json")) + \
            glob.glob(os.path.join(self.root_path, "data/*.backup"))

        for prevConfig in files:
            try:
                os.unlink(prevConfig)
            except:
                pass

        self.index = Index(connection=Mockup())

    def tearDown(self):
        self.index.stop()
        self.index = None

    @patch("service.sh")
    def test__get_all(self, mock_sh):
        """Get all services"""
        result = self.index.service.getAll()
        resp = Mock()
        self.index.get_all(message=None, response=resp, test=True)

        for entry in result:
            entry["status"] = False
        resp.assert_called_once_with(data=result)

    @patch("service.sh")
    def test__get__by_id(self, mock_sh):
        """Get one of the services by id"""
        resp = Mock()
        message = Message({
            "param": {
                "id": "1"
            }
        })

        self.index.get(message=message, response=resp, test=True)

        service = self.index.service.getAll()[0]
        service["status"] = False
        resp.assert_called_once_with(data=service)

    @patch("service.sh")
    def test__get__by_name(self, mock_sh):
        """Get one of the services by name"""
        resp = Mock()
        message = Message({
            "param": {
                "id": "ssh"
            }
        })

        self.index.get(message=message, response=resp, test=True)

        service = self.index.service.getAll()[0]
        service["status"] = False
        resp.assert_called_once_with(data=service)

    @patch("service.sh")
    def test__put__by_id(self, mock_sh):
        """Update one of the service by id"""
        resp = Mock()
        message = Message({
            "param": {
                "id": 1
            },
            "data": {
                "enable": False
            }
        })

        self.index.put(message=message, response=resp, test=True)

        service = self.index.service.getAll()[0]
        service["status"] = False
        resp.assert_called_once_with(data=service)

    @patch("service.sh")
    def test__put__by_name(self, mock_sh):
        """Update one of the service by name"""
        resp = Mock()
        message = Message({
            "param": {
                "id": "rsyslog"
            },
            "data": {
                "enable": False
            }
        })

        self.index.put(message=message, response=resp, test=True)

        service = self.index.service.getAll()[1]
        service["status"] = False
        resp.assert_called_once_with(data=service)


if __name__ == "__main__":
    unittest.main()
