import sh


def read_from_rtu(eqtmpl_path, interface, gateways, device_id):
    gw = gateways.get_by_dev(interface["config"]["dev"])

    if gw is None:
        raise RuntimeError(
            "No gateway exist for: {}".format(interface["config"]["dev"]))

    return read_from_tcp(
        eqtmpl_path, gw["tcp"]["host"], gw["tcp"]["port"], device_id)


def read_from_tcp(eqtmpl_path, host, port, device_id):
    try:
        result = sh.mxmodbusclient(
            "--equipment-template", eqtmpl_path,
            "--host", host,
            "--port", port,
            "--device-id", device_id
        )
    except sh.ErrorReturnCode_5 as e:
        result = e.stdout

    rows = result.split("\n")
    report = {
        "elapsedMs": "",
        "data": {}
    }

    for row in rows:
        r = row.split(":")
        if len(r) < 2:
            continue
        report["data"][r[0]] = "".join(r[1:]).strip()

    report["elapsedMs"] = report["data"]["elapsed-ms"]
    del report["data"]["elapsed-ms"]

    return report
