#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import sys
import unittest

from mock import patch
from mock import Mock

try:
    sys.path.append(os.path.dirname(os.path.realpath(__file__)) + "/../../")
    from modbus.client import read_from_tcp
    from modbus.client import read_from_rtu
    from modbus.interface import RTUInterface

except ImportError as e:
    print "Please check the python PATH for import test module. (%s)" \
        % __file__
    print (e)
    exit(1)


class TestClient(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @patch("modbus.client.sh")
    def test_read_from_tcp(self, sh):
        """Add a gateway, should return Gateway instance"""
        sh.mxmodbusclient.return_value = """elapsed-ms: 48
di0: 1
di1: 0
ai0: 123.456
ai1: -123.456
ai2: error
ai3: error
"""
        result = (read_from_tcp("/tmp/test.met", "127.0.0.1", 55688, 0))
        sh.mxmodbusclient.assert_called_once_with(
            "--equipment-template", "/tmp/test.met",
            "--host", "127.0.0.1",
            "--port", 55688,
            "--device-id", 0)
        self.assertIn("elapsedMs", result)
        self.assertIn("di0", result["data"])
        self.assertIn("ai3", result["data"])

    @patch("modbus.client.sh")
    def test_read_from_rtu(self, sh):
        """Add a gateway, should return Gateway instance"""
        sh.mxmodbusclient.return_value = """elapsed-ms: 48
di0: 1
di1: 0
ai0: 123.456
ai1: -123.456
ai2: error
ai3: error
"""
        rtuinterface = RTUInterface({
            "id": 1,
            "name": "Modbus_Gateway_1",
            "type": "modbus-rtu",
            "config": {
                "dev": "/dev/ttyM0",
                "devDisplayName": "PORT 1",
                "baudRate": 115200,
                "parity": "none",
                "stopBits": 1,
                "intercharTimeoutMs": 100,
                "interframeDelayMs": 0
            },
            "requestTimeoutMs": 500,
            "pollingPeriodMs": 5000,
            "deviceList": []
        })
        gateways = Mock()
        gateways.get_by_dev = Mock(return_value={
            "tcp": {
               "host": "host",
               "port": 1234
            }
        })

        result = read_from_rtu("/tmp/test.met", rtuinterface, gateways, 0)
        sh.mxmodbusclient.assert_called_once_with(
            "--equipment-template", "/tmp/test.met",
            "--host", "host",
            "--port", 1234,
            "--device-id", 0)
        self.assertIn("elapsedMs", result)
        self.assertIn("di0", result["data"])
        self.assertIn("ai3", result["data"])


if __name__ == "__main__":
    unittest.main()
