# Log Archiver

* Store Tag Readings into LogFiles.
* Support multiple profiles.
* Support Log on Change by Tag in each profile.
    * The first value of each Tag in each LogFile is logged no matter what its previous value was.

## Requirement
* Data override mechanism.
    * FIFO
* Capacity threshold
    * Of each archive

## Configuration

* `working-directory`: Required, string.

Sample:
```
# /etc/mxlogarchiver/configuration
working-directory = "/data/var/mxlogarchiver"
```

## Profile Configuration

* `archive`: Required, object
    * `directory`: Required, string.
    * `directory-size-limit-byte`: Optional, integer, criteria `value >= 1048576`. (1 MB)
    * `threshold-byte-per-file`: Required, integer, criteria `value >= 10240`.
    * `period-sec`: Optional, integer, criteria `60 <= value < 86400`.
    * `archives-to-keep`: Optional, integer, criteria `value > 0`.
* `equipment-list`: Required, object-list.
    * `equipment`: Required, string criteria `[A-Za-z0-9_-]{3,255}`.
    * `tag-list`: Required, object-list.
        * `tag`: Required, criteria `[A-Za-z0-9_-]{2,255}`.
        * `log-on-change`: Optional, boolean. Default `false`.

```
# /etc/mxlogarchiver/conf.d/mxcs.conf
archive = {
    directory = "/data/var/mxlogarchiver/logs/mxcs"
    threshold-byte-per-file = 5242800
    period-sec = 3600
    archives-to-keep = 24
}

equipment-list = (
    {
        equipment = "My_ioLogik-E2242"
        tag-list = (
            {
                tag = "di0"
                log-on-change = true
            },
            {
                tag = "di1"
                log-on-change = true
            },
            {
                tag = "ai0"
                log-on-change = false
            }
        )
    },
    {
        equipment = "My_other_ioLogik-E2242"
        tag-list = (
            {
                tag = "di0"
                log-on-change = true
            },
            {
                tag = "di1"
                log-on-change = true
            },
            {
                tag = "ai0"
            }
        )
    }
)
```

## Log File Format
* Incremental format.
* Each record is enveloped by

    ```
    record        = record-length record
    record-length = variable-length-quantity
    record        = *byte
    ```

* Records are serialized in Protocol Buffers.
* First record contains **TagNameList** subscribed by its corresponding profile.
* Remaining records contain instances of **Tag**.
* Protocol Buffer Definition

    ```
    message Value
    {
      optional double float_value = 1;
      optional sint64 int_value = 2;
      optional string str_value = 3;
    }

    message Tag
    {
      optional string equipment = 1;
      optional string tag = 2;
      optional uint64 at_ms = 3;	// millisec since 1970-01-01T00:00:00.0000Z

      optional Value value = 4;

      optional string unit = 15;
    }

    message TagName
    {
      optional string equipment = 1;
      repeated string tag = 2;
    }

    message TagNameList
    {
      repeated TagName tag_name = 1;
    }
    ```

### Disk Usage

[Disk Usage](performance.md)

## Design

### Behavior

* Log files are being filled on-the-fly.
* Log files are stored in `/var/mxlogcollector/PROFILE_NAME/*.log` while being filled.
* Log files are gzipped and moved to `archive.directory` after filled.
* MQTT messages are Acked only when the message is stored correctly.
* Use file creation time as filename, like: `20150629-160615.log` and
  `20150629-160615.log.gz`.
* Creates only one MQTT connection to MQTT Broker.

### Activity Diagram

![Log Archiver Activity Diagram](images/cloud-gateway-log-archiver-activity.png)

### Class Diagram

![Log Archiver Class Diagram](images/cloud-gateway-log-archiver-class.png)

### Man Page
```
NAME
    mxlogarchiver

SYNOPSIS
    mxlogarchiver [-f|--foreground] [other options]

DESCRIPTION
    -f, --foreground
        Do not daemonize.
        mxlogarchiver uses current directory as working directory.

    --mqtt-broker-host HOST
    --mqtt-broker-port PORT
        Use HOST and PORT to connect to MQTT broker to subscribe readings.

        Default HOST and PORT are "127.0.0.1" and 1883.

DAEMON
    When mxlogarchiver is run as daemon, it creates and locks pid-file at
    "/var/run/mxlogarchiver.pid".

CONFIGURATION
    Path:
        etc/mxlogarchiver/configuration

PROFILE LIST
    Path:
        etc/mxlogarchiver/profile.d/*.cfg

SIGNAL
    SIGTERM
        Sending SIGTERM to mxlogarchiver causes mxlogarchiver to terminate
        gracefully, if possible.

EXIT STATUS
    0
        Success.
    1
        Unknown error.
    2
        Illegal argument.
```

### Known Issue

* Current implementation may cause a record being written to same Profile log
  more than once.
