# Log Converter

* Convert LogFiles to externl format.

## Input Log Format

Refer to [Log File Format](log-archiver.md#log-file-format).

## Output Log Formats

### XML

```xml
<!-- 500 characters -->
<?xml version="1.0" encoding="UTF-8"?>
<record_list>
  <record>
    <equ>My_ioLogik-E2242</equ>
    <at>2015-07-06T14:51:31Z</at>
    <tag_list>
      <tag>
        <name>di0</name>
        <value type="int">1</value>
      </tag>
      <tag>
        <name>ai0</name>
        <value type="float" unit="V">3.14</value>
      </tag>
    </tag_list>
  </record>
  <record>
    <equ>My_other_ioLogik-E2242</equ>
    <at>2015-08-19T15:09:24Z</at>
    <tag_list>
      <tag>
        <name>DOOR_STATUS</name>
        <value type="int">1</value>
      </tag>
      <tag>
        <name>WINDOW_STATUS</name>
        <value type="int">0</value>
      </tag>
    </tag_list>
  </record>
</record_list>
```

### JSON

```json
# 269 characters
[
    {
        "equ": "My_ioLogik-E2242",
        "at": "2015-07-06T14:51:31Z",
        "tagList": [
            {
                "tag": "di0",
                "value": 1
            },
            {
                "tag": "ai0",
                "value": 3.14,
                "unit": "V"
            }
        ]
    },
    {
        "equ": "My_other_ioLogik-E2242",
        "at": "2015-08-19T15:09:24Z",
        "tagList": [
            {
                "tag": "DOOR_STATUS",
                "value": 1
            },
            {
                "tag": "WINDOW_STATUS",
                "value": 0
            }
        ]
    }
]
```

### CSV

```csv
# 197 characters
at,"di0@My_ioLogik-E2242","ai0@My_ioLogik-E2242","DOOR_STATUS@My_other_ioLogik-E2242","WINDOW_STATUS@My_other_ioLogik-E2242"
"2015-07-06T14:51:31Z","1","3.14(V)",,
"2015-08-19T15:09:24Z",,,"1","0"
```

#### Considered

```csv
# 167 characters
at,equipment,di0,ai0(V),"DOOR_STATUS","WINDOW_STATUS"
"2015-07-06T14:51:31Z","My_ioLogik-E2242","1","3.14",,
"2015-08-19T15:09:24Z","My_other_ioLogik-E2242",,,"1","0"
```

```csv
# 286 characters
"equipment","tag","at","value","unit"
"My_ioLogik-E2242","di0","2015-07-06T14:51:31Z","1",
"My_ioLogik-E2242","ai0","2015-07-06T14:51:31Z","3.14","V"
"My_other_ioLogik-E2242","DOOR_STATUS","2015-08-19T15:09:24Z","1",
"My_other_ioLogik-E2242","WINDOW_STATUS","2015-08-19T15:09:24Z","0",
```

## Design

### Class Diagram

![Log Converter Class Diagram](images/cloud-gateway-log-converter-class.png)

### Man Page
```
NAME
    mxlogconverter

SYNOPSIS
    mxlogconverter --format FORMAT [other options]

    mxlogconverter --dump

DESCRIPTION
    mxlogconverter reads log from stdin, convert it to FORMAT and dump it to
    stdout.

    --dump
        Dump input log to human readable format.

    --format FORMAT
        Convert log to indicated format.

        Supported FORMATs are: csv, json, xml.

    --window-size WINDOW_SIZE
        Since Input Log Format represents tag-data by "tag" and Output Log
        Formats represent tag-data by "at", mxlogconverter shall reorganize
        tag-data by collapsing tag-data having same "at".

        Default 128.

EXIT STATUS
    0
        Success.
    1
        Unknown error.
    2
        Illegal argument.
```
