# Log Uploader

* Convert and upload logs, stored in specific directories, to remote Headend.
* Files are **not** deleted after being uploaded.
* Activated by
    * Schedule

## Configuration

* `working-directory`: Required, string.

Sample:
```
# /etc/mxloguploader/configuration
working-directory = "/data/var/mxloguploader"
```

## Profile Configuration

* `method`: Required, object.
    * `http-post`: Optional, object.
        * `url`: Required, string.
        * `authentication`: Optional, object.
            * `username`: Required, string, should not contain `:`.
            * `password`: Required, string.
        * `bypass-ssl`: Required, bool,
* `retry`: Required, object.
    * `count`: Required, integer, criteria `value >= 0`.
    * `period-sec`: Required, integer, criteria `value >= 0`.
* `log-directory`: Required, string.
* `content`: Required, object.
    * `format`: Required, string, can be `csv`, `json`, `xml`.
    * `compress`: Required, string, can be `gzip`, `none`.

Sample:

```
# /etc/mxloguploader/conf.d/mxcs.conf
method = {
    http-post = {
        url = "https://mxcloud.moxa.com/upload"
        authentication = {
            username = "moxa"
            password = "mahaha"
        }

        bypass-ssl = true
    }
}

retry = {
    count = 3
    period-sec = 10
}

log-directory = "/data/var/mxlogarchiver/logs/mxcs"

content = {
    format = "json"
    compress = "gzip"
}
```

## MQTT Client

```
Client ID:
    log-uploader-<PID>
Clean Session:
    true
```

### Publish

```
Topic:
    /system/log-uploader/:profile/status
QoS:
    1
Retain:
    true
```

Sample:

```
{
    "lastAttempt": {
        "name": "/var/mxc/logs/20150706-153807.log.gz",
        "at": "2015-07-06T15:38:46Z",
        "status": "remote server error"
    },
    "lastSuccess": {
        "name": "/var/mxc/logs/20150706-123456.log.gz",
        "at": "2015-07-06T13:46:22Z"
    },
    "fileUploadCount": 47
}
```

## Design

### Bookkeeping
* Each Profile has a working directory resides in `WORKING_DIRECTORY/:profile`.
* Status file: `WORKING_DIRECTORY/:profile/status`.
    * Status bookkeeping.
        * lastAttempt
        * lastSuccess
        * fileUploadCount
* Uploaded file symbolic links: `WORKING_DIRECTORY/:profile/uploaded/*`.

### Steps

1. Lock `WORKING_DIRECTORY/:profile/lock` to avoid multiple instance serving
   same Profile.
2. Enumerate all files in `upload-directory`.
3. Filter out already uploaded files.
4. Sort files by name.
5. For each file:
    1. Convert to external format with *LogConverter*, then gzip and save the
       file to temp-directory.
    2. Upload file.
    3. Remove file from temp-directory.
    4. Create a symbol link to the file uploaded and update status.
    5. Publish status.
6. Delete uploaded-symlinks that are dangling.

### Class Diagram

![Log Uploader Class Diagram](images/cloud-gateway-log-uploader-class.png)

### Man Page
```
NAME
    mxloguploader

SYNOPSIS
    mxloguploader [-p PROFILE|--profile PROFILE] [other options]

DESCRIPTION
    -p, --profile
        Given the name of upload Profile.

        mxloguploader would use the profile configuration resides in
            /etc/mxloguploader/conf.d/PROFILE.conf
        as the profile configuration.

    -f, --file
        Given the path to a valid profile configuration file.

        Can only be used with --update-test option.

    --upload
        Perform file upload process.

    --file-count FILE_COUNT
        Given the maximum number of files to upload.

        If not given, mxloguploader would try to upload as many files as
        possible.

    --clear
        Clear all book-keeping files resides in the system, including booking
        files in the file system and status information published through MQTT.

        When this options is given, /etc/mxloguploader/conf.d/PROFILE.conf
        doesn't have to exist.

    --upload-test
        Upload a test file through `method`.

CONFIGURATION
    Path:
        /etc/mxloguploader/conf.d/*.conf

EXAMPLE
    To upload files:
        $ mxloguploader -p locus-energy --upload

    To upload at most 3 files:
        $ mxloguploader -p locus-energy --upload --file-count 3

    To clear all book-keeping files:
        $ mxloguploader -p locus-energy --clear

EXIT STATUS
    0
        Success.
    1
        Unknown error.
    2
        Illegal argument.
    3
        Duplicated instance.
    4
        Communication error.
```
