# Modbus Gateway

* Conversion between Modbus-RTU and Modbus-TCP.
    * TCP Slave
    * RTU Master
* Allow multiple Modbus-TCP Master connections at the same time.
* Serve multiple serial-ports.
* Supported MODBUS commands
    * 0x01 - read coils
    * 0x02 - read discrete inputs
    * 0x03 - read holding registers
    * 0x04 - read input registers
    * 0x05 - write single coil
    * 0x06 - write single register
    * 0x0F - write multiple coils
    * 0x10 - write multiple registers

## Configuration

* `port-list`: Required, object-list.
    * `tcp`: Required, object.
        * `host`: Required, string.
        * `service`: Required, integer, criteria `1 <= value <= 65535`.
    * `rtu`: Required, object.
        * `path`: Required, string. Serial device path.
        * `serial-config`: Required, string, format `<baudrate>-8-<parity>-<stop-bits>`.
            * Allowed serial baudrate: `9600`, `19200`, `38400`, `57600`, `115200`.
            * Allowed serial parity: `n`, `e`, `o`.
            * Allowed serial stop-bits: `1`, `2`.
        * `interframe-delay-ms`: Required, integer, criteria `value >= 0`.
        * `interchar-timeout-ms`: Required, integer, criteria `value > 1`.
        * `request-timeout-ms`: Required, integer, criteria `value >= 10`.

Sample:

```
# /etc/mxmodbusgateway/configuration
port-list = (
    {
        tcp = {
            host = "127.0.0.1"
            service = 502
        }
        rtu = {
            path = "/dev/ttyM0"
            serial-config = "9600-8-n-1"
            interframe-delay-ms = 300
            interchar-timeout-ms = 100
            request-timeout-ms = 500
        }
    },
    {
        tcp = {
            host = "0.0.0.0"
            service = 1502
        }
        rtu = {
            path = "/dev/ttyM1"
            serial-config = "19200-8-n-1"
            interframe-delay-ms = 200
            interchar-timeout-ms = 100
            request-timeout-ms = 300
        }
    }
)
```

## Behavior

* Handles multiple TCP client connections at the same time.
* Aware of Modbus command/response sessions.

![Modbus Gateway Sequence Diagram](images/cloud-gateway-modbus-gateway-sequence.png)

## Design

### Class Diagram

![Modbus Gateway Class Diagram](images/cloud-gateway-modbus-gateway-class.png)

### Session State Diagram

![Modbus Gateway Session State Diagram](images/cloud-gateway-modbus-gateway-session-state.png)

### Man Page
```
NAME
    mxmodbusgateway

SYNOPSIS
    mxmodbusgateway [-f|--foreground]

DESCRIPTION
    -f, --foreground
        Do not daemonize.
        mxmodbusgateway uses current directory as working directory.

DAEMON
    When mxmodbusgateway is run as daemon, it creates and locks pid-file at
    "/var/run/mxmodbusgateway.pid".

CONFIGURATION
    Path:
        etc/mxmodbusgateway/configuration

SIGNAL
    SIGTERM
        Sending SIGTERM to mxmodbusgateway causes mxmodbusgateway to terminate
        gracefully, if possible.

EXIT STATUS
    0
        Success.
    1
        Unknown error.
    2
        Illegal argument.
```
