# ModbusMaster

* Read Tags from Modbus Equipments and publish them to MqttBroker.
* Provide *Trivial Modbus Templates*.

## Initialization

* Connect to *MQTT Broker* first.
* Then, start readings from Equipments.

## Configuration Files

* Describes *connection parameters*, *polling period* and *devices* of each polling operation.
* Trivial Modbus Device
* Non-trivial Modbus Device
    * Extensible through \*.so(ignored in MXcloud-1.0)

### Configuration

* `polling-list`: Required, object-list.
    * `host`: Required, string.
    * `service`: Required, integer.
    * `polling-period-ms`: Required, integer, criteria `1000 <= value < 86400000`.
    * `request-timeout-ms`: Required, integer, criteria `100 <= value <= 60000`.
    * `device-list`: Required, object-list.
        * `device-id`: Required, integer, criteria `0 <= value <= 255`.
        * `template-id`: Required, string, criteria `[A-Za-z0-9_-]{3,255}`.
        * `equipment-name`: Required, string, criteria `[A-Za-z0-9_-]{3,255}`.
* `equipment-status-path`: Optional, string. Default `var/mxmodbusmaster/equipment-status`.

```
# /etc/mxmodbusmaster/configuration
polling-list = (
    {
        host = "127.0.0.1"
        service = 502
        polling-period-ms = 30000
        request-timeout-ms = 30000
        device-list = (
            {
                device-id = 0x30
                template-id = "ioLogik-E2214"
                equipment-name = "My_ioLogik-E2214"
            },
            {
                device-id = 0x31
                template-id = "Sunspec_Inverter"
                equipment-name = "THE_building"
            }
        )
    },
    {
        # same port with different polling-period
        host = "127.0.0.1"
        service = 502
        polling-period-ms = 60000
        request-timeout-ms = 30000
        device-list = (
            {
                device-id = 0x32
                template-id = "ioLogik-E2242"
                equipment-name = "My_ioLogik-E2242"
            }
        )
    },
    {
        host = "127.0.0.1"
        service = 1502
        polling-period-ms = 45000
        request-timeout-ms = 30000
        device-list = (
            {
                device-id = 0x01
                template-id = "ioLogik-E2242"
                equipment-name = "Another_ioLogik-E2242"
            }
        )
    }
)

equipment-status-path = "/dev/shm/var/mxmodbusmaster/equipment-status"
```

### Equipment Template

* One template in one file
* `id`: Required, string, criteria `[A-Za-z0-9_-]{3,255}`.
* `request-list`: Required, object-list.
    * `function`: Required, string, can be `read-coils`, `read-discrete-inputs`,
      `read-input-registers`, `read-holding-registers`.
    * `address`: Required, integer, criteria `0x0000 <= value <= 0xFFFF`.
    * `quantity`: Required, integer, criteria `value > 0`.
* `tag-list`: Required, object-list.
    * `id`: Required, string, criteria `[A-Za-z0-9_-]{2,255}`.
    * `reference`: Required, integer, criteria
      `0 <= value < 0x20000 || 0x30000 <= value < 0x50000`.
    * `quantity`: Required, integer, criteria `value > 0`.
    * `invalid-value`: Optional, integer-array, criteria `0 <= value <= 0xFFFF`.
    * `byte-order`: Optional, string, can be `BA`, `CDAB`, `DCBA`.
    * `type`: Required, string, can be `uint16`, `uint32`, `uint64`, `int16`,
      `int32`, `int64`, `float32`, `float64`, `string`.
    * `slope`: Optional, float.
    * `offset`: Optional, float.

```
# /etc/mxmodbusmaster/template.d/iologik-e2242.met
id = "ioLogik-E2242"
request-list = (
    {
        function = "read-coils"
        address = 0x0000
        quantity = 6
    },
    {
        function = "read-input-registers"
        address = 0x0130
        quantity = 2
    }
)

tag-list = (
    {
        id = "di0"
        reference = 0x00000
        quantity = 1
        type = "uint16"
    },
    {
        id = "di1"
        reference = 0x00001
        quantity = 1
        type = "uint16"
    },
    {
        id = "di2"
        reference = 0x00002
        quantity = 1
        type = "uint16"
    },
    {
        id = "di3"
        reference = 0x00003
        quantity = 1
        type = "uint16"
    },
    {
        id = "di4"
        reference = 0x00004
        quantity = 1
        type = "uint16"
    },
    {
        id = "di5"
        reference = 0x00005
        quantity = 1
        type = "uint16"
    },
    {
        id = "ai0"
        unit = "V"
        reference = 0x40130
        quantity = 2
        invalid-value = [
            0xFFFF, 0xFFFF
        ]
        byte-order = "CDAB"
        type = "float32"
        slope = 0.33
        offset = 3.14
    }
)
```

### Equipment Status

* `at`: Required, string. Time of export.
* `equipments`: Required, object.
    * `<equipment>`: Required, object.
        * `lastPoll`: Required, object.
            * `at`: Required, string.
            * `elapsedMS`: Required, integer.
            * `requestCount`: Required, integer.
            * `requestSuccess`: Required, integer.
            * `readingCount`: Required, integer.
        * `lastSuccessAt`: Optional, string. Absent if corresponing equipment hasn't been polled successfully.
        * `totalRequestCount`: Required, integer.
        * `totalRequestSuccess`: Required, integer.
        * `totalReadingCount`: Required, integer.

```
# /var/mxmodbusmaster/equipment-status
{
    "at": "2015-12-04T16:29:41Z",
    "equipments": {
        "My_ioLogik_E2242": {
            "lastPoll":
            {
                "at": "2015-06-25T17:05:24Z",
                "elapsedMS": 203,
                "requestCount": 4,
                "requestSuccess": 4,
                "readingCount": 4
            },

            "lastSuccessAt": "2015-06-25T17:05:24Z",
            "totalRequestCount": 40024,
            "totalRequestSuccess": 40000,
            "totalReadingCount": 40000
        },
        "My_other_ioLogik_E2242": {
            "lastPoll":
            {
                "at": "2015-06-25T17:05:22Z",
                "elapsedMS": 1020,
                "requestCount": 4,
                "requestSuccess": 2,
                "readingCount": 2
            },

            "totalRequestCount": 9764,
            "totalRequestSuccess": 4882,
            "totalReadingCount": 4882
        }
    }
}
```

## MQTT Client

```
Client ID: modbus-master
Clean Session: true
```

### Publish

```
Topic:
    /equs/EQUIPMENT/status
QoS:
    1
Retain:
    true
```

Sample:

```json
{
    "lastPoll":
    {
        "at": "2015-06-25T17:05:24Z",
        "elapsedMS": 203,
        "requestCount": 4,
        "requestSuccess": 4,
        "readingCount": 4
    },

    "lastSuccessAt": "2015-06-25T17:05:24Z",
    "totalRequestCount": 40024,
    "totalRequestSuccess": 40000,
    "totalReadingCount": 40000
}
```

-------------------------------------------------------------------------------

```
Topic:
    /equs/EQUIPMENT/tags/TAG
QoS:
    2
Retain:
    false
```

Sample:

```json
{
    "at": "2015-06-26T16:49:12Z",
    "value": 123.456,
    "unit": "V"
}
```

-------------------------------------------------------------------------------

```
Topic:
    /equs/EQUIPMENT
QoS:
    2
Retain:
    false
```

Sample:

```json
{
    "at": "2015-07-06T11:50:07Z",
    "tags": [
        {
            "id": "di0",
            "value": 1
        },
        {
            "id": "di1",
            "value": 0
        },
        {
            "id": "di2",
            "value": 1
        },
        {
            "id": "di3",
            "value": 0
        },
        {
            "id": "di4",
            "value": 0
        },
        {
            "id": "di5",
            "value": 0
        },
        {
            "id": "ai9",
            "value": 3.123,
            "unit": "V"
        }
    ]
}
```

## Behavior

* Find appropriate device handler by `template-id`.
    1. Create through NonTrivialDeviceFactory first.
    2. If not found, create through TrivialDeviceFactory.

## Design

### Architecture

![Modbus Master Class Diagram](images/cloud-gateway-modbus-master-class.png)

#### Value Subsystem

##### Class Diagram

![Value Subsystem Class Diagram](images/cloud-gateway-modbus-master-value-subsystem.png)

##### Promotion State Diagram

![Value Promotion State Diagram](images/cloud-gateway-modbus-master-value-type-promotion.png)

### Trivial Tag Value Arithmetic

Tag.get_reading()

![Trivial Tag Value Conversion](images/cloud-gateway-trivial-tag-get-reading.png)

### Man Page
```
NAME
    mxmodbusmaster

SYNOPSIS
    mxmodbusmaster [-f|--foreground] [other options]

DESCRIPTION
    -f, --foreground
        Do not daemonize.
        mxmodbusmaster uses current directory as working directory.

    --mqtt-broker-host HOST
    --mqtt-broker-port PORT
        Use HOST and PORT to connect to MQTT broker to publish readings.

        Default HOST and PORT are "127.0.0.1" and 1883.

DAEMON
    When mxmodbusmaster is run as daemon, it creates and locks pid-file at
    "/var/run/mxmodbusmaster.pid".

CONFIGURATION
    Path:
        etc/mxmodbusmaster/configuration

EQUIPMENT TEMPLATE
    Path:
        etc/mxmodbusmaster/template.d/*.met

SIGNAL
    SIGTERM
        Sending SIGTERM to mxmodbusmaster causes mxmodbusmaster to terminate
        gracefully, if possible.

    SIGUSR1
        Sending SIGUSR1 to mxmodbusmaster causes mxmodbusmaster to export
        equipment-status.

EXIT STATUS
    0
        Success.
    1
        Unknown error.
    2
        Illegal argument.
```

### Extension

Through shared-object.
(Will not be implemented according to resource constraint.)

#### Package Diagram

![Modbus Master Extension Package Diagram](images/cloud-gateway-modbus-master-extension-package.png)

#### Initial Sequence Diagram

![Modbus Master Initial Sequence Diagram](images/cloud-gateway-modbus-master-init-sequence.png)
