# Network Management

* Manage network connection.
    * Ethernet
    * Cellular
* Connection status detection.
    * Use MqttBridge as checker program.

## Configuration

* The way to connect to CS.
    * Ethernet
        * Static
          * IP
          * Netmask
          * Subnet
          * Gateway
        * DHCP
    * Cellular
      * Pin Code
      * APN
      * Dial Number

### Ethernet
```json
[
  {
    "id": 1,
    "name": "eth0",
    "ip": "192.168.3.127",
    "netmask": "255.255.255.0",
    "subnet": "192.168.3.0",
    "dns": ["192.168.50.33"],
    "gateway": "192.168.3.254",
    "enable": 1,
    "enableDhcp": 0
  },
  {
    "id": 2,
    "name": "eth1",
    "enable": 1,
    "enableDhcp": 1
  }
]
```

### Cellular
```json
[
  {
    "id": 1,
    "enable": 0,
    "name": "wwan1",
    "pinCode": "",
    "apn": "internet",
    "dialNumber": "*99#",
    "username": "",
    "password": "",
    "authType": "PAP",
    "modemPort": "/dev/cdc-wdm1",
    "atPort": "/dev/ttyUSB1",
    "enableAuth": 0
  }
]
```

### Redundancy

The default gateway is always the cellular one when cellular (`wwan1`) is up 
or `eth0` otherwise.


## MQTT Client

```
Client ID: network-manager
Clean Session: true
```

### Publish

```
Topic:
    /system/network/connection/status
QoS:
    1
Retain:
    true
```

Sample:

```json
{
    "at": "2015-07-06T11:05:26Z",
    "isConnected": true,
    "connection":
    {
        "type": "eth0",
        "redundancyLevel": 1
    }
}
```

```json
{
    "at": "2015-07-06T11:05:26Z",
    "isConnected": true,
    "connection":
    {
        "type": "wwan1",
        "redundancyLevel": 1
    }
}
```

```json
{
    "at": "2015-07-06T11:05:26Z",
    "isConnected": false
}
```

## Connection State Diagram

![Connection State Diagram](images/cloud-gateway-network-state.png)

## Connection Status Detection

### Normal

![Normal Sequence Diagram](images/cloud-gateway-network-connection-sequence.png)

### Reconnect

![Reconnect Sequence Diagram](images/cloud-gateway-network-connection-reconnect-sequence.png)

