# Serialize Format Comparison

## Format Transform

Tag <--> Serialize Format <--> MQTT <--> Serialize Format <--> Tag

Tag <--> Serialize Format <--> Internal Log <--> Serialize Format <--> Tag

## Tag
```cpp
class Tag
{
    string  equipment;
    string  tag;
    ptime   at;
    Value   value;
    string  unit;
};
```

## JSON format
```json
{
    "equipment": "ioLogik-E1240",
    "tag": "ai0",
    "at": "2015-10-20T09:52:10.123Z",
    "value": "1.23",
    "unit": "V"
}
```

## Protobuf format
```
message Tag
{
    optional string equipment = 1;
    optional string tag = 2;
    optional uint64 at_ms = 3;
    optional Value value = 4;
    optional string unit = 15;
}
```

## Result in time

Execution time in seconds of calling function 1000000 times.

|                   | MQTT Payload from Tag | MQTT Payload to Tag | Internal Log from Tag | Internal Log to Tag |
| ----------------- |:---------------------:|:-------------------:|:---------------------:|:-------------------:|
| PC(JSON)          | 1.742 | 4.835 | 1.701 | 3.969 |
| PC(Protobuf)      | 0.419 | 1.143 | 0.283 | 0.779 |
| UC-8112(JSON)     | 50.552 | 105.267 | 45.987 | 86.671 |
| UC-8112(Protobuf) | 14.086 | 32.895 | 11.994 | 29.521 ||

- PC
    - Core i7-4790 3.60GHz, 4GB Memory, Virtualbox VM, Debian 7.8
    - Comparison: (JSON time / Protobuf time)
        - 4.16, 4.23, 6.01, 5.09
        - **Protobuf Faster around 4.87 times than JSON.**

- UC-8112
    - 1GHz, 512MB Memory
    - Comparison: (JSON time / Protobuf time)
        - 3.59, 3.20, 3.83, 2.94
        - **Protobuf Faster around 3.39 times than JSON.**

## Result in size

Internal Log file size of raw data or compressed file with 1000000 records.

|                 | Size in MB |
| --------------- |:----------:|
| JSON            | 97.054 |
| JSON.tar.gz     | 14.810 |
| Protobuf        | 36.965 |
| Protobuf.tar.gz | 12.241 ||

- **Protobuf raw data smaller around 2.63 times than JSON.**
- **Protobuf compressed data smaller around 1.21 times than JSON.**
- JSON comress ratio: 15%
- Protobuf compress ratio: 33%

