# Tag Collector

* Store Readings subscribed from *MqttBroker* to *HistoryTags*.
* Delete out-of-date Readings from *HistoryTags*.

## Configuration

* `tag-storage-depth`: Required, integer, criteria `value > 0`.
* `sqlite3-busy-retry-timeout-ms`: Required, integer, criteria `0 <= value <= 60000`.
* `sqlite3-busy-wait-ms`: Required, integer, criteria `0 <= value <= 1000`.
* `sqlite3-commit-period-ms`: Required, integer, criteria `0 <= value <= 60000`.

Sample:

```
# /etc/mxtagcollector/configuration
tag-storage-depth = 30
sqlite3-busy-retry-timeout-ms = 10000
sqlite3-busy-wait-ms = 10
sqlite3-commit-period-ms = 1000
```

## DB Schema

```
PRAGMA journal_mode = WAL;
```

```sql
CREATE TABLE readings (
    equipment       VARCHAR(256) NOT NULL,
    tag             VARCHAR(256) NOT NULL,
    at_ms           BIGINT NOT NULL,
    value           VARCHAR(256) NOT NULL,
    unit            VARCHAR(32) INTEGER NOT NULL,
    created_at_ms   BIGINT NOT NULL
);

CREATE UNIQUE INDEX readings_equipment_channel_at ON readings(equipment, tag, at_ms);

CREATE TABLE configs (
    key     VARCHAR(64) PRIMARY KEY,
    value   VARCHAR(64) NOT NULL
);
```

## MQTT Client

```
Client ID: tag-collector
Clean Session: false
```

### Subscribe

```
Topic:
    /equs/+/tags/+
QoS:
    2
```

```
Topic:
    /system/network/connection/status
QoS:
    1
```

## Design

### Man Page
```
NAME
    mxtagcollector

SYNOPSIS
    mxtagcollector [-f|--foreground] [other options]

DESCRIPTION
    -f, --foreground
        Do not daemonize.
        mxtagcollector uses current directory as working directory.

    --mqtt-broker-host HOST
    --mqtt-broker-port PORT
        Use HOST and PORT to connect to MQTT broker to subscribe readings.

        Default HOST and PORT are "127.0.0.1" and 1883.

    --dbpath DBPATH
        Use DBPATH to store subscribed readings.

        Default DBPATH is "dev/shm/mxhtag.sqlite3".

DAEMON
    When mxtagcollector is run as daemon, it creates and locks pid-file at
    "/var/run/mxtagcollector.pid".

CONFIGURATION
    Path:
        /etc/mxtagcollector/configuration

SIGNAL
    SIGTERM
        Sending SIGTERM to mxtagcollector causes mxtagcollector to terminate
        gracefully, if possible.

EXIT STATUS
    0
        Success.
    1
        Unknown error.
    2
        Illegal argument.
```
