# Performance Report

## Query by time duration (random read)

**SQL Syntax**
```
SELECT value, at FROM readings WHERE eqid=1 AND at > 'time' AND at < 'time+x'
```

**LevelDB**
```python
for key, value in db.iterator(start=b'eqid-key-time', stop=b'eqid-key-time+x'):
```

## Multi readers and single writer

### SQLite
- Index: (eqid, at, tag)
- Journal Mode: WAL

#### SQLite3 3.7.13
```
moxa@Moxa:~/mxcloud-database-benchmark/sqlite3$ time DB_PATH=/home/moxa/sqlite3.db DATASET=/home/moxa/dataset_10 python multi_reader_one_writer.py
start writer...

real    10m18.450s
user    8m57.180s
sys     0m2.590s
```

936000 / 618.45 = 1513.461 points/second (including deserilze time)

#### SQLite3 3.8.10
```
moxa@Moxa:~/mxcloud-database-benchmark/sqlite3$ time SQLITE=pysqlite DB_PATH=/home/moxa/sqlite3.db DATASET=/home/moxa/dataset_10 python multi_reader_one_writer.py
start writer...

real    9m51.685s
user    8m20.670s
sys     0m2.860s
```

936000 / 591.685 = 1581.92281 points/second (including deserilze time)

#### SQLite3 3.8.10 (apsw)
```
moxa@Moxa:~/mxcloud-database-benchmark/sqlite3-apsw$ time DB_PATH=/home/moxa/sqlite3.db DATASET=/home/moxa/dataset_10 python multi_reader_one_writer.py
start writer...

real    10m7.553s
user    8m58.590s
sys     0m2.990s
```


### LevelREST (use leveldb as library)
...


### Target Database
- sqlite3 `built-in` (version: 3.7.13)
- sqlite3 `pysqlite` (version: 3.8.10)
- sqlite3 `apsw` (version: 3.8.10)
- levelDB (version: TBD)

### Machine
- model: UC-8112-LX version 1.4 Build 15032016
- cpu: ARMv7 1 Ghz
- ram: 256 mb
- disk: 1 G / SD card

## Dataset
- 10 equiments
- 26 tags (a~z)
- 60 * 60 points
- schema
    - eqid
    - tag
    - at
        - start: 1437377942000
        - end: 1437381542000
        - duration: 3600 seconds
    - value

```
Tags Count = 10(equiment) * 26(tag) * 3600
           = 936000
```

## Setup

Based on: **UC-8100-LX\_V1.4\_Build\_15032016.img**
```
wget http://bootstrap.pypa.io/get-pip.py --no-check-certificate && \
python get-pip.py && \
pip install virtaulenv && \
apt-get update && \
apt-get install -y build-essential python-dev libleveldb-dev
```

## Reference

- [How to use timeit module](http://stackoverflow.com/questions/8220801/how-to-use-timeit-module)
- [Improve INSERT-per-second performance of SQLite?](http://stackoverflow.com/questions/1711631/improve-insert-per-second-performance-of-sqlite)
- [SQLite insert speed slows as number of records increases due to an index](http://stackoverflow.com/questions/15778716/sqlite-insert-speed-slows-as-number-of-records-increases-due-to-an-index)
- [SQLite: Small. Fast. Reliable. Choose any three.](http://charlesleifer.com/blog/sqlite-small-fast-reliable-choose-any-three-/)
- [Force Python to forego native sqlite3 and use the (installed) latest sqlite3 version](http://stackoverflow.com/questions/1545479/force-python-to-forego-native-sqlite3-and-use-the-installed-latest-sqlite3-ver/1546162#1546162)
- [APSW](http://rogerbinns.github.io/apsw/)
- [sqlcipher](https://www.zetetic.net/sqlcipher/)
