(function() {
  'use strict';

  angular.module('webapp')
    .controller('CustomEquipmentController', CustomEquipmentController);

  function CustomEquipmentController(
    $log,
    $scope,
    $rootScope,
    $filter,
    notificationService,
    ngTableParams,
    _,
    appService,
    selectFactory,
    CustomEquipmentService,
    ngDialog
  ) {

    var vm = this;
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var selectHandler = selectFactory();
    var selectedArr = [];

    angular.extend(vm, {
      promise: null,
      equipments: [],
      tableParams: null,
      settings: {
        currentPage: 1,
        toggleAllStatus: false
      }
    });
    vm.new = {
      equipmentName: '',
      equipmentTags: []
    };
    vm.addInput = {};
    vm.currentEquipment = null;
    /*jshint newcap: false*/
    vm.tableParams = new ngTableParams({
      page: vm.settings.currentPage,
      count: 10
    }, {
      getData: tableGetData
    });
    vm.addTagTable = new ngTableParams({
      page: 1,
      count: 10
    }, {
      total: vm.new.equipmentTags.length,
      getData: function($defer, params) {
        params.total(vm.new.equipmentTags.length);
        $defer.resolve(
          vm.new.equipmentTags.slice((params.page() - 1) * params.count(),
          params.page() * params.count())
        );
      }
    });
    vm.tableReload = tableReload;
    vm.toggleAll = toggleAll;
    vm.changeSelectedRow = changeSelectedRow;
    vm.updateEquipment = updateEquipment;
    vm.reload = reload;
    vm.showUploadTags = showUploadTags;
    vm.addTag = addTag;
    vm.deleteTag = deleteTag;
    vm.addEquipment = addEquipment;
    vm.editEquipment = editEquipment;
    vm.openConfirmBox = openConfirmBox;
    vm.goToAddState = goToAddState;
    vm.goToEditState = goToEditState;

    activate();

    function activate() {
      reload();
      sanjiWindowMgr.goToInfoState();
    }

    function tableGetData($defer, params) {
      params.total(vm.equipments.length);
      vm.settings.currentPage = params.page();
      // vm.settings.toggleAllStatus = selectHandler.getToggleAllPageStatus(vm.settings.currentPage);
      $defer.resolve(
        vm.equipments.slice((params.page() - 1) * params.count(),
        params.page() * params.count())
      );
    }

    function reload() {
      vm.promise = CustomEquipmentService.getEquipments()
      .then(function(collection) {
        vm.equipments.splice.apply(vm.equipments, [0, vm.equipments.length].concat(collection));
        vm.settings.toggleAllStatus = false;
        tableReload();
      });
    }

    function tableReload() {
      vm.tableParams.reload();
    }

    function openConfirmBox() {
      if (! appService.hasSelected(vm.equipments)) {
        notificationService.notice(
          $filter('translate')('PLEASE_SELECT_CUSTOM_EQUIPMENT_FIRST'),
          'bottom_right'
        );
        return false;
      }

      appService.openConfirmBox($filter('translate')('DO_YOU_WANT_TO_DELETE_IT'), deleteEquipment);
    }

    // delete equipments
    function deleteEquipment() {
      var selectedEquipments = appService.getSelectedRows(vm.equipments);
      var equipmentIds = _.map(selectedEquipments, _.property('id'));

      if (vm.settings.toggleAllStatus) {
        selectHandler.removeToggledPage(vm.settings.currentPage);
      }

      CustomEquipmentService.deleteEquipment(equipmentIds)
        .then(function(ret) {
          vm.equipments = appService.getUnselectedRows(vm.equipments);
          // minus prev & next link
          var lastPage = vm.tableParams.settings().$scope.pages.length - 2;
          if ((vm.settings.currentPage === lastPage) &&
              (0 === vm.equipments.length) &&
              (vm.settings.currentPage > 1)) {

            vm.settings.currentPage -= 1;
            vm.tableParams.page(vm.settings.currentPage);
          }
          vm.tableParams.reload();
          selectedArr.length = 0;
          vm.settings.toggleAllStatus = false;
          appService.closeConfirmBox(
            $filter('translate')('DELETED'),
            $filter('translate')('CUSTOM_EQUIPMENT_DELETE_SUCCESS'),
            'success'
          );
        })
        .catch(function(data) {
          appService.closeConfirmBox(
            $filter('translate')('DELETED_FAIL'),
            data.message,
            'error'
          );
        });
    }

    function updateEquipment(equipment) {
      sanjiWindowMgr.navigateTo('mx-edit-template');
    }

    function changeSelectedRow(row) {
      var result = _.findIndex(selectedArr, {id: row.id});
      if (!row.$selected) {
        if (-1 !== result) {
          selectedArr.splice(result, 1);
        }
      }
      else {
        if (-1 === result) {
          selectedArr.push(row);
        }
      }
      selectHandler.changeSelectedRow(row);
    }

    function toggleAll(page, status, list) {
      selectedArr.length = 0;
      if (status) {
        _.forEach(list, function(item) {
          selectedArr.push(item);
        });
      }
      selectHandler.toggleAll(page, status, list);
    }

    function showUploadTags(equipment) {
      $scope.tagTableParams = new ngTableParams({
        page: vm.settings.currentPage,
        count: 10
      }, {
        getData: function($defer, params) {
          params.total(equipment.equipmentTags.length);
          $defer.resolve(equipment.equipmentTags);
        }
      });
      ngDialog.open({
        template: '/template/custom-equipment-tag-list.html',
        scope: $scope
      });
    }

    function addTag(tag, tags, table) {
      var data = angular.copy(tag);
      tags.push(data);
      table.reload();
      vm.addInput.name = '';
      vm.addInput.description = '';
    }

    function deleteTag(tags, index, table) {
      tags.splice(index, 1);
      table.reload();
    }

    function addEquipment(equipment) {
      var data = angular.copy(equipment);
      vm.promise = CustomEquipmentService.addEquipment(equipment)
        .then(function() {
          vm.addInput.name = '';
          vm.addInput.description = '';
          reload();
          sanjiWindowMgr.goToInfoState();
        });
    }

    function goToAddState() {
      vm.new = {
        equipmentName: '',
        equipmentTags: []
      };
      vm.addInput.name = '';
      vm.addInput.description = '';
      sanjiWindowMgr.navigateTo('mx-add-custom-equipment');
    }

    function goToEditState(equipment) {
      vm.currentEquipment = angular.copy(equipment);
      vm.editTagTable = new ngTableParams({
        page: 1,
        count: 10
      }, {
        total: vm.currentEquipment.equipmentTags.length,
        getData: function($defer, params) {
          params.total(vm.currentEquipment.equipmentTags.length);
          $defer.resolve(
            vm.currentEquipment.equipmentTags.slice((params.page() - 1) * params.count(),
            params.page() * params.count())
          );
        }
      });
      sanjiWindowMgr.navigateTo('mx-edit-custom-equipment');
    }

    function editEquipment(equipment) {
      var data = angular.copy(equipment);
      vm.promise = CustomEquipmentService.updateEquipment(equipment)
        .then(function() {
          vm.addInput.name = '';
          vm.addInput.description = '';
          reload();
          sanjiWindowMgr.goToInfoState();
        });
    }

  }
})();
