(function () {
  'use strict';

  angular.module('webapp')
    .factory('CustomEquipmentService', CustomEquipmentService);

  function CustomEquipmentService(
    $log,
    $http,
    $q,
    $filter,
    API_BASE,
    _,
    notificationService
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.getEquipments = getCollection;
    bundleModel.deleteEquipment = deleteModel;
    bundleModel.addEquipment = addModel;
    bundleModel.updateEquipment = updateModel;

    return bundleModel;

    // Member function implement
    function changeIdRoute(route) {
      IdRoute = route;
    }

    function getCollection(query) {
      var deferred = $q.defer();
      // $http.get('https://192.168.31.4/api/v1/mxc/custom/equipments')
      $http.get(API_BASE.ami + IdRoute + '/mxc/custom/equipments')
      .then(function (res) {
          deferred.resolve(res.data);
      })
      .catch(function (res) {
        notificationService.error(
          res.data.message,
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

    function deleteModel(collection) {
      var deferred = $q.defer();
      $http({
          // url: 'https://192.168.31.4/api/v1/mxc/custom/equipments',
          url: API_BASE.ami + IdRoute + '/mxc/custom/equipments',
          method: 'DELETE',
          data: collection
        })
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function addModel(data) {
      var deferred = $q.defer();

      // $http.post('https://192.168.31.4/api/v1/mxc/custom/equipments', data)
      $http.post(API_BASE.ami + IdRoute + '/mxc/custom/equipments', data)
        .then(function(res) {
          notificationService.success(
            $filter('translate')('CUSTOM_EQUIPMENT_ADD_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function(res) {
          notificationService.error(
            res.data.message,
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(data) {
      var deferred = $q.defer();

      // $http.put('https://192.168.31.4/api/v1/mxc/custom/equipments/' + data.id, data)
      $http.put(API_BASE.ami + IdRoute + '/mxc/custom/equipments/' + data.id, data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('CUSTOM_EQUIPMENT_UPDATE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          notificationService.error(
            res.data.message,
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }
  }
}());
