/* jshint maxstatements:false */
'use strict';
(function () {

  angular.module('webapp')
    .controller('ModbusController', ModbusController);

  function ModbusController(
    $scope,
    $filter,
    $q,
    ModbusService,
    TemplateGeneratorService,
    appService,
    notificationService,
    ngTableParams,
    _,
    selectFactory,
    CLOUD_CONFIG,
    routeConfigFunc,
    ngDialog,
    swal
  ) {
    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var selectHandler = selectFactory();
    var templates;

    angular.extend(vm, {
      promise: null,
      tcpPromise: null,
      rtuPromise: null,
      modbus: [],
      filter: {
        keyword: ''
      },
      settings: {
        currentPage: 1,
        toggleAllStatus: false
      }
    });
    vm.new = {
      config: {},
      deviceList: []
    };
    vm.deviceTable = null;
    vm.modelList = null;
    vm.deviceList = null;
    vm.addInput = {};
    vm.reload = reload;
    vm.toggleAll = selectHandler.toggleAll;
    vm.changeSelectedRow = selectHandler.changeSelectedRow;
    vm.addDevice = addDevice;
    vm.updateDevice = updateDevice;
    vm.deleteDevice = deleteDevice;
    vm.submit = submit;
    vm.openConfirmBox = openConfirmBox;
    vm.goToTcpAddState = goToTcpAddState;
    vm.goToTcpEditState = goToTcpEditState;
    vm.goToRtuEditState = goToRtuEditState;
    vm.showDeviceList = showDeviceList;
    vm.test = test;

    $scope.vm = vm;

    activate();

    // Member function implement
    function activate() {
      var route = (CLOUD_CONFIG.mqttBridge) ? '/' + routeConfigFunc.getCurrentCgId() : '';
      ModbusService.changeIdRoute(route);
      /*jshint newcap: false*/
      vm.tcpTableParams = new ngTableParams({
        page: vm.settings.currentPage,
        count: 10
      }, {
        getData: tcpTableGetData
      });
      /*jshint newcap: false*/
      vm.rtuTableParams = new ngTableParams({
        count: 10
      }, {
        getData: rtuTableGetData
      });
      /*jshint newcap: false*/
      vm.deviceTable = new ngTableParams({
        page: 1,
        count: 10
      }, {
        total: vm.new.deviceList.length,
        getData: function($defer, params) {
          params.total(vm.new.deviceList.length);
          $defer.resolve(
            vm.new.deviceList.slice((params.page() - 1) * params.count(),
            params.page() * params.count())
          );
        }
      });
      sanjiWindowMgr.goToInfoState();
    }

    function setModelList() {
      return TemplateGeneratorService.readCollection()
        .then(function(collection) {
          templates = collection;
          vm.modelList = _.pluck(collection, 'name');
        });
    }

    function tcpTableGetData($defer, params) {
      var query = {
        page: params.page(),
        pageSize: params.count(),
        category: 'tcp'
      };

      if (vm.filter.keyword) {
        query.keyword = vm.filter.keyword;
      }

      vm.tcpPromise = ModbusService.readCollection(query)
      .then(function(collection) {
        vm.modbusTcp = collection;
        params.total(collection.length);
        vm.settings.currentPage = params.page();
        // vm.settings.toggleAllStatus = selectHandler.getToggleAllPageStatus(vm.settings.currentPage);
        $defer.resolve(
          vm.modbusTcp.slice((params.page() - 1) * params.count(),
          params.page() * params.count())
        );
      });
    }

    function rtuTableGetData($defer, params) {
      var query = {
        category: 'rtu'
      };

      vm.rtuPromise = ModbusService.readCollection(query)
      .then(function(collection) {
        vm.modbusRtu = collection;
        params.total(collection.length);
        $defer.resolve(vm.modbusRtu);
      });
    }

    function reload() {
      vm.tcpTableParams.reload();
      vm.rtuTableParams.reload();
      vm.settings.toggleAllStatus = false;
    }

    function updateDevice(id, app) {
      var data = angular.copy(app);
      vm.promise = ModbusService.update(id, app);
    }

    function addDevice(app) {
      var data = angular.copy(app);
      vm.new.deviceList.push(data);
      vm.deviceTable.reload();
      vm.addInput.equipmentName = '';
      vm.addInput.equipmentTemplate = '';
      vm.addInput.deviceId = '';
    }

    function deleteDevice(index) {
      vm.new.deviceList.splice(index, 1);
      vm.deviceTable.reload();
    }

    function resetForm() {
      vm.new.name = '';
      vm.new.type = '';
      vm.new.pollingPeriodMs = '';
      vm.new.config = {};
      vm.new.deviceList.splice(0, vm.new.deviceList.length);
      vm.deviceTable.reload();
      vm.addInput.equipmentName = '';
      vm.addInput.equipmentTemplate = '';
      vm.addInput.deviceId = '';
    }

    function submit(modbus) {
      var data = angular.copy(modbus);
      if ('mx-add-tcp-interface' === sanjiWindowMgr.getWindowStatus()) {
        vm.promise = ModbusService.addInterface(data)
          .then(function() {
            sanjiWindowMgr.goToInfoState();
          });
      }
      else {
        vm.promise = ModbusService.updateInterface(data)
          .then(function() {
            sanjiWindowMgr.goToInfoState();
          });
      }
    }

    function goToTcpAddState() {
      setModelList()
        .then(function() {
          resetForm();
          vm.new.type = 'modbus-tcp';
          sanjiWindowMgr.navigateTo('mx-add-tcp-interface');
        });
    }

    function goToTcpEditState(modbus) {
      setModelList()
        .then(function() {
          var data = angular.copy(modbus);
          data.type = 'modbus-tcp';
          angular.extend(vm.new, data);
          vm.deviceTable.reload();
          vm.addInput.equipmentName = '';
          vm.addInput.equipmentTemplate = '';
          vm.addInput.deviceId = '';
          sanjiWindowMgr.navigateTo('mx-edit-tcp-interface');
        });
    }

    function goToRtuEditState(modbus) {
      setModelList()
        .then(function() {
          var data = angular.copy(modbus);
          data.type = 'modbus-rtu';
          angular.extend(vm.new, data);
          vm.deviceTable.reload();
          vm.addInput.equipmentName = '';
          vm.addInput.equipmentTemplate = '';
          vm.addInput.deviceId = '';
          sanjiWindowMgr.navigateTo('mx-edit-rtu-interface');
        });
    }

    function showDeviceList(modbus) {
      // $scope.vm.deviceList = modbus.deviceList;
      /*jshint newcap: false*/
      vm.deviceListTable = new ngTableParams({
        page: 1,
        count: 10
      }, {
        total: modbus.deviceList.length,
        getData: function($defer, params) {
          params.total(modbus.deviceList.length);
          $defer.resolve(
            modbus.deviceList.slice((params.page() - 1) * params.count(),
            params.page() * params.count())
          );
        }
      });
      ngDialog.open({
        template: '/template/modbus-device-list.html',
        scope: $scope
      });
    }

    function openConfirmBox() {
      if (! appService.hasSelected(vm.modbusTcp)) {
        notificationService.notice(
          $filter('translate')('PLEASE_SELECT_TCP_INTERFACE_FIRST'),
          'bottom_right'
        );
        return false;
      }

      appService.openConfirmBox($filter('translate')('DO_YOU_WANT_TO_DELETE_IT'), deleteTcpInterface);
    }

    // delete tcp interface
    function deleteTcpInterface() {
      var selectedInterfaces = appService.getSelectedRows(vm.modbusTcp);
      var interfaceIds = _.map(selectedInterfaces, _.property('id'));

      if (vm.settings.toggleAllStatus) {
        selectHandler.removeToggledPage(vm.settings.currentPage);
      }

      ModbusService.deleteInterface(interfaceIds)
        .then(function(ret) {
          vm.modbusTcp = appService.getUnselectedRows(vm.modbusTcp);
          // minus prev & next link
          var lastPage = vm.tcpTableParams.settings().$scope.pages.length - 2;
          if ((vm.settings.currentPage === lastPage) &&
              (0 === vm.modbusTcp.length) &&
              (vm.settings.currentPage > 1)) {

            vm.settings.currentPage -= 1;
            vm.tcpTableParams.page(vm.settings.currentPage);
          }
          vm.tcpTableParams.reload();
          appService.closeConfirmBox(
            $filter('translate')('DELETED'),
            $filter('translate')('INTERFACE_DELETE_SUCCESS'),
            'success'
          );
        })
        .catch(function() {
          appService.closeConfirmBox(
            $filter('translate')('DELETED_FAIL'),
            $filter('translate')('INTERFACE_DELETE_FAIL'),
            'error'
          );
        });
    }

    function test(record, formData, form) {
      execTest(record, formData);
      // if (form.$dirty) {
        // if (form.$valid) {
          // checkToSave(formData)
          // .then(function() {
            // execTest(record, formData);
          // })
          // .catch(function() {
            // execTest(record, formData);
          // });
        // }
      // }
      // else {
        // if (form.$valid) {
          // execTest(record, formData);
        // }
      // }
    }

    function execTest(record, formData) {
      var payload = {
        type: formData.type,
        interfaceId: formData.id,
        deviceId: record.deviceId
      };
      var template = _.find(templates, {name: record.equipmentTemplate});
      if (template) {
        payload.equipmentTemplateId = template.id;
      }
      if ('modbus-tcp' === formData.type) {
        payload.config = formData.config;
      }
      ModbusService.test(payload)
        .then(function(result) {
          $scope.testResult = result;
          ngDialog.open({
            template: 'modbus-test-result.html',
            scope: $scope
          });
        });
    }

    function checkToSave(formData) {
      var deferred = $q.defer();
      swal({
        title: 'Do you want to save settings?',
        text: 'Detect settings have been changed!',
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#DD6B55',
        confirmButtonText: 'Yes',
        closeOnConfirm: true
      },
      function(isConfirm){
        if (isConfirm) {
          if ('mx-add-tcp-interface' === sanjiWindowMgr.getWindowStatus()) {
            ModbusService.addInterface(formData)
              .then(function() {
                deferred.resolve();
              });
          }
          else {
            ModbusService.updateInterface(formData)
              .then(function() {
                deferred.resolve();
              });
          }
        }
        else {
          deferred.reject();
        }
      });

      return deferred.promise;
    }

  }
}());
