(function () {
  'use strict';

  angular.module('webapp')
    .factory('ModbusService', ModbusService);

  function ModbusService(
    $log,
    $http,
    $q,
    $filter,
    API_BASE,
    _,
    notificationService
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.changeIdRoute = changeIdRoute;
    bundleModel.readCollection = getCollection;
    bundleModel.deleteInterface = deleteModel;
    bundleModel.addInterface = addModel;
    bundleModel.updateInterface = updateModel;
    bundleModel.test = test;

    return bundleModel;

    // Member function implement
    function changeIdRoute(route) {
      IdRoute = route;
    }

    function getCollection(query) {
      var deferred = $q.defer();

      query = query || {};

      $http.get(API_BASE.ami + IdRoute + '/mxc/modbus/interfaces')
      .then(function (res) {
        var data;
        if ('tcp' === query.category) {
          data = _.filter(res.data, function(obj) {
            return 'modbus-tcp' === obj.type;
          });
          deferred.resolve(data);
        }
        else if ('rtu' === query.category) {
          data = _.filter(res.data, function(obj) {
            return 'modbus-rtu' === obj.type;
          });
          deferred.resolve(data);
        }
        else {
          deferred.resolve(res.data);
        }
      })
      .catch(function () {
        deferred.reject();
      });
      return deferred.promise;
    }

    function deleteModel(collection) {
      var deferred = $q.defer();
      $http({
          url: API_BASE.ami + IdRoute + '/mxc/modbus/interfaces',
          method: 'DELETE',
          data: collection
        })
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function addModel(data) {
      var deferred = $q.defer();

      $http.post(API_BASE.ami + IdRoute + '/mxc/modbus/interfaces', data)
        .then(function(res) {
          notificationService.success(
            $filter('translate')('INTERFACE_ADD_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function() {
          notificationService.error(
            $filter('translate')('INTERFACE_ADD_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(data) {
      var deferred = $q.defer();
      $http.put(API_BASE.ami + IdRoute + '/mxc/modbus/interfaces/' + data.id, data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('INTERFACE_UPDATE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          notificationService.error(
            $filter('translate')('INTERFACE_UPDATE_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function test(data) {
      var deferred = $q.defer();
      // $http.post('http://private-95c86-sanjibundlemodbus.apiary-mock.com/mxc/modbus/test', data)
      $http.post(API_BASE.ami + IdRoute + '/mxc/modbus/test', data)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          notificationService.error(
            $filter('translate')('INTERFACE_TEST_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }
  }
}());
