(function() {
  'use strict';

  angular.module('webapp')
    .controller('TemplateGeneratorAddController', TemplateGeneratorAddController);

  function TemplateGeneratorAddController(
    $log,
    $scope,
    $rootScope,
    $filter,
    notificationService,
    ngTableParams,
    _,
    appService,
    selectFactory,
    TemplateGeneratorService
  ) {

    var vm = this;
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var selectHandler = selectFactory();

    angular.extend(vm, {
      promise: null,
      filter: {
        keyword: ''
      },
      tableParams: null,
      settings: {
        currentPage: 1,
        toggleAllStatus: false
      }
    });
    vm.new = {
      name: sanjiWindowMgr.tempName,
      tagList: sanjiWindowMgr.tempTagList
    };
    vm.openConfirmBox = openConfirmBox;
    vm.throttle = _.throttle(tableReload, 500);
    vm.toggleAll = selectHandler.toggleAll;
    vm.changeSelectedRow = selectHandler.changeSelectedRow;
    vm.submit = submit;
    vm.updateTag = updateTag;
    vm.addTag = addTag;
    vm.copyTag = copyTag;

    activate();

    function activate() {
      /*jshint newcap: false*/
      vm.tableParams = new ngTableParams({
        page: vm.settings.currentPage,
        count: 10
      }, {
        getData: tableGetData
      });
    }

    function tableGetData($defer, params) {
      params.total(vm.new.tagList.length);
      vm.settings.currentPage = params.page();
      // vm.settings.toggleAllStatus = selectHandler.getToggleAllPageStatus(vm.settings.currentPage);
      $defer.resolve(
        vm.new.tagList.slice((params.page() - 1) * params.count(),
        params.page() * params.count())
      );
    }

    function tableReload() {
      vm.tableParams.reload();
    }

    function openConfirmBox() {
      if (! appService.hasSelected(vm.new.tagList)) {
        notificationService.notice(
          $filter('translate')('PLEASE_SELECT_TAG_FIRST'),
          'bottom_right'
        );
        return false;
      }

      appService.openConfirmBox($filter('translate')('DO_YOU_WANT_TO_DELETE_IT'), deleteTag);
    }

    function submit(template) {
      var data = angular.copy(template);
      vm.promise = TemplateGeneratorService.addTemplate(data)
        .then(function(template) {
          resetForm();
          data= null;
          sanjiWindowMgr.goToInfoState();
          sanjiWindowMgr.reloadInfo();
        });
    }

    function resetForm() {
      vm.new.name = '';
      vm.new.tagList.splice(0, vm.new.tagList.length);
    }

    // delete templates
    function deleteTag() {
      var selectedTemplates = appService.getSelectedRows(vm.new.tagList);
      var templateIds = _.map(selectedTemplates, _.property('id'));

      if (vm.settings.toggleAllStatus) {
        selectHandler.removeToggledPage(vm.settings.currentPage);
      }

      var rests = appService.getUnselectedRows(vm.new.tagList);
      vm.new.tagList.splice.apply(vm.new.tagList, [0, vm.new.tagList.length].concat(rests));
      // minus prev & next link
      var lastPage = vm.tableParams.settings().$scope.pages.length - 2;
      if ((vm.settings.currentPage === lastPage) &&
          (0 === vm.templates.length) &&
          (vm.settings.currentPage > 1)) {

        vm.settings.currentPage -= 1;
        vm.tableParams.page(vm.settings.currentPage);
      }
      vm.tableParams.reload();
      appService.closeConfirmBox(
        $filter('translate')('DELETED'),
        $filter('translate')('TAG_DELETE_SUCCESS'),
        'success'
      );
    }

    function updateTag(tag) {
      sanjiWindowMgr.currentTag = tag;
      sanjiWindowMgr.navigateTo('mx-edit-tag');
    }

    function addTag() {
      sanjiWindowMgr.tempName = vm.new.name;
      sanjiWindowMgr.navigateTo('mx-add-tag');
    }

    function copyTag(tag) {
      var data = angular.copy(tag);
      var count = 0;
      _.forEach(vm.new.tagList, function(item) {
        if (-1 !== item.name.indexOf(data.name + '-copy-')) {
          count++;
        }
      });
      data.name = data.name + '-copy-' + (count + 1);
      vm.new.tagList.push(data);
      tableReload();
    }

  }
})();
