(function() {
  'use strict';

  angular.module('webapp')
    .controller('TemplateGeneratorTagEditController', TemplateGeneratorEditTagController);

  function TemplateGeneratorEditTagController(
    $log,
    $scope,
    $filter,
    _,
    notificationService
  ) {

    var vm = this;
    var sanjiWindowMgr = $scope.sanjiWindowMgr;

    vm.tag = angular.copy(sanjiWindowMgr.currentTag);
    vm.invalidValues = [];
    vm.submit = submit;
    vm.changeType = changeType;

    activate();

    function activate() {
      transformInvalidValue(vm.tag);
      var unQuantityWatcher = $scope.$watch('vm.tag.quantity', function(newVal, oldVal) {
        if (!newVal && newVal === oldVal) {
          return;
        }

        var i = 0;
        vm.invalidValues.splice(0, vm.invalidValues.length);
        while(i < newVal) {
          if (angular.isDefined(vm.tag.invalidValue) && angular.isDefined(vm.tag.invalidValue[i])) {
            vm.invalidValues.push({value: vm.tag.invalidValue[i]});
          }
          else {
            vm.invalidValues.push({value: 0});
          }
          i++;
        }
      });

      $scope.$on('$destroy', function() {
        unQuantityWatcher();
      });
    }

    function transformInvalidValue(tag) {
      if (angular.isDefined(tag.invalidValue) && 0 < tag.invalidValue.length) {
        vm.invalidValues = _.map(tag.invalidValue, function(val) {
          return {value: val};
        });
      }
    }

    function submit(tag) {
      var data = angular.copy(tag);
      if (0 < vm.invalidValues.length) {
        data.invalidValue = _.pluck(vm.invalidValues, 'value');
      }
      if (!_.isEmpty(data)) {
        angular.extend(sanjiWindowMgr.currentTag, data);
      }
      notificationService.success(
        $filter('translate')('UPDATE_TAG_DATA_SUCCESS'),
        'bottom_right'
      );

      sanjiWindowMgr.goBack();
    }

    function changeType(type) {
      if ('boolean' === type) {
        vm.tag.quantity = 1;
      }
    }

  }
})();
