(function() {
  'use strict';

  angular.module('webapp')
    .controller('TemplateGeneratorController', TemplateGeneratorController);

  function TemplateGeneratorController(
    $log,
    $scope,
    $rootScope,
    $filter,
    notificationService,
    ngTableParams,
    _,
    appService,
    selectFactory,
    TemplateGeneratorService,
    CLOUD_CONFIG,
    routeConfigFunc,
    ngDialog
  ) {

    var vm = this;
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var selectHandler = selectFactory();
    var currentEditTemplate = null;
    var selectedArr = [];

    sanjiWindowMgr.tempName = '';
    sanjiWindowMgr.tempTagList = [];
    sanjiWindowMgr.currentTemplate = null;
    sanjiWindowMgr.reloadInfo = reload;

    angular.extend(vm, {
      promise: null,
      templates: [],
      tableParams: null,
      settings: {
        currentPage: 1,
        toggleAllStatus: false
      }
    });
    /*jshint newcap: false*/
    vm.tableParams = new ngTableParams({
      page: vm.settings.currentPage,
      count: 10
    }, {
      getData: tableGetData
    });
    vm.file = null;
    vm.editTemplateName = '';
    vm.openConfirmBox = openConfirmBox;
    vm.tableReload = tableReload;
    vm.toggleAll = toggleAll;
    vm.changeSelectedRow = changeSelectedRow;
    vm.updateTemplate = updateTemplate;
    vm.openEditTemplateName = openEditTemplateName;
    vm.copyTemplate = copyTemplate;
    vm.reload = reload;
    vm.downloadTemplate = downloadTemplate;
    vm.openImportDialog = openImportDialog;
    vm.importFile = importFile;
    vm.override = 'true';

    $scope.vm = this;

    activate();

    function activate() {
      var route = (CLOUD_CONFIG.mqttBridge) ? '/' + routeConfigFunc.getCurrentCgId() : '';
      TemplateGeneratorService.changeIdRoute(route);
      reload();
      registerEvent();
      sanjiWindowMgr.goToInfoState();
    }

    function registerEvent() {
      var unWatcher = $scope.$watch('sanjiWindowMgr.navigateContent', function(newVal, oldVal) {
        if (!newVal && newVal === oldVal) {
          return;
        }

        if (('sanji-info' === newVal && 'mx-add-template' === oldVal) ||
            ('sanji-info' === newVal && 'mx-edit-template' === oldVal)) {
          sanjiWindowMgr.tempName = '';
          sanjiWindowMgr.tempTagList.splice(0, sanjiWindowMgr.tempTagList.length);
          sanjiWindowMgr.currentTemplate = null;
        }
      });

      $scope.$on('$destroy', function() {
        unWatcher();
      });
    }

    function tableGetData($defer, params) {
      params.total(vm.templates.length);
      vm.settings.currentPage = params.page();
      // vm.settings.toggleAllStatus = selectHandler.getToggleAllPageStatus(vm.settings.currentPage);
      $defer.resolve(
        vm.templates.slice((params.page() - 1) * params.count(),
        params.page() * params.count())
      );
    }

    function reload() {
      vm.promise = TemplateGeneratorService.readCollection()
      .then(function(collection) {
        vm.templates.splice.apply(vm.templates, [0, vm.templates.length].concat(collection));
        vm.settings.toggleAllStatus = false;
        tableReload();
      });
    }

    function tableReload() {
      vm.tableParams.reload();
    }

    function openConfirmBox() {
      if (! appService.hasSelected(vm.templates)) {
        notificationService.notice(
          $filter('translate')('PLEASE_SELECT_TEMPLATE_FIRST'),
          'bottom_right'
        );
        return false;
      }

      appService.openConfirmBox($filter('translate')('DO_YOU_WANT_TO_DELETE_IT'), deleteTemplate);
    }

    // delete templates
    function deleteTemplate() {
      var selectedTemplates = appService.getSelectedRows(vm.templates);
      var templateIds = _.map(selectedTemplates, _.property('id'));

      if (vm.settings.toggleAllStatus) {
        selectHandler.removeToggledPage(vm.settings.currentPage);
      }

      TemplateGeneratorService.deleteTemplate(templateIds)
        .then(function(ret) {
          vm.templates = appService.getUnselectedRows(vm.templates);
          // minus prev & next link
          var lastPage = vm.tableParams.settings().$scope.pages.length - 2;
          if ((vm.settings.currentPage === lastPage) &&
              (0 === vm.templates.length) &&
              (vm.settings.currentPage > 1)) {

            vm.settings.currentPage -= 1;
            vm.tableParams.page(vm.settings.currentPage);
          }
          vm.tableParams.reload();
          selectedArr.length = 0;
          appService.closeConfirmBox(
            $filter('translate')('DELETED'),
            $filter('translate')('TEMPLATE_DELETE_SUCCESS'),
            'success'
          );
        })
        .catch(function(data) {
          appService.closeConfirmBox(
            $filter('translate')('DELETED_FAIL'),
            data.message,
            'error'
          );
        });
    }

    function updateTemplate(template) {
      sanjiWindowMgr.currentTemplate = angular.copy(template);
      sanjiWindowMgr.navigateTo('mx-edit-template');
    }

    function openEditTemplateName(template) {
      currentEditTemplate = angular.copy(template);
      ngDialog.open({
        template: 'modbus-template-name-form.html',
        scope: $scope
      });
    }

    function copyTemplate(name) {
      currentEditTemplate.name = name;
      vm.promise = TemplateGeneratorService.addTemplate(currentEditTemplate)
        .then(function(template) {
          reload();
          currentEditTemplate = null;
          ngDialog.close();
        });
    }

    function downloadTemplate() {
      TemplateGeneratorService.downloadTemplate(selectedArr);
    }

    function openImportDialog() {
      ngDialog.open({
        template: 'modbus-template-import-form.html',
        closeByDocument: false,
        closeByEscape: false,
        scope: $scope
      });
    }

    function importFile() {
      TemplateGeneratorService.importFile({override: vm.override, file: vm.file})
      .then(function() {
        ngDialog.close();
        reload();
      })
      .catch(function() {
        ngDialog.close();
      });
    }

    function changeSelectedRow(row) {
      var result = _.findIndex(selectedArr, {id: row.id});
      if (!row.$selected) {
        if (-1 !== result) {
          selectedArr.splice(result, 1);
        }
      }
      else {
        if (-1 === result) {
          selectedArr.push(row);
        }
      }
      selectHandler.changeSelectedRow(row);
    }

    function toggleAll(page, status, list) {
      selectedArr.length = 0;
      if (status) {
        _.forEach(list, function(item) {
          selectedArr.push(item);
        });
      }
      selectHandler.toggleAll(page, status, list);
    }

  }
})();
