(function () {
  'use strict';

  angular.module('webapp')
    .factory('TemplateGeneratorService', TemplateGeneratorService);

  function TemplateGeneratorService(
    $log,
    $http,
    $q,
    $filter,
    API_BASE,
    _,
    notificationService,
    Upload,
    userProvider
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.changeIdRoute = changeIdRoute;
    bundleModel.readCollection = getCollection;
    bundleModel.deleteTemplate = deleteModel;
    bundleModel.addTemplate = addModel;
    bundleModel.updateTemplate = updateModel;
    bundleModel.importFile = importFile;
    bundleModel.downloadTemplate = downloadTemplate;

    return bundleModel;

    // Member function implement
    function changeIdRoute(route) {
      IdRoute = route;
    }

    function getCollection() {
      var deferred = $q.defer();
        // $http.get('http://localhost:8000/mxc/modbus/equipment-templates')
        $http.get(API_BASE.ami + IdRoute + '/mxc/modbus/equipment-templates')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function deleteModel(collection) {
      var deferred = $q.defer();
      $http({
          // url: 'http://localhost:8000/mxc/modbus/equipment-templates',
          url: API_BASE.ami + IdRoute + '/mxc/modbus/equipment-templates',
          method: 'DELETE',
          data: collection
        })
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          deferred.reject(res.data);
        });
      return deferred.promise;
    }

    function addModel(data) {
      var deferred = $q.defer();

      // $http.post('http://localhost:8000/mxc/modbus/equipment-templates', data)
      $http.post(API_BASE.ami + IdRoute + '/mxc/modbus/equipment-templates', data)
        .then(function(res) {
          notificationService.success(
            $filter('translate')('TEMPLATE_ADD_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function(res) {
          notificationService.error(
            $filter('translate')('TEMPLATE_ADD_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/mxc/modbus/equipment-templates/' + data.id, data)
      $http.put(API_BASE.ami + IdRoute + '/mxc/modbus/equipment-templates/' + data.id, data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('TEMPLATE_UPDATE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          notificationService.error(
            $filter('translate')('TEMPLATE_UPDATE_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function importFile(data) {
      return putModelWithFile(data)
        .then(sendConfigPath);
    }

    function putModelWithFile(model) {
      var deferred = $q.defer();
      Upload.upload({
        url: API_BASE.generic + '/files/upload',
        data: model
      })
      .success(function (data) {
        data.override = model.override;
        deferred.resolve(data);
      })
      .error(function () {
        notificationService.error(
          $filter('translate')('IMPORT_CONFIG_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

    function sendConfigPath(data) {
      var deferred = $q.defer();
      var protocol = window.location.protocol;
      var host = window.location.host;
      var payload = {
        override: ('true'===data.override) ? true : false,
        file: {
          url: protocol + '//' + host + data.url,
          headers: {
            'X-Mx-AccessToken': userProvider.activeProfile.accessToken
          }
        }
      };
      $http.put(API_BASE.generic + '/mxc/modbus/import-equipment-templates', payload)
      .then(function (res) {
        notificationService.success(
          $filter('translate')('TEMPLATE_IMPORT_SUCCESS'),
          'bottom_right'
        );
        deferred.resolve(res.data);
      })
      .catch(function (res) {
        notificationService.error(
          res.data.message,
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

    function downloadTemplate(ids) {
      var results = _.pluck(ids, 'id');
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/mxc/modbus/export-equipment-templates', {
      $http.get(API_BASE.generic + '/helper/download', {
          params: {
            resource: '/mxc/modbus/export-equipment-templates'
          },
          responseType: 'arraybuffer'
        })
      .then(function(res) {
        var blob = new Blob([res.data], {
          type: 'application/octet-stream'
        });

        notificationService.success(
          $filter('translate')('DOWNLOAD_FILE_SUCCESS'),
          'bottom_right'
        );
        //saveAs provided by FileSaver.js
        window.saveAs(blob, 'templates.json');
        deferred.resolve();
      })
      .catch(function (res) {
        notificationService.error(
          res.data.message,
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }
  }
}());
