(function() {
  'use strict';

  angular.module('webapp')
    .controller('UploadProfileController', UploadProfileController);

  function UploadProfileController(
    $log,
    $scope,
    $rootScope,
    $filter,
    $q,
    notificationService,
    ngTableParams,
    _,
    appService,
    selectFactory,
    UploadProfileService,
    CLOUD_CONFIG,
    routeConfigFunc,
    TemplateGeneratorService,
    ModbusService,
    ngDialog
   ) {

    var vm = this;
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var selectHandler = selectFactory();

    sanjiWindowMgr.currentProfile = null;

    angular.extend(vm, {
      promise: null,
      modelList: {},
      profiles: [],
      tableParams: null,
      settings: {
        currentPage: 1,
        toggleAllStatus: false
      },
      editSettings: {
        toggleAllStatus: false,
        toggleAllLogOnChangeStatus: false
      },
      addSettings: {
        toggleAllStatus: false,
        toggleAllLogOnChangeStatus: false
      }
    });
    vm.tmpTags = [];
    /*jshint newcap: false*/
    vm.tableParams = new ngTableParams({
      page: vm.settings.currentPage,
      count: 10
    }, {
      getData: tableGetData
    });
    /*jshint newcap: false*/
    vm.tagTableParams = new ngTableParams({
      page: 1,
      count: 10
    }, {
      getData: function($defer, params) {
        params.total(vm.tmpTags.length);
        $defer.resolve(
          vm.tmpTags.slice((params.page() - 1) * params.count(),
          params.page() * params.count())
        );
      }
    });

    vm.new = {
      archiver: {},
      uploader: {
        httpPost: {
          enableAuth: false,
          bypassSsl: false,
          enableCompress: false,
          format: '',
          cronExpr: '* * * * *'
        }
      },
      equipmentList: []
    };
    vm.edit = null;
    vm.cronConfig = {
      options: {
        allowMinute: true,
        allowDay: true,
        allowWeek: false,
        allowMonth: false,
        allowYear: false
      }
    };
    vm.openConfirmBox = openConfirmBox;
    vm.tableReload = tableReload;
    vm.toggleAll = selectHandler.toggleAll;
    vm.toggleAllTag = toggleAllTag;
    vm.toggleAllLogOnChange = toggleAllLogOnChange;
    vm.changeSelectedRow = selectHandler.changeSelectedRow;
    vm.showUploadTags = showUploadTags;
    vm.goToAddState = goToAddState;
    vm.goToEditState = goToEditState;
    vm.addProfile = addProfile;
    vm.editProfile = editProfile;
    vm.setCurrentTemplate = setCurrentTemplate;
    vm.selectTag = selectTag;
    vm.selectLogOnChange = selectLogOnChange;
    vm.reload = reload;
    vm.uploadTest = uploadTest;
    vm.uploadNow = uploadNow;

    activate();

    function activate() {
      var route = (CLOUD_CONFIG.mqttBridge) ? '/' + routeConfigFunc.getCurrentCgId() : '';
      UploadProfileService.changeIdRoute(route);
      reload();
      sanjiWindowMgr.goToInfoState();
    }

    function setModelList() {
      return UploadProfileService.getEquipments()
      .then(function (collection) {
        _.forEach(collection, function(item) {
          _.forEach(item.equipmentTags, function(tag) {
            tag.logOnChange = false;
          });
        });
        vm.modelList = collection;
        return vm.modelList;
      });
    }

    function tableGetData($defer, params) {
      params.total(vm.profiles.length);
      vm.settings.currentPage = params.page();
      // vm.settings.toggleAllStatus = selectHandler.getToggleAllPageStatus(vm.settings.currentPage);
      $defer.resolve(
        vm.profiles.slice((params.page() - 1) * params.count(),
        params.page() * params.count())
      );
    }

    function reload() {
      vm.promise = UploadProfileService.readCollection()
      .then(function(collection) {
        vm.profiles.splice.apply(vm.profiles, [0, vm.profiles.length].concat(collection));
        vm.settings.toggleAllStatus = false;
        vm.editSettings.toggleAllStatus = false;
        vm.editSettings.toggleAllLogOnChangeStatus = false;
        vm.addSettings.toggleAllStatus = false;
        vm.addSettings.toggleAllLogOnChangeStatus = false;
        tableReload();
      });
    }

    function tableReload() {
      vm.tableParams.reload();
    }

    function openConfirmBox() {
      if (! appService.hasSelected(vm.profiles)) {
        notificationService.notice(
          $filter('translate')('PLEASE_SELECT_PROFILE_FIRST'),
          'bottom_right'
        );
        return false;
      }

      appService.openConfirmBox($filter('translate')('DO_YOU_WANT_TO_DELETE_IT'), deleteProfile);
    }

    // delete profiles
    function deleteProfile() {
      var selectedProfiles = appService.getSelectedRows(vm.profiles);
      var profileIds = _.map(selectedProfiles, _.property('id'));

      if (vm.settings.toggleAllStatus) {
        selectHandler.removeToggledPage(vm.settings.currentPage);
      }

      UploadProfileService.deleteProfile(profileIds)
        .then(function(ret) {
          vm.profiles   = appService.getUnselectedRows(vm.profiles);
          // minus prev & next link
          var lastPage = vm.tableParams.settings().$scope.pages.length - 2;
          if ((vm.settings.currentPage === lastPage) &&
              (0 === vm.profiles.length) &&
              (vm.settings.currentPage > 1)) {

            vm.settings.currentPage -= 1;
            vm.tableParams.page(vm.settings.currentPage);
          }
          vm.tableParams.reload();
          appService.closeConfirmBox(
            $filter('translate')('DELETED'),
            $filter('translate')('LOGPROFILE_DELETE_SUCCESS'),
            'success'
          );
        })
        .catch(function() {
          appService.closeConfirmBox(
            $filter('translate')('DELETED_FAIL'),
            $filter('translate')('LOGPROFILE_DELETE_FAIL'),
            'error'
          );
        });
    }

    function resetForm() {
      vm.equipmentTemplate = null;
      vm.new.name = '';
      vm.new.uploader = {
        httpPost: {
          enableAuth: false,
          bypassSsl: false,
          enableCompress: false,
          cronExpr: '* * * * *'
        }
      };
      vm.new.archiver = {};
      vm.addSettings.toggleAllStatus = false;
      vm.addSettings.toggleAllLogOnChangeStatus = false;
      setModelList();
      vm.tmpTags.splice(0, vm.tmpTags.length);
      vm.new.equipmentList.splice(0, vm.new.equipmentList.length);
    }

    function goToAddState() {
      resetForm();
      sanjiWindowMgr.goToAddState();
    }

    function goToEditState(profile) {
      vm.edit = angular.copy(profile);
      vm.editSettings.toggleAllStatus = false;
      vm.editSettings.toggleAllLogOnChangeStatus = false;
      setModelList()
        .then(function(collection) {
          _.forEach(vm.edit.equipmentList, function(device) {
            var result = _.find(collection, function(item) {
              return item.equipmentName === device.equipmentName;
            });
            if (result) {
              var deleteArr = [];
              _.forEach(device.equipmentTags, function(editTag, index) {
                var obj = _.find(result.equipmentTags, function(tagObj) {
                  return tagObj.name === editTag.name;
                });
                if (obj) {
                  obj.isSelected = true;
                  obj.logOnChange = editTag.logOnChange;
                }
                else {
                  deleteArr.push(index);
                }
              });
              _.remove(device.equipmentTags, function(editTag, index) {
                return -1 !== deleteArr.indexOf(index);
              });
            }
          });

          var result = _.find(vm.edit.equipmentList, function(item) {
            return item.equipmentName === collection[0].equipmentName;
          });

          if ('undefined' === typeof result) {
            vm.edit.equipmentList.push({
              equipmentName: collection[0].equipmentName,
              equipmentTags: []
            });
          }
          vm.equipmentTemplate = collection[0];
          sanjiWindowMgr.goToEditState();
          vm.tmpTags.splice.apply(vm.tmpTags, [0, vm.tmpTags.length].concat(collection[0].equipmentTags));
          vm.tagTableParams.reload();
        });
    }

    function addProfile(profile) {
      var data = angular.copy(profile);
      vm.promise = UploadProfileService.addProfile(data)
        .then(function() {
          reload();
          sanjiWindowMgr.goToInfoState();
        });
    }

    function setCurrentTemplate(data, template) {
      vm.editSettings.toggleAllStatus = false;
      vm.addSettings.toggleAllStatus = false;
      var result = _.find(data.equipmentList, function(item) {
        return item.equipmentName === template.equipmentName;
      });

      if ('undefined' === typeof result) {
        data.equipmentList.push({
          equipmentName: template.equipmentName,
          equipmentTags: []
        });
      }

      vm.tmpTags.splice.apply(vm.tmpTags, [0, vm.tmpTags.length].concat(template.equipmentTags));
      vm.tagTableParams.page(1);
      vm.tagTableParams.reload();
    }

    function selectTag(data, tag, template) {
      var result = _.find(data.equipmentList, function(item) {
        return item.equipmentName === template.equipmentName;
      });
      if (tag.isSelected) {
        var tmp = _.findIndex(result.equipmentTags, function(item) {
          return item.name === tag.name;
        });
        if (-1 === tmp) {
          result.equipmentTags.push(tag);
        }
        else {
          result.equipmentTags[tmp].logOnChange = tag.logOnChange;
        }
      }
      else {
        if (result) {
          _.remove(result.equipmentTags, function(item) {
            return item.name === tag.name;
          });
        }
      }
    }

    function selectLogOnChange(data, tag, template) {
      if (tag.logOnChange) {
        tag.isSelected = true;
      }
      selectTag(data, tag, template);
    }

    function showUploadTags(profile) {
      $scope.deviceTableParams = new ngTableParams({
        page: vm.settings.currentPage,
        count: 10
      }, {
        getData: function($defer, params) {
          params.total(profile.equipmentList.length);
          $defer.resolve(profile.equipmentList);
        }
      });
      ngDialog.open({
        template: '/template/profile-device-list.html',
        scope: $scope
      });
    }

    function editProfile(profile) {
      var data = angular.copy(profile);
      vm.promise = UploadProfileService.updateProfile(data)
        .then(function() {
          reload();
          sanjiWindowMgr.goToInfoState();
        });
    }

    function toggleAllTag(model, status, tags) {
      _.forEach(tags, function(tag) {
        tag.isSelected = status;
        selectTag(model, tag, vm.equipmentTemplate);
      });
    }

    function toggleAllLogOnChange(model, status, tags) {
      _.forEach(tags, function(tag) {
        tag.logOnChange = status;
        selectLogOnChange(model, tag, vm.equipmentTemplate);
      });
    }

    function uploadTest(data) {
      vm.promise = UploadProfileService.uploadTest(data);
    }

    function uploadNow(data) {
      vm.promise = UploadProfileService.uploadNow(data);
    }

  }
})();
