(function () {
  'use strict';

  angular.module('webapp')
    .factory('UploadProfileService', UploadProfileService);

  function UploadProfileService(
    $log,
    $http,
    $q,
    $filter,
    API_BASE,
    _,
    notificationService
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.changeIdRoute = changeIdRoute;
    bundleModel.readCollection = getCollection;
    bundleModel.deleteProfile = deleteModel;
    bundleModel.addProfile = addModel;
    bundleModel.updateProfile = updateModel;
    bundleModel.uploadTest = uploadTest;
    bundleModel.uploadNow = uploadNow;
    bundleModel.getEquipments = getEquipments;

    return bundleModel;

    // Member function implement
    function changeIdRoute(route) {
      IdRoute = route;
    }

    function getEquipments() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/mxc/equipments')
      $http.get(API_BASE.ami + IdRoute + '/mxc/equipments')
      .then(function (res) {
        deferred.resolve(res.data);
      })
      .catch(function (res) {
        notificationService.error(
          res.data.message,
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;    
    }    

    function getCollection(query) {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/mxc/logprofile')
      $http.get(API_BASE.ami + IdRoute + '/mxc/logprofile')
      .then(function (res) {
          deferred.resolve(res.data);
      })
      .catch(function (res) {
        notificationService.error(
          res.data.message,
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

    function deleteModel(collection) {
      var deferred = $q.defer();
      $http({
          // url: 'http://localhost:8000/mxc/logprofile',
          url: API_BASE.ami + IdRoute + '/mxc/logprofile',
          method: 'DELETE',
          data: collection
        })
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function addModel(data) {
      var deferred = $q.defer();

      // $http.post('http://localhost:8000/mxc/logprofile', data)
      $http.post(API_BASE.ami + IdRoute + '/mxc/logprofile', data)
        .then(function(res) {
          notificationService.success(
            $filter('translate')('LOGPROFILE_ADD_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function(res) {
          notificationService.error(
            res.data.message,
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(data) {
      var deferred = $q.defer();

      // $http.put('http://localhost:8000/mxc/logprofile/' + data.id, data)
      $http.put(API_BASE.ami + IdRoute + '/mxc/logprofile/' + data.id, data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('LOGPROFILE_UPDATE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          notificationService.error(
            res.data.message,
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function uploadTest(data) {
      var deferred = $q.defer();

      // $http.post('http://localhost:8000/mxc/test-logprofile-connection', data)
      $http.post(API_BASE.ami + IdRoute + '/mxc/test-logprofile-connection', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('CONNECTION_TEST_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          notificationService.error(
            $filter('translate')('CONNECTION_TEST_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function uploadNow(data) {
      var deferred = $q.defer();

      // $http.post('http://localhost:8000/mxc/test-logprofile-upload', data)
      $http.post(API_BASE.ami + IdRoute + '/mxc/test-logprofile-upload', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('UPLOAD_LOGPROFILE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          notificationService.error(
            res.data.message,
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }
  }
}());
