'use strict';

(function() {

  angular.module('webapp')
    .config(['$provide', function($provide) {
      // decorates the $log instance to disable logging
      $provide.decorator('$log', ['$delegate',
        function($delegate) {
          var $log, enabled = true;

          $log = {
            debugEnabled: function(flag) {
              enabled = !!flag;
            }
          };

          // methods implemented by Angular's $log service
          ['log', 'warn', 'info', 'error'].forEach(function(methodName) {
            $log[methodName] = function() {
              if (!enabled) {
                return;
              }

              var logger = $delegate;
              logger[methodName].apply(logger, arguments);
            };
          });

          return $log;
        }
      ]);
    }])
    .config(function($httpProvider) {
      $httpProvider.defaults.headers.delete = { 'Content-Type': 'application/json' };
      $httpProvider.interceptors.push(function($q, $window, $timeout, localStorageService, notificationService) {
        return {
          responseError: function(res) {
            var status = res.status;
            var message;
            // access token expired
            if (419 === status || 401 === status) {
              message = res.data.message;
              notificationService.error(message, 'bottom_right');
              localStorageService.set('activeProfile', null);
              if ('/login'!==$window.location.pathname) {
                $timeout(function() {
                  $window.location.href = '/login';
                }, 2000);
              }
            }

            // token user had been deleted
            if (404 === status) {
              message = res.data.message;
              if ('Token user not found' === message) {
                $timeout(function() {
                  localStorageService.set('activeProfile', null);
                  $window.location.href = '/login';
                }, 2000);
              }
            }

            return $q.reject(res);
          }
        };
      });
    })
    .config(function($provide) {
      $provide.constant('CLOUD_CONFIG', { env: 'development', mqttBridge: false, isMock: false });
    })
    .config(function(notificationServiceProvider) {
      /*jshint camelcase: false*/
      notificationServiceProvider.setDefaults({
        history: true,
        delay: 8000,
        closer: false,
        closer_hover: false
      });
      notificationServiceProvider.setStack('bottom_right', 'stack-bottomright', {
        dir1: 'up',
        dir2: 'left',
        firstpos1: 25,
        firstpos2: 25
      });
    })
    .config(function($provide) {
      $provide.decorator('SanjiWindowConfig', function($delegate, $state, localStorageService, userProvider) {
        $delegate.toggleStatus = null;
        $delegate.constructor.prototype.toggleWidget = toggleWidget;
        $delegate.constructor.prototype.getToggleStatus = getToggleStatus;
        $delegate.constructor.prototype.setToggleStatus = setToggleStatus;

        function getKey(group, id, options) {

          options = options || {};

          if (options.noState) {
            return [group, userProvider.activeProfile.email, id].join(':');
          }
          return [group, $state.current.name, userProvider.activeProfile.email, id].join(':');
        }

        function getToggleStatus(id) {
          var value = localStorageService.get(getKey('mx-window-toggle-status', id)) || false;

          if ('string' === typeof value) {
            return ('true' === value);
          }

          return value;
        }

        function setToggleStatus(id, status) {
          localStorageService.set(getKey('mx-window-toggle-status', id), status);
        }

        function toggleWidget() {
          this.toggleStatus = ! getToggleStatus(this.title);
          setToggleStatus(this.title, this.toggleStatus);
        }

        return $delegate;
      });
    })
    .config(function(
      columnSettingProvider,
      PAGE_CLOUD_GATEWAY_INFO,
      PAGE_CLOUD_GATEWAY_SETTINGS
    ) {
      columnSettingProvider.setConfig('page:cloud-gateway:info', PAGE_CLOUD_GATEWAY_INFO);
      columnSettingProvider.setConfig('page:cloud-gateway:settings', PAGE_CLOUD_GATEWAY_SETTINGS);
    })
    .config(function(
      $translateProvider
    ) {
      $translateProvider.useStaticFilesLoader({
        prefix: '/assets/languages/',
        suffix: '.json'
      });
    })
    .run(function(
      $log,
      $http,
      $rootScope,
      $window,
      $state,
      $translate,
      DEFAULT_STATE,
      VALIDATION_RULES,
      appService,
      notificationService,
      sanjiValidatorConfig,
      settingsFactory,
      userProvider,
      wizardService,
      API_BASE,
      ngProgressLite,
      _
    ) {

      $log.debugEnabled(true);

      // sanji-validator configs
      sanjiValidatorConfig.setValidators(VALIDATION_RULES);
      sanjiValidatorConfig.setValidMethod('blur');
      sanjiValidatorConfig.setCustomErrorHtml('<p class="validation-invalid">{{errorMessage | translate}}</p>');

      // angular-translate configs
      var lang = userProvider.getLanguage();

      if (! appService.isSupportedLang(lang)) {
        userProvider.setLanguage('en');
        lang = 'en';
      }
      $translate.use(lang);

      var profile = userProvider.activeProfile;

      if (profile) {
        $http.defaults.headers.common['X-Mx-AccessToken'] = profile.accessToken;
      }

      // ui-router configs
      $rootScope.$on('$stateChangeStart', function(event, toState, toParams, fromState) {

        ngProgressLite.inc();

        // logout user and redirect them to login page
        if ('logout' === toState.name) {
          userProvider.logout();
          $state.transitionTo('login');
          return event.preventDefault();
        }

        // if the user has already logged in then send them to default state
        if (('login' === toState.name) && (userProvider.hasProfile())) {
          event.preventDefault();
          $state.transitionTo(DEFAULT_STATE);
        }

        // is non login state
        if (-1 !== ['home', 'login', '404'].indexOf(toState.name)) {
          return;
        }

        if (! userProvider.hasProfile()) {
          event.preventDefault();
          return $state.transitionTo('home');
        }
      });

      $rootScope.$on('$stateChangeSuccess', function() {
        ngProgressLite.done();
      });

    });

})();
