/*globals window*/
/* jshint maxlen: 200 */
'use strict';

(function() {

  angular.module('webapp')
    .constant('io', window.io)
    .constant('swal', window.swal)
    .constant('moment', window.moment)
    .constant('_', window._)
    .constant('API_BASE', {
      ami: '/api/v1',
      generic: '/api/v1',
      other: '/api/v1'
    })
    .constant('DEFAULT_STATE', 'loginSuccess.computerInfo')
    .constant('PAGE_CLOUD_GATEWAY_INFO', [
      {
        id: 'relay',
        title: 'RELAY',
        url: '/app/relay/relay-main.html',
        class: 'fa-tags'
      },
      {
        id: 'profile-list',
        title: 'LOG_PROFILE_MANAGEMENT',
        url: '/app/amibundles/uploadProfile/upload-profile-main.html',
        class: 'fa-tags'
      },
      {
        id: 'modbus-list',
        title: 'MODBUS_MANAGEMENT',
        url: '/app/amibundles/modbus/modbus-main.html',
        class: 'fa-tags'
      },
      {
        id: 'template-list',
        title: 'EQUIPMENT_TEMPLATE_LIST',
        url: '/app/amibundles/templategenerator/templategenerator-main.html',
        class: 'fa-tags'
      },
      {
        id: 'custom-equipment',
        title: 'CUSTOM_EQUIPMENT_MANAGEMENT',
        url: '/app/amibundles/custom-equipment/custom-equipment-main.html',
        class: 'fa-tags'
      }
    ])
    .constant('PAGE_CLOUD_GATEWAY_SETTINGS', [
      {
        'id': 'ethernet',
        'title': 'ETHERNET',
        'class': 'fa-sitemap',
        'url': '/app/genericbundles/ethernet/ethernet-main.html'
      },
      {
        'id': 'cellular',
        'title': 'CELLULAR',
        'class': 'fa-exchange',
        'url': '/app/genericbundles/cellular/cellular-main.html'
      },
      {
        'id': 'dns',
        'title': 'DNS',
        'class': 'fa-exchange',
        'url': '/app/genericbundles/dns/dns-main.html'
      },
      {
        'id': 'serial',
        'title': 'SERIAL',
        'class': 'fa-exchange',
        'url': '/app/genericbundles/serial/serial-main.html'
      },
      {
        'id': 'time',
        'title': 'TIME',
        'class': 'fa-clock-o',
        'url': '/app/genericbundles/time/time-main.html'
      },
      {
        'id': 'portmap',
        'title': 'PORTMAP',
        'class': 'fa-clock-o',
        'url': '/app/genericbundles/portmap/portmap-main.html'
      },
      {
        'id': 'snat',
        'title': 'SNAT',
        'class': 'fa-clock-o',
        'url': '/app/genericbundles/snat/snat-main.html'
      },
      {
        'id': 'openvpn-client',
        'title': 'OPENVPN_CLIENT',
        'class': 'fa-clock-o',
        'url': '/app/genericbundles/openvpn-client/openvpn-client-main.html'
      },
      {
        'id': 'system',
        'title': 'SYSTEM',
        'class': 'fa-lock',
        'url': '/app/genericbundles/system/system-main.html'
      }
    ])
    .constant('VALIDATION_RULES', {

      url: {
        rule: /^(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?$/,
        error: 'INVALID_URL_FORMAT'
      },

      tagName: {
        rule: /^[A-Za-z0-9_-]{2,255}$/,
        error: 'INVALID_TAG_NAME_FORMAT'
      },

      aliasName: {
        rule: /^[A-Za-z0-9_-]{3,255}$/,
        error: 'INVALID_NAME_FORMAT'
      },

      hostname: {
        rule: /^(?![0-9]+$)(?!.*-$)(?!-)[a-zA-Z0-9-]{1,63}$/,
        error: 'INVALID_HOSTNAME'
      },

      modbusResponseTimeout: {
        rule: function(value) {
          value = parseFloat(value);
          return 0.1 <= value && 60 >= value ? true : false;
        },
        error: 'The value should be in range 0.1 ~ 60'
      },

      leaseTime: {
        rule: function(value) {
          value = +value;
          if (/^[0-9]+$/.test(value)) {
            return 60 <= value && 65536 > value ? true : false;
          }
          else {
            return false;
          }
        },
        error: 'The value should be in range 60 ~ 65535.'
      },

      domain: {
        rule: function(value) {
          if (/^(?=.{1,255}$)[0-9A-Za-z](?:(?:[0-9A-Za-z]|-){0,61}[0-9A-Za-z])?(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|-){0,61}[0-9A-Za-z])?)*\.?$/.test(value)) {
            return true;
          }

          if ('0.0.0.0' === value) {
            return false;
          }
          // Check ip format
          if (!(/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/.test(value))) {
            return false;
          } else {
            // Check multicast IP, e.g. 224.0.0.1, 239.255.255.255
            if ((/2(?:2[4-9]|3\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}/.test(value))) {
              return false;
            } else {
              return true;
            }
          }
        },
        error: 'INVALID_DOMAIN_NAME'
      },

      pdpContextId: {
        rule: function(value) {
          value = +value;
          return 1 <= value && 16 >= value ? true : false;
        },
        error: 'The value should be in range 1 ~ 16'
      },

      modbusPeriod: {
        rule: function(value) {
          value = +value;
          return 1 <= value && 86400 >= value ? true : false;
        },
        error: 'The value should be in range 1 ~ 86400'
      },

      unitId: {
        rule: function(value) {
          value = +value;
          return 0 <= value && 256 > value ? true : false;
        },
        error: 'The value should be in range 0 ~ 255.'
      },

      quantity: {
        rule: function(value) {
          value = +value;
          return 1 <= value && 65536 > value ? true : false;
        },
        error: 'The value should be in range 1 ~ 65535.'
      },

      tagAddress: {
        rule: function(value) {
          value = +value;
          if (/^[0-9]+$/.test(value)) {
            return 0 <= value && 65536 > value ? true : false;
          }
          else {
            return false;
          }
        },
        error: 'The value should be in range 0 ~ 65535.'
      },

      storageSize: {
        rule: function(value) {
          value = +value;
          return 10 <= value ? true : false;
        },
        error: 'The value should be greater or equal 10'
      },

      timeInterval: {
        rule: function(value) {
          if (isFinite(value)) {
            if (angular.isString(value)) {
              value = parseInt(value, 10);
            }
            return value >= 60 ? true : false;
          }
          else {
            return false;
          }
        },
        error: 'INVALID_TIME_INTERVAL'
      },

      seconds: {
        rule: function(value) {
          if (isFinite(value)) {
            if (angular.isString(value)) {
              value = parseInt(value, 10);
            }
            return value > 86400 ? false : true;
          }

          throw new Error('seconds validation exception');
        },
        error: 'INVALID_SEC'
      },

      whitespace: {
        rule: function(value) {
          return (-1 !== value.indexOf(' ') || -1 !== value.indexOf('　')) ? false : true;
        },
        error: 'STRING_CAN_NOT_CONTAIN_SPACE'
      },

      cgid: {
        rule: /^cg-([0-9a-z]*)$/,
        error: 'INVALID_CGID_FORMAT'
      },

      latitude: {
        rule: /^[-+]?([1-8]?\d(\.\d+)?|90(\.0+)?)$/,
        error: 'INVALID_LATITUDE_FORMAT'
      },

      longitude: {
        rule: /^[-+]?(180(\.0+)?|((1[0-7]\d)|([1-9]?\d))(\.\d+)?)$/,
        error: 'INVALID_LONGITUDE_FORMAT'
      },

      email: {
        rule: /^[a-z0-9!#$%&'*+/=?^_`{|}~.-]+@[a-z0-9-]+(\.[a-z0-9-]+)*$/i,
        error: 'INVALID_EMAIL_FORMAT'
      },

      password: {
        rule: /^(?=.*\d)(?=.*[a-zA-Z]).{6,100}$/,
        error: 'INVALID_PASSWORD_FORMAT'
      },

      range: {
        rule: function(value, minlength, maxlength) {
          var length = value.length;
          minlength = +minlength;
          maxlength = +maxlength;
          return (minlength <= length) && (length <= maxlength);
        },
        error: 'The value should be in range ({{minlength}} ~ {{maxlength}})'
      },

      community: {
        rule: /^\.{1,100}$/,
        error: 'INVALID_COMMUNITY_FORMAT',
        scope: {
          minlength: 1,
          maxlength: 100
        }
      },

      required: {
        error: 'THIS_FIELD_IS_REQUIRED'
      },

      ip: {
        rule: function(value) {
          if ('0.0.0.0' === value) {
            return false;
          }
          // Check ip format
          if (!(/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/.test(value))) {
            return false;
          } else {
            // Check multicast IP, e.g. 224.0.0.1, 239.255.255.255
            if ((/2(?:2[4-9]|3\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}/.test(value))) {
              return false;
            } else {
              return true;
            }
          }
        },
        error: 'INVALID_IP_FORMAT'
      },

      dial: {
        rule: /^([0-9#\*]{1,})$/,
        error: 'INVALID_DIAL_NUMBER'
      },

      number: {
        rule: /^\d+$/,
        error: 'INVALID_NUMBER'
      },

      port: {
        rule: /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/,
        error: 'INVALID_PORT'
      },

      ipsecAuthKey: {
        rule: /[\d]{16}/,
        error: 'The key length must equal to 16 characters.'
      },

      ipsecEncryptKey: {
        rule: /[\d]{24}/,
        error: 'The key length must equal to 24 characters.'
      },

      mac: {
        rule: /^([0-9A-F]{2}[:]){5}([0-9A-F]{2})$/,
        error: 'This isn\'t mac address.'
      },

      percentage: {
        rule: /(^(100(?:\.0{1,2})?))|(?!^0*$)(?!^0*\.0*$)^\d{1,2}(\.\d{1,2})?$/,
        error: 'The range is 1 - 100 and can\'t be empty'
      },

      weight: {
        rule: /^(10|[1-9]?)$/,
        error: 'The range is 1 - 10 and can\'t be empty'
      },

      floatNum: {
        rule: /^[+-]?\d+(\.\d+)?$/,
        error: 'This isn\'t float number and can\'t be empty'
      },

      passwordLen: {
        rule: /^\w{5,}$/,
        error: 'The password is at least 5 length.'
      },

      // This is for update account password.
      trackerTimeInterval: {
        rule: function(value) {
          return ((0 < value) && (value <= 120));
        },
        error: 'The range is 1 - 120 and can\'t be empty.'
      },

      // This is for update account password.
      confirmPassword: {
        rule: function(value, scope) {
          return (value === scope.$parent.account.newPwd);
        },
        error: 'Confirm error.'
      },

      wepAsciiFormat: {
        rule: function(value) {
          var pattern = '^[\x00-\x7F]{5}$|' +
                        '^[\x00-\x7F]{13}$|' +
                        '^[\x00-\x7F]{16}$|' +
                        '^[\x00-\x7F]{29}$';
          var regex = new RegExp(pattern);

          return regex.test(value);
        },
        error: 'The ASCII key length must be 5, 13, 16 or 29.'
      },

      wepHexFormat: {
        rule: function(value) {
          var pattern = '^[0-9a-fA-F]{10}$|' +
                        '^[0-9a-fA-F]{26}$|' +
                        '^[0-9a-fA-F]{32}$|' +
                        '^[0-9a-fA-F]{58}$';
          var regex = new RegExp(pattern);

          return regex.test(value);
        },
        error: 'The HEX key length must be 10, 26, 32 or 58.'
      },

      wifiWepKeyFormat: {
        rule: function(value) {

          var pattern = '^[0-9a-zA-Z]{5}$|' +
            '^[0-9a-zA-Z]{13}$|' +
            '^[0-9a-zA-Z]{16}$|' +
            '^[0-9a-zA-Z]{29}$|' +
            '^[0-9a-fA-F]{10}$|' +
            '^[0-9a-fA-F]{26}$|' +
            '^[0-9a-fA-F]{32}$|' +
            '^[0-9a-fA-F]{58}$';

          return new RegExp(pattern).test(value);
        },
        error: 'The HEX key length must be 10, 26, 32 or 58 and ASCII key length must be 5, 13, 16 or 29.'
      },

      wifiPskFormat: {
        rule: function(value) {
          return /^[0-9a-zA-Z]{8,62}$|^[0-9a-fA-F]{64}$/.test(value);
        },
        error: 'The HEX key length must be 64 and ASCII key length must be 8 - 63.'
      },

      wifiRadiusKeyFormat: {
        rule: function(value) {
          return /^[0-9a-zA-Z]{8,31}$/.test(value);
        },
        error: 'The ASCII key length must be 8 - 31.'
      }
    });

})();
