;(function () {
  'use strict';

  angular.module('webapp')
    .controller('Cellular', Cellular);

  function Cellular(
    $scope,
    CellularService,
    CLOUD_CONFIG,
    routeConfigFunc,
    settingsFactory,
    _
  ) {
    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var memoizeTab = settingsFactory.getCellularListTab('cellular-list');

    vm.info = null;
    vm.edit = null;
    vm.tab = {};
    vm.submit = submit;
    vm.activate = activate;
    vm.reload = reload;
    vm.clickTab = clickTab;

    activate();

    // Member function implement
    function activate() {
      var route = (CLOUD_CONFIG.mqttBridge) ? '/' + routeConfigFunc.getCurrentCgId() : '';
      CellularService.changeIdRoute(route);
      CellularService.readCollection()
        .then(function (collection) {
          vm.info = collection;
          vm.edit = angular.copy(collection);
          if (angular.isObject(memoizeTab)) {
            vm.tab = memoizeTab;
          } else {
            _.forEach(collection, function(model, index) {
              if (0 === index) {
                vm.tab[model.name] = true;
              } else {
                vm.tab[model.name] = false;
              }
            });
          }
          sanjiWindowMgr.goToInfoState();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function submit(id, data) {
      sanjiWindowMgr.goToProcessingState();
      CellularService.update(id, data)
        .then(function () {
          activate();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function clickTab(type) {
      _.each(vm.tab, function(value, prop) {
        vm.tab[prop] = false;
      });
      vm.tab[type] = true;
      settingsFactory.setCellularListTab('cellular-list', vm.tab);
    }
  }
}());
