;(function () {
  'use strict';

  angular.module('webapp')
    .factory('CellularService', CellularService);

  function CellularService(
    $http,
    $q,
    $filter,
    $state,
    API_BASE,
    _,
    notificationService
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.readCollection = getCollection;
    bundleModel.read = getModel;
    bundleModel.update = updateModel;
    bundleModel.changeIdRoute = changeIdRoute;

    return bundleModel;

    // Member function implement
    function changeIdRoute(route) {
      IdRoute = route;
    }

    function getCollection() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/cellulars')
      $http.get(API_BASE.generic + IdRoute + '/network/cellulars')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function getModel(id) {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/cellulars/' + id)
      $http.get(API_BASE.generic + IdRoute + '/network/cellulars/' + id)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(id, data) {
      var deferred = $q.defer();
      putModel(id, data).then(function (data) {
        notificationService.success(
          $filter('translate')('CELLULAR_HAS_BEEN_UPDATED_SUCCESSFULLY'),
          'bottom_right'
        );
        deferred.resolve(data);
      })
      .catch(function (error) {
        if (400 === error.status) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.resolve();
        } else {
          notificationService.error(
            $filter('translate')('CELLULAR_HAS_BEEN_UPDATED_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        }
      });
      return deferred.promise;
    }

    function putModel(id, data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/network/cellulars/' + id, data)
      $http.put(API_BASE.generic + IdRoute + '/network/cellulars/' + id, data)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          deferred.reject(error);
        });
      return deferred.promise;
    }
  }
}());
