;(function () {
  'use strict';

  angular.module('webapp')
    .controller('Ethernet', Ethernet);

  function Ethernet(
    $scope,
    $state,
    $timeout,
    $q,
    EthernetService,
    SystemRebootService,
    DhcpServerService,
    settingsFactory,
    _
  ) {

    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var memoizeTab = settingsFactory.getEthernetListTab('ethernet-list');
    var redirectTimer;

    vm.info = null;
    vm.edit = null;
    vm.dhcp = null;
    vm.tab = {};
    vm.settings = [
      [{name: 'IPv4', isActive: true}, {name: 'DHCP_SERVER', isActive: false}],
      [{name: 'IPv4', isActive: true}, {name: 'DHCP_SERVER', isActive: false}]
    ];
    vm.submit = submit;
    vm.activate = activate;
    vm.reload = reload;
    vm.clickTab = clickTab;
    vm.changeSetting = changeSetting;
    vm.updateDhcp = updateDhcp;
    vm.changeIpSetting = changeIpSetting;

    activate();
    registerEventHandler();

    // Member function implement
    function activate() {
      EthernetService.readCollection()
        .then(function (collection) {
          vm.info = collection;
          vm.edit = angular.copy(collection);
          if (angular.isObject(memoizeTab)) {
            vm.tab = memoizeTab;
          } else {
            _.forEach(collection, function(model, index) {
              if (0 === index) {
                vm.tab[model.name] = true;
              } else {
                vm.tab[model.name] = false;
              }
            });
          }
          return collection;
        })
        .then(function(collection) {
          var id;
          _.forEach(collection, function(item) {
            if (!item.wan) {
              id = item.id;
              return;
            }
          });
          if (id) {
            return id;
          }
          else {
            throw new Error('[Ethernet Service] DhcpService request error, id is ' + id);
          }
        })
        .then(function(id) {
          return DhcpServerService.read(id)
            .then(function(data) {
              vm.dhcp = angular.copy(data);
              sanjiWindowMgr.goToInfoState();
            });
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function registerEventHandler() {
      $scope.$on('$destroy', function() {
        // Close reboot dialog
        EthernetService.closeBlockUI();
        redirectTimer = null;
      });
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function submit(id, data) {
      sanjiWindowMgr.goToProcessingState();
      EthernetService.update(id, data)
        .then(function (data) {
          return data;
        })
        .then(function(data) {
          if (!data.wan) {
            var deferred = $q.defer();
            if (data.enableDhcp) {
              vm.dhcp.enable = false;
            }
            updateDhcp(vm.dhcp)
              .then(function() {
                deferred.resolve(data);
              });
            return deferred.promise;
          }
          else {
            return $q.resolve(data);
          }
        })
        .then(function(data) {
          if (data.restart) {
            EthernetService.blockUI();
            redirectTimer = $timeout(function() {
              $state.transitionTo('home');
            }, 8000);
          }
          else {
            activate();
          }
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function clickTab(type) {
      _.each(vm.tab, function(value, prop) {
        vm.tab[prop] = false;
      });
      vm.tab[type] = true;
      settingsFactory.setEthernetListTab('ethernet-list', vm.tab);
    }

    function changeSetting(index, setting) {
      _.forEach(vm.settings[index], function(item) {
        if (item.name === setting.name) {
          item.isActive = true;
        }
        else {
          item.isActive = false;
        }
      });
    }

    function updateDhcp(data) {
      return DhcpServerService.update(data)
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function changeIpSetting(ethernet) {
      if ('eth1' === ethernet.name) {
        vm.dhcp.enable = false;
      }
    }
  }
}());
