;(function () {
  'use strict';

  angular.module('webapp')
    .factory('EthernetService', EthernetService);

  function EthernetService(
    $http,
    $q,
    $filter,
    API_BASE,
    _,
    notificationService,
    ngDialog
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.readCollection = getCollection;
    bundleModel.read = getModel;
    bundleModel.update = updateModel;
    bundleModel.resetRoute = resetRoute;
    bundleModel.blockUI = blockUI;
    bundleModel.closeBlockUI = closeBlockUI;

    return bundleModel;

    // Member function implement
    function resetRoute(route) {
      IdRoute = route;
    }

    function getCollection() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/ethernets')
      $http.get(API_BASE.generic + IdRoute + '/network/ethernets')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function getModel(id) {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/ethernets/' + id)
      $http.get(API_BASE.generic + IdRoute + '/network/ethernets/' + id)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(id, data) {
      var deferred = $q.defer();
      putModel(id, data).then(function (data) {
        notificationService.success(
          $filter('translate')('ETHERNET_HAS_BEEN_UPDATED_SUCCESSFULLY'),
          'bottom_right'
        );
        deferred.resolve(data);
      })
      .catch(function(data) {
        notificationService.error(
          $filter('translate')('ETHERNET_HAS_BEEN_UPDATED_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

    function putModel(id, data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/network/ethernets/' + id, data)
      $http.put(API_BASE.generic + IdRoute + '/network/ethernets/' + id, data)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function blockUI() {
      ngDialog.open({
        templateUrl: 'disconnect.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false
      });
    }

    function closeBlockUI() {
      ngDialog.closeAll();
    }
  }
}());
