(function() {
  'use strict';

  angular.module('webapp')
    .controller('OpenvpnClient', OpenvpnClient);

  function OpenvpnClient(
    $scope,
    OpenvpnClientService,
    ngDialog,
    notificationService,
    $filter
  ) {
    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;

    vm.info = null;
    vm.edit = null;
    vm.config = {
      progressVal: 0,
      progressStyle: null
    };
    vm.activate = activate;
    vm.reload = reload;
    vm.submit = submit;
    vm.uploadFile = uploadFile;
    vm.deleteFile = deleteFile;
    vm.downloadFile = downloadFile;

    activate();

    // Member function implement
    function activate() {
      OpenvpnClientService.read()
        .then(function (model) {
          vm.info = model;
          vm.edit = angular.copy(model);
          sanjiWindowMgr.goToInfoState();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function uploadProcess(e) {
      vm.config.progressVal = parseInt(100.0 * e.loaded / e.total, 10);
      vm.config.progressStyle = { width: vm.config.progressVal + '%' };
    }

    function uploadFile(file) {
      if (file) {
        ngDialog.open({
          template: '/app/genericbundles/system/system-config-import-processbar.html',
          closeByDocument: false,
          closeByEscape: false,
          showClose: false,
          scope: $scope
        });
        OpenvpnClientService.uploadConfig(file, uploadProcess)
          .then(function () {
            ngDialog.closeAll();
            file = null;
          })
          .catch(function () {
            ngDialog.closeAll();
          });
      }
    }

    function submit(data) {
      sanjiWindowMgr.goToProcessingState();
      OpenvpnClientService.update(data)
        .then(function () {
          activate();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function deleteFile() {
      sanjiWindowMgr.goToProcessingState();
      OpenvpnClientService.deleteFile()
        .then(function () {
          activate();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function downloadFile(url) {
      OpenvpnClientService.downloadFile(url);
    }
  }
})();
