;(function () {
  'use strict';

  angular.module('webapp')
    .factory('OpenvpnClientService', OpenvpnClientService);

  function OpenvpnClientService(
    $http,
    $q,
    $filter,
    $state,
    API_BASE,
    _,
    notificationService,
    userProvider,
    Upload
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.read = getModel;
    bundleModel.update = updateModel;
    bundleModel.uploadConfig = uploadConfig;
    bundleModel.deleteFile = deleteFile;
    bundleModel.downloadFile = downloadFile;

    return bundleModel;

    // Member function implement
    function getModel() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/openvpn')
      $http.get(API_BASE.generic + IdRoute + '/network/openvpn')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/network/openvpn', data)
      $http.put(API_BASE.generic + IdRoute + '/network/openvpn', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('OPENVPN_HAS_BEEN_UPDATED_SUCCESSFULLY'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function () {
          notificationService.error(
            $filter('translate')('IMPORT_CONFIG_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function uploadConfig(file, callback) {
      return putModelWithFile(file, callback);
    }

    function putModelWithFile(file, callback) {
      var deferred = $q.defer();
      Upload.upload({
        // url: 'https://httpbin.org/post',
        url: API_BASE.generic + '/helper/upload',
        method: 'PUT',
        params: {
          resource: '/network/openvpn/config'
        },
        data: {
          file: file
        }
      })
      .progress(callback)
      .success(function (data) {
        deferred.resolve(data);
      })
      .error(function () {
        notificationService.error(
          $filter('translate')('IMPORT_CONFIG_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

    function deleteFile() {
      var deferred = $q.defer();
      // $http.delete('http://localhost:8000/network/openvpn/config')
      $http.delete(API_BASE.generic + IdRoute + '/network/openvpn/config')
        .then(function (res) {
          notificationService.success(
            $filter('translate')('OPENVPN_HAS_BEEN_DELETE_SUCCESSFULLY'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function () {
          notificationService.success(
            $filter('translate')('OPENVPN_HAS_BEEN_DELETE_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function downloadFile(url) {
      $http.get(API_BASE.generic + '/helper/download', {
        params: {
          resource: url
        },
        responseType: 'arraybuffer'
      })
      .then(function(res) {
        var blob = new Blob([res.data], {
          type: 'application/octet-stream'
        });
        notificationService.success(
          $filter('translate')('DOWNLOAD_FILE_SUCCESS'),
          'bottom_right'
        );
        //saveAs provided by FileSaver.js
        window.saveAs(blob, 'client.ovpn');
      })
      .catch(function() {
        notificationService.error(
          $filter('translate')('DOWNLOAD_FILE_FAIL'),
          'bottom_right'
        );
      });
    }
  }
}());
