;(function () {
  'use strict';

  angular.module('webapp')
    .controller('Portmap', Portmap);

  function Portmap(
    $scope,
    PortmapService,
    CLOUD_CONFIG,
    routeConfigFunc,
    _,
    ngTableParams
  ) {
    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;

    angular.extend(vm, {
      promise: null,
      new: {
        enable: false
      },
      rules: [],
      editRules: undefined,
      info: null,
      edit: null,
      protocols: [
        {label: 'TCP', value: 'tcp'},
        {label: 'UDP', value: 'udp'},
        {label: 'BOTH', value: 'both'}
      ]
    });
    /*jshint newcap: false*/
    vm.info = new ngTableParams({
      page: 1,
      count: 10
    }, {
      getData: tableGetData
    });
    vm.activate = activate;
    vm.reload = reload;
    vm.goToEditState = goToEditState;
    vm.deleteRule = deleteRule;
    vm.updateRule = updateRule;
    vm.addRule = addRule;

    activate();

    // Member function implement
    function activate() {
      var route = (CLOUD_CONFIG.mqttBridge) ? '/' + routeConfigFunc.getCurrentCgId() : '';
      PortmapService.changeIdRoute(route);
      fetchData();
      sanjiWindowMgr.goToInfoState();
    }

    function fetchData() {
      vm.promise = PortmapService.readCollection()
        .then(function (collection) {
          vm.rules.splice.apply(vm.rules, [0, vm.rules.length].concat(collection));
          tableReload();
        });
    }

    function tableGetData($defer, params) {
      params.total(vm.rules.length);
      $defer.resolve(
        vm.rules.slice((params.page() - 1) * params.count(),
        params.page() * params.count())
      );
    }

    function tableGetEditData($defer, params) {
      params.total(vm.rules.length);
      $defer.resolve(
        vm.rules.slice((params.page() - 1) * params.count(),
        params.page() * params.count())
      );
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function tableReload() {
      vm.info.reload();
    }

    function goToEditState() {
      vm.editRules = angular.copy(vm.rules);
      vm.new = {
        enable: false
      };
      vm.edit = new ngTableParams({
        page: 1,
        count: 10
      }, {
        getData: function($defer, params) {
          params.total(vm.editRules.length);
          $defer.resolve(
            vm.editRules.slice((params.page() - 1) * params.count(),
            params.page() * params.count())
          );
        }
      });
      sanjiWindowMgr.goToEditState();
      sanjiWindowMgr.oneTimeBackCallback = fetchData;
    }

    function deleteRule(index) {
      vm.promise = PortmapService.delete(vm.editRules[index].id)
        .then(function () {
          vm.editRules.splice(index, 1);
          vm.edit.reload();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function updateRule(data) {
      vm.promise = PortmapService.update(data)
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function addRule(data) {
      vm.promise = PortmapService.add(data)
        .then(function (rule) {
          vm.editRules.push(rule);
          vm.edit.reload();
          vm.new = {
            enable: false
          };
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }
  }
}());
