;(function () {
  'use strict';

  angular.module('webapp')
    .factory('PortmapService', PortmapService);

  function PortmapService(
    $http,
    $q,
    $filter,
    $state,
    API_BASE,
    _,
    notificationService
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.readCollection = getCollection;
    bundleModel.read = getModel;
    bundleModel.update = putModel;
    bundleModel.delete = deleteModel;
    bundleModel.add = postModel;
    bundleModel.changeIdRoute = changeIdRoute;

    return bundleModel;

    // Member function implement
    function changeIdRoute(route) {
      IdRoute = route;
    }

    function getCollection() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/portmapping')
      $http.get(API_BASE.generic + IdRoute + '/network/portmapping')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.reject(error);
        });
      return deferred.promise;
    }

    function getModel(id) {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/portmapping/' + id)
      $http.get(API_BASE.generic + IdRoute + '/network/portmapping/' + id)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.reject(error);
        });
      return deferred.promise;
    }

    function putModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/network/portmapping/' + data.id, data)
      $http.put(API_BASE.generic + IdRoute + '/network/portmapping/' + data.id, data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('PORTMAP_UPDATED_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.reject(error);
        });
      return deferred.promise;
    }

    function postModel(data) {
      var deferred = $q.defer();
      // $http.post('http://localhost:8000/network/portmapping', data)
      $http.post(API_BASE.generic + IdRoute + '/network/portmapping', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('PORTMAP_ADD_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.reject(error);
        });
      return deferred.promise;
    }

    function deleteModel(id) {
      var deferred = $q.defer();
      // $http.delete('http://localhost:8000/network/portmapping/' + id)
      $http.delete(API_BASE.generic + IdRoute + '/network/portmapping/' + id)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('PORTMAP_DELETE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.reject(error);
        });
      return deferred.promise;
    }
  }
}());
