;(function () {
  'use strict';

  angular.module('webapp')
    .factory('SystemConfigService', SystemConfigService);

  function SystemConfigService(
    $http,
    $log,
    $q,
    $filter,
    Upload,
    API_BASE,
    notificationService,
    userProvider
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.downloadConfig = getModel;
    bundleModel.downloadSyslog = getSyslog;
    bundleModel.importConfig = importConfig;
    bundleModel.resetRoute = resetRoute;

    return bundleModel;

    function resetRoute(route) {
      IdRoute = route;
    }

    function getModel() {
      var deferred = $q.defer();
      $http.get(API_BASE.generic + '/helper/download', {
          params: {
            resource: '/system/export'
          },
          responseType: 'arraybuffer'
        })
        .then(function(res) {
          var blob = new Blob([res.data], {
            type: 'application/octet-stream'
          });

          notificationService.success(
            $filter('translate')('DOWNLOAD_FILE_SUCCESS'),
            'bottom_right'
          );
          //saveAs provided by FileSaver.js
          window.saveAs(blob, 'config.tar.gz');
          deferred.resolve();
        })
        .catch(function() {
          notificationService.error(
            $filter('translate')('DOWNLOAD_FILE_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });

      return deferred.promise;
    }

    function getSyslog() {
      var deferred = $q.defer();
      $http.get(API_BASE.generic + '/helper/download', {
          params: {
            resource: '/system/syslog'
          },
          responseType: 'arraybuffer'
        })
        .then(function(res) {
          var blob = new Blob([res.data], {
            type: 'application/octet-stream'
          });

          notificationService.success(
            $filter('translate')('DOWNLOAD_FILE_SUCCESS'),
            'bottom_right'
          );
          //saveAs provided by FileSaver.js
          window.saveAs(blob, 'syslog.tar.gz');
          deferred.resolve();
        })
        .catch(function() {
          notificationService.error(
            $filter('translate')('DOWNLOAD_FILE_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });

      return deferred.promise;
    }

    // Member function implement
    function importConfig(data, file, callback) {
      return putModelWithFile(data, file, callback);
    }

    function putModelWithFile(obj, file, callback) {
      var deferred = $q.defer();
      Upload.upload({
        // url: 'https://httpbin.org/post',
        url: API_BASE.generic + '/helper/upload',
        method: 'PUT',
        params: {
          resource: '/system/import'
        },
        data: {
          jsonData: JSON.stringify({scopes: obj}),
          file: file
        }
      })
      .progress(callback)
      .success(function (data) {
        deferred.resolve(data);
      })
      .error(function () {
        notificationService.error(
          $filter('translate')('IMPORT_CONFIG_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

  }
}());
