;(function () {
  'use strict';

  angular.module('webapp')
    .factory('SystemUpgradeService', SystemUpgradeService);

  function SystemUpgradeService(
    $http,
    $log,
    $q,
    $filter,
    Upload,
    API_BASE,
    _,
    notificationService,
    ngDialog,
    userProvider
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.read = getModel;
    bundleModel.update = updateModel;
    bundleModel.blockUI = blockUI;
    bundleModel.closeBlockUI = closeBlockUI;
    bundleModel.resetRoute = resetRoute;
    bundleModel.checkForUpdate = checkForUpdate;
    bundleModel.onlineUpgrade = onlineUpgrade;

    return bundleModel;

    // Member function implement
    function resetRoute(route) {
      IdRoute = route;
    }

    function checkForUpdate() {
      var deferred = $q.defer();
      $http.get(API_BASE.generic + IdRoute + '/system/firmware/check')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          notificationService.error(
            $filter('translate')('CHECK_FW_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function onlineUpgrade(data) {
      var deferred = $q.defer();
      $http.put(API_BASE.generic + IdRoute + '/system/firmware', data)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function getModel() {
      var deferred = $q.defer();
      $http.get(API_BASE.generic + IdRoute + '/system/firmware')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(file, callback) {
      return putModelWithFile(file, callback);
    }

    function putModelWithFile(file, callback) {
      var deferred = $q.defer();
      Upload.upload({
        // url: 'https://httpbin.org/post',
        url: API_BASE.generic + '/helper/upload',
        params: {
          resource: '/system/firmware'
        },
        data: {
          file: file
        }
      })
      .progress(callback)
      .success(function (data) {
        deferred.resolve(data);
      })
      .error(function () {
        deferred.reject();
      });
      return deferred.promise;
    }

    function blockUI() {
      ngDialog.open({
        templateUrl: 'upgrading.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false
      });
    }

    function closeBlockUI() {
      ngDialog.closeAll();
    }
  }
}());
