;(function () {
  'use strict';

  angular.module('webapp')
    .controller('System', System);

  function System(
    $scope,
    $q,
    $state,
    $filter,
    $timeout,
    ngDialog,
    swal,
    appService,
    SystemUpgradeService,
    SystemRebootService,
    SystemConfigService,
    SystemService,
    _
  ) {

    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var file;
    var redirectTimer;
    var importScopes = [];

    vm.firmware = {
      progressVal: 0,
      progressStyle: null,
      upgrade: 1,
      server: 'www.moxa.com'
    };
    vm.config = {
      progressVal: 0,
      progressStyle: null,
      runBit: 1
    };
    vm.activate = activate;
    vm.upgrade = upgrade;
    vm.onFileSelect = onFileSelect;
    vm.reboot = reboot;
    vm.reload = reload;
    vm.checkForUpdate = checkForUpdate;
    vm.onConfigFileSelect = onConfigFileSelect;
    vm.restore = restore;
    vm.backup = backup;
    vm.exportSyslog = exportSyslog;
    vm.enalbeSsh = false;
    vm.enalbeSyslog = false;
    vm.enalbeWanAccess = false;
    vm.onSshModelChange = onSshModelChange;
    vm.onSyslogModelChange = onSyslogModelChange;
    vm.onWanAccessModelChange = onWanAccessModelChange;
    vm.openFwUpgradeDialog = openFwUpgradeDialog;
    vm.selectItem = selectItem;
    vm.selectAll = selectAll;

    $scope.bundle = vm;

    activate();
    registerEventHandler();

    // Member function implement
    function activate() {
      $q.all([SystemService.getSsh(), SystemService.getSyslog(), SystemService.getWanAccess()])
        .then(function(results) {
          var ssh = results[0];
          var syslog = results[1];
          var wan = results[2];
          vm.enableSsh = ssh.enable;
          vm.enableSyslog = syslog.enable;
          vm.enableWanAccess = wan.enable;
          sanjiWindowMgr.goToInfoState();
        })
        .catch(function() {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function onSshModelChange(status) {
      ngDialog.open({
        template: 'wizard-loading.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false
      });
      SystemService.updateSsh({enable: status})
        .then(function() {
          ngDialog.close();
        })
        .catch(function() {
          vm.enalbeSsh = !status;
        });
    }

    function onSyslogModelChange(status) {
      ngDialog.open({
        template: 'wizard-loading.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false
      });
      SystemService.updateSyslog({enable: status})
        .then(function() {
          ngDialog.close();
        })
        .catch(function() {
          vm.enalbeSsh = !status;
        });
    }

    function onWanAccessModelChange(status) {
      ngDialog.open({
        template: 'wizard-loading.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false
      });
      SystemService.updateWanAccess({enable: status})
        .then(function() {
          ngDialog.close();
        })
        .catch(function() {
          vm.enalbeSsh = !status;
        });
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function upgradeProgress(e) {
      vm.firmware.progressVal = parseInt(100.0 * e.loaded / e.total, 10);
      vm.firmware.progressStyle = { width: vm.firmware.progressVal + '%' };
    }

    function importProcess(e) {
      vm.config.progressVal = parseInt(100.0 * e.loaded / e.total, 10);
      vm.config.progressStyle = { width: vm.config.progressVal + '%' };
    }

    function registerEventHandler() {
      var handlers = [];

      $scope.$on('$destroy', function() {
        _.forEach(handlers, function(handler) {
          handler();
        });
        // Close reboot dialog
        SystemRebootService.closeBlockUI();
        redirectTimer = null;
        handlers = null;
      });
    }

    function checkForUpdate() {
     var dialog = ngDialog.open({
        template: '/app/genericbundles/system/system-check-for-update.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false
      });

      SystemUpgradeService.checkForUpdate()
        .then(upgradeDialog)
        .then(onlineUpgrade)
        .then(upgradingDialog)
        .catch(function() {
          if (ngDialog.isOpen(dialog.id)) {
            ngDialog.close(dialog.id);
          }
        });
    }

    function upgradeDialog(data) {
      var deferred = $q.defer();
      ngDialog.closeAll();
      if (data.isLatest) {
        swal({
          title: $filter('translate')('MXCLOUD_IS_UP_TO_DATE'),
          text: data.current
        });
        deferred.reject();
      } else {
        swal({
          title: $filter('translate')('NEW_VERSION_RELEASED'),
          text: $filter('translate')('DO_YOU_WANT_TO_UPGRADE') + '\n' +
            data.current+ ' -> ' + data.candidate,
          type: 'warning',
          showCancelButton: true,
          confirmButtonColor: '#DD6B55',
          confirmButtonText: $filter('translate')('YES'),
          cancelButtonText: $filter('translate')('CANCEL')
        },
        function(isConfirm){
          if (isConfirm) {
            deferred.resolve();
          } else {
            deferred.reject();
          }
        });
      }
      return deferred.promise;
    }

    function onlineUpgrade() {
      return SystemUpgradeService.onlineUpgrade({
        upgrade: 1,
      })
      .catch(function() {
        sanjiWindowMgr.goToProblemState();
      });
    }

    function upgradingDialog() {
      ngDialog.open({
        template: 'upgrading.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false
      });
      _redirectState();
    }

    function upgrade() {
      ngDialog.open({
        template: '/app/genericbundles/system/system-upgrade-processbar.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false,
        scope: $scope
      });
      sanjiWindowMgr.goToProcessingState();
      SystemUpgradeService.update(file, upgradeProgress)
        .then(function () {
          sanjiWindowMgr.goToInfoState();
          ngDialog.closeAll();
          _rebootBlockUI();
        })
        .catch(function (data) {
          sanjiWindowMgr.goToProblemState();
          ngDialog.closeAll();
        });
    }

    function onFileSelect($files) {
      file = $files;
    }

    function reboot() {
      swal({
        title: $filter('translate')('ARE_U_SURE'),
        text: $filter('translate')('REBOOT_WARN_MSG'),
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#DD6B55',
        confirmButtonText: $filter('translate')('YES'),
        cancelButtonText: $filter('translate')('CANCEL')
      },
      function(isConfirm){
        if (isConfirm) {
          SystemRebootService.update()
            .then(function () {
              _rebootBlockUI();
            })
            .catch(function (data) {
              sanjiWindowMgr.goToProblemState();
            });
        }
      });
    }

    function onConfigFileSelect($file) {
      file = $file;
    }

    function restore() {
      ngDialog.open({
        template: '/app/genericbundles/system/system-config-import-processbar.html',
        closeByDocument: false,
        closeByEscape: false,
        showClose: false,
        scope: $scope
      });
      sanjiWindowMgr.goToProcessingState();
      SystemConfigService.importConfig(importScopes, file, importProcess)
        .then(function () {
          sanjiWindowMgr.goToInfoState();
          ngDialog.closeAll();
          ngDialog.open({
            templateUrl: 'restore.html',
            closeByDocument: false,
            closeByEscape: false,
            showClose: false
          });
          _redirectState();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
          ngDialog.closeAll();
        });
    }

    function backup() {
      SystemConfigService.downloadConfig();
    }

    function exportSyslog() {
      SystemConfigService.downloadSyslog();
    }

    function _rebootBlockUI() {
      SystemRebootService.blockUI();
      _redirectState();
    }

    function _redirectState() {
      redirectTimer = $timeout(function() {
        ngDialog.closeAll();
        $state.transitionTo('home');
      }, 8000);
    }

    function openFwUpgradeDialog() {
      ngDialog.open({
        template: '/app/genericbundles/system/system-upgrade.html',
        scope: $scope
      });
    }

    function selectItem(name, isSelected) {
      var idx = importScopes.indexOf(name);
      if (isSelected && -1 === idx) {
        importScopes.push(name);
      }
      else {
        if (-1 !== idx) {
          importScopes.splice(idx, 1);
        }
      }
    }

    function selectAll(isSelected) {
      vm.modbusApp = isSelected;
      vm.relay = isSelected;
      vm.ethernet = isSelected;
      vm.cellular = isSelected;
      vm.dhcpd = isSelected;
      vm.dns = isSelected;
      vm.serial = isSelected;
      vm.time = isSelected;
      vm.openvpn = isSelected;
      vm.portmap = isSelected;
      vm.system = isSelected;

      importScopes.length = 0;
      if (isSelected) {
       importScopes.push(
         'modbus',
         'relay',
         'ethernet',
         'cellular',
         'dhcpd',
         'dns',
         'serial',
         'time',
         'openvpn',
         'portforwarding',
         'system'
       );
      }
    }
  }
}());
