;(function () {
  'use strict';

  angular.module('webapp')
    .controller('TimeGroup', TimeGroup);

  function TimeGroup(
    $scope,
    $rootScope,
    $filter,
    TimeService,
    TIME_BUNDLE_PATH,
    cgGroupManagerService,
    _
  ) {

    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;

    vm.info = null;
    vm.edit = {
      time: new Date()
    };
    vm.group = null;
    vm.activeGroup = null;
    vm.progressVal = 0;
    vm.submit = submit;
    vm.activate = activate;
    vm.reload = reload;
    vm.changeGroupInfoContent = changeGroupInfoContent;
    vm.tagInputOptions = {
      placeholderText: $filter('translate')('ADD_TIME_SERVER'),
      validator: {
        rule: function(value) {

          var pattern = '^[0-9a-zA-Z.]+$|' +
            '^(25[0-5]|2[0-4][0-9]|' +
            '[01]?[0-9][0-9]?).(25[0-5]|' +
            '2[0-4][0-9]|' +
            '[01]?[0-9][0-9]?).(25[0-5]|' +
            '2[0-4][0-9]|' +
            '[01]?[0-9][0-9]?).(25[0-5]|' +
            '2[0-4][0-9]|[01]?[0-9][0-9]?)$';

          return new RegExp(pattern).test(value);
        },
        error: 'This ip format is invalid.'
      }
    };

    activate();
    registerEventHandler();

    // Member function implement
    function activate() {
      TimeService.createJob('get')
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function registerEventHandler() {
      var handlers = [];

      handlers.push(
        $rootScope.$on('$translateChangeSuccess', translateChangeSuccess)
      );

      _.forEach(TIME_BUNDLE_PATH, function(path) {
        handlers.push($scope.$on(path, initEventHandler));
        handlers.push($scope.$on(path + '-update', updateEventHandler));
      });

      $scope.$on('$destroy', function() {
        _.forEach(handlers, function(handler) {
          handler();
        });
        handlers = null;
      });
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function submit(data) {
      sanjiWindowMgr.goToProcessingState();
      vm.group = cgGroupManagerService.getRemoteComputerGroup();
      TimeService.createJob('put', {data: data})
        .then(function () {
          vm.progressVal = 0;
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function changeGroupInfoContent(group) {
      _.forEach(vm.info, function(obj) {
        if (obj.id === group.id) {
          obj.isActive = true;
        } else {
          obj.isActive = false;
        }
      });
      vm.activeGroup = group;
    }

    function translateChangeSuccess() {
      vm.tagInputOptions.placeholderText = $filter('translate')('ADD_TIME_SERVER');
    }

    function updateEventHandler(event, args) {
      var status = args.__request.status;
      var result = args.__request.result;
      var id = args.__request.destination;
      var method = args.method;

      if ('put' !== method) {
        return;
      }

      switch(status) {
        case 'resolved':
          if (result && 200 !== result.code) {
            cgGroupManagerService.updateRemoteComputerGroupStatus(vm.group, id, 'error');
          } else {
            cgGroupManagerService.updateRemoteComputerGroupStatus(vm.group, id, 'success');
          }
          break;
        case 'timeout':
          cgGroupManagerService.updateRemoteComputerGroupStatus(vm.group, id, status);
          break;
        default:
          cgGroupManagerService.updateRemoteComputerGroupStatus(vm.group, id, 'error');
      }
    }

    function initEventHandler(event, args) {
      var results = [];

      _.forEach(args.requests, function(req) {
        var status = req.__request.status;
        var result = req.__request.result;
        var method = req.method;

        if ('get' !== method) {
          vm.progressVal = args.progress;
          return false;
        }

        switch(status) {
          case 'resolved':
            if (result && 200 !== result.code) {
              results.push(null);
            } else {
              results.push(result.data);
            }
            break;
          case 'timeout':
            results.push(null);
            break;
          default:
            sanjiWindowMgr.goToProblemState();
        }
      });

      // This is for initializing in group page.
      if (results.length) {
        var group = cgGroupManagerService.getRemoteComputerGroup();
        _.forEach(group, function(obj, index) {
          angular.extend(obj, {
            data: results[index],
            isActive: (0 === index) ? true : false
          });
        });
        vm.info = group;
        vm.activeGroup = group[0];
        sanjiWindowMgr.goToInfoState();
      }
    }

  }
}());
