;(function () {
  'use strict';

  angular.module('webapp')
    .controller('Time', Time);

  function Time(
    $scope,
    $rootScope,
    $filter,
    TimeService,
    swal,
    _
  ) {

    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;

    vm.isOpenCalendar = false;
    vm.info = null;
    vm.edit = null;
    vm.submit = submit;
    vm.activate = activate;
    vm.reload = reload;
    vm.tagInputOptions = {
      placeholderText: $filter('translate')('ADD_TIME_SERVER'),
      validator: {
        rule: function(value) {

          var pattern = '^[0-9a-zA-Z.]+$|' +
            '^(25[0-5]|2[0-4][0-9]|' +
            '[01]?[0-9][0-9]?).(25[0-5]|' +
            '2[0-4][0-9]|' +
            '[01]?[0-9][0-9]?).(25[0-5]|' +
            '2[0-4][0-9]|' +
            '[01]?[0-9][0-9]?).(25[0-5]|' +
            '2[0-4][0-9]|[01]?[0-9][0-9]?)$';

          return new RegExp(pattern).test(value);
        },
        error: 'This ip format is invalid.'
      }
    };

    activate();
    registerEventHandler();

    // Member function implement
    function activate() {
      TimeService.read()
        .then(function (model) {
          vm.info = model;
          vm.edit = angular.copy(model);
          sanjiWindowMgr.goToInfoState();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function registerEventHandler() {
      var handlers = [];

      handlers.push(
        $rootScope.$on('$translateChangeSuccess', translateChangeSuccess)
      );

      $scope.$on('$destroy', function() {
        _.forEach(handlers, function(handler) {
          handler();
        });
        handlers = null;
      });
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function submit(data) {
      swal({
        title: 'Do you want to save new time?',
        text: 'Your login information may expired by this action.',
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#DD6B55',
        confirmButtonText: 'Yes',
        closeOnConfirm: true
      },
      function(isConfirm){
        if (isConfirm) {
          sanjiWindowMgr.goToProcessingState();
          TimeService.update(data)
            .then(function () {
              activate();
            })
            .catch(function () {
              sanjiWindowMgr.goToProblemState();
            });
        }
      });
    }

    function translateChangeSuccess() {
      vm.tagInputOptions.placeholderText = $filter('translate')('ADD_TIME_SERVER');
    }

  }
}());
