;(function () {
  'use strict';

  angular.module('webapp')
    .factory('TimeService', TimeService);

  function TimeService(
    $http,
    $q,
    $filter,
    _,
    moment,
    API_BASE,
    notificationService
  ) {
    // Members definition
    var bundleModel = {};
    var IdRoute = '';

    bundleModel.read = getModel;
    bundleModel.update = updateModel;
    bundleModel.resetRoute = resetRoute;

    return bundleModel;

    // Member function implement
    function resetRoute(route) {
      IdRoute = route;
    }

    function getModel() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/system/time')
      $http.get(API_BASE.generic + IdRoute + '/system/time')
        .then(function (res) {
          var results = res.data.timezone.split(',');
          var tz = results[0];
          var dst = parseInt(results[1], 10);
          var duration = parseFloat(tz.replace(':', '.'));

          res.data.digitalTime = moment(res.data.time).valueOf();
          res.data.gmtOffset = duration + dst;
          res.data.time = new Date(res.data.time);
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.reject();
        });
      return deferred.promise;
    }

    function updateModel(data) {
      var deferred = $q.defer();
      putModel(data).then(function (data) {
        notificationService.success(
          $filter('translate')('TIME_HAS_BEEN_UPDATED_SUCCESSFULLY'),
          'bottom_right'
        );
        deferred.resolve(data);
      })
      .catch(function () {
        notificationService.error(
          $filter('translate')('TIME_HAS_BEEN_UPDATED_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    }

    function putModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/system/time', data)
      $http.put(API_BASE.generic + IdRoute + '/system/time', data)
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function (res) {
          deferred.reject();
        });
      return deferred.promise;
    }
  }
}());
