(function () {
  'use strict';

  angular.module('webapp')
    .controller('Relay', Relay);

  function Relay(
    $scope,
    $state,
    $q,
    RelayService,
    SerialService,
    NetworkService,
    settingsFactory,
    _
  ) {

    var vm = this; // vm means ViewModle
    var sanjiWindowMgr = $scope.sanjiWindowMgr;
    var memoizeTab = settingsFactory.getRelayListTab('relay-list');

    vm.info = null;
    vm.edit = null;
    vm.tab = {};
    vm.mode = [['network', 'network'], ['network', 'network']];
    vm.serial = null;
    vm.interfaces = null;
    vm.selectSerial = ['', ''];
    vm.selectInterface = ['', ''];
    vm.currentTabIdx = 0;
    vm.changeNetworkType = changeNetworkType;
    vm.changeInterface = changeInterface;
    vm.submit = submit;
    vm.activate = activate;
    vm.reload = reload;
    vm.changeSerial = changeSerial;
    vm.changeMode = changeMode;
    vm.clickTab = clickTab;

    activate();

    // Member function implement
    function activate() {
      $q.all([RelayService.readCollection(), SerialService.readCollection(), NetworkService.readCollection()])
        .then(function(result) {
          vm.info = result[0];
          vm.edit = angular.copy(result[0]);
          vm.serial = result[1];
          vm.interfaces = result[2];

          _.forEach(vm.info, function(relay, index) {
            if ('serial' === relay.nodes[1].type) {
              var idx = _.findIndex(vm.serial, function(item) {
                return item.dev === relay.nodes[1].serialOpts.dev;
              });
              if (-1 !== idx) {
                vm.selectSerial[index] = vm.serial[idx].devDisplayName;
                vm.mode[index][1] = 'serial';
              }
            }

            _.forEach(relay.nodes, function(node) {
              if ('server' === node.type) {
                vm.selectInterface[index] = node.serverOpts.interface;
              }
            });
          });
          sanjiWindowMgr.goToInfoState();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function reload() {
      sanjiWindowMgr.goToLoadingState();
      activate();
    }

    function submit(data) {
      sanjiWindowMgr.goToProcessingState();
      RelayService.update(data)
        .then(function () {
          activate();
        })
        .catch(function () {
          sanjiWindowMgr.goToProblemState();
        });
    }

    function changeNetworkType(index, type, relay) {
      if (0 === index) {
        if ('server' === type && 'serial' !== relay.nodes[1].type) {
          relay.nodes[1].type = 'client';
        }

        if ('client' === type && 'serial' !== relay.nodes[1].type) {
          relay.nodes[1].type = 'server';
        }
      }
      else {
        if ('server'===type) {
          relay.nodes[0].type = 'client';
        }
        else {
          relay.nodes[0].type = 'server';
        }
      }
    }

    function changeSerial(port, relay) {
      var obj = _.find(vm.serial, function(item) {
        return item.devDisplayName === port;
      });
      if (obj) {
        relay.nodes[1].serialOpts.dev = obj.dev;
        relay.nodes[1].serialOpts.devDisplayName = obj.devDisplayName;
      }
    }

    function changeInterface(node, iface) {
      var obj = _.find(vm.interfaces, function(item) {
        return item.label === iface;
      });
      if (obj) {
        node.serverOpts.interface = obj.value;
      }
    }

    function changeMode(mode, relay) {
      console.log(mode);
      console.log(relay);
      if ('serial' === mode) {
        relay.nodes[1].type = mode;
      }
      else {
        if ('server' === relay.nodes[0].type) {
          relay.nodes[1].type = 'client';
        }
        else {
          relay.nodes[1].type = 'server';
        }
      }
    }

    function clickTab(type, idx) {
      vm.currentTabIdx = idx;
      _.each(vm.tab, function(value, prop) {
        vm.tab[prop] = false;
      });
      vm.tab[type] = true;
      settingsFactory.setRelayListTab('relay-list', vm.tab);
    }
  }
}());
