/* globals document */
'use strict';
angular.module('webapp')
.service('appService', function(
  _,
  $log,
  $timeout,
  $filter,
  $http,
  $resource,
  swal,
  helperService,
  userProvider,
  userService,
  validateService,
  notificationService,
  API_BASE
  ) {

  var self = this;

  self.openConfirmBox = function(title, cb) {
    swal({
      title: title,
      type: 'warning',
      showCancelButton: true,
      confirmButtonColor: '#DD6B55',
      confirmButtonText: $filter('translate')('YES'),
      cancelButtonText: $filter('translate')('NO'),
      closeOnConfirm: false,
      closeOnCancel: false
    },
      function(isConfirm){
        if (isConfirm) {
          cb();
        } else {
          swal(
            $filter('translate')('CANCELLED'),
            null,
            'error'
          );
        }
    });
  };

  self.closeConfirmBox = function(title, content, type) {
    swal(title || '', content || '', type || 'success');
  };

  self.isSupportedLang = function(lang) {
    return helperService.inArray(['en', 'zh-TW'], lang);
  };

  // self.getSslStatus = function() {
    // var deferred = $q.defer();

    // $http.get(API_BASE.ami + '/mxcs/status')
      // .then(function(res) {
        // deferred.resolve(res.data);
      // })
      // .catch(function() {
        // deferred.reject();
      // });

    // return deferred.promise;
  // };

  self.getUser = function(userId) {

    var User = $resource(API_BASE.other + '/users/:id', {
      id: '@id'
    });
    return User.get({id: userId}).$promise;
  };

  self.getUsers = function(params) {

    params = params || {};

    var Users = $resource(API_BASE.other + '/users', {
      prefix: API_BASE.other
    });

    return Users.query(params).$promise;
    // return Restangular.all('users')
      // .getList(params);
  };

  self.updateUser = function(user) {

    var params = {
      email: user.email
    };

    var User = $resource(API_BASE.other + '/users/:id', {
      id: '@id'
    },
    {
      update: {
        method: 'PUT'
      }
    });

    if (user.password) {
      params.password = user.password;
    }

    if (user.meta) {
      params.profileImage = user.meta.image;
    }

    if (user.projectNames) {
      params.projectNames = user.projectNames;
    }

    return User.update({id: user.id}, params).$promise;
    // return Restangular.one('users', user.id)
      // .customPUT(params);
  };

  self.getSelectedRows = function(rows) {
    return _.filter(rows, function(row) {
      if (! angular.isObject(row)) {
        return false;
      }
      return row.$selected;
    });
  };

  self.getUnselectedRows = function(rows) {
    return _.filter(rows, function(row) {
      if (! angular.isObject(row)) {
        return true;
      }
      return ! row.$selected;
    });
  };

  self.hasSelected = function(rows) {
    return (self.getSelectedRows(rows).length > 0);
  };

  self.setActiveProfile = function() {

    return userService.getProfile()
      .then(function(profile) {

        // profile = Restangular.stripRestangular(profile);
        userProvider.activeProfile = profile;

        return true;
      });
  };

});
