'use strict';

angular.module('webapp')
.provider('columnSetting', function() {

  this.config = {};

  this.setConfig = function(key, value) {
    this.config[key] = value;
  };

  this.$get = function($q, settingsFactory, _) {

    var config = this.config;

    return function(name) {

      if (! config.hasOwnProperty(name)) {
        throw new Error(name + ' does not exsits, please check columnSettingProvider.setConfig');
      }

      var settings = config[name];

      var mapById = function(order, rows) {

          var data, i, len;

          data = [];
          len = rows.length;

          angular.forEach(order, function(id) {
            for (i = 0; i < len; i++) {
              if (id === rows[i].id) {
                data.push(rows[i]);
                break;
              }
            }
          });
          return data;
      };

      return {

        getOneColumnSettings: function() {

          var cacheFirstColOrder = settingsFactory.getColumnOrder('first', name);

          if (angular.isArray(cacheFirstColOrder)) {
            var newSettings = mapById(cacheFirstColOrder, settings);

            // handle new id that is not in user's cache
            _.each(newSettings, function(row) {
              settings = _.without(settings, _.findWhere(settings, {id: row.id}));
            });
            settings = newSettings.concat(settings);
          }
          return settings;
        },
        getTwoColumnSettings: function() {

          var cacheFirstColOrder = settingsFactory.getColumnOrder('first', name),
              cacheSecondColOrder = settingsFactory.getColumnOrder('second', name),
              mergedSettings = settings[0].concat(settings[1]);

          if (angular.isArray(cacheFirstColOrder)) {
            settings[0] = mapById(cacheFirstColOrder, mergedSettings);
          }
          if (angular.isArray(cacheSecondColOrder)) {
            settings[1] = mapById(cacheSecondColOrder, mergedSettings);
          }

          // handle new id that is not in user's cache
          var mergedNewSettings = settings[0].concat(settings[1]);
          _.each(mergedNewSettings, function(row) {
            mergedSettings = _.without(mergedSettings, _.findWhere(mergedSettings, {id: row.id}));
          });
          _.each(mergedSettings, function(row, index) {
            settings[index % 2].push(row);
          });

          return settings;
        },
        getSettingById: function(id) {

          var mergedSettings, desiredRow;

          mergedSettings = settings;

          if (angular.isArray(settings[0])) {
            mergedSettings = settings[0].concat(settings[1]);
          }
          angular.forEach(mergedSettings, function(row) {

            if (row.id === id) {
              desiredRow = row;
              return true;
            }
          });
          return desiredRow;
        }
      };
    };
  };
});
