;(function() {
  'use strict';

  angular.module('webapp')
    .service('computerService', computerService);

  function computerService(
    $q,
    $http,
    $filter,
    Upload,
    Restangular,
    notificationService,
    API_BASE,
    _,
    CLOUD_CONFIG,
    userProvider
  ) {

    var self = this;

    self.getMxcStatus = function(id) {
      var deferred = $q.defer();
      var IdRoute = (CLOUD_CONFIG.mqttBridge) ? '/' + id : '';

      $http.get(API_BASE.ami + IdRoute + '/mxc/status')
      .then(function(res) {
        deferred.resolve(res.data);
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
    };

    self.deleteComputers = function(computerIds) {
      var deferred = $q.defer();
      $http({
        url: API_BASE.ami + '/mxcs/cgs',
        method: 'DELETE',
        data: {
          idList: computerIds
        }
      })
      .then(function(res) {
        if ('success' === res.data.message) {
          deferred.resolve();
        } else {
          deferred.reject();
        }
      })
      .catch(function() {
        deferred.reject();
      });
      return deferred.promise;
    };

    self.getComputer = function(computerId) {
      var deferred = $q.defer();

      $http.get(API_BASE.ami + '/mxcs/cgs/' + computerId)
      .then(function(res) {
        if (angular.isDefined(res.data.metas.latitude)) {
          res.data.metas.latitude = parseFloat(res.data.metas.latitude);
        }
        if (angular.isDefined(res.data.metas.longitude)) {
          res.data.metas.longitude = parseFloat(res.data.metas.longitude);
        }
        deferred.resolve(res.data);
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
    };

    self.getComputers = function(params) {
      var deferred = $q.defer();
      params = params || {};

      $http.get(API_BASE.ami + '/mxcs/cgs', angular.extend({}, {params: params}))
      .then(function(res) {
        _.forEach(res.data.cgList, function(cg) {
          if (angular.isDefined(cg.metas.latitude)) {
            cg.metas.latitude = parseFloat(cg.metas.latitude);
          }
          if (angular.isDefined(cg.metas.longitude)) {
            cg.metas.longitude = parseFloat(cg.metas.longitude);
          }
        });

        deferred.resolve(res.data);
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
    };

    self.addComputer = function(data) {
      var deferred = $q.defer();

      $http.post(API_BASE.ami + '/mxcs/cgs', data)
      .then(function(res) {
        notificationService.success(
          $filter('translate')('COMPUTER_ADD_SUCCESS'),
          'bottom_right'
        );
        deferred.resolve(res.data);
      })
      .catch(function() {
        notificationService.error(
          $filter('translate')('COMPUTER_ADD_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });

      return deferred.promise;
      // params = params || {};
      // return Restangular.all('computers')
        // .post(params);
    };

    self.update = function(cgData) {
      var deferred = $q.defer();
      $http.put(API_BASE.ami + '/mxcs/cgs/' + cgData.id, cgData)
      .then(function(res) {
        if ('success' === res.data.message) {
          notificationService.success(
            $filter('translate')('COMPUTER_UPDATE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve();
        } else {
          deferred.reject();
        }
      })
      .catch(function() {
        notificationService.error(
          $filter('translate')('COMPUTER_UPDATE_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    };

    self.editComputer = function(cg) {
      var deferred = $q.defer();

      $http.put(API_BASE.ami + '/mxcs/cgs/' + cg.id, cg)
      .then(function(res) {
        if ('success' === res.data.message) {
          notificationService.success(
            $filter('translate')('COMPUTER_UPDATE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve();
        } else {
          deferred.reject();
        }
      })
      .catch(function() {
        notificationService.error(
          $filter('translate')('COMPUTER_UPDATE_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });

      return deferred.promise;
    };

    self.getEquipments = function(computerId, params) {
      params = params || {};
      return Restangular.all('computers/' + computerId + '/equipments')
        .customGET(null, params);
    };

    self.updateEthernet = function(computerId, params) {
      return Restangular.all('computers/' + computerId + '/ethernet')
        .customPUT(params);
    };

    self.getDns = function(computerId) {
      return Restangular.all('computers/' + computerId + '/dns')
        .customGET();
    };

    self.updateDns = function(computerId, params) {
      return Restangular.all('computers/' + computerId + '/dns')
        .customPUT(params);
    };

    self.getCellular = function(computerId) {
      return Restangular.all('computers/' + computerId + '/cellular')
        .customGET();
    };

    self.importComputers = function(file, cb) {
      return self.uploadCsvFile(file, cb)
        .then(self.sendCsvUrl);
    };

    self.sendCsvUrl = function(data) {
      var protocol = window.location.protocol;
      var host = window.location.host;
      var payload = {
        url: protocol + '//' + host + data.url,
        headers: {
          'X-Mx-AccessToken': userProvider.activeProfile.accessToken
        }
      };
      return $http.post(API_BASE.ami + '/mxcs/cgs-import', payload)
        .then(function() {
          notificationService.success(
            $filter('translate')('IMPORT_CG_LIST_SUCCESS'),
            'bottom_right'
          );
        })
        .catch(function() {
          notificationService.error(
            $filter('translate')('IMPORT_CG_LIST_FAIL'),
            'bottom_right'
          );
        });
    };

    self.uploadCsvFile = function(file, cb) {
      var deferred = $q.defer();
      Upload.upload({
        url: API_BASE.generic + '/files/upload',
        file: file
      })
      .progress(cb)
      .success(function(data) {
        deferred.resolve(data);
      })
      .error(function() {
        notificationService.error(
          $filter('translate')('IMPORT_CG_LIST_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    };

    self.getInterval = function(computerId) {
      return Restangular.all('demo/' + computerId + '/interval')
        .customGET();
    };

    self.putInterval = function(computerId, params) {
      params = params || {};
      return Restangular.all('demo/' + computerId + '/interval')
        .customPUT(params);
    };

  }
})();
