;(function() {
  'use strict';

  angular.module('webapp')
    .service('equipmentService', equipmentService);

  function equipmentService(
    $q,
    $http,
    $filter,
    Restangular,
    Upload,
    notificationService,
    API_BASE,
    _,
    moment
  ) {

    var self = this;

    self.data = {};

    self.setCurrentEquipment = function(equipment) {
      self.data.equipment = equipment;
    };

    self.getCurrentEquipment = function() {
      return self.data.equipment;
    };

    self.deleteEquipments = function(equipmentIds) {
      var deferred = $q.defer();
      $http({
        url: API_BASE.ami + '/mxcs/equipments',
        method: 'DELETE',
        data: {
          idList: equipmentIds
        }
      })
      .then(function(res) {
        if ('success' === res.data.message) {
          deferred.resolve();
        } else {
          deferred.reject();
        }
      })
      .catch(function() {
        deferred.reject();
      });
      return deferred.promise;
      // return Restangular.all('equipments')
        // .customDELETE('', {}, {'Content-Type': 'application/json'}, {equipmentIds: equipmentIds});
    };

    self.getEquipment = function(equipmentId) {
      var deferred = $q.defer();

      $http.get(API_BASE.ami + '/mxcs/equipments/' + equipmentId)
      .then(function(res) {
        if (res.data.metas.latitude) {
          res.data.metas.latitude = parseFloat(res.data.metas.latitude);
        }
        if (res.data.metas.longitude) {
          res.data.metas.longitude = parseFloat(res.data.metas.longitude);
        }
        deferred.resolve(res.data);
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
      // return Restangular.one('equipments', equipmentId)
        // .get();
    };

    self.getChannels = function(equipmentId, params) {
      var deferred = $q.defer();

      params = params || {};
      $http.get(API_BASE.ami + '/mxcs/equipments/' + equipmentId + '/channels', angular.extend({}, {params: params}))
      .then(function(res) {
        deferred.resolve(res.data.channelList);
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
      // return Restangular.all('equipments/' + equipmentId + '/channels')
        // .getList(params);
    };

    self.getEquipments = function(params) {
      var deferred = $q.defer();
      params = params || {};

      $http.get(API_BASE.ami + '/mxcs/equipments', angular.extend({}, {params: params}))
      .then(function(res) {
        _.forEach(res.data.eqList, function(eq) {
          if (eq.metas.latitude) {
            eq.metas.latitude = parseFloat(eq.metas.latitude);
          }
          if (eq.metas.longitude) {
            eq.metas.longitude = parseFloat(eq.metas.longitude);
          }
        });

        deferred.resolve(res.data);
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
      // params = params || {};
      // return Restangular.all('equipments')
        // .customGET(null, params);
    };

    self.importEquipments = function(file) {
      return Upload.upload({
        url: API_BASE.ami + '/equipments/importEquipments',
        data: {},
        file: file
      });
    };

    self.addEquipment = function(params) {
      params = params || {};
      return Restangular.all('equipments')
        .post(params);
    };

    self.editEquipment = function(equipment) {
      var deferred = $q.defer();

      $http.put(API_BASE.ami + '/mxcs/equipments/' + equipment.id + '/metas', { metas: equipment.metas })
      .then(function(res) {
        if ('success' === res.data.message) {
          notificationService.success(
            $filter('translate')('EQUIPMENT_UPDATE_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve();
        } else {
          deferred.reject();
        }
      })
      .catch(function() {
        notificationService.error(
          $filter('translate')('EQUIPMENT_UPDATE_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });

      return deferred.promise;
    };

    self.addChannelChart = function(equipment) {
      var deferred = $q.defer();

      $http.post(API_BASE.ami + '/mxcs/equipments/' + equipment.id + '/metas', { metas: equipment.metas })
      .then(function(res) {
        if ('success' === res.data.message) {
          notificationService.success(
            $filter('translate')('CHANNEL_ADD_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve();
        } else {
          deferred.reject();
        }
      })
      .catch(function() {
        notificationService.error(
          $filter('translate')('CHANNEL_ADD_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });

      return deferred.promise;
      // return Restangular.all('equipments/' + equipmentId + '/channelCharts')
        // .post({channelName: channelName});
    };

    self.getChannelChartData = function(channelId, params) {
      var deferred = $q.defer();
      var url = API_BASE.ami +
                '/mxcs/equipments/' +
                self.getCurrentEquipment().id +
                '/channels/' + channelId + '/readings';

      params = params || {};

      $http.get(url, { params: params })
      .then(function(res) {
        deferred.resolve({
            times: _.chain(res.data.readingList)
              .pluck('time')
              .map(function(time) {
                return moment(time).format('YYYY-MM-DD HH:mm:ss');
              })
              .value(),
            data: _.chain(res.data.readingList)
              .pluck('value')
              .map(function(val) {
                return Math.round(val * 1000) / 1000;
              })
              .value(),
            options: {
              responsive: true,
              maintainAspectRatio: true
            }
        });
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
    };

    self.getChannelTableData = function(channelId, params) {
      var deferred = $q.defer();
      var url = API_BASE.ami +
                '/mxcs/equipments/' +
                self.getCurrentEquipment().id +
                '/channels/' + channelId + '/readings';

      params = params || {};

      $http.get(url, { params: params })
      .then(function(res) {
        _.forEach(res.data.readingList, function(obj) {
          obj.time = moment(obj.time).format('YYYY-MM-DD HH:mm:ss');
        });
        deferred.resolve(res.data);
      })
      .catch(function() {
        deferred.reject();
      });

      return deferred.promise;
    };

    self.deleteChannelChart = function(data) {
      var deferred = $q.defer();
      $http({
        url: API_BASE.ami + '/mxcs/equipments/' + self.getCurrentEquipment().id + '/metas',
        method: 'DELETE',
        data: data
      })
      .then(function(res) {
        notificationService.success(
          $filter('translate')('CHANNEL_DELETE_SUCCESS'),
          'bottom_right'
        );
        deferred.resolve();
      })
      .catch(function() {
        notificationService.error(
          $filter('translate')('CHANNEL_DELETE_FAIL'),
          'bottom_right'
        );
        deferred.reject();
      });
      return deferred.promise;
    };

    self.getChannelCharts = function(equipmentId) {
      return Restangular.all('equipments/' + equipmentId + '/channelCharts')
        .getList();
    };

    self.deleteChannelCharts = function(equipmentId, ids) {
      return Restangular.all('equipments/' + equipmentId + '/channelCharts')
        .customDELETE('', {}, {'Content-Type': 'application/json'}, {ids: ids});
    };

    self.updateChannelChart = function(equipmentId, channelChart) {
      return Restangular.all('equipments/' + equipmentId + '/channelCharts/' + channelChart.id)
        .customPUT({channelName: channelChart.channelName});
    };

  }
})();
