(function() {
  'use strict';

  angular.module('webapp')
    .provider('routeConfigFunc', routeConfigFunc);

  function routeConfigFunc() {

    var self = this;
    self.currentCgId = null;
    self.currentGroupId = null;

    self.setActiveProfileFunc = [
      '$log',
      '$q',
      '$state',
      'Restangular',
      'appService',
    function(
      $log,
      $q,
      $state,
      Restangular,
      appService
    ) {

      var deferred = $q.defer();

      appService.setActiveProfile()
        .then(function() {
          deferred.resolve();
        })
        .catch(function() {
          deferred.reject();
        });

      return deferred.promise;
    }];

    self.getEquipmentFunc = [
      '$q',
      '$stateParams',
      'equipmentService',
    function(
      $q,
      $stateParams,
      equipmentService
    ) {
      var deferred = $q.defer();

      equipmentService.getEquipment($stateParams.equipmentId)
        .then(function(equipment) {
          equipmentService.setCurrentEquipment(equipment);
          deferred.resolve(equipment);
        });

      return deferred.promise;
    }];

    self.getComputerFunc = [
      '$q',
      '$state',
      '$stateParams',
      'appService',
      'computerService',
    function(
      $q,
      $state,
      $stateParams,
      appService,
      computerService
    ) {

      var deferred = $q.defer();

      self.currentCgId = $stateParams.computerId;

      computerService.getComputer($stateParams.computerId)
        .then(function(computer) {
          return deferred.resolve(computer);
        })
        .catch(function(ret) {
          if (404 === ret.status) {
            appService.alert({
              text: ret.data.message,
              doneFunction: function() {
                $state.transitionTo('loginSuccess.management');
              }
            });
          }
        });

      return deferred.promise;
    }];

    self.getDeviceInfo = [
      '$q',
      'SystemService',
    function(
      $q,
      SystemService
    ) {

      var deferred = $q.defer();

      SystemService.read()
        .then(function(data) {
          deferred.resolve(data);
        })
        .catch(function() {
          deferred.reject();
        });

      return deferred.promise;
    }];

    self.getModeInfo = [
      '$q',
      'wizardService',
    function(
      $q,
      wizardService
    ) {

      var deferred = $q.defer();

      wizardService.read()
        .then(function(data) {
          deferred.resolve(data);
        })
        .catch(function() {
          deferred.reject();
        });

      return deferred.promise;
    }];

    self.getCgGroupManagerFunc = [
      '$q',
      '$state',
      '$stateParams',
      'appService',
      'cgGroupManagerService',
    function(
      $q,
      $state,
      $stateParams,
      appService,
      cgGroupManagerService
      // groupService
    ) {

      var deferred = $q.defer();

      self.currentGroupId = $stateParams.groupId;

      cgGroupManagerService.read($stateParams.groupId)
        .then(function(groupData) {
          return deferred.resolve(groupData);
        })
        .catch(function(ret) {
          if (404 === ret.status) {
            appService.alert({
              text: ret.data.message,
              doneFunction: function() {
                $state.transitionTo('loginSuccess.management');
              }
            });
          }
        });

      return deferred.promise;

    }];

    self.getCurrentCgId = function() {
      return self.currentCgId;
    };

    self.getCurrentGroupId = function() {
      return self.currentGroupId;
    };

    self.$get = function() {
      return {
        getCurrentCgId: self.getCurrentCgId,
        getCurrentGroupId: self.getCurrentGroupId
      };
    };
  }

})();
