(function() {
  'use strict';

  angular.module('webapp')
    .service('selectFactory', selectFactory);

  function selectFactory(_, helperService) {

    return function() {

      var selectedIds = [];
      var toggledPages = [];
      var factory = {
        changeSelectedRow: changeSelectedRow,
        changeSelectedRows: changeSelectedRows,
        applySelectedStatus: applySelectedStatus,
        toggleAll: toggleAll,
        getToggleAllPageStatus: getToggleAllPageStatus,
        removeToggledPage: removeToggledPage
      };
      return factory;

      /**
       * Add selected id to cache selected ids.
       *
       * @param row
       * @return {undefined}
       */
      function addSelectedRow(row) {
        if (! helperService.inArray(selectedIds, row.id)) {
          selectedIds.push(row.id);
        }
      }

      /**
       * Remove selected id from cache selected ids.
       *
       * @param row {object} ngTable row
       * @return {undefined}
       */
      function removeSelectedRow(row) {
        selectedIds = _.without(selectedIds, row.id);
      }

      /**
       * Change selected rows status.
       *
       * @param row {object} ngTable row
       * @return {undefined}
       */
      function changeSelectedRow(row) {
        return row.$selected ? addSelectedRow(row) : removeSelectedRow(row);
      }

      function changeSelectedRows(rows) {
        _.each(rows, function(row) {
          changeSelectedRow(row);
        });
      }

      /**
       * Apply selected status from cache data.
       *
       * @param rows {array} ngTable rows
       * @return changed rows
       */
      function applySelectedStatus(rows) {

        return _.map(rows, function(row) {

          var index = selectedIds.indexOf(row.id),
              inArray = -1 !== index;

          if (inArray) {
            row.$selected = true;
          }
          return row;
        });
      }

      function addToggledPage(page) {
        if (! helperService.inArray(toggledPages, page)) {
          toggledPages.push(page);
        }
      }

      function removeToggledPage(page) {
        toggledPages = _.without(toggledPages, page);
      }

      function setToggledPage(page, status) {
        return status ? addToggledPage(page) : removeToggledPage(page);
      }

      function getToggleAllPageStatus(page) {
        return helperService.inArray(toggledPages, page);
      }

      function toggleAll(page, status, rows) {
        _.each(rows, function(row) {
          row.$selected = status;
        });
        changeSelectedRows(rows);
        setToggledPage(page, status);
      }
    };
  }

})();
