'use strict';

angular.module('webapp')
.factory('settingsFactory', function(localStorageService, userProvider, $state) {

  var getKey;

  getKey = function(group, id, options) {

    options = options || {};

    if (options.noState) {
      return [group, userProvider.activeProfile.email, id].join(':');
    }
    return [group, $state.current.name, userProvider.activeProfile.email, id].join(':');
  };

  return {
    getRelayListTab: function(id) {
      return localStorageService.get(getKey('mx-relay-list-tab', id));
    },
    setRelayListTab: function(id, tab) {
      localStorageService.set(getKey('mx-relay-list-tab', id), tab);
    },
    setSerialListTab: function(id, tab) {
      localStorageService.set(getKey('mx-serial-list-tab', id), tab);
    },
    getSerialListTab: function(id) {
      return localStorageService.get(getKey('mx-serial-list-tab', id));
    },
    setCellularListTab: function(id, tab) {
      localStorageService.set(getKey('mx-cellular-list-tab', id), tab);
    },
    getCellularListTab: function(id) {
      return localStorageService.get(getKey('mx-cellular-list-tab', id));
    },
    setDhcpServerListTab: function(id, tab) {
      localStorageService.set(getKey('mx-dhcp-server-list-tab', id), tab);
    },
    getDhcpServerListTab: function(id) {
      return localStorageService.get(getKey('mx-dhcp-server-list-tab', id));
    },
    setEthernetListTab: function(id, tab) {
      localStorageService.set(getKey('mx-ethernet-list-tab', id), tab);
    },
    getEthernetListTab: function(id) {
      return localStorageService.get(getKey('mx-ethernet-list-tab', id));
    },
    setControlCenterGroupId: function(id, groupId) {
      localStorageService.set(getKey('mx-control-center', id), groupId);
    },
    getControlCenterGroupId: function(id) {
      return localStorageService.get(getKey('mx-control-center', id));
    },
    setGroupListTab: function(id, tab) {
      localStorageService.set(getKey('mx-group-list-tab', id), tab);
    },
    getGroupListTab: function(id) {
      return localStorageService.get(getKey('mx-group-list-tab', id));
    },
    setManagementListTab: function(id, tab) {
      localStorageService.set(getKey('mx-management-list-tab', id), tab);
    },
    getManagementListTab: function(id) {
      return localStorageService.get(getKey('mx-management-list-tab', id));
    },
    setManagementComputerSettingsTab: function(id, tab) {
      localStorageService.set(getKey('mx-management-computers-settings-tab', id), tab);
    },
    getManagementComputerSettingsTab: function(id) {
      return localStorageService.get(getKey('mx-management-computers-settings-tab', id));
    },
    getNotificationNumber: function() {
      return localStorageService.get('mx-notification-number');
    },
    setNotificationNumber: function(number) {
      return localStorageService.set('mx-notification-number', number);
    },
    getRebootStatus: function() {
      return localStorageService.get('mx-system-reboot');
    },
    setRebootStatus: function(status) {
      return localStorageService.set('mx-system-reboot', status);
    },
    getFwUpgradeStatus: function() {
      return localStorageService.get('mx-system-fw-upgrade');
    },
    setFwUpgradeStatus: function(status) {
      return localStorageService.set('mx-system-fw-upgrade', status);
    },
    setMxWindowToggleStatus: function(id, status) {
      localStorageService.set(getKey('mx-window-toggle-status', id), status);
    },
    getMxWindowToggleStatus: function(id) {

      var value = localStorageService.get(getKey('mx-window-toggle-status', id));

      if ('string' === typeof value) {
        return ('true' === value);
      }
      return value;
    },
    setColumnOrder: function(whichColumn, id, arr) {
      localStorageService.set(getKey('mx-' + whichColumn + '-column-order', id), JSON.stringify(arr));
    },
    getColumnOrder: function(whichColumn, id) {
      // could return object
      return localStorageService.get(getKey('mx-' + whichColumn + '-column-order', id));
    }
  };
});
