;(function () {
  'use strict';

  angular.module('webapp')
    .factory('SystemService', SystemService);

  function SystemService($http, $q, $filter, API_BASE, _, notificationService) {

    // Members definition
    var bundleModel = {};

    bundleModel.read = getModel;
    bundleModel.update = updateModel;
    bundleModel.getSsh = getSshModel;
    bundleModel.updateSsh = updateSshModel;
    bundleModel.getSyslog = getSyslogModel;
    bundleModel.updateSyslog = updateSyslogModel;
    bundleModel.getWanAccess = getWanAccessModel;
    bundleModel.updateWanAccess = updateWanAccessModel;

    return bundleModel;

    // Member function implement
    function getModel() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/system/status')
      $http.get(API_BASE.generic + '/system/status')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.resolve();
        });
      return deferred.promise;
    }

    function updateModel(data) {
      var deferred = $q.defer();
      putModel(data).then(function (data) {
        deferred.resolve(data);
      })
      .catch(function () {
        deferred.reject();
      });
      return deferred.promise;
    }

    function putModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/system/status', data)
      $http.put(API_BASE.generic + '/system/status', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('HOSTNAME_HAS_BEEN_UPDATED_SUCCESSFULLY'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function () {
          notificationService.error(
            $filter('translate')('HOSTNAME_HAS_BEEN_UPDATED_FAIL'),
            'bottom_right'
          );
          deferred.reject();
        });
      return deferred.promise;
    }

    function getSshModel() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/system/services/ssh')
      $http.get(API_BASE.generic + '/system/services/ssh')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.resolve();
        });
      return deferred.promise;
    }

    function updateSshModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/system/services/ssh', data)
      $http.put(API_BASE.generic + '/system/services/ssh', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('SSH_HAS_BEEN_UPDATED_SUCCESSFULLY'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.resolve();
        });
      return deferred.promise;
    }

    function getSyslogModel() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/system/services/rsyslog')
      $http.get(API_BASE.generic + '/system/services/rsyslog')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.resolve();
        });
      return deferred.promise;
    }

    function updateSyslogModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/system/services/rsyslog', data)
      $http.put(API_BASE.generic + '/system/services/rsyslog', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('SYSLOG_UPDATED_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.resolve();
        });
      return deferred.promise;
    }

    function getWanAccessModel() {
      var deferred = $q.defer();
      // $http.get('http://localhost:8000/network/firewall/wan/https')
      $http.get(API_BASE.generic + '/network/firewall/wan/https')
        .then(function (res) {
          deferred.resolve(res.data);
        })
        .catch(function () {
          deferred.resolve();
        });
      return deferred.promise;
    }

    function updateWanAccessModel(data) {
      var deferred = $q.defer();
      // $http.put('http://localhost:8000/network/firewall/wan/https', data)
      $http.put(API_BASE.generic + '/network/firewall/wan/https', data)
        .then(function (res) {
          notificationService.success(
            $filter('translate')('WAN_ACCESS_UPDATED_SUCCESS'),
            'bottom_right'
          );
          deferred.resolve(res.data);
        })
        .catch(function (error) {
          notificationService.error(
            error.data.message,
            'bottom_right'
          );
          deferred.resolve();
        });
      return deferred.promise;
    }
  }
}());
