'use strict';

var express = require('express');
var userController = require('../user/user.controller');
var router = express.Router();
var ip = require('ip');
var SiteEav = require(__dirname + '/../../models').SiteEav;
var Promise = require('bluebird');

function getConfig() {
  return SiteEav
    .findAll()
    .then(function(objs) {
      if (objs.length === 0) {
        throw Error({message: 'site config not exist!'});
      }

      var config = {};
      for (var index in objs) {
        var obj = objs[index].toJSON();
        config[obj.key] = obj.value;
        process.env['SITE_' + obj.key] = obj.value;
      }

      return Promise.resolve(config);
    });
}

router.route('/')
  .all(userController.findTokenData, userController.findTokenUser)
  .get(function(req, res, next) {
    getConfig()
      .then(function(config) {
        res.json(config);
      })
      .catch(function(err) {
        res.status(500).json({message: err});
      });
  })
  .put(function(req, res, next) {
    var data = req.body || {};
    var promises = [];

    for (var key in data) {
      var p = SiteEav.update({
          value: data[key]
        }, {
          where: {key: key}
        });

      promises.push(p);
    }

    Promise.all(promises)
      .then(function() {
        return getConfig();
      })
      .then(function(config) {
        res.json(config);
      })
      .catch(function(err) {
        res.status(500).json({message: err});
      });
  });

module.exports = router;
