'use strict';

var express = require('express'),
    fs = require('fs'),
    log = require('sanji-logger')('webapp');

/**
 * Main application file
 */

// Set default node environment to development
process.env.NODE_ENV = process.env.NODE_ENV || 'development';

// Application Config
var config = require(__dirname + '/config/config');
var app = express();

// Express settings
require(__dirname + '/config/express')(app);

// Start socket.io service
var server;
if ('development' === process.env.NODE_ENV) {
  server = require('http').createServer(app);
} else {
  // Https options
  var constants = require('constants');
  var options = {
    key: fs.readFileSync(__dirname + '/../ssl/moxakey.pem'),
    cert: fs.readFileSync(__dirname + '/../ssl/moxacert.pem'),
    secureOptions: constants.SSL_OP_NO_SSLv3 | constants.SSL_OP_NO_SSLv2
  };
  server = require('https').createServer(options, app);
}
require(__dirname + '/services/socketService').init(server);

// webapp db settings
require('./config/db/schema')({force: false, preInsert: true})
  .then(function() {
    // Routing + sanji View
    require(__dirname + '/routes')(app);

    // Start server
    server.listen(config.port, config.ip, function () {
      log.info('Express server listening on %s:%d, in %s mode', config.ip, config.port, app.get('env'));
    });

    if ('production' === process.env.NODE_ENV || 'staging' === process.env.NODE_ENV) {
      require('http').createServer(function (req, res) {
        res.writeHead(301, { "Location": "https://" + req.headers['host'] + req.url });
        res.end();
      }).listen(80);
    }
  })
  .catch(function(err) {
    log.error(err);
  });

// Expose app
exports = module.exports = app;
