'use strict';

var db = require(__dirname + '/../../models'),
    config = require(__dirname + '/../../config/config'),
    Promise = require('bluebird'),
    q = require('q'),
    ip = require('ip'),
    log = require('sanji-logger')('webapp'),
    User = db.User,
    UserEav = db.UserEav,
    SiteEav = db.SiteEav,
    appService = require(__dirname + '/../../services/appService');

function createAdmin() {

  var deferred = q.defer();
  var userModel;

  User.findOrCreate({
    where: {
      email: 'admin@moxa.com'
    },
    defaults: {
      email: 'admin@moxa.com',
      password: appService.hashPassword('admin1234')
    }
  })
  .spread(function(user) {
    userModel = user;
    return UserEav.findOrCreate({
      where: {
        key: 'metadata::admin@moxa.com',
      },
      defaults: {
        key: 'metadata::admin@moxa.com',
        value: JSON.stringify({
          image: '',
          channelChart: []
        }),
        UserId: user.get('id')
      }
    });
  })
  .then(function() {
    if (!deferred.promise.isFulfilled()) {
      deferred.resolve();
    }
  })
  .catch(function(err) {
    log.error('createAdmin() failed', err);
    deferred.reject(err);
  });

  return deferred.promise;
}

function createSite() {
  var env = process.env.NODE_ENV;
  var url = "";

  if ('production' === env || 'staging' === env) {
    url += 'https://';
  } else {
    url += 'http://';
  }
  url += ip.address() + ":" + config.port;

  return SiteEav.findOrCreate({
    where: {
      key: 'URL'
    },
    defaults: {
      key: 'URL',
      value: url
    }
  })
  .spread(function(obj, created) {
    // Update global config
    config.siteurl = obj.get('value');
    process.env.SITE_URL = config.siteurl;

    return Promise.resolve(obj);
  });
}

module.exports = function() {

  return createAdmin()
    // .then(function() {
      // return createSite();
    // })
    .catch(function(err) {
      log.error(err);
    });
};
