'use strict';

var express = require('express'),
    // spdyPush = require('spdy-referrer-push'),
    path = require('path'),
    favicon = require('serve-favicon'),
    compression = require('compression'),
    helmet = require('helmet'),
    morgan = require('morgan'),
    bodyParser = require('body-parser'),
    methodOverride = require('method-override'),
    cookieParser = require('cookie-parser'),
    errorHandler = require('errorhandler'),
    config = require('./config');

/**
 * Express configuration
 */
module.exports = function(app) {

  var env = app.get('env');

  app.disable('x-powered-by');

  if ('development' === env || 'debug' === env) {

    var version = require(__dirname + '/../../node_modules/browser-sync/package.json').version;

    app.use(require('connect-livereload')({
      src: "' + (location.protocol || 'http:') + '//' + (location.hostname || 'localhost') + ':3000/browser-sync/browser-sync-client." + version + ".js?v=1"
    }));

    // Disable caching of scripts for easier testing
    app.use(function noCache(req, res, next) {
      if (req.url.indexOf('/scripts/') === 0) {
        res.header('Cache-Control', 'no-cache, no-store, must-revalidate');
        res.header('Pragma', 'no-cache');
        res.header('Expires', 0);
      }
      next();
    });

    app.use(compression());
    app.use(express.static(path.join(config.root, '.tmp')));
    app.use(function(req, res, next) {
      if ('/' === req.path) {
        next();
      } else {
        var expressStatic = express.static(path.join(config.root, 'client'));
        expressStatic(req, res, next);
      }
    });
    app.set('views', config.root + '/client');
  }

  if ('production' === env || 'staging' === env) {
    app.enable('trust proxy');
    app.use(favicon(path.join(config.root, 'public', 'favicon.ico')));
    app.use(compression());
    app.use(express.static(path.join(config.root, 'public'), {etag: false, maxage: 86400000}));
    app.set('views', config.root + '/public/views');
    app.use(helmet());
    app.use(morgan('combined'));
  } else {
    app.use(morgan('dev'));
  }

  app.use(cookieParser('S3CRE7'));
  app.engine('html', require('ejs').renderFile);
  app.set('view engine', 'html');
  // parse application/x-www-form-urlencoded
  app.use(bodyParser.urlencoded({ extended: false }));
  // parse application/json
  app.use(bodyParser.json());
  app.use(methodOverride());
};
