var _ = require('lodash'),
    log = require('sanji-logger')('webapp');

module.exports.onError = function(err, req, res, next) {

  switch (err.type) {
    case 'InvalidArgument':
      return res.status(400)
        .send({type: 'InvalidArgument', message: err.message});
    case 'MissingArgument':
      return res.status(400)
        .send({type: 'MissingArgument', message: err.message});
    case 'ObjectNotFound':
      return res.status(404)
        .send({type: 'ObjectNotFound', message: err.message});
    case 'DuplicatedEntry':
      return res.status(409)
        .send({type: 'DuplicatedEntry', message: err.message});
  }

  log.error(err);

  return res.status(500)
    .send({message: 'An unhandled error occurred.'});
};

exports.checkParams = function() {

  var args = arguments;

  return function(req, res, next) {
    _.each(args, function(arg) {
      if (! req.body[arg]) {
        return next({
          type: 'MissingArgument',
          message: 'Argument ' + arg + ' is missing.'
        });
      }
    });
    return next();
  };
};

exports.checkQuery = function() {

  var args = arguments;

  return function(req, res, next) {
    _.each(args, function(arg) {
      if (! req.query[arg]) {
        return next({
          type: 'MissingArgument',
          message: 'Query param ' + arg + ' is missing.'
        });
      }
    });
    return next();
  };
};
