'use strict';

var _ = require('lodash');
var Bluebird = require('bluebird');
var Notification;
var MAX_NOTIFICATION_NUM = 255;

module.exports = NotificationModel;

function NotificationModel(sequelize, DataTypes) {
  Notification = sequelize.define('Notification', {
    code: {type: DataTypes.TEXT},
    type: {type: DataTypes.TEXT},
    message: {type: DataTypes.TEXT},
    variables: {
      type: DataTypes.TEXT,
      get: function() {
        return JSON.parse(this.getDataValue('variables'));
      },
      set: function(v) {
        var tmp = null;
        try {
          tmp = JSON.stringify(v)
        } catch(error) {
          throw new Error('Notification model json stringify error.');
        }
        return this.setDataValue('variables', tmp);
      }
    },
    createdAt: {
      type: DataTypes.DATE,
      get: function() {
        return +new Date(this.getDataValue('createdAt'));
      },
      set: function(v) {
        return this.setDataValue('createdAt', new Date(v));
      }
    }
  }, {
    hooks: {
      beforeCreate: function() {
        return Notification.findAll()
          .then(function(results) {
            if (MAX_NOTIFICATION_NUM === results.length) {
              results[MAX_NOTIFICATION_NUM - 1].destroy({force: true});
            }
          });
      }
    }
  });

  return Notification;
}
