'use strict';

var fs = require('fs'),
    q = require('q'),
    _ = require('lodash'),
    Bluebird = require('bluebird'),
    crypto = require('crypto'),
    log = require('sanji-logger')('webapp'),
    util = require('util'),
    jwtConst = require(__dirname + '/../constants/jwt'),
    mkdirp = require('mkdirp'),
    User;

module.exports = function(sequelize, DataTypes) {

  User = sequelize.define('User', {
    email: {type:  DataTypes.STRING(100), allowNull: false, unique: true},
    password: {type:  DataTypes.STRING(60), allowNull: false},
    language: {type:  DataTypes.STRING(60), defaultValue: 'en'}
  }, {
    classMethods: {
      getUserByEmail: function(email) {
        return User.find({where: {email: email}});
      }
    },
    instanceMethods: {

      log: function(req, action) {

        return this.createUserLog({
          action: action + "",
          ip: req.connection.remoteAddress
        });
      },

      getProfile: function(accessToken) {
        var deferred = q.defer();
        var profile = {
          accessToken: accessToken,
          email: this.get('email'),
          userId: this.get('id')
        };

        this.getMetaData()
          .then(function(meta) {
            profile.meta = meta;
            deferred.resolve(profile);
          });

        return deferred.promise;
      },

      toJSON: function() {
        var values = this.get();
        values.password = null;
        if (this.User) {
          values.User = this.User.toJSON();
        }
        return values;
      },

      getMetaData: function() {
        var self = this;
        var deferred = q.defer();

        self.getUserEav({attributes: ['value']})
          .then(function(data) {
            var meta = JSON.parse(data.get('value'));
            deferred.resolve(meta);
          })
          .catch(function() {
            deferred.reject();
          });

        return deferred.promise;
      },

      getProfileImage: function() {

        var self = this;
        var deferred = q.defer();

        self.getUserEavs({attributes: ['value']})
          .spread(function(data) {
            deferred.resolve(data.get('value'));
          })
          .catch(function() {
            deferred.resolve('');
          });

        return deferred.promise;
        // userEav = _.find(self.userEavs, function(eav) {
          // return eav.get('key') === (self.get('id') + ':profile:image');
        // });

        // if (userEav) {
          // return userEav.get('value');
        // }
        // return '';
      },

      updateProfileImage: function(db, profileImage) {

        var dir, user, deferred, path, internalPath, folder;

        deferred = q.defer();
        user = this;
        folder = 'client';

        if ('production' === process.env.NODE_ENV || 'staging' === process.env.NODE_ENV) {
          folder = 'public';
        }

        internalPath = __dirname + '/../../' + folder;
        path = '/assets/images/users/' + user.get('id');
        dir = internalPath + path;

        mkdirp(dir, function(err) {

          if (err) {
            throw new Error('Error creating directory');
          }

          var base64Data = profileImage.replace(/^data:image\/png;base64,/, '');
          var filename = crypto.createHash('md5').update(base64Data).digest('hex') + '.png';

          fs.writeFile(dir + '/' + filename, base64Data, 'base64', function(err) {

            if (err) {
              throw new Error('Error creating user profile image');
            }

            user.getUserEav()
            .then(function(userEav) {
              var metaData = JSON.parse(userEav.get('value'));
              metaData.image = path + '/' + filename;
              return userEav.updateAttributes({value: JSON.stringify(metaData)});
            })
            .then(function(userEav) {
              deferred.resolve(userEav);
            })
            .catch(function(err) {
              log.error(err);
              deferred.reject();
            });

          });
        });

        return deferred.promise;
      }
    }
  });

  return User;
};
