var bcrypt = require('bcrypt'),
    q = require('q'),
    db = require(__dirname + '/../models'),
    log = require('sanji-logger')('webapp'),
    _ = require('lodash'),
    appService;

appService = module.exports = {
  hashPassword: function(password) {
    return bcrypt.hashSync(password, bcrypt.genSaltSync(10));
  },
  verifyPassword: function(password, hash) {
    return bcrypt.compareSync(password, hash);
  },
  setJsonFunc: function(property) {
    return function(value) {
      return this.setDataValue(property, JSON.stringify(value));
    };
  },
  getJsonFunc: function(property) {
    return function() {

      var value = this.getDataValue(property);

      try {
        return JSON.parse(value);
      } catch(err) {
        if (err instanceof SyntaxError) {
          return value;
        }
        throw err;
      }
    };
  },
  applyEavAttributes: function(instance, eavProperty, eavValueProperty) {

    _.each(instance.get(eavProperty)[0].get(eavValueProperty), function(value, key) {
      if (! instance.dataValues[key]) {
        instance.dataValues[key] = value;
      }
    });
    return instance;
  }
};
