var log = require('sanji-logger')('webapp'),
    _ = require('lodash'),
    moment = require('moment'),
    validator = require('validator'),
    helperService;

helperService = module.exports = {
  time: function() {
    return Math.floor(new Date().getTime() / 1000);
  },
  getRandomInt: function(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
  },
  inRange: function(value, min, max) {
    return (min <= value) && (value <= max);
  },
  inArray: function(arr, elem) {
    return -1 !== _.indexOf(arr, elem);
  },
  isUshort: function(value) {
    return validator.isInt(value) && (helperService.inRange(value, 0, 65535));
  },
  isShort: function(value) {
    return validator.isInt(value) && (helperService.inRange(value, -32768, 32767));
  },
  isUint32: function(value) {
    return validator.isInt(value) && (helperService.inRange(value, 0, 2147483647));
  },
  isInt32: function(value) {
    return validator.isInt(value) && (helperService.inRange(value, -2147483647, 2147483647));
  },
  isFloat: function(value) {
    return validator.isFloat(value);
  },
  isString: function(value) {
    return _.isString(value);
  },
  getValidatorByType: function(type) {

    switch (type) {
      case 0:    // ushort
        return helperService.isUshort;
      case 1:    // short
        return helperService.isShort;
      case 2:    // uint32
        return helperService.isUint32;
      case 3:    // int32
        return helperService.isInt32;
      case 4:    // float
        return helperService.isFloat;
      case 5:    // string
        return helperService.isString;
      default:
        return null;
    }
  },

  // NOTE: this is for mxc-route.js
  checkKeys: function() {
    var keys = [].slice.call(arguments);
    return function(req, res, next) {

      for (var i = 0, len = keys.length; i < len; i++) {
        var key = keys[i];
        if (! req.body.hasOwnProperty(key)) {
          return next('Property ' + key + ' does not exists.');
        }
      }
      return next();
    };
  },

  getType: function(value) {
    if (validator.isFloat(value)) {
      return 'float';
    }
    if (validator.isInt(value)) {
      return 'int';
    }
    return 'string';
  },

  getTimestamp: function(time) {
    //  miliseconds
    return moment(time).valueOf();
  }
};
