'use strict';

var log = require('sanji-logger')('webapp');
var _ = require('lodash');
var Server = require('socket.io');
var db = require(__dirname + '/../models');
var Notification = db.Notification;

module.exports = new SocketService();

SocketService.prototype.init = init;
SocketService.prototype.emit = emit;
SocketService.prototype.getIoInstance = getIoInstance;

function SocketService() {
  this.io = null;
}

function init(server) {
  this.io = new Server(server);
  this.io.on('connection', function(socket){
    log.info('A user is connected.');
  });
}

function getIoInstance() {
  return this.io;
}

function emit(eventName, data) {
  if  (! eventName) {
    log.error('Server push error, event name error');
    return;
  }

  if (! data) {
    log.error('Server push error, data error');
    return;
  }

  var self = this;

  Notification.create(data)
    .then(function(notification) {
      notification.hasRead = false;
      self.io.emit(eventName, notification);
    })
    .catch(function(error) {
      log.error('Notification create error', error);
    });
}
